@@ -24,7 +24,7 @@
  * - central directory
  */
 
-#include "busybox.h"
+#include "libbb.h"
 #include "unarchive.h"
 
 #define ZIP_FILEHEADER_MAGIC		SWAP_LE32(0x04034b50)
@@ -93,6 +93,7 @@ static int unzip_extract(zip_header_t *zip_header, int src_fd, int dst_fd)
 	return 0;
 }
 
+int unzip_main(int argc, char **argv);
 int unzip_main(int argc, char **argv)
 {
 	zip_header_t zip_header;
@@ -133,7 +134,8 @@ int unzip_main(int argc, char **argv)
 				break;
 
 			case 1 : /* The zip file */
-				src_fn = xstrndup(optarg, strlen(optarg)+4);
+				src_fn = xmalloc(strlen(optarg)+4);
+				strcpy(src_fn, optarg);
 				opt_range++;
 				break;
 
@@ -194,7 +196,7 @@ int unzip_main(int argc, char **argv)
 			src_fd = open(src_fn, O_RDONLY);
 		}
 		if (src_fd == -1) {
-			src_fn[orig_src_fn_len] = 0;
+			src_fn[orig_src_fn_len] = '\0';
 			bb_error_msg_and_die("cannot open %s, %s.zip, %s.ZIP", src_fn, src_fn, src_fn);
 		}
 	}
