@@ -10,19 +10,20 @@
 
 /* no options, no getopt */
 
-#include "busybox.h"
+#include "libbb.h"
 
 /* From <linux/vt.h> */
 enum { VT_DISALLOCATE = 0x5608 }; /* free memory associated to vt */
 
-int deallocvt_main(int argc, char *argv[])
+int deallocvt_main(int argc, char **argv);
+int deallocvt_main(int argc, char **argv)
 {
 	/* num = 0 deallocate all unused consoles */
 	int num = 0;
 
 	switch (argc) {
 	case 2:
-		num = xatoul_range(argv[1], 1, 63);
+		num = xatou_range(argv[1], 1, 63);
 		/* Fallthrough */
 	case 1:
 		break;
@@ -30,8 +31,7 @@ int deallocvt_main(int argc, char *argv[])
 		bb_show_usage();
 	}
 
-	if (-1 == ioctl(get_console_fd(), VT_DISALLOCATE, num)) {
-		bb_perror_msg_and_die("VT_DISALLOCATE");
-	}
+	/* double cast suppresses "cast to ptr from int of different size */
+	xioctl(get_console_fd(), VT_DISALLOCATE, (void *)(ptrdiff_t)num);
 	return EXIT_SUCCESS;
 }
