@@ -8,7 +8,7 @@
  *
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
 #define BINARY_KEYMAP_MAGIC "bkeymap"
 
@@ -25,6 +25,7 @@ struct kbentry {
 #define NR_KEYS         128
 #define MAX_NR_KEYMAPS  256
 
+int loadkmap_main(int argc, char **argv);
 int loadkmap_main(int argc, char **argv)
 {
 	struct kbentry ke;
