@@ -9,9 +9,10 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
-extern int setlogcons_main(int argc, char **argv)
+int setlogcons_main(int argc, char **argv);
+int setlogcons_main(int argc, char **argv)
 {
 	struct {
 		char fn;
@@ -24,8 +25,7 @@ extern int setlogcons_main(int argc, char **argv)
 	if (argc == 2)
 		arg.subarg = xatoul_range(argv[1], 0, 63);
 
-	if (ioctl(xopen(VC_1, O_RDONLY), TIOCLINUX, &arg))
-		bb_perror_msg_and_die("TIOCLINUX");
+	xioctl(xopen(VC_1, O_RDONLY), TIOCLINUX, &arg);
 
 	return 0;
 }
