@@ -10,21 +10,30 @@
 /* BB_AUDIT SUSv3 compliant */
 /* http://www.opengroup.org/onlinepubs/007904975/utilities/cat.html */
 
-#include "busybox.h"
+#include "libbb.h"
+
+/* This is a NOFORK applet. Be very careful! */
+
 
 int bb_cat(char **argv)
 {
-	static char *const argv_dash[] = { "-", NULL };
-	FILE *f;
+	static const char *const argv_dash[] = { "-", NULL };
+
+	int fd;
 	int retval = EXIT_SUCCESS;
 
-	if (!*argv) argv = (char**) &argv_dash;
+	if (!*argv)
+		argv = (char**) &argv_dash;
 
 	do {
-		f = fopen_or_warn_stdin(*argv);
-		if (f) {
-			off_t r = bb_copyfd_eof(fileno(f), STDOUT_FILENO);
-			fclose_if_not_stdin(f);
+		fd = STDIN_FILENO;
+		if (!LONE_DASH(*argv))
+			fd = open_or_warn(*argv, O_RDONLY);
+		if (fd >= 0) {
+			/* This is not an xfunc - never exits */
+			off_t r = bb_copyfd_eof(fd, STDOUT_FILENO);
+			if (fd != STDIN_FILENO)
+				close(fd);
 			if (r >= 0)
 				continue;
 		}
@@ -34,9 +43,10 @@ int bb_cat(char **argv)
 	return retval;
 }
 
+int cat_main(int argc, char **argv);
 int cat_main(int argc, char **argv)
 {
-	getopt32(argc, argv, "u");
+	getopt32(argv, "u");
 	argv += optind;
 	return bb_cat(argv);
 }
