@@ -7,15 +7,16 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-/* BB_AUDIT SUSv3 defects - unsupported options -H, -L, and -P. */
+/* BB_AUDIT SUSv3 defects - none? */
 /* BB_AUDIT GNU defects - unsupported long options. */
 /* http://www.opengroup.org/onlinepubs/007904975/utilities/chgrp.html */
 
-#include "busybox.h"
+#include "libbb.h"
 
-/* FIXME - move to .h */
-extern int chown_main(int argc, char **argv);
+/* This is a NOEXEC applet. Be very careful! */
 
+
+int chgrp_main(int argc, char **argv);
 int chgrp_main(int argc, char **argv)
 {
 	/* "chgrp [opts] abc file(s)" == "chown [opts] :abc file(s)" */
