@@ -14,17 +14,20 @@
 
 #include "libbb.h"
 
-enum ConvType {
+enum {
 	CT_UNIX2DOS = 1,
 	CT_DOS2UNIX
-} ConvType;
+};
 
 /* if fn is NULL then input is stdin and output is stdout */
-static int convert(char *fn)
+static int convert(char *fn, int conv_type)
 {
 	FILE *in, *out;
 	int i;
+#define name_buf bb_common_bufsiz1
 
+	in = stdin;
+	out = stdout;
 	if (fn != NULL) {
 		in = xfopen(fn, "rw");
 		/*
@@ -32,31 +35,24 @@ static int convert(char *fn)
 		   permissions 0666 for glibc 2.0.6 and earlier or
 		   0600 for glibc 2.0.7 and later.
 		 */
-		snprintf(bb_common_bufsiz1, sizeof(bb_common_bufsiz1), "%sXXXXXX", fn);
-		/*
-		   sizeof bb_common_bufsiz1 is 4096, so it should be big enough to
-		   hold the full path.  However if the output is truncated the
-		   subsequent call to mkstemp would fail.
-		 */
-		i = mkstemp(&bb_common_bufsiz1[0]);
-		if (i == -1 || chmod(bb_common_bufsiz1, 0600) == -1) {
+		snprintf(name_buf, sizeof(name_buf), "%sXXXXXX", fn);
+		i = mkstemp(&name_buf[0]);
+		if (i == -1 || chmod(name_buf, 0600) == -1) {
 			bb_perror_nomsg_and_die();
 		}
 		out = fdopen(i, "w+");
 		if (!out) {
 			close(i);
-			remove(bb_common_bufsiz1);
+			remove(name_buf);
+			return -2;
 		}
-	} else {
-		in = stdin;
-		out = stdout;
 	}
 
 	while ((i = fgetc(in)) != EOF) {
 		if (i == '\r')
 			continue;
 		if (i == '\n') {
-			if (ConvType == CT_UNIX2DOS)
+			if (conv_type == CT_UNIX2DOS)
 				fputc('\r', out);
 			fputc('\n', out);
 			continue;
@@ -67,14 +63,14 @@ static int convert(char *fn)
 	if (fn != NULL) {
 		if (fclose(in) < 0 || fclose(out) < 0) {
 			bb_perror_nomsg();
-			remove(bb_common_bufsiz1);
+			remove(name_buf);
 			return -2;
 		}
 		/* Assume they are both on the same filesystem (which
 		 * should be true since we put them into the same directory
 		 * so we _should_ be ok, but you never know... */
-		if (rename(bb_common_bufsiz1, fn) < 0) {
-			bb_perror_msg("cannot rename '%s' as '%s'", bb_common_bufsiz1, fn);
+		if (rename(name_buf, fn) < 0) {
+			bb_perror_msg("cannot rename '%s' as '%s'", name_buf, fn);
 			return -1;
 		}
 	}
@@ -85,32 +81,31 @@ static int convert(char *fn)
 int dos2unix_main(int argc, char **argv);
 int dos2unix_main(int argc, char **argv)
 {
-	int o;
+	int o, conv_type;
 
 	/* See if we are supposed to be doing dos2unix or unix2dos */
 	if (applet_name[0] == 'd') {
-		ConvType = CT_DOS2UNIX;	/*2 */
+		conv_type = CT_DOS2UNIX;	/* 2 */
 	} else {
-		ConvType = CT_UNIX2DOS;	/*1 */
+		conv_type = CT_UNIX2DOS;	/* 1 */
 	}
-	/* -u and -d are mutally exclusive */
-	opt_complementary = "?:u--d:d--u";
-	/* process parameters */
-	/* -u convert to unix */
-	/* -d convert to dos  */
-	o = getopt32(argc, argv, "du");
+
+	/* -u convert to unix, -d convert to dos */
+	opt_complementary = "u--d:d--u"; /* mutually exclusive */
+	o = getopt32(argv, "du");
 
 	/* Do the conversion requested by an argument else do the default
 	 * conversion depending on our name.  */
 	if (o)
-		ConvType = o;
+		conv_type = o;
 
-	if (optind < argc) {
-		while (optind < argc)
-			if ((o = convert(argv[optind++])) < 0)
-				break;
-	} else
-		o = convert(NULL);
+	do {
+		/* might be convert(NULL) if there is no filename given */
+		o = convert(argv[optind], conv_type);
+		if (o < 0)
+			break;
+		optind++;
+	} while (optind < argc);
 
 	return o;
 }
