@@ -23,7 +23,7 @@
  * 4) Fixed busybox bug #1284 involving long overflow with human_readable.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
 #if ENABLE_FEATURE_HUMAN_READABLE
 # if ENABLE_FEATURE_DU_DEFAULT_BLOCKSIZE_1K
@@ -48,7 +48,7 @@ static int one_file_system;
 static dev_t dir_dev;
 
 
-static void print(long size, const char * const filename)
+static void print(long size, const char *const filename)
 {
 	/* TODO - May not want to defer error checking here. */
 #if ENABLE_FEATURE_HUMAN_READABLE
@@ -64,7 +64,7 @@ static void print(long size, const char * const filename)
 }
 
 /* tiny recursive du */
-static long du(const char * const filename)
+static long du(const char *const filename)
 {
 	struct stat statbuf;
 	long sum;
@@ -101,7 +101,7 @@ static long du(const char * const filename)
 
 	if (statbuf.st_nlink > count_hardlinks) {
 		/* Add files/directories with links only once */
-		if (is_in_ino_dev_hashtable(&statbuf, NULL)) {
+		if (is_in_ino_dev_hashtable(&statbuf)) {
 			return 0;
 		}
 		add_to_ino_dev_hashtable(&statbuf, NULL);
@@ -143,6 +143,7 @@ static long du(const char * const filename)
 	return sum;
 }
 
+int du_main(int argc, char **argv);
 int du_main(int argc, char **argv)
 {
 	long total;
@@ -169,7 +170,7 @@ int du_main(int argc, char **argv)
 	 */
 #if ENABLE_FEATURE_HUMAN_READABLE
 	opt_complementary = "h-km:k-hm:m-hk:H-L:L-H:s-d:d-s";
-	opt = getopt32(argc, argv, "aHkLsx" "d:" "lc" "hm", &smax_print_depth);
+	opt = getopt32(argv, "aHkLsx" "d:" "lc" "hm", &smax_print_depth);
 	if (opt & (1 << 9)) {
 		/* -h opt */
 		disp_hr = 0;
@@ -184,7 +185,7 @@ int du_main(int argc, char **argv)
 	}
 #else
 	opt_complementary = "H-L:L-H:s-d:d-s";
-	opt = getopt32(argc, argv, "aHkLsx" "d:" "lc", &smax_print_depth);
+	opt = getopt32(argv, "aHkLsx" "d:" "lc", &smax_print_depth);
 #if !ENABLE_FEATURE_DU_DEFAULT_BLOCKSIZE_1K
 	if (opt & (1 << 2)) {
 		/* -k opt */
@@ -222,7 +223,7 @@ int du_main(int argc, char **argv)
 	/* go through remaining args (if any) */
 	argv += optind;
 	if (optind >= argc) {
-		*--argv = ".";
+		*--argv = (char*)".";
 		if (slink_depth == 1) {
 			slink_depth = 0;
 		}
@@ -234,10 +235,8 @@ int du_main(int argc, char **argv)
 		total += du(*argv);
 		slink_depth = slink_depth_save;
 	} while (*++argv);
-#if ENABLE_FEATURE_CLEAN_UP
-	reset_ino_dev_hashtable();
-#endif
-
+	if (ENABLE_FEATURE_CLEAN_UP)
+		reset_ino_dev_hashtable();
 	if (print_final_total) {
 		print(total, "total");
 	}
