@@ -25,7 +25,7 @@
 
 /* no getopt needed */
 
-#include "busybox.h"
+#include "libbb.h"
 #include "xregex.h"
 
 /* The kinds of value we can have.  */
@@ -62,39 +62,14 @@ struct valinfo {
 typedef struct valinfo VALUE;
 
 /* The arguments given to the program, minus the program name.  */
-static char **args;
+struct globals {
+	char **args;
+};
+#define G (*(struct globals*)&bb_common_bufsiz1)
 
-static VALUE *docolon(VALUE * sv, VALUE * pv);
+/* forward declarations */
 static VALUE *eval(void);
-static VALUE *int_value(arith_t i);
-static VALUE *str_value(char *s);
-static int nextarg(char *str);
-static int null(VALUE * v);
-static int toarith(VALUE * v);
-static void freev(VALUE * v);
-static void tostring(VALUE * v);
 
-int expr_main(int argc, char **argv)
-{
-	VALUE *v;
-
-	if (argc == 1) {
-		bb_error_msg_and_die("too few arguments");
-	}
-
-	args = argv + 1;
-
-	v = eval();
-	if (*args)
-		bb_error_msg_and_die("syntax error");
-
-	if (v->type == integer)
-		printf("%" PF_REZ "d\n", PF_REZ_TYPE v->u.i);
-	else
-		puts(v->u.s);
-
-	fflush_stdout_and_exit(null(v));
-}
 
 /* Return a VALUE for I.  */
 
@@ -110,7 +85,7 @@ static VALUE *int_value(arith_t i)
 
 /* Return a VALUE for S.  */
 
-static VALUE *str_value(char *s)
+static VALUE *str_value(const char *s)
 {
 	VALUE *v;
 
@@ -135,8 +110,8 @@ static int null(VALUE * v)
 {
 	if (v->type == integer)
 		return v->u.i == 0;
-	else				/* string: */
-		return v->u.s[0] == '\0' || LONE_CHAR(v->u.s, '0');
+	/* string: */
+	return v->u.s[0] == '\0' || LONE_CHAR(v->u.s, '0');
 }
 
 /* Coerce V to a string value (can't fail).  */
@@ -151,7 +126,7 @@ static void tostring(VALUE * v)
 
 /* Coerce V to an integer value.  Return 1 on success, 0 on failure.  */
 
-static int toarith(VALUE * v)
+static bool toarith(VALUE * v)
 {
 	if (v->type == string) {
 		arith_t i;
@@ -172,11 +147,11 @@ static int toarith(VALUE * v)
 /* Return nonzero if the next token matches STR exactly.
    STR must not be NULL.  */
 
-static int nextarg(char *str)
+static bool nextarg(const char *str)
 {
-	if (*args == NULL)
+	if (*G.args == NULL)
 		return 0;
-	return strcmp(*args, str) == 0;
+	return strcmp(*G.args, str) == 0;
 }
 
 /* The comparison operator handling functions.  */
@@ -193,16 +168,16 @@ static int cmp_common(VALUE * l, VALUE * r, int op)
 		cmpval = l->u.i - r->u.i;
 	if (op == '<')
 		return cmpval < 0;
-	else if (op == ('L' + 'E'))
+	if (op == ('L' + 'E'))
 		return cmpval <= 0;
-	else if (op == '=')
+	if (op == '=')
 		return cmpval == 0;
-	else if (op == '!')
+	if (op == '!')
 		return cmpval != 0;
-	else if (op == '>')
+	if (op == '>')
 		return cmpval > 0;
-	else				/* >= */
-		return cmpval >= 0;
+	/* >= */
+	return cmpval >= 0;
 }
 
 /* The arithmetic operator handling functions.  */
@@ -244,15 +219,14 @@ static VALUE *docolon(VALUE * sv, VALUE * pv)
 	tostring(pv);
 
 	if (pv->u.s[0] == '^') {
-		fprintf(stderr, "\
+		bb_error_msg("\
 warning: unportable BRE: `%s': using `^' as the first character\n\
 of a basic regular expression is not portable; it is being ignored", pv->u.s);
 	}
 
 	memset(&re_buffer, 0, sizeof(re_buffer));
 	memset(re_regs, 0, sizeof(*re_regs));
-	if (regcomp(&re_buffer, pv->u.s, 0) != 0)
-		bb_error_msg_and_die("invalid regular expression");
+	xregcomp(&re_buffer, pv->u.s, 0);
 
 	/* expr uses an anchored pattern match, so check that there was a
 	 * match and that the match starts at offset 0. */
@@ -271,6 +245,7 @@ of a basic regular expression is not portable; it is being ignored", pv->u.s);
 		else
 			v = int_value(0);
 	}
+//FIXME: sounds like here is a bit missing: regfree(&re_buffer);
 	return v;
 }
 
@@ -280,53 +255,59 @@ static VALUE *eval7(void)
 {
 	VALUE *v;
 
-	if (!*args)
+	if (!*G.args)
 		bb_error_msg_and_die("syntax error");
 
 	if (nextarg("(")) {
-		args++;
+		G.args++;
 		v = eval();
 		if (!nextarg(")"))
 			bb_error_msg_and_die("syntax error");
-		args++;
+		G.args++;
 		return v;
 	}
 
 	if (nextarg(")"))
 		bb_error_msg_and_die("syntax error");
 
-	return str_value(*args++);
+	return str_value(*G.args++);
 }
 
 /* Handle match, substr, index, length, and quote keywords.  */
 
 static VALUE *eval6(void)
 {
-	VALUE *l, *r, *v, *i1, *i2;
+	static const char keywords[] ALIGN1 =
+		"quote\0""length\0""match\0""index\0""substr\0";
 
-	if (nextarg("quote")) {
-		args++;
-		if (!*args)
+	VALUE *r, *i1, *i2;
+	VALUE *l = l; /* silence gcc */
+	VALUE *v = v; /* silence gcc */
+	int key = *G.args ? index_in_strings(keywords, *G.args) + 1 : 0;
+
+	if (key == 0) /* not a keyword */
+		return eval7();
+	G.args++; /* We have a valid token, so get the next argument.  */
+	if (key == 1) { /* quote */
+		if (!*G.args)
 			bb_error_msg_and_die("syntax error");
-		return str_value(*args++);
-	} else if (nextarg("length")) {
-		args++;
+		return str_value(*G.args++);
+	}
+	if (key == 2) { /* length */
 		r = eval6();
 		tostring(r);
 		v = int_value(strlen(r->u.s));
 		freev(r);
-		return v;
-	} else if (nextarg("match")) {
-		args++;
+	} else
 		l = eval6();
+
+	if (key == 3) { /* match */
 		r = eval6();
 		v = docolon(l, r);
 		freev(l);
 		freev(r);
-		return v;
-	} else if (nextarg("index")) {
-		args++;
-		l = eval6();
+	}
+	if (key == 4) { /* index */
 		r = eval6();
 		tostring(l);
 		tostring(r);
@@ -335,16 +316,14 @@ static VALUE *eval6(void)
 			v->u.i = 0;
 		freev(l);
 		freev(r);
-		return v;
-	} else if (nextarg("substr")) {
-		args++;
-		l = eval6();
+	}
+	if (key == 5) { /* substr */
 		i1 = eval6();
 		i2 = eval6();
 		tostring(l);
 		if (!toarith(i1) || !toarith(i2)
-			|| i1->u.i > (arith_t) strlen(l->u.s)
-			|| i1->u.i <= 0 || i2->u.i <= 0)
+		 || i1->u.i > (arith_t) strlen(l->u.s)
+		 || i1->u.i <= 0 || i2->u.i <= 0)
 			v = str_value("");
 		else {
 			v = xmalloc(sizeof(VALUE));
@@ -354,9 +333,9 @@ static VALUE *eval6(void)
 		freev(l);
 		freev(i1);
 		freev(i2);
-		return v;
-	} else
-		return eval7();
+	}
+	return v;
+
 }
 
 /* Handle : operator (pattern matching).
@@ -368,7 +347,7 @@ static VALUE *eval5(void)
 
 	l = eval6();
 	while (nextarg(":")) {
-		args++;
+		G.args++;
 		r = eval6();
 		v = docolon(l, r);
 		freev(l);
@@ -396,7 +375,7 @@ static VALUE *eval4(void)
 			op = '%';
 		else
 			return l;
-		args++;
+		G.args++;
 		r = eval5();
 		val = arithmetic_common(l, r, op);
 		freev(l);
@@ -421,7 +400,7 @@ static VALUE *eval3(void)
 			op = '-';
 		else
 			return l;
-		args++;
+		G.args++;
 		r = eval4();
 		val = arithmetic_common(l, r, op);
 		freev(l);
@@ -454,7 +433,7 @@ static VALUE *eval2(void)
 			op = '>';
 		else
 			return l;
-		args++;
+		G.args++;
 		r = eval3();
 		toarith(l);
 		toarith(r);
@@ -473,7 +452,7 @@ static VALUE *eval1(void)
 
 	l = eval2();
 	while (nextarg("&")) {
-		args++;
+		G.args++;
 		r = eval2();
 		if (null(l) || null(r)) {
 			freev(l);
@@ -493,7 +472,7 @@ static VALUE *eval(void)
 
 	l = eval1();
 	while (nextarg("|")) {
-		args++;
+		G.args++;
 		r = eval1();
 		if (null(l)) {
 			freev(l);
@@ -503,3 +482,26 @@ static VALUE *eval(void)
 	}
 	return l;
 }
+
+int expr_main(int argc, char **argv);
+int expr_main(int argc, char **argv)
+{
+	VALUE *v;
+
+	if (argc == 1) {
+		bb_error_msg_and_die("too few arguments");
+	}
+
+	G.args = argv + 1;
+
+	v = eval();
+	if (*G.args)
+		bb_error_msg_and_die("syntax error");
+
+	if (v->type == integer)
+		printf("%" PF_REZ "d\n", PF_REZ_TYPE v->u.i);
+	else
+		puts(v->u.s);
+
+	fflush_stdout_and_exit(null(v));
+}
