@@ -10,26 +10,23 @@
 /* BB_AUDIT SUSv3 _NOT_ compliant -- option -G is not currently supported. */
 /* Hacked by Tito Ragusa (C) 2004 to handle usernames of whatever length and to
  * be more similar to GNU id.
+ * -Z option support: by Yuichi Nakamura <ynakam@hitachisoft.jp>
  */
 
-#include "busybox.h"
-#include <stdio.h>
-#include <unistd.h>
-#include <sys/types.h>
-
-#ifdef CONFIG_SELINUX
-#include <selinux/selinux.h>          /* for is_selinux_enabled() */
-#endif
+#include "libbb.h"
 
 #define PRINT_REAL        1
 #define NAME_NOT_NUMBER   2
 #define JUST_USER         4
 #define JUST_GROUP        8
+#if ENABLE_SELINUX
+#define JUST_CONTEXT     16
+#endif
 
-static short printf_full(unsigned int id, const char *arg, const char prefix)
+static int printf_full(unsigned int id, const char *arg, const char prefix)
 {
 	const char *fmt = "%cid=%u";
-	short status = EXIT_FAILURE;
+	int status = EXIT_FAILURE;
 
 	if (arg) {
 		fmt = "%cid=%u(%s)";
@@ -39,6 +36,7 @@ static short printf_full(unsigned int id, const char *arg, const char prefix)
 	return status;
 }
 
+int id_main(int argc, char **argv);
 int id_main(int argc, char **argv)
 {
 	struct passwd *p;
@@ -46,11 +44,13 @@ int id_main(int argc, char **argv)
 	gid_t gid;
 	unsigned long flags;
 	short status;
-
+#if ENABLE_SELINUX
+	security_context_t scontext;
+#endif
 	/* Don't allow -n -r -nr -ug -rug -nug -rnug */
 	/* Don't allow more than one username */
-	opt_complementary = "?1:?:u--g:g--u:r?ug:n?ug";
-	flags = getopt32(argc, argv, "rnug");
+	opt_complementary = "?1:u--g:g--u:r?ug:n?ug" USE_SELINUX(":u--Z:Z--u:g--Z:Z--g");
+	flags = getopt32(argv, "rnug" USE_SELINUX("Z"));
 
 	/* This values could be overwritten later */
 	uid = geteuid();
@@ -68,26 +68,44 @@ int id_main(int argc, char **argv)
 		/* in this case PRINT_REAL is the same */
 	}
 
-	if (flags & (JUST_GROUP | JUST_USER)) {
+	if (flags & (JUST_GROUP | JUST_USER USE_SELINUX(| JUST_CONTEXT))) {
 		/* JUST_GROUP and JUST_USER are mutually exclusive */
 		if (flags & NAME_NOT_NUMBER) {
-			/* bb_getpwuid and bb_getgrgid exit on failure so puts cannot segfault */
-			puts((flags & JUST_USER) ? bb_getpwuid(NULL, uid, -1 ) : bb_getgrgid(NULL, gid, -1 ));
+			/* bb_getXXXid(-1) exit on failure, puts cannot segfault */
+			puts((flags & JUST_USER) ? bb_getpwuid(NULL, -1, uid) : bb_getgrgid(NULL, -1, gid));
 		} else {
-			printf("%u\n", (flags & JUST_USER) ? uid : gid);
+			if (flags & JUST_USER) {
+				printf("%u\n", uid);
+			}
+			if (flags & JUST_GROUP) {
+				printf("%u\n", gid);
+			}
 		}
+
+#if ENABLE_SELINUX
+		if (flags & JUST_CONTEXT) {
+			selinux_or_die();
+			if (argc - optind == 1) {
+				bb_error_msg_and_die("user name can't be passed with -Z");
+			}
+
+			if (getcon(&scontext)) {
+				bb_error_msg_and_die("can't get process context");
+			}
+			printf("%s\n", scontext);
+		}
+#endif
 		/* exit */
 		fflush_stdout_and_exit(EXIT_SUCCESS);
 	}
 
 	/* Print full info like GNU id */
-	/* bb_getpwuid doesn't exit on failure here */
-	status = printf_full(uid, bb_getpwuid(NULL, uid, 0), 'u');
+	/* bb_getpwuid(0) doesn't exit on failure (returns NULL) */
+	status = printf_full(uid, bb_getpwuid(NULL, 0, uid), 'u');
 	putchar(' ');
-	/* bb_getgrgid doesn't exit on failure here */
-	status |= printf_full(gid, bb_getgrgid(NULL, gid, 0), 'g');
+	status |= printf_full(gid, bb_getgrgid(NULL, 0, gid), 'g');
 
-#ifdef CONFIG_SELINUX
+#if ENABLE_SELINUX
 	if (is_selinux_enabled()) {
 		security_context_t mysid;
 		const char *context;
