@@ -20,9 +20,6 @@
  *
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include "libbb.h"
 #include "coreutils.h"
 
@@ -30,11 +27,22 @@ mode_t getopt_mk_fifo_nod(int argc, char **argv)
 {
 	mode_t mode = 0666;
 	char *smode = NULL;
-
-	getopt32(argc, argv, "m:", &smode);
-	if(smode) {
+#if ENABLE_SELINUX
+	security_context_t scontext;
+#endif
+	int opt;
+	opt = getopt32(argv, "m:" USE_SELINUX("Z:"), &smode USE_SELINUX(,&scontext));
+	if (opt & 1) {
 		if (bb_parse_mode(smode, &mode))
 			umask(0);
 	}
+
+#if ENABLE_SELINUX
+	if (opt & 2) {
+		selinux_or_die();
+		setfscreatecon_or_die(scontext);
+	}
+#endif
+
 	return mode;
 }
