@@ -6,7 +6,7 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
 typedef enum { HASH_SHA1, HASH_MD5 } hash_algo_t;
 
@@ -38,9 +38,8 @@ static uint8_t *hash_file(const char *filename, hash_algo_t hash_algo)
 
 	src_fd = STDIN_FILENO;
 	if (NOT_LONE_DASH(filename)) {
-		src_fd = open(filename, O_RDONLY);
+		src_fd = open_or_warn(filename, O_RDONLY);
 		if (src_fd < 0) {
-			bb_perror_msg("%s", filename);
 			return NULL;
 		}
 	}
@@ -78,17 +77,18 @@ static uint8_t *hash_file(const char *filename, hash_algo_t hash_algo)
 	return hash_value;
 }
 
+int md5_sha1_sum_main(int argc, char **argv);
 int md5_sha1_sum_main(int argc, char **argv)
 {
 	int return_value = EXIT_SUCCESS;
 	uint8_t *hash_value;
 	unsigned flags;
 	hash_algo_t hash_algo = ENABLE_MD5SUM
-		? (ENABLE_SHA1SUM ? (**argv=='m' ? HASH_MD5 : HASH_SHA1) : HASH_MD5)
+		? (ENABLE_SHA1SUM ? (applet_name[0] == 'm' ? HASH_MD5 : HASH_SHA1) : HASH_MD5)
 		: HASH_SHA1;
 
 	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK)
-		flags = getopt32(argc, argv, "scw");
+		flags = getopt32(argv, "scw");
 	else optind = 1;
 
 	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK && !(flags & FLAG_CHECK)) {
@@ -102,7 +102,7 @@ int md5_sha1_sum_main(int argc, char **argv)
 	}
 
 	if (argc == optind) {
-		argv[argc++] = "-";
+		argv[argc++] = (char*)"-";
 	}
 
 	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK && (flags & FLAG_CHECK)) {
