@@ -10,8 +10,9 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
+int realpath_main(int argc, char **argv);
 int realpath_main(int argc, char **argv)
 {
 	int retval = EXIT_SUCCESS;
