@@ -12,7 +12,10 @@
  * http://www.opengroup.org/onlinepubs/007904975/utilities/sort.html
  */
 
-#include "busybox.h"
+#include "libbb.h"
+
+/* This is a NOEXEC applet. Be very careful! */
+
 
 /*
 	sort [-m][-o output][-bdfinru][-t char][-k keydef]... [file...]
@@ -20,7 +23,7 @@
 */
 
 /* These are sort types */
-static const char OPT_STR[] = "ngMucszbrdfimS:T:o:k:t:";
+static const char OPT_STR[] ALIGN1 = "ngMucszbrdfimS:T:o:k:t:";
 enum {
 	FLAG_n  = 1,            /* Numeric sort */
 	FLAG_g  = 2,            /* Sort using strtod() */
@@ -271,11 +274,13 @@ static unsigned str2u(char **str)
 }
 #endif
 
+int sort_main(int argc, char **argv);
 int sort_main(int argc, char **argv)
 {
 	FILE *fp, *outfile = stdout;
 	char *line, **lines = NULL;
-	char *str_ignored, *str_o, *str_k, *str_t;
+	char *str_ignored, *str_o, *str_t;
+	llist_t *lst_k = NULL;
 	int i, flag;
 	int linecount = 0;
 
@@ -283,8 +288,9 @@ int sort_main(int argc, char **argv)
 
 	/* Parse command line options */
 	/* -o and -t can be given at most once */
-	opt_complementary = "?:o--o:t--t";
-	getopt32(argc, argv, OPT_STR, &str_ignored, &str_ignored, &str_o, &str_k, &str_t);
+	opt_complementary = "o--o:t--t:" /* -t, -o: maximum one of each */
+			"k::"; /* -k takes list */
+	getopt32(argv, OPT_STR, &str_ignored, &str_ignored, &str_o, &lst_k, &str_t);
 #if ENABLE_FEATURE_SORT_BIG
 	if (option_mask32 & FLAG_o) outfile = xfopen(str_o, "w");
 	if (option_mask32 & FLAG_t) {
@@ -293,7 +299,7 @@ int sort_main(int argc, char **argv)
 		key_separator = str_t[0];
 	}
 	/* parse sort key */
-	if (option_mask32 & FLAG_k) {
+	while (lst_k) {
 		enum {
 			FLAG_allowed_for_k =
 				FLAG_n | /* Numeric sort */
@@ -307,6 +313,7 @@ int sort_main(int argc, char **argv)
 			0
 		};
 		struct sort_key *key = add_key();
+		char *str_k = lst_k->data;
 		const char *temp2;
 
 		i = 0; /* i==0 before comma, 1 after (-k3,6) */
@@ -336,6 +343,8 @@ int sort_main(int argc, char **argv)
 				str_k++;
 			}
 		}
+		/* leaking lst_k... */
+		lst_k = lst_k->link;
 	}
 #endif
 	/* global b strips leading and trailing spaces */
