@@ -21,7 +21,7 @@
 
    */
 
-#include "busybox.h"
+#include "libbb.h"
 
 #ifndef _POSIX_VDISABLE
 # define _POSIX_VDISABLE ((unsigned char) 0)
@@ -70,15 +70,15 @@
 
 /* SunOS 5.3 loses (^Z doesn't work) if 'swtch' is the same as 'susp'.
    So the default is to disable 'swtch.'  */
-#if defined (__sparc__) && defined (__svr4__)
+#if defined(__sparc__) && defined(__svr4__)
 # undef CSWTCH
 # define CSWTCH _POSIX_VDISABLE
 #endif
 
-#if defined(VWERSE) && !defined (VWERASE)       /* AIX-3.2.5 */
+#if defined(VWERSE) && !defined(VWERASE)       /* AIX-3.2.5 */
 # define VWERASE VWERSE
 #endif
-#if defined(VDSUSP) && !defined (CDSUSP)
+#if defined(VDSUSP) && !defined(CDSUSP)
 # define CDSUSP Control('y')
 #endif
 #if !defined(VREPRINT) && defined(VRPRNT)       /* Irix 4.0.5 */
@@ -127,28 +127,28 @@ enum {
 	control, input, output, local, combination
 };
 
-static const char evenp     [] = "evenp";
-static const char raw       [] = "raw";
-static const char stty_min  [] = "min";
-static const char stty_time [] = "time";
-static const char stty_swtch[] = "swtch";
-static const char stty_eol  [] = "eol";
-static const char stty_eof  [] = "eof";
-static const char parity    [] = "parity";
-static const char stty_oddp [] = "oddp";
-static const char stty_nl   [] = "nl";
-static const char stty_ek   [] = "ek";
-static const char stty_sane [] = "sane";
-static const char cbreak    [] = "cbreak";
-static const char stty_pass8[] = "pass8";
-static const char litout    [] = "litout";
-static const char cooked    [] = "cooked";
-static const char decctlq   [] = "decctlq";
-static const char stty_tabs [] = "tabs";
-static const char stty_lcase[] = "lcase";
-static const char stty_LCASE[] = "LCASE";
-static const char stty_crt  [] = "crt";
-static const char stty_dec  [] = "dec";
+static const char evenp     [] ALIGN1 = "evenp";
+static const char raw       [] ALIGN1 = "raw";
+static const char stty_min  [] ALIGN1 = "min";
+static const char stty_time [] ALIGN1 = "time";
+static const char stty_swtch[] ALIGN1 = "swtch";
+static const char stty_eol  [] ALIGN1 = "eol";
+static const char stty_eof  [] ALIGN1 = "eof";
+static const char parity    [] ALIGN1 = "parity";
+static const char stty_oddp [] ALIGN1 = "oddp";
+static const char stty_nl   [] ALIGN1 = "nl";
+static const char stty_ek   [] ALIGN1 = "ek";
+static const char stty_sane [] ALIGN1 = "sane";
+static const char cbreak    [] ALIGN1 = "cbreak";
+static const char stty_pass8[] ALIGN1 = "pass8";
+static const char litout    [] ALIGN1 = "litout";
+static const char cooked    [] ALIGN1 = "cooked";
+static const char decctlq   [] ALIGN1 = "decctlq";
+static const char stty_tabs [] ALIGN1 = "tabs";
+static const char stty_lcase[] ALIGN1 = "lcase";
+static const char stty_LCASE[] ALIGN1 = "LCASE";
+static const char stty_crt  [] ALIGN1 = "crt";
+static const char stty_dec  [] ALIGN1 = "dec";
 
 /* Flags for 'struct mode_info' */
 #define SANE_SET 1              /* Set in 'sane' mode                  */
@@ -158,7 +158,7 @@ static const char stty_dec  [] = "dec";
 
 /* Each mode */
 struct mode_info {
-	const char * const name;      /* Name given on command line           */
+	const char *const name;      /* Name given on command line           */
 	const unsigned char type;     /* Which structure element to change    */
 	const unsigned char flags;    /* Setting and display options          */
 	/* were using short here, but ppc32 was unhappy: */
@@ -319,12 +319,12 @@ static const struct mode_info mode_info[] = {
 };
 
 enum {
-	NUM_mode_info = (sizeof(mode_info) / sizeof(mode_info[0]))
+	NUM_mode_info = ARRAY_SIZE(mode_info)
 };
 
 /* Control character settings */
 struct control_info {
-	const char * const name;               /* Name given on command line */
+	const char *const name;               /* Name given on command line */
 	const unsigned char saneval;          /* Value to set for 'stty sane' */
 	const unsigned char offset;           /* Offset in c_cc */
 };
@@ -371,21 +371,26 @@ static const struct control_info control_info[] = {
 };
 
 enum {
-	NUM_control_info = (sizeof(control_info) / sizeof(control_info[0]))
+	NUM_control_info = ARRAY_SIZE(control_info)
 };
 
 /* The width of the screen, for output wrapping */
-static unsigned max_col = 80; /* default */
-/* Current position, to know when to wrap */
-static unsigned current_col;
+unsigned max_col = 80; /* default */
+
+struct globals {
+	/* Current position, to know when to wrap */
+	unsigned current_col;
+	char buf[10];
+};
+#define G (*(struct globals*)&bb_common_bufsiz1)
+
 static const char *device_name = bb_msg_standard_input;
 
 /* Return a string that is the printable representation of character CH */
 /* Adapted from 'cat' by Torbjorn Granlund */
-static const char *visible(unsigned int ch)
+static const char *visible(unsigned ch)
 {
-	static char buf[10];
-	char *bpout = buf;
+	char *bpout = G.buf;
 
 	if (ch == _POSIX_VDISABLE)
 		return "<undef>";
@@ -407,16 +412,16 @@ static const char *visible(unsigned int ch)
 	}
 
 	*bpout = '\0';
-	return buf;
+	return G.buf;
 }
 
 static tcflag_t *mode_type_flag(unsigned type, const struct termios *mode)
 {
-	static const unsigned char tcflag_offsets[] = {
+	static const unsigned char tcflag_offsets[] ALIGN1 = {
 		offsetof(struct termios, c_cflag), /* control */
 		offsetof(struct termios, c_iflag), /* input */
 		offsetof(struct termios, c_oflag), /* output */
-		offsetof(struct termios, c_lflag), /* local */
+		offsetof(struct termios, c_lflag)  /* local */
 	};
 
 	if (type <= local) {
@@ -425,7 +430,7 @@ static tcflag_t *mode_type_flag(unsigned type, const struct termios *mode)
 	return NULL;
 }
 
-static void set_speed_or_die(enum speed_setting type, const char * const arg,
+static void set_speed_or_die(enum speed_setting type, const char *const arg,
 					struct termios * const mode)
 {
 	speed_t baud;
@@ -466,20 +471,20 @@ static void wrapf(const char *message, ...)
 	   somebody failed to adhere to this assumption just to be sure.  */
 	if (!buflen || buflen >= sizeof(buf)) return;
 
-	if (current_col > 0) {
-		current_col++;
+	if (G.current_col > 0) {
+		G.current_col++;
 		if (buf[0] != '\n') {
-			if (current_col + buflen >= max_col) {
+			if (G.current_col + buflen >= max_col) {
 				putchar('\n');
-				current_col = 0;
+				G.current_col = 0;
 			} else
 				putchar(' ');
 		}
 	}
 	fputs(buf, stdout);
-	current_col += buflen;
+	G.current_col += buflen;
 	if (buf[buflen-1] == '\n')
-		current_col = 0;
+		G.current_col = 0;
 }
 
 static void set_window_size(const int rows, const int cols)
@@ -519,10 +524,10 @@ static void display_window_size(const int fancy)
 }
 
 static const struct suffix_mult stty_suffixes[] = {
-	{"b",  512 },
-	{"k",  1024},
-	{"B",  1024},
-	{NULL, 0   }
+	{ "b",  512 },
+	{ "k", 1024 },
+	{ "B", 1024 },
+	{ }
 };
 
 static const struct mode_info *find_mode(const char *name)
@@ -545,41 +550,39 @@ static const struct control_info *find_control(const char *name)
 
 enum {
 	param_need_arg = 0x80,
-	param_line   = 1 | 0x80,
-	param_rows   = 2 | 0x80,
-	param_cols   = 3 | 0x80,
-	param_size   = 4,
-	param_speed  = 5,
-	param_ispeed = 6 | 0x80,
-	param_ospeed = 7 | 0x80,
+	param_line    = 1 | 0x80,
+	param_rows    = 2 | 0x80,
+	param_cols    = 3 | 0x80,
+	param_columns = 4 | 0x80,
+	param_size    = 5,
+	param_speed   = 6,
+	param_ispeed  = 7 | 0x80,
+	param_ospeed  = 8 | 0x80,
 };
 
-static int find_param(const char * const name)
+static int find_param(const char *const name)
 {
-	const char * const params[] = {
-		"line",
-		"rows",
-		"cols",
-		"columns",
-		"size",
-		"speed",
-		"ispeed",
-		"ospeed",
-		NULL
-	};
-	int i = index_in_str_array(params, name);
-	if (i < 0)
+	static const char params[] ALIGN1 =
+		"line\0"    /* 1 */
+		"rows\0"    /* 2 */
+		"cols\0"    /* 3 */
+		"columns\0" /* 4 */
+		"size\0"    /* 5 */
+		"speed\0"   /* 6 */
+		"ispeed\0"
+		"ospeed\0";
+	int i = index_in_strings(params, name) + 1;
+	if (i == 0)
 		return 0;
-	if (!(i == 4 || i == 5))
+	if (i != 5 && i != 6)
 		i |= 0x80;
-
 	return i;
 }
 
 static int recover_mode(const char *arg, struct termios *mode)
 {
 	int i, n;
-	unsigned int chr;
+	unsigned chr;
 	unsigned long iflag, oflag, cflag, lflag;
 
 	/* Scan into temporaries since it is too much trouble to figure out
@@ -607,7 +610,7 @@ static int recover_mode(const char *arg, struct termios *mode)
 }
 
 static void display_recoverable(const struct termios *mode,
-								const int ATTRIBUTE_UNUSED dummy)
+				int ATTRIBUTE_UNUSED dummy)
 {
 	int i;
 	printf("%lx:%lx:%lx:%lx",
@@ -627,7 +630,7 @@ static void display_speed(const struct termios *mode, int fancy)
 	ospeed = ispeed = cfgetispeed(mode);
 	if (ispeed == 0 || ispeed == (ospeed = cfgetospeed(mode))) {
 		ispeed = ospeed;                /* in case ispeed was 0 */
-	                 //0123 4 5 6 7 8 9
+		         //0123 4 5 6 7 8 9
 		fmt_str = "%lu\n\0\0\0\0\0speed %lu baud;";
 	}
 	if (fancy) fmt_str += 9;
@@ -669,14 +672,14 @@ static void do_display(const struct termios *mode, const int all)
 	if ((mode->c_lflag & ICANON) == 0)
 #endif
 		wrapf("min = %d; time = %d;", mode->c_cc[VMIN], mode->c_cc[VTIME]);
-	if (current_col) wrapf("\n");
+	if (G.current_col) wrapf("\n");
 
 	for (i = 0; i < NUM_mode_info; ++i) {
 		if (mode_info[i].flags & OMIT)
 			continue;
 		if (mode_info[i].type != prev_type) {
 			/* wrapf("\n"); */
-			if (current_col) wrapf("\n");
+			if (G.current_col) wrapf("\n");
 			prev_type = mode_info[i].type;
 		}
 
@@ -692,7 +695,7 @@ static void do_display(const struct termios *mode, const int all)
 				wrapf("-%s", mode_info[i].name);
 		}
 	}
-	if (current_col) wrapf("\n");
+	if (G.current_col) wrapf("\n");
 }
 
 static void sane_mode(struct termios *mode)
@@ -908,6 +911,7 @@ static void set_control_char_or_die(const struct control_info *info,
 #define STTY_verbose_output     (1<<2)
 #define STTY_recoverable_output (1<<3)
 #define STTY_noargs             (1<<4)
+int stty_main(int argc, char **argv);
 int stty_main(int argc, char **argv)
 {
 	struct termios mode;
@@ -969,7 +973,7 @@ int stty_main(int argc, char **argv)
 					goto invalid_argument;
 				}
 			}
-end_option:
+ end_option:
 			continue;
 		}
 
@@ -1008,6 +1012,7 @@ end_option:
 #ifdef TIOCGWINSZ
 		case param_rows:
 		case param_cols:
+		case param_columns:
 			xatoul_range_sfx(argnext, 1, INT_MAX, stty_suffixes);
 			break;
 		case param_size:
@@ -1025,7 +1030,7 @@ end_option:
 		default:
 			if (recover_mode(arg, &mode) == 1) break;
 			if (tty_value_to_baud(xatou(arg)) != (speed_t) -1) break;
-invalid_argument:
+ invalid_argument:
 			bb_error_msg_and_die("invalid argument '%s'", arg);
 		}
 		stty_state &= ~STTY_noargs;
