@@ -146,7 +146,6 @@ static const struct t_op {
 	{ ")"  , RPAREN , PAREN  },
 };
 
-enum { NUM_OPS = sizeof(ops) / sizeof(ops[0]) };
 
 #if ENABLE_FEATURE_TEST_64
 typedef int64_t arith_t;
@@ -155,7 +154,7 @@ typedef int arith_t;
 #endif
 
 /* Cannot eliminate these static data (do the G trick)
- * because of bb_test usage from other applets */
+ * because of test_main usage from other applets */
 static char **t_wp;
 static struct t_op const *t_wp_op;
 static gid_t *group_array;
@@ -179,22 +178,25 @@ static int test_eaccess(char *path, int mode);
 static int is_a_group_member(gid_t gid);
 static void initialize_group_array(void);
 
-int bb_test(int argc, char **argv)
+int test_main(int argc, char **argv)
 {
 	int res;
+	const char *arg0;
+	bool _off;
 
-	if (LONE_CHAR(argv[0], '[')) {
+	arg0 = bb_basename(argv[0]);
+	if (arg0[0] == '[') {
 		--argc;
-		if (NOT_LONE_CHAR(argv[argc], ']')) {
-			bb_error_msg("missing ]");
-			return 2;
-		}
-		argv[argc] = NULL;
-	} else if (strcmp(argv[0], "[[") == 0) {
-		--argc;
-		if (strcmp(argv[argc], "]]")) {
-			bb_error_msg("missing ]]");
-			return 2;
+		if (!arg0[1]) { /* "[" ? */
+			if (NOT_LONE_CHAR(argv[argc], ']')) {
+				bb_error_msg("missing ]");
+				return 2;
+			}
+		} else { /* assuming "[[" */
+			if (strcmp(argv[argc], "]]") != 0) {
+				bb_error_msg("missing ]]");
+				return 2;
+			}
 		}
 		argv[argc] = NULL;
 	}
@@ -219,15 +221,19 @@ int bb_test(int argc, char **argv)
 	if (argc == 2)
 		return *argv[1] == '\0';
 //assert(argc);
-	if (LONE_CHAR(argv[1], '!')) {
-		bool _off;
+	/* remember if we saw argc==4 which wants *no* '!' test */
+	_off = argc - 4;
+	if (_off ?
+		(LONE_CHAR(argv[1], '!'))
+		: (argv[1][0] != '!' || argv[1][1] != '\0'))
+	{
 		if (argc == 3)
 			return *argv[2] != '\0';
-		_off = argc - 4;
+
 		t_lex(argv[2 + _off]);
 		if (t_wp_op && t_wp_op->op_type == BINOP) {
 			t_wp = &argv[1 + _off];
-			return binop() == 0;
+			return binop() == _off;
 		}
 	}
 	t_wp = &argv[1];
@@ -378,7 +384,7 @@ static int binop(void)
 static int filstat(char *nm, enum token mode)
 {
 	struct stat s;
-	int i;
+	int i = i; /* gcc 3.x thinks it can be used uninitialized */
 
 	if (mode == FILSYM) {
 #ifdef S_IFLNK
@@ -463,7 +469,7 @@ static enum token t_lex(char *s)
 			return op->op_num;
 		}
 		op++;
-	} while (op < ops + NUM_OPS);
+	} while (op < ops + ARRAY_SIZE(ops));
 
 	return OPERAND;
 }
@@ -561,7 +567,7 @@ static void initialize_group_array(void)
 	if (ngroups > 0) {
 		/* FIXME: ash tries so hard to not die on OOM,
 		 * and we spoil it with just one xrealloc here */
-		/* We realloc, because bb_test can be entered repeatedly by shell.
+		/* We realloc, because test_main can be entered repeatedly by shell.
 		 * Testcase (ash): 'while true; do test -x some_file; done'
 		 * and watch top. (some_file must have owner != you) */
 		group_array = xrealloc(group_array, ngroups * sizeof(gid_t));
@@ -591,12 +597,3 @@ static int is_a_group_member(gid_t gid)
 
 	return 0;
 }
-
-
-/* applet entry point */
-
-int test_main(int argc, char **argv);
-int test_main(int argc, char **argv)
-{
-	return bb_test(argc, argv);
-}
