@@ -46,7 +46,7 @@
 
 #include "e2fsbb.h"
 
-#include "busybox.h"
+#include "libbb.h"
 
 #ifndef _PATH_MNTTAB
 #define _PATH_MNTTAB    "/etc/fstab"
@@ -115,7 +115,7 @@ struct fsck_instance {
  * Required for the uber-silly devfs /dev/ide/host1/bus2/target3/lun3
  * pathames.
  */
-static const char * const devfs_hier[] = {
+static const char *const devfs_hier[] = {
 	"host", "bus", "target", "lun", 0
 };
 #endif
@@ -124,7 +124,7 @@ static char *base_device(const char *device)
 {
 	char *str, *cp;
 #ifdef CONFIG_FEATURE_DEVFS
-	const char * const *hier;
+	const char *const *hier;
 	const char *disk;
 	int len;
 #endif
@@ -226,7 +226,7 @@ errout:
 }
 
 
-static const char * const ignored_types[] = {
+static const char *const ignored_types[] = {
 	"ignore",
 	"iso9660",
 	"nfs",
@@ -238,7 +238,7 @@ static const char * const ignored_types[] = {
 	NULL
 };
 
-static const char * const really_wanted[] = {
+static const char *const really_wanted[] = {
 	"minix",
 	"ext2",
 	"ext3",
@@ -372,7 +372,6 @@ static void free_instance(struct fsck_instance *i)
 	if (i->base_device)
 		free(i->base_device);
 	free(i);
-	return;
 }
 
 static struct fs_info *create_fs_device(const char *device, const char *mntpnt,
@@ -1172,7 +1171,7 @@ static void signal_cancel(int sig FSCK_ATTR((unused)))
 	cancel_requested++;
 }
 
-static void PRS(int argc, char *argv[])
+static void PRS(int argc, char **argv)
 {
 	int     i, j;
 	char    *arg, *dev, *tmp = 0;
@@ -1322,7 +1321,8 @@ static void PRS(int argc, char *argv[])
 	    max_running = atoi(tmp);
 }
 
-int fsck_main(int argc, char *argv[])
+int fsck_main(int argc, char **argv);
+int fsck_main(int argc, char **argv)
 {
 	int i, status = 0;
 	int interactive = 0;
@@ -1357,7 +1357,7 @@ int fsck_main(int argc, char *argv[])
 		interactive++;
 		return check_all();
 	}
-	for (i = 0 ; i < num_devices; i++) {
+	for (i = 0; i < num_devices; i++) {
 		if (cancel_requested) {
 			if (!kill_sent) {
 				kill_all(SIGTERM);
