@@ -21,7 +21,7 @@
   add_cmd() is called on each line of sed command text (from a file or from
   the command line).  It calls get_address() and parse_cmd_args().  The
   resulting sed_cmd_t structures are appended to a linked list
-  (bbg.sed_cmd_head/bbg.sed_cmd_tail).
+  (G.sed_cmd_head/G.sed_cmd_tail).
 
   add_input_file() adds a FILE * to the list of input files.  We need to
   know all input sources ahead of time to find the last line for the $ match.
@@ -58,12 +58,13 @@
 	Reference http://www.opengroup.org/onlinepubs/007904975/utilities/sed.html
 */
 
-#include "busybox.h"
+#include "libbb.h"
 #include "xregex.h"
 
 /* Each sed command turns into one of these structures. */
 typedef struct sed_cmd_s {
 	/* Ordered by alignment requirements: currently 36 bytes on x86 */
+	struct sed_cmd_s *next; /* Next command (linked list, NULL terminated) */
 
 	/* address storage */
 	regex_t *beg_match;     /* sed -e '/match/cmd' */
@@ -72,26 +73,25 @@ typedef struct sed_cmd_s {
 	int beg_line;           /* 'sed 1p'   0 == apply commands to all lines */
 	int end_line;           /* 'sed 1,3p' 0 == one line only. -1 = last line ($) */
 
-	FILE *file;             /* File (sw) command writes to, -1 for none. */
+	FILE *sw_file;          /* File (sw) command writes to, -1 for none. */
 	char *string;           /* Data string for (saicytb) commands. */
 
-	unsigned short which_match;     /* (s) Which match to replace (0 for all) */
+	unsigned short which_match; /* (s) Which match to replace (0 for all) */
 
 	/* Bitfields (gcc won't group them if we don't) */
-	unsigned int invert:1;          /* the '!' after the address */
-	unsigned int in_match:1;        /* Next line also included in match? */
-	unsigned int sub_p:1;           /* (s) print option */
+	unsigned invert:1;      /* the '!' after the address */
+	unsigned in_match:1;    /* Next line also included in match? */
+	unsigned sub_p:1;       /* (s) print option */
 
-	int last_char;                  /* Last line written by (sw) had no '\n' */
+	char sw_last_char;      /* Last line written by (sw) had no '\n' */
 
 	/* GENERAL FIELDS */
 	char cmd;               /* The command char: abcdDgGhHilnNpPqrstwxy:={} */
-	struct sed_cmd_s *next; /* Next command (linked list, NULL terminated) */
 } sed_cmd_t;
 
-static const char *const semicolon_whitespace = "; \n\r\t\v";
+static const char semicolon_whitespace[] ALIGN1 = "; \n\r\t\v";
 
-struct sed_globals {
+struct globals {
 	/* options */
 	int be_quiet, regex_type;
 	FILE *nonstdout;
@@ -117,21 +117,28 @@ struct sed_globals {
 		int idx;	/* Space used */
 		int len;	/* Space allocated */
 	} pipeline;
-} bbg;
+};
+#define G (*(struct globals*)&bb_common_bufsiz1)
+void BUG_sed_globals_too_big(void);
+#define INIT_G() do { \
+	if (sizeof(struct globals) > COMMON_BUFSIZE) \
+		BUG_sed_globals_too_big(); \
+	G.sed_cmd_tail = &G.sed_cmd_head; \
+} while (0)
 
 
 #if ENABLE_FEATURE_CLEAN_UP
 static void sed_free_and_close_stuff(void)
 {
-	sed_cmd_t *sed_cmd = bbg.sed_cmd_head.next;
+	sed_cmd_t *sed_cmd = G.sed_cmd_head.next;
 
-	llist_free(bbg.append_head, free);
+	llist_free(G.append_head, free);
 
 	while (sed_cmd) {
 		sed_cmd_t *sed_cmd_next = sed_cmd->next;
 
-		if (sed_cmd->file)
-			xprint_and_close_file(sed_cmd->file);
+		if (sed_cmd->sw_file)
+			xprint_and_close_file(sed_cmd->sw_file);
 
 		if (sed_cmd->beg_match) {
 			regfree(sed_cmd->beg_match);
@@ -150,10 +157,10 @@ static void sed_free_and_close_stuff(void)
 		sed_cmd = sed_cmd_next;
 	}
 
-	if (bbg.hold_space) free(bbg.hold_space);
+	if (G.hold_space) free(G.hold_space);
 
-	while (bbg.current_input_file < bbg.input_file_count)
-		fclose(bbg.input_file_list[bbg.current_input_file++]);
+	while (G.current_input_file < G.input_file_count)
+		fclose(G.input_file_list[G.current_input_file++]);
 }
 #else
 void sed_free_and_close_stuff(void);
@@ -163,12 +170,12 @@ void sed_free_and_close_stuff(void);
 
 static void cleanup_outname(void)
 {
-	if (bbg.outname) unlink(bbg.outname);
+	if (G.outname) unlink(G.outname);
 }
 
 /* strdup, replacing "\n" with '\n', and "\delimiter" with 'delimiter' */
 
-static void parse_escapes(char *dest, char *string, int len, char from, char to)
+static void parse_escapes(char *dest, const char *string, int len, char from, char to)
 {
 	int i = 0;
 
@@ -186,7 +193,7 @@ static void parse_escapes(char *dest, char *string, int len, char from, char to)
 	*dest = 0;
 }
 
-static char *copy_parsing_escapes(char *string, int len)
+static char *copy_parsing_escapes(const char *string, int len)
 {
 	char *dest = xmalloc(len + 1);
 
@@ -201,7 +208,7 @@ static char *copy_parsing_escapes(char *string, int len)
  * expression delimiter (typically a forward * slash ('/')) not preceded by
  * a backslash ('\').  A negative delimiter disables square bracket checking.
  */
-static int index_of_next_unescaped_regexp_delim(int delimiter, char *str)
+static int index_of_next_unescaped_regexp_delim(int delimiter, const char *str)
 {
 	int bracket = -1;
 	int escaped = 0;
@@ -235,9 +242,9 @@ static int index_of_next_unescaped_regexp_delim(int delimiter, char *str)
 /*
  *  Returns the index of the third delimiter
  */
-static int parse_regex_delim(char *cmdstr, char **match, char **replace)
+static int parse_regex_delim(const char *cmdstr, char **match, char **replace)
 {
-	char *cmdstr_ptr = cmdstr;
+	const char *cmdstr_ptr = cmdstr;
 	char delimiter;
 	int idx = 0;
 
@@ -262,12 +269,12 @@ static int parse_regex_delim(char *cmdstr, char **match, char **replace)
 /*
  * returns the index in the string just past where the address ends.
  */
-static int get_address(char *my_str, int *linenum, regex_t ** regex)
+static int get_address(const char *my_str, int *linenum, regex_t ** regex)
 {
-	char *pos = my_str;
+	const char *pos = my_str;
 
 	if (isdigit(*my_str)) {
-		*linenum = strtol(my_str, &pos, 10);
+		*linenum = strtol(my_str, (char**)&pos, 10);
 		/* endstr shouldnt ever equal NULL */
 	} else if (*my_str == '$') {
 		*linenum = -1;
@@ -282,7 +289,7 @@ static int get_address(char *my_str, int *linenum, regex_t ** regex)
 		next = index_of_next_unescaped_regexp_delim(delimiter, ++pos);
 		temp = copy_parsing_escapes(pos, next);
 		*regex = xmalloc(sizeof(regex_t));
-		xregcomp(*regex, temp, bbg.regex_type|REG_NEWLINE);
+		xregcomp(*regex, temp, G.regex_type|REG_NEWLINE);
 		free(temp);
 		/* Move position to next character after last delimiter */
 		pos += (next+1);
@@ -291,30 +298,34 @@ static int get_address(char *my_str, int *linenum, regex_t ** regex)
 }
 
 /* Grab a filename.  Whitespace at start is skipped, then goes to EOL. */
-static int parse_file_cmd(sed_cmd_t *sed_cmd, char *filecmdstr, char **retval)
+static int parse_file_cmd(sed_cmd_t *sed_cmd, const char *filecmdstr, char **retval)
 {
 	int start = 0, idx, hack = 0;
 
 	/* Skip whitespace, then grab filename to end of line */
-	while (isspace(filecmdstr[start])) start++;
+	while (isspace(filecmdstr[start]))
+		start++;
 	idx = start;
-	while (filecmdstr[idx] && filecmdstr[idx] != '\n') idx++;
+	while (filecmdstr[idx] && filecmdstr[idx] != '\n')
+		idx++;
 
 	/* If lines glued together, put backslash back. */
-	if (filecmdstr[idx] == '\n') hack = 1;
+	if (filecmdstr[idx] == '\n')
+		hack = 1;
 	if (idx == start)
 		bb_error_msg_and_die("empty filename");
 	*retval = xstrndup(filecmdstr+start, idx-start+hack+1);
-	if (hack) (*retval)[idx] = '\\';
+	if (hack)
+		(*retval)[idx] = '\\';
 
 	return idx;
 }
 
-static int parse_subst_cmd(sed_cmd_t *sed_cmd, char *substr)
+static int parse_subst_cmd(sed_cmd_t *sed_cmd, const char *substr)
 {
-	int cflags = bbg.regex_type;
+	int cflags = G.regex_type;
 	char *match;
-	int idx = 0;
+	int idx;
 
 	/*
 	 * A substitution command should look something like this:
@@ -337,9 +348,9 @@ static int parse_subst_cmd(sed_cmd_t *sed_cmd, char *substr)
 		if (isdigit(substr[idx])) {
 			if (match[0] != '^') {
 				/* Match 0 treated as all, multiple matches we take the last one. */
-				char *pos = substr + idx;
-				/* FIXME: error check? */
-				sed_cmd->which_match = (unsigned short)strtol(substr+idx, &pos, 10);
+				const char *pos = substr + idx;
+/* FIXME: error check? */
+				sed_cmd->which_match = (unsigned short)strtol(substr+idx, (char**) &pos, 10);
 				idx = pos - substr;
 			}
 			continue;
@@ -361,7 +372,6 @@ static int parse_subst_cmd(sed_cmd_t *sed_cmd, char *substr)
 		{
 			char *temp;
 			idx += parse_file_cmd(sed_cmd, substr+idx, &temp);
-
 			break;
 		}
 		/* Ignore case (gnu exension) */
@@ -395,7 +405,7 @@ out:
 /*
  *  Process the commands arguments
  */
-static char *parse_cmd_args(sed_cmd_t *sed_cmd, char *cmdstr)
+static const char *parse_cmd_args(sed_cmd_t *sed_cmd, const char *cmdstr)
 {
 	/* handle (s)ubstitution command */
 	if (sed_cmd->cmd == 's')
@@ -422,8 +432,10 @@ static char *parse_cmd_args(sed_cmd_t *sed_cmd, char *cmdstr)
 		if (sed_cmd->end_line || sed_cmd->end_match)
 			bb_error_msg_and_die("command only uses one address");
 		cmdstr += parse_file_cmd(sed_cmd, cmdstr, &sed_cmd->string);
-		if (sed_cmd->cmd == 'w')
-			sed_cmd->file = xfopen(sed_cmd->string, "w");
+		if (sed_cmd->cmd == 'w') {
+			sed_cmd->sw_file = xfopen(sed_cmd->string, "w");
+			sed_cmd->sw_last_char = '\n';
+		}
 	/* handle branch commands */
 	} else if (strchr(":btT", sed_cmd->cmd)) {
 		int length;
@@ -467,24 +479,24 @@ static char *parse_cmd_args(sed_cmd_t *sed_cmd, char *cmdstr)
 
 /* Parse address+command sets, skipping comment lines. */
 
-static void add_cmd(char *cmdstr)
+static void add_cmd(const char *cmdstr)
 {
 	sed_cmd_t *sed_cmd;
 	int temp;
 
 	/* Append this line to any unfinished line from last time. */
-	if (bbg.add_cmd_line) {
-		cmdstr = xasprintf("%s\n%s", bbg.add_cmd_line, cmdstr);
-		free(bbg.add_cmd_line);
-		bbg.add_cmd_line = cmdstr;
+	if (G.add_cmd_line) {
+		char *tp = xasprintf("%s\n%s", G.add_cmd_line, cmdstr);
+		free(G.add_cmd_line);
+		cmdstr = G.add_cmd_line = tp;
 	}
 
 	/* If this line ends with backslash, request next line. */
 	temp = strlen(cmdstr);
-	if (temp && cmdstr[temp-1] == '\\') {
-		if (!bbg.add_cmd_line)
-			bbg.add_cmd_line = xstrdup(cmdstr);
-		bbg.add_cmd_line[temp-1] = 0;
+	if (temp && cmdstr[--temp] == '\\') {
+		if (!G.add_cmd_line)
+			G.add_cmd_line = xstrdup(cmdstr);
+		G.add_cmd_line[temp] = '\0';
 		return;
 	}
 
@@ -500,7 +512,7 @@ static void add_cmd(char *cmdstr)
 		if (*cmdstr == '#') {
 			/* "#n" is the same as using -n on the command line */
 			if (cmdstr[1] == 'n')
-				bbg.be_quiet++;
+				G.be_quiet++;
 			cmdstr = strpbrk(cmdstr, "\n\r");
 			if (!cmdstr) break;
 			continue;
@@ -547,13 +559,13 @@ static void add_cmd(char *cmdstr)
 		cmdstr = parse_cmd_args(sed_cmd, cmdstr);
 
 		/* Add the command to the command array */
-		bbg.sed_cmd_tail->next = sed_cmd;
-		bbg.sed_cmd_tail = bbg.sed_cmd_tail->next;
+		G.sed_cmd_tail->next = sed_cmd;
+		G.sed_cmd_tail = G.sed_cmd_tail->next;
 	}
 
 	/* If we glued multiple lines together, free the memory. */
-	free(bbg.add_cmd_line);
-	bbg.add_cmd_line = NULL;
+	free(G.add_cmd_line);
+	G.add_cmd_line = NULL;
 }
 
 /* Append to a string, reallocating memory as necessary. */
@@ -562,12 +574,12 @@ static void add_cmd(char *cmdstr)
 
 static void pipe_putc(char c)
 {
-	if (bbg.pipeline.idx == bbg.pipeline.len) {
-		bbg.pipeline.buf = xrealloc(bbg.pipeline.buf,
-				bbg.pipeline.len + PIPE_GROW);
-		bbg.pipeline.len += PIPE_GROW;
+	if (G.pipeline.idx == G.pipeline.len) {
+		G.pipeline.buf = xrealloc(G.pipeline.buf,
+				G.pipeline.len + PIPE_GROW);
+		G.pipeline.len += PIPE_GROW;
 	}
-	bbg.pipeline.buf[bbg.pipeline.idx++] = c;
+	G.pipeline.buf[G.pipeline.idx++] = c;
 }
 
 static void do_subst_w_backrefs(char *line, char *replace)
@@ -580,10 +592,10 @@ static void do_subst_w_backrefs(char *line, char *replace)
 		if (replace[i] == '\\') {
 			unsigned backref = replace[++i] - '0';
 			if (backref <= 9) {
-				/* print out the text held in bbg.regmatch[backref] */
-				if (bbg.regmatch[backref].rm_so != -1) {
-					j = bbg.regmatch[backref].rm_so;
-					while (j < bbg.regmatch[backref].rm_eo)
+				/* print out the text held in G.regmatch[backref] */
+				if (G.regmatch[backref].rm_so != -1) {
+					j = G.regmatch[backref].rm_so;
+					while (j < G.regmatch[backref].rm_eo)
 						pipe_putc(line[j++]);
 				}
 				continue;
@@ -597,8 +609,8 @@ static void do_subst_w_backrefs(char *line, char *replace)
 		}
 		/* if we find an unescaped '&' print out the whole matched text. */
 		if (replace[i] == '&') {
-			j = bbg.regmatch[0].rm_so;
-			while (j < bbg.regmatch[0].rm_eo)
+			j = G.regmatch[0].rm_so;
+			while (j < G.regmatch[0].rm_eo)
 				pipe_putc(line[j++]);
 			continue;
 		}
@@ -616,20 +628,20 @@ static int do_subst_command(sed_cmd_t *sed_cmd, char **line)
 
 	/* Handle empty regex. */
 	if (sed_cmd->sub_match == NULL) {
-		current_regex = bbg.previous_regex_ptr;
+		current_regex = G.previous_regex_ptr;
 		if (!current_regex)
 			bb_error_msg_and_die("no previous regexp");
 	} else
-		bbg.previous_regex_ptr = current_regex = sed_cmd->sub_match;
+		G.previous_regex_ptr = current_regex = sed_cmd->sub_match;
 
 	/* Find the first match */
-	if (REG_NOMATCH == regexec(current_regex, oldline, 10, bbg.regmatch, 0))
+	if (REG_NOMATCH == regexec(current_regex, oldline, 10, G.regmatch, 0))
 		return 0;
 
 	/* Initialize temporary output buffer. */
-	bbg.pipeline.buf = xmalloc(PIPE_GROW);
-	bbg.pipeline.len = PIPE_GROW;
-	bbg.pipeline.idx = 0;
+	G.pipeline.buf = xmalloc(PIPE_GROW);
+	G.pipeline.len = PIPE_GROW;
+	G.pipeline.idx = 0;
 
 	/* Now loop through, substituting for matches */
 	do {
@@ -639,7 +651,7 @@ static int do_subst_command(sed_cmd_t *sed_cmd, char **line)
 		   echo " a.b" | busybox sed 's [^ .]* x g'
 		   The match_count check is so not to break
 		   echo "hi" | busybox sed 's/^/!/g' */
-		if (!bbg.regmatch[0].rm_so && !bbg.regmatch[0].rm_eo && match_count) {
+		if (!G.regmatch[0].rm_so && !G.regmatch[0].rm_eo && match_count) {
 			pipe_putc(*oldline++);
 			continue;
 		}
@@ -649,26 +661,26 @@ static int do_subst_command(sed_cmd_t *sed_cmd, char **line)
 		/* If we aren't interested in this match, output old line to
 		   end of match and continue */
 		if (sed_cmd->which_match && sed_cmd->which_match != match_count) {
-			for (i = 0; i < bbg.regmatch[0].rm_eo; i++)
+			for (i = 0; i < G.regmatch[0].rm_eo; i++)
 				pipe_putc(*oldline++);
 			continue;
 		}
 
 		/* print everything before the match */
-		for (i = 0; i < bbg.regmatch[0].rm_so; i++)
+		for (i = 0; i < G.regmatch[0].rm_so; i++)
 			pipe_putc(oldline[i]);
 
 		/* then print the substitution string */
 		do_subst_w_backrefs(oldline, sed_cmd->string);
 
 		/* advance past the match */
-		oldline += bbg.regmatch[0].rm_eo;
+		oldline += G.regmatch[0].rm_eo;
 		/* flag that something has changed */
 		altered++;
 
 		/* if we're not doing this globally, get out now */
 		if (sed_cmd->which_match) break;
-	} while (*oldline && (regexec(current_regex, oldline, 10, bbg.regmatch, 0) != REG_NOMATCH));
+	} while (*oldline && (regexec(current_regex, oldline, 10, G.regmatch, 0) != REG_NOMATCH));
 
 	/* Copy rest of string into output pipeline */
 
@@ -677,7 +689,7 @@ static int do_subst_command(sed_cmd_t *sed_cmd, char **line)
 	pipe_putc(0);
 
 	free(*line);
-	*line = bbg.pipeline.buf;
+	*line = G.pipeline.buf;
 	return altered;
 }
 
@@ -686,7 +698,7 @@ static sed_cmd_t *branch_to(char *label)
 {
 	sed_cmd_t *sed_cmd;
 
-	for (sed_cmd = bbg.sed_cmd_head.next; sed_cmd; sed_cmd = sed_cmd->next) {
+	for (sed_cmd = G.sed_cmd_head.next; sed_cmd; sed_cmd = sed_cmd->next) {
 		if (sed_cmd->cmd == ':' && sed_cmd->string && !strcmp(sed_cmd->string, label)) {
 			return sed_cmd;
 		}
@@ -696,7 +708,7 @@ static sed_cmd_t *branch_to(char *label)
 
 static void append(char *s)
 {
-	llist_add_to_end(&bbg.append_head, xstrdup(s));
+	llist_add_to_end(&G.append_head, xstrdup(s));
 }
 
 static void flush_append(void)
@@ -704,60 +716,79 @@ static void flush_append(void)
 	char *data;
 
 	/* Output appended lines. */
-	while ((data = (char *)llist_pop(&bbg.append_head))) {
-		fprintf(bbg.nonstdout, "%s\n", data);
+	while ((data = (char *)llist_pop(&G.append_head))) {
+		fprintf(G.nonstdout, "%s\n", data);
 		free(data);
 	}
 }
 
 static void add_input_file(FILE *file)
 {
-	bbg.input_file_list = xrealloc(bbg.input_file_list,
-			(bbg.input_file_count + 1) * sizeof(FILE *));
-	bbg.input_file_list[bbg.input_file_count++] = file;
+	G.input_file_list = xrealloc(G.input_file_list,
+			(G.input_file_count + 1) * sizeof(FILE *));
+	G.input_file_list[G.input_file_count++] = file;
 }
 
-/* Get next line of input from bbg.input_file_list, flushing append buffer and
+/* Get next line of input from G.input_file_list, flushing append buffer and
  * noting if we ran out of files without a newline on the last line we read.
  */
-static char *get_next_line(int *last_char)
+enum {
+	NO_EOL_CHAR = 1,
+	LAST_IS_NUL = 2,
+};
+static char *get_next_line(char *gets_char)
 {
 	char *temp = NULL;
-	int len, lc;
+	int len;
+	char gc;
 
-	lc = 0;
 	flush_append();
-	while (bbg.current_input_file < bbg.input_file_count) {
+
+	/* will be returned if last line in the file
+	 * doesn't end with either '\n' or '\0' */
+	gc = NO_EOL_CHAR;
+	while (G.current_input_file < G.input_file_count) {
+		FILE *fp = G.input_file_list[G.current_input_file];
 		/* Read line up to a newline or NUL byte, inclusive,
 		 * return malloc'ed char[]. length of the chunk read
 		 * is stored in len. NULL if EOF/error */
-		temp = bb_get_chunk_from_file(
-			bbg.input_file_list[bbg.current_input_file], &len);
+		temp = bb_get_chunk_from_file(fp, &len);
 		if (temp) {
 			/* len > 0 here, it's ok to do temp[len-1] */
 			char c = temp[len-1];
 			if (c == '\n' || c == '\0') {
 				temp[len-1] = '\0';
-				lc |= (unsigned char)c;
-				break;
+				gc = c;
+				if (c == '\0') {
+					int ch = fgetc(fp);
+					if (ch != EOF)
+						ungetc(ch, fp);
+					else
+						gc = LAST_IS_NUL;
+				}
 			}
-			/* will be returned if last line in the file
-			 * doesn't end with either '\n' or '\0' */
-			lc |= 0x100;
+			/* else we put NO_EOL_CHAR into *gets_char */
 			break;
+
+		/* NB: I had the idea of peeking next file(s) and returning
+		 * NO_EOL_CHAR only if it is the *last* non-empty
+		 * input file. But there is a case where this won't work:
+		 * file1: "a woo\nb woo"
+		 * file2: "c no\nd no"
+		 * sed -ne 's/woo/bang/p' input1 input2 => "a bang\nb bang"
+		 * (note: *no* newline after "b bang"!) */
 		}
 		/* Close this file and advance to next one */
-		fclose(bbg.input_file_list[bbg.current_input_file++]);
-		/* "this is the first line from new input file" */
-		lc |= 0x200;
+		fclose(fp);
+		G.current_input_file++;
 	}
-	*last_char = lc;
+	*gets_char = gc;
 	return temp;
 }
 
 /* Output line of text. */
 /* Note:
- * The tricks with 0x200 and last_puts_char are there to emulate gnu sed.
+ * The tricks with NO_EOL_CHAR and last_puts_char are there to emulate gnu sed.
  * Without them, we had this:
  * echo -n thingy >z1
  * echo -n again >z2
@@ -769,50 +800,63 @@ static char *get_next_line(int *last_char)
  * bbox:
  * 00000000  74 68 7a 6e 67 79 61 67  61 7a 6e                 |thzngyagazn|
  */
-
-static int puts_maybe_newline(char *s, FILE *file, int prev_last_char, int last_char)
+static void puts_maybe_newline(char *s, FILE *file, char *last_puts_char, char last_gets_char)
 {
-	static char last_puts_char;
+	char lpc = *last_puts_char;
 
-	/* Is this a first line from new file
-	 * and old file didn't end with '\n'? */
-	if ((last_char & 0x200) && last_puts_char != '\n') {
+	/* Need to insert a '\n' between two files because first file's
+	 * last line wasn't terminated? */
+	if (lpc != '\n' && lpc != '\0') {
 		fputc('\n', file);
-		last_puts_char = '\n';
+		lpc = '\n';
 	}
 	fputs(s, file);
-	/* why 'x'? - just something which is not '\n' */
+
+	/* 'x' - just something which is not '\n', '\0' or NO_EOL_CHAR */
 	if (s[0])
-		last_puts_char = 'x';
-	if (!(last_char & 0x100)) { /* had trailing '\n' or '\0'? */
-		last_char &= 0xff;
-		fputc(last_char, file);
-		last_puts_char = last_char;
+		lpc = 'x';
+
+	/* had trailing '\0' and it was last char of file? */
+	if (last_gets_char == LAST_IS_NUL) {
+		fputc('\0', file);
+		lpc = 'x'; /* */
+	} else
+	/* had trailing '\n' or '\0'? */
+	if (last_gets_char != NO_EOL_CHAR) {
+		fputc(last_gets_char, file);
+		lpc = last_gets_char;
 	}
 
 	if (ferror(file)) {
 		xfunc_error_retval = 4;  /* It's what gnu sed exits with... */
 		bb_error_msg_and_die(bb_msg_write_error);
 	}
-
-	return last_char;
+	*last_puts_char = lpc;
 }
 
-#define sed_puts(s, n) \
-	(prev_last_char = puts_maybe_newline(s, bbg.nonstdout, prev_last_char, n))
+#define sed_puts(s, n) (puts_maybe_newline(s, G.nonstdout, &last_puts_char, n))
+
+static int beg_match(sed_cmd_t *sed_cmd, const char *pattern_space)
+{
+	int retval = sed_cmd->beg_match && !regexec(sed_cmd->beg_match, pattern_space, 0, NULL, 0);
+	if (retval)
+		G.previous_regex_ptr = sed_cmd->beg_match;
+	return retval;
+}
 
 /* Process all the lines in all the files */
 
 static void process_files(void)
 {
 	char *pattern_space, *next_line;
-	int linenum = 0, prev_last_char = 0;
-	int last_char, next_last_char = 0;
+	int linenum = 0;
+	char last_puts_char = '\n';
+	char last_gets_char, next_gets_char;
 	sed_cmd_t *sed_cmd;
 	int substituted;
 
 	/* Prime the pump */
-	next_line = get_next_line(&next_last_char);
+	next_line = get_next_line(&next_gets_char);
 
 	/* go through every line in each file */
 again:
@@ -821,15 +865,15 @@ again:
 	/* Advance to next line.  Stop if out of lines. */
 	pattern_space = next_line;
 	if (!pattern_space) return;
-	last_char = next_last_char;
+	last_gets_char = next_gets_char;
 
 	/* Read one line in advance so we can act on the last line,
 	 * the '$' address */
-	next_line = get_next_line(&next_last_char);
+	next_line = get_next_line(&next_gets_char);
 	linenum++;
 restart:
 	/* for every line, go through all the commands */
-	for (sed_cmd = bbg.sed_cmd_head.next; sed_cmd; sed_cmd = sed_cmd->next) {
+	for (sed_cmd = G.sed_cmd_head.next; sed_cmd; sed_cmd = sed_cmd->next) {
 		int old_matched, matched;
 
 		old_matched = sed_cmd->in_match;
@@ -844,8 +888,7 @@ restart:
 			/* Or did we match the start of a numerical range? */
 			|| (sed_cmd->beg_line > 0 && (sed_cmd->beg_line == linenum))
 			/* Or does this line match our begin address regex? */
-			|| (sed_cmd->beg_match &&
-			    !regexec(sed_cmd->beg_match, pattern_space, 0, NULL, 0))
+			|| (beg_match(sed_cmd, pattern_space))
 			/* Or did we match last line of input? */
 			|| (sed_cmd->beg_line == -1 && next_line == NULL);
 
@@ -867,16 +910,19 @@ restart:
 				/* or does this line matches our last address regex */
 				|| (sed_cmd->end_match && old_matched
 				     && (regexec(sed_cmd->end_match,
-				    	         pattern_space, 0, NULL, 0) == 0))
+				                 pattern_space, 0, NULL, 0) == 0))
 			);
 		}
 
 		/* Skip blocks of commands we didn't match. */
 		if (sed_cmd->cmd == '{') {
-			if (sed_cmd->invert ? matched : !matched)
-				while (sed_cmd && sed_cmd->cmd != '}')
+			if (sed_cmd->invert ? matched : !matched) {
+				while (sed_cmd->cmd != '}') {
 					sed_cmd = sed_cmd->next;
-			if (!sed_cmd) bb_error_msg_and_die("unterminated {");
+					if (!sed_cmd)
+						bb_error_msg_and_die("unterminated {");
+				}
+			}
 			continue;
 		}
 
@@ -884,7 +930,7 @@ restart:
 		if (sed_cmd->invert ? !matched : matched) {
 			/* Update last used regex in case a blank substitute BRE is found */
 			if (sed_cmd->beg_match) {
-				bbg.previous_regex_ptr = sed_cmd->beg_match;
+				G.previous_regex_ptr = sed_cmd->beg_match;
 			}
 
 			/* actual sedding */
@@ -892,7 +938,7 @@ restart:
 
 			/* Print line number */
 			case '=':
-				fprintf(bbg.nonstdout, "%d\n", linenum);
+				fprintf(G.nonstdout, "%d\n", linenum);
 				break;
 
 			/* Write the current pattern space up to the first newline */
@@ -902,7 +948,8 @@ restart:
 
 				if (tmp) {
 					*tmp = '\0';
-					sed_puts(pattern_space, 1);
+					/* TODO: explain why '\n' below */
+					sed_puts(pattern_space, '\n');
 					*tmp = '\n';
 					break;
 				}
@@ -911,7 +958,11 @@ restart:
 
 			/* Write the current pattern space to output */
 			case 'p':
-				sed_puts(pattern_space, last_char);
+				/* NB: we print this _before_ the last line
+				 * (of current file) is printed. Even if
+				 * that line is nonterminated, we print
+				 * '\n' here (gnu sed does the same) */
+				sed_puts(pattern_space, '\n');
 				break;
 			/* Delete up through first newline */
 			case 'D':
@@ -937,12 +988,12 @@ restart:
 
 				/* handle p option */
 				if (sed_cmd->sub_p)
-					sed_puts(pattern_space, last_char);
+					sed_puts(pattern_space, last_gets_char);
 				/* handle w option */
-				if (sed_cmd->file)
-					sed_cmd->last_char = puts_maybe_newline(
-						pattern_space, sed_cmd->file,
-						sed_cmd->last_char, last_char);
+				if (sed_cmd->sw_file)
+					puts_maybe_newline(
+						pattern_space, sed_cmd->sw_file,
+						&sed_cmd->sw_last_char, last_gets_char);
 				break;
 
 			/* Append line to linked list to be printed later */
@@ -952,14 +1003,14 @@ restart:
 
 			/* Insert text before this line */
 			case 'i':
-				sed_puts(sed_cmd->string, 1);
+				sed_puts(sed_cmd->string, '\n');
 				break;
 
 			/* Cut and paste text (replace) */
 			case 'c':
 				/* Only triggers on last line of a matching range. */
 				if (!sed_cmd->in_match)
-					sed_puts(sed_cmd->string, 0);
+					sed_puts(sed_cmd->string, NO_EOL_CHAR);
 				goto discard_line;
 
 			/* Read file, append contents to output */
@@ -982,20 +1033,20 @@ restart:
 
 			/* Write pattern space to file. */
 			case 'w':
-				sed_cmd->last_char = puts_maybe_newline(
-					pattern_space, sed_cmd->file,
-					sed_cmd->last_char, last_char);
+				puts_maybe_newline(
+					pattern_space, sed_cmd->sw_file,
+					&sed_cmd->sw_last_char, last_gets_char);
 				break;
 
 			/* Read next line from input */
 			case 'n':
-				if (!bbg.be_quiet)
-					sed_puts(pattern_space, last_char);
+				if (!G.be_quiet)
+					sed_puts(pattern_space, last_gets_char);
 				if (next_line) {
 					free(pattern_space);
 					pattern_space = next_line;
-					last_char = next_last_char;
-					next_line = get_next_line(&next_last_char);
+					last_gets_char = next_gets_char;
+					next_line = get_next_line(&next_gets_char);
 					linenum++;
 					break;
 				}
@@ -1024,8 +1075,8 @@ restart:
 				pattern_space = realloc(pattern_space, len + strlen(next_line) + 2);
 				pattern_space[len] = '\n';
 				strcpy(pattern_space + len+1, next_line);
-				last_char = next_last_char;
-				next_line = get_next_line(&next_last_char);
+				last_gets_char = next_gets_char;
+				next_line = get_next_line(&next_gets_char);
 				linenum++;
 				break;
 			}
@@ -1062,7 +1113,7 @@ restart:
 			}
 			case 'g':	/* Replace pattern space with hold space */
 				free(pattern_space);
-				pattern_space = xstrdup(bbg.hold_space ? bbg.hold_space : "");
+				pattern_space = xstrdup(G.hold_space ? G.hold_space : "");
 				break;
 			case 'G':	/* Append newline and hold space to pattern space */
 			{
@@ -1071,49 +1122,49 @@ restart:
 
 				if (pattern_space)
 					pattern_space_size += strlen(pattern_space);
-				if (bbg.hold_space)
-					hold_space_size = strlen(bbg.hold_space);
+				if (G.hold_space)
+					hold_space_size = strlen(G.hold_space);
 				pattern_space = xrealloc(pattern_space,
 						pattern_space_size + hold_space_size);
 				if (pattern_space_size == 2)
 					pattern_space[0] = 0;
 				strcat(pattern_space, "\n");
-				if (bbg.hold_space)
-					strcat(pattern_space, bbg.hold_space);
-				last_char = '\n';
+				if (G.hold_space)
+					strcat(pattern_space, G.hold_space);
+				last_gets_char = '\n';
 
 				break;
 			}
 			case 'h':	/* Replace hold space with pattern space */
-				free(bbg.hold_space);
-				bbg.hold_space = xstrdup(pattern_space);
+				free(G.hold_space);
+				G.hold_space = xstrdup(pattern_space);
 				break;
 			case 'H':	/* Append newline and pattern space to hold space */
 			{
 				int hold_space_size = 2;
 				int pattern_space_size = 0;
 
-				if (bbg.hold_space)
-					hold_space_size += strlen(bbg.hold_space);
+				if (G.hold_space)
+					hold_space_size += strlen(G.hold_space);
 				if (pattern_space)
 					pattern_space_size = strlen(pattern_space);
-				bbg.hold_space = xrealloc(bbg.hold_space,
+				G.hold_space = xrealloc(G.hold_space,
 						hold_space_size + pattern_space_size);
 
 				if (hold_space_size == 2)
-					*bbg.hold_space = 0;
-				strcat(bbg.hold_space, "\n");
+					*G.hold_space = 0;
+				strcat(G.hold_space, "\n");
 				if (pattern_space)
-					strcat(bbg.hold_space, pattern_space);
+					strcat(G.hold_space, pattern_space);
 
 				break;
 			}
 			case 'x': /* Exchange hold and pattern space */
 			{
 				char *tmp = pattern_space;
-				pattern_space = bbg.hold_space ? : xzalloc(1);
-				last_char = '\n';
-				bbg.hold_space = tmp;
+				pattern_space = G.hold_space ? : xzalloc(1);
+				last_gets_char = '\n';
+				G.hold_space = tmp;
 				break;
 			}
 			}
@@ -1123,13 +1174,14 @@ restart:
 	/*
 	 * exit point from sedding...
 	 */
-discard_commands:
+ discard_commands:
 	/* we will print the line unless we were told to be quiet ('-n')
 	   or if the line was suppressed (ala 'd'elete) */
-	if (!bbg.be_quiet) sed_puts(pattern_space, last_char);
+	if (!G.be_quiet)
+		sed_puts(pattern_space, last_gets_char);
 
 	/* Delete and such jump here. */
-discard_line:
+ discard_line:
 	flush_append();
 	free(pattern_space);
 
@@ -1137,46 +1189,41 @@ discard_line:
 }
 
 /* It is possible to have a command line argument with embedded
-   newlines.  This counts as multiple command lines. */
+ * newlines.  This counts as multiple command lines.
+ * However, newline can be escaped: 's/e/z\<newline>z/'
+ * We check for this.
+ */
 
 static void add_cmd_block(char *cmdstr)
 {
-	int go = 1;
-	char *temp = xstrdup(cmdstr), *temp2 = temp;
+	char *sv, *eol;
 
-	while (go) {
-		int len = strcspn(temp2, "\n");
-		if (!temp2[len]) go = 0;
-		else temp2[len] = 0;
-		add_cmd(temp2);
-		temp2 += len+1;
-	}
-	free(temp);
-}
+	cmdstr = sv = xstrdup(cmdstr);
+	do {
+		eol = strchr(cmdstr, '\n');
+ next:
+		if (eol) {
+			/* Count preceding slashes */
+			int slashes = 0;
+			char *sl = eol;
 
-static void add_cmds_link(llist_t *opt_e)
-{
-	if (!opt_e) return;
-	add_cmds_link(opt_e->link);
-	add_cmd_block(opt_e->data);
-	free(opt_e);
-}
-
-static void add_files_link(llist_t *opt_f)
-{
-	char *line;
-	FILE *cmdfile;
-	if (!opt_f) return;
-	add_files_link(opt_f->link);
-	cmdfile = xfopen(opt_f->data, "r");
-	while ((line = xmalloc_getline(cmdfile)) != NULL) {
-		add_cmd(line);
-		free(line);
-	}
-	xprint_and_close_file(cmdfile);
-	free(opt_f);
+			while (sl != cmdstr && *--sl == '\\')
+				slashes++;
+			/* Odd number of preceding slashes - newline is escaped */
+			if (slashes & 1) {
+				strcpy(eol-1, eol);
+				eol = strchr(eol, '\n');
+				goto next;
+			}
+			*eol = '\0';
+		}
+		add_cmd(cmdstr);
+		cmdstr = eol + 1;
+	} while (eol);
+	free(sv);
 }
 
+int sed_main(int argc, char **argv);
 int sed_main(int argc, char **argv)
 {
 	enum {
@@ -1186,7 +1233,7 @@ int sed_main(int argc, char **argv)
 	llist_t *opt_e, *opt_f;
 	int status = EXIT_SUCCESS;
 
-	bbg.sed_cmd_tail = &bbg.sed_cmd_head;
+	INIT_G();
 
 	/* destroy command strings on exit */
 	if (ENABLE_FEATURE_CLEAN_UP) atexit(sed_free_and_close_stuff);
@@ -1201,22 +1248,31 @@ int sed_main(int argc, char **argv)
 	opt_e = opt_f = NULL;
 	opt_complementary = "e::f::" /* can occur multiple times */
 	                    "nn"; /* count -n */
-	opt = getopt32(argc, argv, "irne:f:", &opt_e, &opt_f,
-			    &bbg.be_quiet); /* counter for -n */
+	opt = getopt32(argv, "irne:f:", &opt_e, &opt_f,
+			    &G.be_quiet); /* counter for -n */
 	argc -= optind;
 	argv += optind;
 	if (opt & OPT_in_place) { // -i
 		atexit(cleanup_outname);
 	}
-	if (opt & 0x2) bbg.regex_type |= REG_EXTENDED; // -r
-	//if (opt & 0x4) bbg.be_quiet++; // -n
-	if (opt & 0x8) { // -e
-		/* getopt32 reverses order of arguments, handle it */
-		add_cmds_link(opt_e);
+	if (opt & 0x2) G.regex_type |= REG_EXTENDED; // -r
+	//if (opt & 0x4) G.be_quiet++; // -n
+	while (opt_e) { // -e
+		add_cmd_block(opt_e->data);
+		opt_e = opt_e->link;
+		/* we leak opt_e here... */
 	}
-	if (opt & 0x10) { // -f
-		/* getopt32 reverses order of arguments, handle it */
-		add_files_link(opt_f);
+	while (opt_f) { // -f
+		char *line;
+		FILE *cmdfile;
+		cmdfile = xfopen(opt_f->data, "r");
+		while ((line = xmalloc_getline(cmdfile)) != NULL) {
+			add_cmd(line);
+			free(line);
+		}
+		fclose(cmdfile);
+		opt_f = opt_f->link;
+		/* we leak opt_f here... */
 	}
 	/* if we didn't get a pattern from -e or -f, use argv[0] */
 	if (!(opt & 0x18)) {
@@ -1229,7 +1285,7 @@ int sed_main(int argc, char **argv)
 	add_cmd("");
 
 	/* By default, we write to stdout */
-	bbg.nonstdout = stdout;
+	G.nonstdout = stdout;
 
 	/* argv[0..(argc-1)] should be names of file to process. If no
 	 * files were specified or '-' was specified, take input from stdin.
@@ -1262,11 +1318,11 @@ int sed_main(int argc, char **argv)
 				continue;
 			}
 
-			bbg.outname = xasprintf("%sXXXXXX", argv[i]);
-			nonstdoutfd = mkstemp(bbg.outname);
+			G.outname = xasprintf("%sXXXXXX", argv[i]);
+			nonstdoutfd = mkstemp(G.outname);
 			if (-1 == nonstdoutfd)
-				bb_error_msg_and_die("no temp file");
-			bbg.nonstdout = fdopen(nonstdoutfd, "w");
+				bb_perror_msg_and_die("cannot create temp file %s", G.outname);
+			G.nonstdout = fdopen(nonstdoutfd, "w");
 
 			/* Set permissions of output file */
 
@@ -1274,16 +1330,16 @@ int sed_main(int argc, char **argv)
 			fchmod(nonstdoutfd, statbuf.st_mode);
 			add_input_file(file);
 			process_files();
-			fclose(bbg.nonstdout);
+			fclose(G.nonstdout);
 
-			bbg.nonstdout = stdout;
+			G.nonstdout = stdout;
 			/* unlink(argv[i]); */
 			// FIXME: error check / message?
-			rename(bbg.outname, argv[i]);
-			free(bbg.outname);
-			bbg.outname = 0;
+			rename(G.outname, argv[i]);
+			free(G.outname);
+			G.outname = 0;
 		}
-		if (bbg.input_file_count > bbg.current_input_file)
+		if (G.input_file_count > G.current_input_file)
 			process_files();
 	}
 
