@@ -7,10 +7,11 @@
  * Licensed under GPL version 2, see file LICENSE in this tarball for details.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 #include <sys/reboot.h>
 
-int halt_main(int argc, char *argv[])
+int halt_main(int argc, char **argv);
+int halt_main(int argc, char **argv)
 {
 	static const int magic[] = {
 #ifdef RB_HALT_SYSTEM
@@ -34,7 +35,7 @@ RB_AUTOBOOT
 	for (which = 0; "hpr"[which] != *applet_name; which++);
 
 	/* Parse and handle arguments */
-	flags = getopt32(argc, argv, "d:nf", &delay);
+	flags = getopt32(argv, "d:nf", &delay);
 	if (flags & 1) sleep(xatou(delay));
 	if (!(flags & 2)) sync();
 
