@@ -9,56 +9,20 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include "busybox.h"
-#include <errno.h>
+#include "libbb.h"
 #include <paths.h>
-#include <signal.h>
-#include <sys/ioctl.h>
-#include <sys/wait.h>
+//#include <signal.h>
+//#include <sys/ioctl.h>
+//#include <sys/wait.h>
 #include <sys/reboot.h>
 
-#include "init_shared.h"
-
-#if ENABLE_SYSLOGD
+#if ENABLE_FEATURE_INIT_SYSLOG
 # include <sys/syslog.h>
 #endif
 
 #define INIT_BUFFS_SIZE 256
-
-/* From <linux/vt.h> */
-struct vt_stat {
-	unsigned short v_active;	/* active vt */
-	unsigned short v_signal;	/* signal to send */
-	unsigned short v_state;	/* vt bitmask */
-};
-enum { VT_GETSTATE = 0x5603 };	/* get global vt state info */
-
-/* From <linux/serial.h> */
-struct serial_struct {
-	int	type;
-	int	line;
-	unsigned int	port;
-	int	irq;
-	int	flags;
-	int	xmit_fifo_size;
-	int	custom_divisor;
-	int	baud_base;
-	unsigned short	close_delay;
-	char	io_type;
-	char	reserved_char[1];
-	int	hub6;
-	unsigned short	closing_wait; /* time to wait before closing */
-	unsigned short	closing_wait2; /* no longer used... */
-	unsigned char	*iomem_base;
-	unsigned short	iomem_reg_shift;
-	unsigned int	port_high;
-	unsigned long	iomap_base;	/* cookie passed into ioremap */
-	int	reserved[1];
-};
-
-#ifndef _PATH_STDPATH
-#define _PATH_STDPATH	"/usr/bin:/bin:/usr/sbin:/sbin"
-#endif
+#define CONSOLE_NAME_SIZE 32
+#define MAXENV	16		/* Number of env. vars */
 
 #if ENABLE_FEATURE_INIT_COREDUMPS
 /*
@@ -76,10 +40,6 @@ struct serial_struct {
 #define INIT_SCRIPT  "/etc/init.d/rcS"	/* Default sysinit script. */
 #endif
 
-#define MAXENV	16		/* Number of env. vars */
-
-#define CONSOLE_BUFF_SIZE 32
-
 /* Allowed init action types */
 #define SYSINIT     0x001
 #define RESPAWN     0x002
@@ -110,32 +70,31 @@ static const struct init_action_type actions[] = {
 
 /* Set up a linked list of init_actions, to be read from inittab */
 struct init_action {
-	pid_t pid;
-	char command[INIT_BUFFS_SIZE];
-	char terminal[CONSOLE_BUFF_SIZE];
 	struct init_action *next;
 	int action;
+	pid_t pid;
+	char command[INIT_BUFFS_SIZE];
+	char terminal[CONSOLE_NAME_SIZE];
 };
 
 /* Static variables */
 static struct init_action *init_action_list = NULL;
-static char console[CONSOLE_BUFF_SIZE] = CONSOLE_DEV;
 
-#if !ENABLE_SYSLOGD
-static char *log_console = VC_5;
+#if !ENABLE_FEATURE_INIT_SYSLOG
+static const char *log_console = VC_5;
 #endif
 #if !ENABLE_DEBUG_INIT
 static sig_atomic_t got_cont = 0;
 #endif
 
 enum {
-	LOG = 0x1,
-	CONSOLE = 0x2,
+	L_LOG = 0x1,
+	L_CONSOLE = 0x2,
 
 #if ENABLE_FEATURE_EXTRA_QUIET
 	MAYBE_CONSOLE = 0x0,
 #else
-	MAYBE_CONSOLE = CONSOLE,
+	MAYBE_CONSOLE = L_CONSOLE,
 #endif
 
 #ifndef RB_HALT_SYSTEM
@@ -147,9 +106,9 @@ enum {
 #endif
 };
 
-static const char * const environment[] = {
+static const char *const environment[] = {
 	"HOME=/",
-	"PATH=" _PATH_STDPATH,
+	bb_PATH_root_path,
 	"SHELL=/bin/sh",
 	"USER=root",
 	NULL
@@ -171,7 +130,7 @@ static void loop_forever(void)
 #endif
 
 /* Print a message to the specified device.
- * Device may be bitwise-or'd from LOG | CONSOLE */
+ * Device may be bitwise-or'd from L_LOG | L_CONSOLE */
 #if ENABLE_DEBUG_INIT
 #define messageD message
 #else
@@ -181,71 +140,64 @@ static void message(int device, const char *fmt, ...)
 	__attribute__ ((format(printf, 2, 3)));
 static void message(int device, const char *fmt, ...)
 {
-	va_list arguments;
-	int l;
-	RESERVE_CONFIG_BUFFER(msg, 1024);
-#if !ENABLE_SYSLOGD
+#if !ENABLE_FEATURE_INIT_SYSLOG
 	static int log_fd = -1;
 #endif
 
-	msg[0] = '\r';
-		va_start(arguments, fmt);
-	l = vsnprintf(msg + 1, 1024 - 2, fmt, arguments) + 1;
-		va_end(arguments);
+	va_list arguments;
+	int l;
+	char msg[128];
 
-#if ENABLE_SYSLOGD
+	msg[0] = '\r';
+	va_start(arguments, fmt);
+	vsnprintf(msg + 1, sizeof(msg) - 2, fmt, arguments);
+	va_end(arguments);
+	msg[sizeof(msg) - 2] = '\0';
+	l = strlen(msg);
+
+#if ENABLE_FEATURE_INIT_SYSLOG
 	/* Log the message to syslogd */
-	if (device & LOG) {
-		/* don`t out "\r\n" */
+	if (device & L_LOG) {
+		/* don't out "\r" */
 		openlog(applet_name, 0, LOG_DAEMON);
-		syslog(LOG_INFO, "%s", msg + 1);
+		syslog(LOG_INFO, "init: %s", msg + 1);
 		closelog();
 	}
-
 	msg[l++] = '\n';
-	msg[l] = 0;
+	msg[l] = '\0';
 #else
-
 	msg[l++] = '\n';
-	msg[l] = 0;
+	msg[l] = '\0';
 	/* Take full control of the log tty, and never close it.
 	 * It's mine, all mine!  Muhahahaha! */
 	if (log_fd < 0) {
-		if ((log_fd = device_open(log_console, O_RDWR | O_NONBLOCK | O_NOCTTY)) < 0) {
-			log_fd = -2;
-			bb_error_msg("bummer, can't write to log on %s!", log_console);
-			device = CONSOLE;
+		if (!log_console) {
+			log_fd = 2;
 		} else {
-			fcntl(log_fd, F_SETFD, FD_CLOEXEC);
+			log_fd = device_open(log_console, O_WRONLY | O_NONBLOCK | O_NOCTTY);
+			if (log_fd < 0) {
+				bb_error_msg("can't log to %s", log_console);
+				device = L_CONSOLE;
+			} else {
+				fcntl(log_fd, F_SETFD, FD_CLOEXEC);
+			}
 		}
 	}
-	if ((device & LOG) && (log_fd >= 0)) {
+	if (device & L_LOG) {
 		full_write(log_fd, msg, l);
+		if (log_fd == 2)
+			return; /* don't print dup messages */
 	}
 #endif
 
-	if (device & CONSOLE) {
-		int fd = device_open(CONSOLE_DEV,
-					O_WRONLY | O_NOCTTY | O_NONBLOCK);
-		/* Always send console messages to /dev/console so people will see them. */
-		if (fd >= 0) {
-			full_write(fd, msg, l);
-			close(fd);
-#if ENABLE_DEBUG_INIT
-		/* all descriptors may be closed */
-		} else {
-			bb_error_msg("bummer, can't print: ");
-			va_start(arguments, fmt);
-			vfprintf(stderr, fmt, arguments);
-			va_end(arguments);
-#endif
-		}
+	if (device & L_CONSOLE) {
+		/* Send console messages to console so people will see them. */
+		full_write(2, msg, l);
 	}
-	RELEASE_CONFIG_BUFFER(msg);
 }
 
 /* Set terminal settings to reasonable defaults */
-static void set_term(void)
+static void set_sane_term(void)
 {
 	struct termios tty;
 
@@ -268,7 +220,6 @@ static void set_term(void)
 	tty.c_cflag &= CBAUD | CBAUDEX | CSIZE | CSTOPB | PARENB | PARODD;
 	tty.c_cflag |= CREAD | HUPCL | CLOCAL;
 
-
 	/* input modes */
 	tty.c_iflag = ICRNL | IXON | IXOFF;
 
@@ -282,292 +233,276 @@ static void set_term(void)
 	tcsetattr(STDIN_FILENO, TCSANOW, &tty);
 }
 
+/* From <linux/serial.h> */
+struct serial_struct {
+	int	type;
+	int	line;
+	unsigned int	port;
+	int	irq;
+	int	flags;
+	int	xmit_fifo_size;
+	int	custom_divisor;
+	int	baud_base;
+	unsigned short	close_delay;
+	char	io_type;
+	char	reserved_char[1];
+	int	hub6;
+	unsigned short	closing_wait; /* time to wait before closing */
+	unsigned short	closing_wait2; /* no longer used... */
+	unsigned char	*iomem_base;
+	unsigned short	iomem_reg_shift;
+	unsigned int	port_high;
+	unsigned long	iomap_base;	/* cookie passed into ioremap */
+	int	reserved[1];
+	/* Paranoia (imagine 64bit kernel overwriting 32bit userspace stack) */
+	uint32_t bbox_reserved[16];
+};
 static void console_init(void)
 {
-	int fd;
-	int tried = 0;
-	struct vt_stat vt;
 	struct serial_struct sr;
 	char *s;
 
-	if ((s = getenv("CONSOLE")) != NULL || (s = getenv("console")) != NULL) {
-		safe_strncpy(console, s, sizeof(console));
-	} else {
-		/* 2.2 kernels: identify the real console backend and try to use it */
-		if (ioctl(0, TIOCGSERIAL, &sr) == 0) {
-			/* this is a serial console */
-			snprintf(console, sizeof(console) - 1, SC_FORMAT, sr.line);
-		} else if (ioctl(0, VT_GETSTATE, &vt) == 0) {
-			/* this is linux virtual tty */
-			snprintf(console, sizeof(console) - 1, VC_FORMAT, vt.v_active);
-		} else {
-			safe_strncpy(console, CONSOLE_DEV, sizeof(console));
-			tried++;
+	s = getenv("CONSOLE");
+	if (!s) s = getenv("console");
+	if (s) {
+		int fd = open(s, O_RDWR | O_NONBLOCK | O_NOCTTY);
+		if (fd >= 0) {
+			dup2(fd, 0);
+			dup2(fd, 1);
+			dup2(fd, 2);
+			while (fd > 2) close(fd--);
 		}
+		messageD(L_LOG, "console='%s'", s);
+	} else {
+		/* Make sure fd 0,1,2 are not closed */
+		bb_sanitize_stdio();
 	}
 
-	while ((fd = open(console, O_RDONLY | O_NONBLOCK)) < 0 && tried < 2) {
-		/* Can't open selected console -- try
-			logical system console and VT_MASTER */
-		safe_strncpy(console, (tried == 0 ? CONSOLE_DEV : CURRENT_VC),
-							sizeof(console));
-		tried++;
-	}
-	if (fd < 0) {
-		/* Perhaps we should panic here? */
-#if !ENABLE_SYSLOGD
-		log_console =
+	s = getenv("TERM");
+	if (ioctl(0, TIOCGSERIAL, &sr) == 0) {
+		/* Force the TERM setting to vt102 for serial console --
+		 * if TERM is set to linux (the default) */
+		if (!s || strcmp(s, "linux") == 0)
+			putenv((char*)"TERM=vt102");
+#if !ENABLE_FEATURE_INIT_SYSLOG
+		log_console = NULL;
 #endif
-		safe_strncpy(console, bb_dev_null, sizeof(console));
-	} else {
-		s = getenv("TERM");
-		/* check for serial console */
-		if (ioctl(fd, TIOCGSERIAL, &sr) == 0) {
-			/* Force the TERM setting to vt102 for serial console --
-			 * if TERM is set to linux (the default) */
-			if (s == NULL || strcmp(s, "linux") == 0)
-				putenv("TERM=vt102");
-#if !ENABLE_SYSLOGD
-			log_console = console;
-#endif
-		} else {
-			if (s == NULL)
-				putenv("TERM=linux");
-		}
-		close(fd);
-	}
-	messageD(LOG, "console=%s", console);
+	} else if (!s)
+		putenv((char*)"TERM=linux");
 }
 
-static void fixup_argv(int argc, char **argv, char *new_argv0)
+static void fixup_argv(char **argv)
 {
-	int len;
-
 	/* Fix up argv[0] to be certain we claim to be init */
-	len = strlen(argv[0]);
-	memset(argv[0], 0, len);
-	safe_strncpy(argv[0], new_argv0, len + 1);
+	strncpy(argv[0], "init", strlen(argv[0]));
 
 	/* Wipe argv[1]-argv[N] so they don't clutter the ps listing */
-	len = 1;
-	while (argc > len) {
-		memset(argv[len], 0, strlen(argv[len]));
-		len++;
-	}
+	while (*++argv)
+		memset(*argv, 0, strlen(*argv));
 }
 
 /* Open the new terminal device */
-static void open_new_terminal(const char * const device, const int fail) {
-	struct stat sb;
-
-	if ((device_open(device, O_RDWR)) < 0) {
-		if (stat(device, &sb) != 0) {
-			message(LOG | CONSOLE, "device '%s' does not exist.", device);
-		} else {
-			message(LOG | CONSOLE, "Bummer, can't open %s", device);
-		}
-		if (fail)
-			_exit(1);
-		/* else */
+static void open_stdio_to_tty(const char* tty_name, int fail)
+{
+	/* empty tty_name means "use init's tty", else... */
+	if (tty_name[0]) {
+		int fd = device_open(tty_name, O_RDWR);
+		if (fd < 0) {
+			message(L_LOG | L_CONSOLE, "Can't open %s: %s",
+				tty_name, strerror(errno));
+			if (fail)
+				_exit(1);
 #if !ENABLE_DEBUG_INIT
-		shutdown_signal(SIGUSR1);
+			shutdown_signal(SIGUSR1);
 #else
-		_exit(2);
+			_exit(2);
 #endif
+		} else {
+			dup2(fd, 0);
+			dup2(fd, 1);
+			dup2(fd, 2);
+			if (fd > 2) close(fd);
+		}
 	}
+	set_sane_term();
 }
 
 static pid_t run(const struct init_action *a)
 {
 	int i;
 	pid_t pid;
-	char *s, *tmpCmd, *cmd[INIT_BUFFS_SIZE], *cmdpath;
+	char *s, *tmpCmd, *cmdpath;
+	char *cmd[INIT_BUFFS_SIZE];
 	char buf[INIT_BUFFS_SIZE + 6];	/* INIT_BUFFS_SIZE+strlen("exec ")+1 */
 	sigset_t nmask, omask;
-	static const char press_enter[] =
-#ifdef CUSTOMIZED_BANNER
-#include CUSTOMIZED_BANNER
-#endif
-		"\nPlease press Enter to activate this console. ";
 
 	/* Block sigchild while forking.  */
 	sigemptyset(&nmask);
 	sigaddset(&nmask, SIGCHLD);
 	sigprocmask(SIG_BLOCK, &nmask, &omask);
+	pid = fork();
+	sigprocmask(SIG_SETMASK, &omask, NULL);
 
-	if ((pid = fork()) == 0) {
+	if (pid)
+		return pid;
 
-		/* Clean up */
-		close(0);
-		close(1);
-		close(2);
-		sigprocmask(SIG_SETMASK, &omask, NULL);
+	/* Reset signal handlers that were set by the parent process */
+	signal(SIGUSR1, SIG_DFL);
+	signal(SIGUSR2, SIG_DFL);
+	signal(SIGINT, SIG_DFL);
+	signal(SIGTERM, SIG_DFL);
+	signal(SIGHUP, SIG_DFL);
+	signal(SIGQUIT, SIG_DFL);
+	signal(SIGCONT, SIG_DFL);
+	signal(SIGSTOP, SIG_DFL);
+	signal(SIGTSTP, SIG_DFL);
 
-		/* Reset signal handlers that were set by the parent process */
-		signal(SIGUSR1, SIG_DFL);
-		signal(SIGUSR2, SIG_DFL);
-		signal(SIGINT, SIG_DFL);
-		signal(SIGTERM, SIG_DFL);
-		signal(SIGHUP, SIG_DFL);
-		signal(SIGQUIT, SIG_DFL);
-		signal(SIGCONT, SIG_DFL);
-		signal(SIGSTOP, SIG_DFL);
-		signal(SIGTSTP, SIG_DFL);
+	/* Create a new session and make ourself the process
+	 * group leader */
+	setsid();
 
-		/* Create a new session and make ourself the process
-		 * group leader */
-		setsid();
+	/* Open the new terminal device */
+	open_stdio_to_tty(a->terminal, 1);
 
-		/* Open the new terminal device */
-		open_new_terminal(a->terminal, 1);
+	/* If the init Action requires us to wait, then force the
+	 * supplied terminal to be the controlling tty. */
+	if (a->action & (SYSINIT | WAIT | CTRLALTDEL | SHUTDOWN | RESTART)) {
 
-		/* Make sure the terminal will act fairly normal for us */
-		set_term();
-		/* Setup stdout, stderr for the new process so
-		 * they point to the supplied terminal */
-		dup(0);
-		dup(0);
+		/* Now fork off another process to just hang around */
+		if ((pid = fork()) < 0) {
+			message(L_LOG | L_CONSOLE, "Can't fork");
+			_exit(1);
+		}
 
-		/* If the init Action requires us to wait, then force the
-		 * supplied terminal to be the controlling tty. */
-		if (a->action & (SYSINIT | WAIT | CTRLALTDEL | SHUTDOWN | RESTART)) {
+		if (pid > 0) {
 
-			/* Now fork off another process to just hang around */
+			/* We are the parent -- wait till the child is done */
+			signal(SIGINT, SIG_IGN);
+			signal(SIGTSTP, SIG_IGN);
+			signal(SIGQUIT, SIG_IGN);
+			signal(SIGCHLD, SIG_DFL);
+
+			waitfor(NULL, pid);
+			/* See if stealing the controlling tty back is necessary */
+			if (tcgetpgrp(0) != getpid())
+				_exit(0);
+
+			/* Use a temporary process to steal the controlling tty. */
 			if ((pid = fork()) < 0) {
-				message(LOG | CONSOLE, "Can't fork!");
+				message(L_LOG | L_CONSOLE, "Can't fork");
 				_exit(1);
 			}
-
-			if (pid > 0) {
-
-				/* We are the parent -- wait till the child is done */
-				signal(SIGINT, SIG_IGN);
-				signal(SIGTSTP, SIG_IGN);
-				signal(SIGQUIT, SIG_IGN);
-				signal(SIGCHLD, SIG_DFL);
-
-				waitfor(NULL, pid);
-				/* See if stealing the controlling tty back is necessary */
-				if (tcgetpgrp(0) != getpid())
-					_exit(0);
-
-				/* Use a temporary process to steal the controlling tty. */
-				if ((pid = fork()) < 0) {
-					message(LOG | CONSOLE, "Can't fork!");
-					_exit(1);
-				}
-				if (pid == 0) {
-					setsid();
-					ioctl(0, TIOCSCTTY, 1);
-					_exit(0);
-				}
-				waitfor(NULL, pid);
+			if (pid == 0) {
+				setsid();
+				ioctl(0, TIOCSCTTY, 1);
 				_exit(0);
 			}
-
-			/* Now fall though to actually execute things */
+			waitfor(NULL, pid);
+			_exit(0);
 		}
 
-		/* See if any special /bin/sh requiring characters are present */
-		if (strpbrk(a->command, "~`!$^&*()=|\\{}[];\"'<>?") != NULL) {
-			cmd[0] = (char *)DEFAULT_SHELL;
-			cmd[1] = "-c";
-			cmd[2] = strcat(strcpy(buf, "exec "), a->command);
-			cmd[3] = NULL;
+		/* Now fall though to actually execute things */
+	}
+
+	/* See if any special /bin/sh requiring characters are present */
+	if (strpbrk(a->command, "~`!$^&*()=|\\{}[];\"'<>?") != NULL) {
+		cmd[0] = (char*)DEFAULT_SHELL;
+		cmd[1] = (char*)"-c";
+		cmd[2] = strcat(strcpy(buf, "exec "), a->command);
+		cmd[3] = NULL;
+	} else {
+		/* Convert command (char*) into cmd (char**, one word per string) */
+		strcpy(buf, a->command);
+		s = buf;
+		for (tmpCmd = buf, i = 0; (tmpCmd = strsep(&s, " \t")) != NULL;) {
+			if (*tmpCmd != '\0') {
+				cmd[i] = tmpCmd;
+				i++;
+			}
+		}
+		cmd[i] = NULL;
+	}
+
+	cmdpath = cmd[0];
+
+	/*
+	 * Interactive shells want to see a dash in argv[0].  This
+	 * typically is handled by login, argv will be setup this
+	 * way if a dash appears at the front of the command path
+	 * (like "-/bin/sh").
+	 */
+	if (*cmdpath == '-') {
+		/* skip over the dash */
+		++cmdpath;
+
+		/* find the last component in the command pathname */
+		s = bb_get_last_path_component(cmdpath);
+
+		/* make a new argv[0] */
+		if ((cmd[0] = malloc(strlen(s) + 2)) == NULL) {
+			message(L_LOG | L_CONSOLE, bb_msg_memory_exhausted);
+			cmd[0] = cmdpath;
 		} else {
-			/* Convert command (char*) into cmd (char**, one word per string) */
-			strcpy(buf, a->command);
-			s = buf;
-			for (tmpCmd = buf, i = 0; (tmpCmd = strsep(&s, " \t")) != NULL;) {
-				if (*tmpCmd != '\0') {
-					cmd[i] = tmpCmd;
-					i++;
-				}
-			}
-			cmd[i] = NULL;
+			cmd[0][0] = '-';
+			strcpy(cmd[0] + 1, s);
 		}
-
-		cmdpath = cmd[0];
-
-		/*
-		   Interactive shells want to see a dash in argv[0].  This
-		   typically is handled by login, argv will be setup this
-		   way if a dash appears at the front of the command path
-		   (like "-/bin/sh").
-		 */
-
-		if (*cmdpath == '-') {
-
-			/* skip over the dash */
-			++cmdpath;
-
-			/* find the last component in the command pathname */
-			s = bb_get_last_path_component(cmdpath);
-
-			/* make a new argv[0] */
-			if ((cmd[0] = malloc(strlen(s) + 2)) == NULL) {
-				message(LOG | CONSOLE, bb_msg_memory_exhausted);
-				cmd[0] = cmdpath;
-			} else {
-				cmd[0][0] = '-';
-				strcpy(cmd[0] + 1, s);
-			}
 #if ENABLE_FEATURE_INIT_SCTTY
-			/* Establish this process as session leader and
-			 * (attempt) to make the tty (if any) a controlling tty.
-			 */
-			(void) setsid();
-			(void) ioctl(0, TIOCSCTTY, 0/*don't steal it*/);
+		/* Establish this process as session leader and
+		 * (attempt) to make the tty (if any) a controlling tty.
+		 */
+		setsid();
+		ioctl(0, TIOCSCTTY, 0 /*don't steal it*/);
 #endif
-		}
+	}
 
 #if !defined(__UCLIBC__) || defined(__ARCH_HAS_MMU__)
-		if (a->action & ASKFIRST) {
-			char c;
-			/*
-			 * Save memory by not exec-ing anything large (like a shell)
-			 * before the user wants it. This is critical if swap is not
-			 * enabled and the system has low memory. Generally this will
-			 * be run on the second virtual console, and the first will
-			 * be allowed to start a shell or whatever an init script
-			 * specifies.
-			 */
-			messageD(LOG, "Waiting for enter to start '%s'"
-						"(pid %d, terminal %s)\n",
-					  cmdpath, getpid(), a->terminal);
-			full_write(1, press_enter, sizeof(press_enter) - 1);
-			while (read(0, &c, 1) == 1 && c != '\n')
-				;
-		}
+	if (a->action & ASKFIRST) {
+		static const char press_enter[] ALIGN1 =
+#ifdef CUSTOMIZED_BANNER
+#include CUSTOMIZED_BANNER
 #endif
-
-		/* Log the process name and args */
-		message(LOG, "Starting pid %d, console %s: '%s'",
-				  getpid(), a->terminal, cmdpath);
+			"\nPlease press Enter to activate this console. ";
+		char c;
+		/*
+		 * Save memory by not exec-ing anything large (like a shell)
+		 * before the user wants it. This is critical if swap is not
+		 * enabled and the system has low memory. Generally this will
+		 * be run on the second virtual console, and the first will
+		 * be allowed to start a shell or whatever an init script
+		 * specifies.
+		 */
+		messageD(L_LOG, "waiting for enter to start '%s'"
+					"(pid %d, tty '%s')\n",
+				  cmdpath, getpid(), a->terminal);
+		full_write(1, press_enter, sizeof(press_enter) - 1);
+		while (read(0, &c, 1) == 1 && c != '\n')
+			;
+	}
+#endif
+	/* Log the process name and args */
+	message(L_LOG, "starting pid %d, tty '%s': '%s'",
+			  getpid(), a->terminal, cmdpath);
 
 #if ENABLE_FEATURE_INIT_COREDUMPS
-		{
-			struct stat sb;
-			if (stat(CORE_ENABLE_FLAG_FILE, &sb) == 0) {
-				struct rlimit limit;
+	{
+		struct stat sb;
+		if (stat(CORE_ENABLE_FLAG_FILE, &sb) == 0) {
+			struct rlimit limit;
 
-				limit.rlim_cur = RLIM_INFINITY;
-				limit.rlim_max = RLIM_INFINITY;
-				setrlimit(RLIMIT_CORE, &limit);
-			}
+			limit.rlim_cur = RLIM_INFINITY;
+			limit.rlim_max = RLIM_INFINITY;
+			setrlimit(RLIMIT_CORE, &limit);
 		}
-#endif
-
-		/* Now run it.  The new program will take over this PID,
-		 * so nothing further in init.c should be run. */
-		execv(cmdpath, cmd);
-
-		/* We're still here?  Some error happened. */
-		message(LOG | CONSOLE, "Bummer, cannot run '%s': %m", cmdpath);
-		_exit(-1);
 	}
-	sigprocmask(SIG_SETMASK, &omask, NULL);
-	return pid;
+#endif
+	/* Now run it.  The new program will take over this PID,
+	 * so nothing further in init.c should be run. */
+	BB_EXECVP(cmdpath, cmd);
+
+	/* We're still here?  Some error happened. */
+	message(L_LOG | L_CONSOLE, "Cannot run '%s': %s",
+			cmdpath, strerror(errno));
+	_exit(-1);
 }
 
 static int waitfor(const struct init_action *a, pid_t pid)
@@ -577,7 +512,7 @@ static int waitfor(const struct init_action *a, pid_t pid)
 
 	runpid = (NULL == a)? pid : run(a);
 	while (1) {
-		wpid = waitpid(runpid,&status,0);
+		wpid = waitpid(runpid, &status, 0);
 		if (wpid == runpid)
 			break;
 		if (wpid == -1 && errno == ECHILD) {
@@ -598,7 +533,8 @@ static void run_actions(int action)
 	for (a = init_action_list; a; a = tmp) {
 		tmp = a->next;
 		if (a->action == action) {
-			if (access(a->terminal, R_OK | W_OK)) {
+			/* a->terminal of "" means "init's console" */
+			if (a->terminal[0] && access(a->terminal, R_OK | W_OK)) {
 				delete_init_action(a);
 			} else if (a->action & (SYSINIT | WAIT | CTRLALTDEL | SHUTDOWN | RESTART)) {
 				waitfor(a, 0);
@@ -655,23 +591,21 @@ static void shutdown_system(void)
 	sigaddset(&block_signals, SIGTSTP);
 	sigprocmask(SIG_BLOCK, &block_signals, NULL);
 
+	message(L_CONSOLE | L_LOG, "The system is going down NOW!");
+
 	/* Allow Ctrl-Alt-Del to reboot system. */
 	init_reboot(RB_ENABLE_CAD);
 
-	message(CONSOLE | LOG, "The system is going down NOW !!");
-	sync();
-
 	/* Send signals to every process _except_ pid 1 */
-	message(CONSOLE | LOG, init_sending_format, "TERM");
+	message(L_CONSOLE | L_LOG, "Sending SIG%s to all processes", "TERM");
 	kill(-1, SIGTERM);
-	sleep(1);
 	sync();
+	sleep(1);
 
-	message(CONSOLE | LOG, init_sending_format, "KILL");
+	message(L_CONSOLE | L_LOG, "Sending SIG%s to all processes", "KILL");
 	kill(-1, SIGKILL);
-	sleep(1);
-
 	sync();
+	sleep(1);
 }
 
 static void exec_signal(int sig ATTRIBUTE_UNUSED)
@@ -698,26 +632,14 @@ static void exec_signal(int sig ATTRIBUTE_UNUSED)
 			sigaddset(&unblock_signals, SIGTSTP);
 			sigprocmask(SIG_UNBLOCK, &unblock_signals, NULL);
 
-			/* Close whatever files are open. */
-			close(0);
-			close(1);
-			close(2);
-
 			/* Open the new terminal device */
-			open_new_terminal(a->terminal, 0);
+			open_stdio_to_tty(a->terminal, 0);
 
-			/* Make sure the terminal will act fairly normal for us */
-			set_term();
-			/* Setup stdout, stderr on the supplied terminal */
-			dup(0);
-			dup(0);
+			messageD(L_CONSOLE | L_LOG, "Trying to re-exec %s", a->command);
+			BB_EXECLP(a->command, a->command, NULL);
 
-			messageD(CONSOLE | LOG, "Trying to re-exec %s", a->command);
-			execl(a->command, a->command, NULL);
-
-			message(CONSOLE | LOG, "exec of '%s' failed: %m",
-					a->command);
-			sync();
+			message(L_CONSOLE | L_LOG, "Cannot run '%s': %s",
+					a->command, strerror(errno));
 			sleep(2);
 			init_reboot(RB_HALT_SYSTEM);
 			loop_forever();
@@ -727,27 +649,23 @@ static void exec_signal(int sig ATTRIBUTE_UNUSED)
 
 static void shutdown_signal(int sig)
 {
-	char *m;
+	const char *m;
 	int rb;
 
 	shutdown_system();
 
+	m = "halt";
+	rb = RB_HALT_SYSTEM;
 	if (sig == SIGTERM) {
 		m = "reboot";
 		rb = RB_AUTOBOOT;
 	} else if (sig == SIGUSR2) {
 		m = "poweroff";
 		rb = RB_POWER_OFF;
-	} else {
-		m = "halt";
-		rb = RB_HALT_SYSTEM;
 	}
-	message(CONSOLE | LOG, "Requesting system %s.", m);
-	sync();
-
+	message(L_CONSOLE | L_LOG, "Requesting system %s", m);
 	/* allow time for last message to reach serial console */
 	sleep(2);
-
 	init_reboot(rb);
 	loop_forever();
 }
@@ -775,32 +693,29 @@ static void cont_handler(int sig ATTRIBUTE_UNUSED)
 	got_cont = 1;
 }
 
-#endif							/* ! ENABLE_DEBUG_INIT */
+#endif	/* !ENABLE_DEBUG_INIT */
 
 static void new_init_action(int action, const char *command, const char *cons)
 {
 	struct init_action *new_action, *a, *last;
 
-	if (*cons == '\0')
-		cons = console;
-
 	if (strcmp(cons, bb_dev_null) == 0 && (action & ASKFIRST))
 		return;
 
-	new_action = xzalloc(sizeof(struct init_action));
-
 	/* Append to the end of the list */
 	for (a = last = init_action_list; a; a = a->next) {
 		/* don't enter action if it's already in the list,
 		 * but do overwrite existing actions */
-		if ((strcmp(a->command, command) == 0) &&
-		    (strcmp(a->terminal, cons) ==0)) {
+		if ((strcmp(a->command, command) == 0)
+		 && (strcmp(a->terminal, cons) == 0)
+		) {
 			a->action = action;
-			free(new_action);
 			return;
 		}
 		last = a;
 	}
+
+	new_action = xzalloc(sizeof(struct init_action));
 	if (last) {
 		last->next = new_action;
 	} else {
@@ -809,7 +724,7 @@ static void new_init_action(int action, const char *command, const char *cons)
 	strcpy(new_action->command, command);
 	new_action->action = action;
 	strcpy(new_action->terminal, cons);
-	messageD(LOG|CONSOLE, "command='%s' action='%d' terminal='%s'\n",
+	messageD(L_LOG | L_CONSOLE, "command='%s' action=%d tty='%s'\n",
 		new_action->command, new_action->action, new_action->terminal);
 }
 
@@ -842,23 +757,22 @@ static void parse_inittab(void)
 #if ENABLE_FEATURE_USE_INITTAB
 	FILE *file;
 	char buf[INIT_BUFFS_SIZE], lineAsRead[INIT_BUFFS_SIZE];
-	char tmpConsole[CONSOLE_BUFF_SIZE];
+	char tmpConsole[CONSOLE_NAME_SIZE];
 	char *id, *runlev, *action, *command, *eol;
 	const struct init_action_type *a = actions;
 
-
 	file = fopen(INITTAB, "r");
 	if (file == NULL) {
 		/* No inittab file -- set up some default behavior */
 #endif
 		/* Reboot on Ctrl-Alt-Del */
-		new_init_action(CTRLALTDEL, "/sbin/reboot", "");
+		new_init_action(CTRLALTDEL, "reboot", "");
 		/* Umount all filesystems on halt/reboot */
-		new_init_action(SHUTDOWN, "/bin/umount -a -r", "");
+		new_init_action(SHUTDOWN, "umount -a -r", "");
 		/* Swapoff on halt/reboot */
-		if(ENABLE_SWAPONOFF) new_init_action(SHUTDOWN, "/sbin/swapoff -a", "");
+		if (ENABLE_SWAPONOFF) new_init_action(SHUTDOWN, "swapoff -a", "");
 		/* Prepare to restart init when a HUP is received */
-		new_init_action(RESTART, "/sbin/init", "");
+		new_init_action(RESTART, "init", "");
 		/* Askfirst shell on tty1-4 */
 		new_init_action(ASKFIRST, bb_default_login_shell, "");
 		new_init_action(ASKFIRST, bb_default_login_shell, VC_2);
@@ -880,6 +794,7 @@ static void parse_inittab(void)
 			continue;
 
 		/* Trim the trailing \n */
+		//XXX: chomp() ?
 		eol = strrchr(id, '\n');
 		if (eol != NULL)
 			*eol = '\0';
@@ -890,7 +805,7 @@ static void parse_inittab(void)
 		/* Separate the ID field from the runlevels */
 		runlev = strchr(id, ':');
 		if (runlev == NULL || *(runlev + 1) == '\0') {
-			message(LOG | CONSOLE, "Bad inittab entry: %s", lineAsRead);
+			message(L_LOG | L_CONSOLE, "Bad inittab entry: %s", lineAsRead);
 			continue;
 		} else {
 			*runlev = '\0';
@@ -900,7 +815,7 @@ static void parse_inittab(void)
 		/* Separate the runlevels from the action */
 		action = strchr(runlev, ':');
 		if (action == NULL || *(action + 1) == '\0') {
-			message(LOG | CONSOLE, "Bad inittab entry: %s", lineAsRead);
+			message(L_LOG | L_CONSOLE, "Bad inittab entry: %s", lineAsRead);
 			continue;
 		} else {
 			*action = '\0';
@@ -910,7 +825,7 @@ static void parse_inittab(void)
 		/* Separate the action from the command */
 		command = strchr(action, ':');
 		if (command == NULL || *(command + 1) == '\0') {
-			message(LOG | CONSOLE, "Bad inittab entry: %s", lineAsRead);
+			message(L_LOG | L_CONSOLE, "Bad inittab entry: %s", lineAsRead);
 			continue;
 		} else {
 			*command = '\0';
@@ -921,11 +836,11 @@ static void parse_inittab(void)
 		for (a = actions; a->name != 0; a++) {
 			if (strcmp(a->name, action) == 0) {
 				if (*id != '\0') {
-					if(strncmp(id, "/dev/", 5) == 0)
+					if (strncmp(id, "/dev/", 5) == 0)
 						id += 5;
 					strcpy(tmpConsole, "/dev/");
 					safe_strncpy(tmpConsole + 5, id,
-						CONSOLE_BUFF_SIZE - 5);
+						sizeof(tmpConsole) - 5);
 					id = tmpConsole;
 				}
 				new_init_action(a->action, command, id);
@@ -934,11 +849,10 @@ static void parse_inittab(void)
 		}
 		if (a->name == 0) {
 			/* Choke on an unknown action */
-			message(LOG | CONSOLE, "Bad inittab entry: %s", lineAsRead);
+			message(L_LOG | L_CONSOLE, "Bad inittab entry: %s", lineAsRead);
 		}
 	}
 	fclose(file);
-	return;
 #endif /* FEATURE_USE_INITTAB */
 }
 
@@ -947,7 +861,7 @@ static void reload_signal(int sig ATTRIBUTE_UNUSED)
 {
 	struct init_action *a, *tmp;
 
-	message(LOG, "Reloading /etc/inittab");
+	message(L_LOG, "reloading /etc/inittab");
 
 	/* disable old entrys */
 	for (a = init_action_list; a; a = a->next ) {
@@ -959,16 +873,15 @@ static void reload_signal(int sig ATTRIBUTE_UNUSED)
 	/* remove unused entrys */
 	for (a = init_action_list; a; a = tmp) {
 		tmp = a->next;
-		if (a->action & (ONCE | SYSINIT | WAIT ) &&
-				a->pid == 0 ) {
+		if ((a->action & (ONCE | SYSINIT | WAIT)) && a->pid == 0) {
 			delete_init_action(a);
 		}
 	}
 	run_actions(RESPAWN);
-	return;
 }
 #endif  /* FEATURE_USE_INITTAB */
 
+int init_main(int argc, char **argv);
 int init_main(int argc, char **argv)
 {
 	struct init_action *a;
@@ -977,13 +890,13 @@ int init_main(int argc, char **argv)
 	die_sleep = 30 * 24*60*60; /* if xmalloc will ever die... */
 
 	if (argc > 1 && !strcmp(argv[1], "-q")) {
-		return kill(1,SIGHUP);
+		return kill(1, SIGHUP);
 	}
 #if !ENABLE_DEBUG_INIT
 	/* Expect to be invoked as init with PID=1 or be invoked as linuxrc */
-	if (getpid() != 1 &&
-		(!ENABLE_FEATURE_INITRD || !strstr(applet_name, "linuxrc")))
-	{
+	if (getpid() != 1
+	 && (!ENABLE_FEATURE_INITRD || !strstr(applet_name, "linuxrc"))
+	) {
 		bb_show_usage();
 	}
 	/* Set up sig handlers  -- be sure to
@@ -1003,23 +916,14 @@ int init_main(int argc, char **argv)
 	init_reboot(RB_DISABLE_CAD);
 #endif
 
+
 	/* Figure out where the default console should be */
 	console_init();
-
-	/* Close whatever files are open, and reset the console. */
-	close(0);
-	close(1);
-	close(2);
-
-	if (device_open(console, O_RDWR | O_NOCTTY) == 0) {
-		set_term();
-		close(0);
-	}
-
+	set_sane_term();
 	chdir("/");
 	setsid();
 	{
-		const char * const *e;
+		const char *const *e;
 		/* Make sure environs is set to something sane */
 		for (e = environment; *e; e++)
 			putenv((char *) *e);
@@ -1028,7 +932,7 @@ int init_main(int argc, char **argv)
 	if (argc > 1) setenv("RUNLEVEL", argv[1], 1);
 
 	/* Hello world */
-	message(MAYBE_CONSOLE | LOG, "init started:  %s", bb_msg_full_version);
+	message(MAYBE_CONSOLE | L_LOG, "init started: %s", bb_banner);
 
 	/* Make sure there is enough memory to do something useful. */
 	if (ENABLE_SWAPONOFF) {
@@ -1037,11 +941,11 @@ int init_main(int argc, char **argv)
 		if (!sysinfo(&info) &&
 			(info.mem_unit ? : 1) * (long long)info.totalram < 1024*1024)
 		{
-			message(CONSOLE,"Low memory: forcing swapon.");
+			message(L_CONSOLE, "Low memory, forcing swapon");
 			/* swapon -a requires /proc typically */
-			new_init_action(SYSINIT, "/bin/mount -t proc proc /proc", "");
+			new_init_action(SYSINIT, "mount -t proc proc /proc", "");
 			/* Try to turn on swap */
-			new_init_action(SYSINIT, "/sbin/swapon -a", "");
+			new_init_action(SYSINIT, "swapon -a", "");
 			run_actions(SYSINIT);   /* wait and removing */
 		}
 	}
@@ -1066,20 +970,21 @@ int init_main(int argc, char **argv)
 	if (getenv("SELINUX_INIT") == NULL) {
 		int enforce = 0;
 
-		putenv("SELINUX_INIT=YES");
+		putenv((char*)"SELINUX_INIT=YES");
 		if (selinux_init_load_policy(&enforce) == 0) {
-			execv(argv[0], argv);
+			BB_EXECVP(argv[0], argv);
 		} else if (enforce > 0) {
 			/* SELinux in enforcing mode but load_policy failed */
 			/* At this point, we probably can't open /dev/console, so log() won't work */
-			message(CONSOLE,"Unable to load SELinux Policy. Machine is in enforcing mode. Halting now.");
+			message(L_CONSOLE, "Cannot load SELinux Policy. "
+				"Machine is in enforcing mode. Halting now.");
 			exit(1);
 		}
 	}
 #endif /* CONFIG_SELINUX */
 
 	/* Make the command line just say "init"  -- thats all, nothing else */
-	fixup_argv(argc, argv, "init");
+	fixup_argv(argv);
 
 	/* Now run everything that needs to be run */
 
@@ -1099,7 +1004,6 @@ int init_main(int argc, char **argv)
 	signal(SIGHUP, SIG_IGN);
 #endif /* FEATURE_USE_INITTAB */
 
-
 	/* Now run the looping stuff for the rest of forever */
 	while (1) {
 		/* run the respawn stuff */
@@ -1120,7 +1024,7 @@ int init_main(int argc, char **argv)
 					/* Set the pid to 0 so that the process gets
 					 * restarted by run_actions() */
 					a->pid = 0;
-					message(LOG, "Process '%s' (pid %d) exited.  "
+					message(L_LOG, "process '%s' (pid %d) exited. "
 							"Scheduling it for restart.",
 							a->command, wpid);
 				}
