@@ -17,7 +17,7 @@
 
 char *concat_subpath_file(const char *path, const char *f)
 {
-	if (f && *f == '.' && (!f[1] || (f[1] == '.' && !f[2])))
+	if (f && DOT_OR_DOTDOT(f))
 		return NULL;
 	return concat_path_file(path, f);
 }
