@@ -31,34 +31,42 @@
 #include "libbb.h"
 
 /* Ask the user for a password.
-   Return 1 if the user gives the correct password for entry PW,
-   0 if not.  Return 1 without asking for a password if run by UID 0
-   or if PW has an empty password.  */
+ * Return 1 if the user gives the correct password for entry PW,
+ * 0 if not.  Return 1 without asking if PW has an empty password.
+ *
+ * NULL pw means "just fake it for login with bad username" */
 
 int correct_password(const struct passwd *pw)
 {
-	char *unencrypted, *encrypted, *correct;
+	char *unencrypted, *encrypted;
+	const char *correct;
 
-#ifdef CONFIG_FEATURE_SHADOWPASSWDS
-	if (LONE_CHAR(pw->pw_passwd, 'x') || LONE_CHAR(pw->pw_passwd, '*')) {
-		struct spwd *sp = getspnam(pw->pw_name);
-
-		if (!sp)
-			bb_error_msg_and_die("no valid shadow password");
-
-		correct = sp->sp_pwdp;
-	} else
+	/* fake salt. crypt() can choke otherwise. */
+	correct = "aa";
+	if (!pw) {
+		/* "aa" will never match */
+		goto fake_it;
+	}
+	correct = pw->pw_passwd;
+#if ENABLE_FEATURE_SHADOWPASSWDS
+	if ((correct[0] == 'x' || correct[0] == '*') && !correct[1]) {
+		/* Using _r function to avoid pulling in static buffers */
+		struct spwd spw;
+		struct spwd *result;
+		char buffer[256];
+		correct = (getspnam_r(pw->pw_name, &spw, buffer, sizeof(buffer), &result)) ? "aa" : spw.sp_pwdp;
+	}
 #endif
-		correct = pw->pw_passwd;
 
-	if (!correct || correct[0] == '\0')
+	if (!correct[0]) /* empty password field? */
 		return 1;
 
+ fake_it:
 	unencrypted = bb_askpass(0, "Password: ");
 	if (!unencrypted) {
 		return 0;
 	}
 	encrypted = crypt(unencrypted, correct);
 	memset(unencrypted, 0, strlen(unencrypted));
-	return (!strcmp(encrypted, correct)) ? 1 : 0;
+	return strcmp(encrypted, correct) == 0;
 }
