@@ -26,12 +26,12 @@ static int exitval;			/* final exit value */
 int bb_dump_blocksize;			/* data block size */
 int bb_dump_length = -1;		/* max bytes to read */
 
-static const char index_str[] = ".#-+ 0123456789";
+static const char index_str[] ALIGN1 = ".#-+ 0123456789";
 
-static const char size_conv_str[] =
+static const char size_conv_str[] ALIGN1 =
 "\x1\x4\x4\x4\x4\x4\x4\x8\x8\x8\x8\010cdiouxXeEfgG";
 
-static const char lcc[] = "diouxX";
+static const char lcc[] ALIGN1 = "diouxX";
 
 int bb_dump_size(FS * fs)
 {
@@ -291,7 +291,7 @@ static void rewrite(FS * fs)
 	}
 }
 
-static void do_skip(char *fname, int statok)
+static void do_skip(const char *fname, int statok)
 {
 	struct stat sbuf;
 
@@ -317,7 +317,8 @@ static void do_skip(char *fname, int statok)
 
 static int next(char **argv)
 {
-	static int done;
+	static smallint done;
+
 	int statok;
 
 	if (argv) {
@@ -332,10 +333,11 @@ static int next(char **argv)
 				++_argv;
 				continue;
 			}
-			statok = done = 1;
+			done = statok = 1;
 		} else {
-			if (done++)
+			if (done)
 				return 0;
+			done = 1;
 			statok = 0;
 		}
 		if (bb_dump_skip)
@@ -350,8 +352,9 @@ static int next(char **argv)
 
 static unsigned char *get(void)
 {
-	static int ateof = 1;
-	static unsigned char *curp=NULL, *savp; /*DBU:[dave@cray.com]initialize curp */
+	static smallint ateof = 1;
+	static unsigned char *curp = NULL, *savp; /*DBU:[dave@cray.com]initialize curp */
+
 	int n;
 	int need, nread;
 	unsigned char *tmpp;
@@ -399,7 +402,8 @@ static unsigned char *get(void)
 		if (bb_dump_length != -1) {
 			bb_dump_length -= n;
 		}
-		if (!(need -= n)) {
+		need -= n;
+		if (!need) {
 			if (bb_dump_vflag == ALL || bb_dump_vflag == FIRST
 				|| memcmp(curp, savp, bb_dump_blocksize)) {
 				if (bb_dump_vflag == DUP || bb_dump_vflag == FIRST) {
@@ -436,7 +440,7 @@ static void bpad(PR * pr)
 	while ((*p2++ = *p1++) != 0);
 }
 
-static const char conv_str[] =
+static const char conv_str[] ALIGN1 =
 	"\0\\0\0"
 	"\007\\a\0"				/* \a */
 	"\b\\b\0"
@@ -445,7 +449,7 @@ static const char conv_str[] =
 	"\r\\r\0"
 	"\t\\t\0"
 	"\v\\v\0"
-	"\0";
+	;
 
 
 static void conv_c(PR * pr, unsigned char * p)
@@ -475,7 +479,7 @@ static void conv_c(PR * pr, unsigned char * p)
 
 static void conv_u(PR * pr, unsigned char * p)
 {
-	static const char list[] =
+	static const char list[] ALIGN1 =
 		"nul\0soh\0stx\0etx\0eot\0enq\0ack\0bel\0"
 		"bs\0_ht\0_lf\0_vt\0_ff\0_cr\0_so\0_si\0_"
 		"dle\0dcl\0dc2\0dc3\0dc4\0nak\0syn\0etb\0"
