@@ -7,9 +7,6 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include <stdarg.h>
-#include <stdlib.h>
-
 #include "libbb.h"
 
 void bb_herror_msg(const char *s, ...)
@@ -17,6 +14,6 @@ void bb_herror_msg(const char *s, ...)
 	va_list p;
 
 	va_start(p, s);
-	bb_vherror_msg(s, p);
+	bb_verror_msg(s, p, hstrerror(h_errno));
 	va_end(p);
 }
