@@ -5,7 +5,6 @@
  *
  * Heavily modified by Manuel Novoa III       Mar 12, 2001
  *
- * Version:     $Id: inet_common.c,v 1.8 2004/03/10 07:42:38 mjn3 Exp $
  *
  */
 
@@ -15,7 +14,9 @@
 int INET_resolve(const char *name, struct sockaddr_in *s_in, int hostfirst)
 {
 	struct hostent *hp;
+#if ENABLE_FEATURE_ETC_NETWORKS
 	struct netent *np;
+#endif
 
 	/* Grmpf. -FvK */
 	s_in->sin_family = AF_INET;
@@ -33,22 +34,28 @@ int INET_resolve(const char *name, struct sockaddr_in *s_in, int hostfirst)
 	/* If we expect this to be a hostname, try hostname database first */
 #ifdef DEBUG
 	if (hostfirst) {
-		bb_error_msg("gethostbyname (%s)", name);
+		bb_error_msg("gethostbyname(%s)", name);
 	}
 #endif
-	if (hostfirst && (hp = gethostbyname(name)) != (struct hostent *) NULL) {
-		memcpy((char *) &s_in->sin_addr, (char *) hp->h_addr_list[0],
-			   sizeof(struct in_addr));
-		return 0;
+	if (hostfirst) {
+		hp = gethostbyname(name);
+		if (hp != NULL) {
+			memcpy(&s_in->sin_addr, hp->h_addr_list[0],
+				sizeof(struct in_addr));
+			return 0;
+		}
 	}
+#if ENABLE_FEATURE_ETC_NETWORKS
 	/* Try the NETWORKS database to see if this is a known network. */
 #ifdef DEBUG
-	bb_error_msg("getnetbyname (%s)", name);
+	bb_error_msg("getnetbyname(%s)", name);
 #endif
-	if ((np = getnetbyname(name)) != (struct netent *) NULL) {
+	np = getnetbyname(name);
+	if (np != NULL) {
 		s_in->sin_addr.s_addr = htonl(np->n_net);
 		return 1;
 	}
+#endif
 	if (hostfirst) {
 		/* Don't try again */
 		return -1;
@@ -59,116 +66,105 @@ int INET_resolve(const char *name, struct sockaddr_in *s_in, int hostfirst)
 #endif
 
 #ifdef DEBUG
-	bb_error_msg("gethostbyname (%s)", name);
+	bb_error_msg("gethostbyname(%s)", name);
 #endif
-	if ((hp = gethostbyname(name)) == (struct hostent *) NULL) {
+	hp = gethostbyname(name);
+	if (hp == NULL) {
 		return -1;
 	}
-	memcpy((char *) &s_in->sin_addr, (char *) hp->h_addr_list[0],
-		   sizeof(struct in_addr));
-
+	memcpy(&s_in->sin_addr, hp->h_addr_list[0], sizeof(struct in_addr));
 	return 0;
 }
 
-/* cache */
-struct addr {
-	struct sockaddr_in addr;
-	char *name;
-	int host;
-	struct addr *next;
-};
-
-static struct addr *INET_nn = NULL;	/* addr-to-name cache           */
 
 /* numeric: & 0x8000: default instead of *,
  *          & 0x4000: host instead of net,
  *          & 0x0fff: don't resolve
  */
-int INET_rresolve(char *name, size_t len, struct sockaddr_in *s_in,
-				  int numeric, unsigned int netmask)
+char *INET_rresolve(struct sockaddr_in *s_in, int numeric, uint32_t netmask)
 {
-	struct hostent *ent;
-	struct netent *np;
+	/* addr-to-name cache */
+	struct addr {
+		struct addr *next;
+		struct sockaddr_in addr;
+		int host;
+		char name[1];
+	};
+	static struct addr *cache = NULL;
+
 	struct addr *pn;
-	unsigned long ad, host_ad;
+	char *name;
+	uint32_t ad, host_ad;
 	int host = 0;
 
 	/* Grmpf. -FvK */
 	if (s_in->sin_family != AF_INET) {
 #ifdef DEBUG
-		bb_error_msg("rresolve: unsupport address family %d !",
+		bb_error_msg("rresolve: unsupported address family %d!",
 				  s_in->sin_family);
 #endif
 		errno = EAFNOSUPPORT;
-		return -1;
+		return NULL;
 	}
-	ad = (unsigned long) s_in->sin_addr.s_addr;
+	ad = s_in->sin_addr.s_addr;
 #ifdef DEBUG
-	bb_error_msg("rresolve: %08lx, mask %08x, num %08x", ad, netmask, numeric);
+	bb_error_msg("rresolve: %08x, mask %08x, num %08x", (unsigned)ad, netmask, numeric);
 #endif
 	if (ad == INADDR_ANY) {
 		if ((numeric & 0x0FFF) == 0) {
 			if (numeric & 0x8000)
-				safe_strncpy(name, bb_str_default, len);
-			else
-				safe_strncpy(name, "*", len);
-			return 0;
+				return xstrdup(bb_str_default);
+			return xstrdup("*");
 		}
 	}
-	if (numeric & 0x0FFF) {
-		safe_strncpy(name, inet_ntoa(s_in->sin_addr), len);
-		return 0;
-	}
+	if (numeric & 0x0FFF)
+		return xstrdup(inet_ntoa(s_in->sin_addr));
 
 	if ((ad & (~netmask)) != 0 || (numeric & 0x4000))
 		host = 1;
-	pn = INET_nn;
-	while (pn != NULL) {
+	pn = cache;
+	while (pn) {
 		if (pn->addr.sin_addr.s_addr == ad && pn->host == host) {
-			safe_strncpy(name, pn->name, len);
 #ifdef DEBUG
-			bb_error_msg("rresolve: found %s %08lx in cache",
-					  (host ? "host" : "net"), ad);
+			bb_error_msg("rresolve: found %s %08x in cache",
+					  (host ? "host" : "net"), (unsigned)ad);
 #endif
-			return 0;
+			return xstrdup(pn->name);
 		}
 		pn = pn->next;
 	}
 
 	host_ad = ntohl(ad);
-	np = NULL;
-	ent = NULL;
+	name = NULL;
 	if (host) {
+		struct hostent *ent;
 #ifdef DEBUG
-		bb_error_msg("gethostbyaddr (%08lx)", ad);
+		bb_error_msg("gethostbyaddr (%08x)", (unsigned)ad);
 #endif
 		ent = gethostbyaddr((char *) &ad, 4, AF_INET);
-		if (ent != NULL) {
-			safe_strncpy(name, ent->h_name, len);
-		}
-	} else {
+		if (ent)
+			name = xstrdup(ent->h_name);
+	} else if (ENABLE_FEATURE_ETC_NETWORKS) {
+		struct netent *np;
 #ifdef DEBUG
-		bb_error_msg("getnetbyaddr (%08lx)", host_ad);
+		bb_error_msg("getnetbyaddr (%08x)", (unsigned)host_ad);
 #endif
 		np = getnetbyaddr(host_ad, AF_INET);
-		if (np != NULL) {
-			safe_strncpy(name, np->n_name, len);
-		}
+		if (np)
+			name = xstrdup(np->n_name);
 	}
-	if ((ent == NULL) && (np == NULL)) {
-		safe_strncpy(name, inet_ntoa(s_in->sin_addr), len);
-	}
-	pn = xmalloc(sizeof(struct addr));
+	if (!name)
+		name = xstrdup(inet_ntoa(s_in->sin_addr));
+	pn = xmalloc(sizeof(*pn) + strlen(name)); /* no '+ 1', it's already accounted for */
+	pn->next = cache;
 	pn->addr = *s_in;
-	pn->next = INET_nn;
 	pn->host = host;
-	pn->name = xstrdup(name);
-	INET_nn = pn;
-
-	return 0;
+	strcpy(pn->name, name);
+	cache = pn;
+	return name;
 }
 
-#ifdef CONFIG_FEATURE_IPV6
+#if ENABLE_FEATURE_IPV6
 
 int INET6_resolve(const char *name, struct sockaddr_in6 *sin6)
 {
@@ -183,9 +179,7 @@ int INET6_resolve(const char *name, struct sockaddr_in6 *sin6)
 		return -1;
 	}
 	memcpy(sin6, ai->ai_addr, sizeof(struct sockaddr_in6));
-
 	freeaddrinfo(ai);
-
 	return 0;
 }
 
@@ -196,9 +190,9 @@ int INET6_resolve(const char *name, struct sockaddr_in6 *sin6)
 #endif
 
 
-int INET6_rresolve(char *name, size_t len, struct sockaddr_in6 *sin6,
-				   int numeric)
+char *INET6_rresolve(struct sockaddr_in6 *sin6, int numeric)
 {
+	char name[128];
 	int s;
 
 	/* Grmpf. -FvK */
@@ -208,28 +202,25 @@ int INET6_rresolve(char *name, size_t len, struct sockaddr_in6 *sin6,
 				  sin6->sin6_family);
 #endif
 		errno = EAFNOSUPPORT;
-		return -1;
+		return NULL;
 	}
 	if (numeric & 0x7FFF) {
-		inet_ntop(AF_INET6, &sin6->sin6_addr, name, len);
-		return 0;
+		inet_ntop(AF_INET6, &sin6->sin6_addr, name, sizeof(name));
+		return xstrdup(name);
 	}
 	if (IN6_IS_ADDR_UNSPECIFIED(&sin6->sin6_addr)) {
-		if (numeric & 0x8000) {
-			strcpy(name, bb_str_default);
-		} else {
-			name[0] = '*';
-			name[1] = '\0';
-		}
-		return 0;
+		if (numeric & 0x8000)
+			return xstrdup(bb_str_default);
+		return xstrdup("*");
 	}
 
-	s = getnameinfo((struct sockaddr *) sin6, sizeof(struct sockaddr_in6), name, len, NULL, 0, 0);
+	s = getnameinfo((struct sockaddr *) sin6, sizeof(struct sockaddr_in6),
+				name, sizeof(name), NULL, 0, 0);
 	if (s) {
 		bb_error_msg("getnameinfo failed");
-		return -1;
+		return NULL;
 	}
-	return 0;
+	return xstrdup(name);
 }
 
 #endif							/* CONFIG_FEATURE_IPV6 */
