@@ -8,21 +8,23 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
-#define DONT_SET_PASS			(1 << 4)
-#define DONT_MAKE_HOME			(1 << 6)
+#define OPT_DONT_SET_PASS  (1 << 4)
+#define OPT_DONT_MAKE_HOME (1 << 6)
 
 
 /* remix */
 /* EDR recoded such that the uid may be passed in *p */
 static int passwd_study(const char *filename, struct passwd *p)
 {
-	struct passwd *pw;
+	enum { min = 500, max = 65000 };
 	FILE *passwd;
-
-	const int min = 500;
-	const int max = 65000;
+	/* We are using reentrant fgetpwent_r() in order to avoid
+	 * pulling in static buffers from libc (think static build here) */
+	char buffer[256];
+	struct passwd pw;
+	struct passwd *result;
 
 	passwd = xfopen(filename, "r");
 
@@ -34,14 +36,14 @@ static int passwd_study(const char *filename, struct passwd *p)
 	 * make sure login isn't taken;
 	 * find free uid and gid;
 	 */
-	while ((pw = fgetpwent(passwd))) {
-		if (strcmp(pw->pw_name, p->pw_name) == 0) {
+	while (!fgetpwent_r(passwd, &pw, buffer, sizeof(buffer), &result)) {
+		if (strcmp(pw.pw_name, p->pw_name) == 0) {
 			/* return 0; */
 			return 1;
 		}
-		if ((pw->pw_uid >= p->pw_uid) && (pw->pw_uid < max)
-			&& (pw->pw_uid >= min)) {
-			p->pw_uid = pw->pw_uid + 1;
+		if ((pw.pw_uid >= p->pw_uid) && (pw.pw_uid < max)
+			&& (pw.pw_uid >= min)) {
+			p->pw_uid = pw.pw_uid + 1;
 		}
 	}
 
@@ -79,13 +81,14 @@ static void passwd_wrapper(const char *login) ATTRIBUTE_NORETURN;
 
 static void passwd_wrapper(const char *login)
 {
-	static const char prog[] = "passwd";
-	execlp(prog, prog, login, NULL);
+	static const char prog[] ALIGN1 = "passwd";
+
+	BB_EXECLP(prog, prog, login, NULL);
 	bb_error_msg_and_die("failed to execute '%s', you must set the password for '%s' manually", prog, login);
 }
 
 /* putpwent(3) remix */
-static int adduser(struct passwd *p, unsigned long flags)
+static int adduser(struct passwd *p)
 {
 	FILE *file;
 	int addgroup = !p->pw_gid;
@@ -101,25 +104,29 @@ static int adduser(struct passwd *p, unsigned long flags)
 			bb_error_msg_and_die("illegal uid or no uids left");
 		case 3:
 			bb_error_msg_and_die("%s: group name already in use", p->pw_name);
- 	}
+	}
 
 	/* add to passwd */
 	if (putpwent(p, file) == -1) {
 		bb_perror_nomsg_and_die();
 	}
+	/* Do fclose even if !ENABLE_FEATURE_CLEAN_UP.
+	 * We will exec passwd, files must be flushed & closed before that! */
 	fclose(file);
 
 #if ENABLE_FEATURE_SHADOWPASSWDS
 	/* add to shadow if necessary */
-	file = xfopen(bb_path_shadow_file, "a");
-	fseek(file, 0, SEEK_END);
-	fprintf(file, "%s:!:%ld:%d:%d:%d:::\n",
-			p->pw_name,             /* username */
-			time(NULL) / 86400,     /* sp->sp_lstchg */
-			0,                      /* sp->sp_min */
-			99999,                  /* sp->sp_max */
-			7);                     /* sp->sp_warn */
-	fclose(file);
+	file = fopen_or_warn(bb_path_shadow_file, "a");
+	if (file) {
+		fseek(file, 0, SEEK_END);
+		fprintf(file, "%s:!:%ld:%d:%d:%d:::\n",
+				p->pw_name,             /* username */
+				time(NULL) / 86400,     /* sp->sp_lstchg */
+				0,                      /* sp->sp_min */
+				99999,                  /* sp->sp_max */
+				7);                     /* sp->sp_warn */
+		fclose(file);
+	}
 #endif
 
 	/* add to group */
@@ -128,9 +135,9 @@ static int adduser(struct passwd *p, unsigned long flags)
 	if (addgroup) addgroup_wrapper(p);
 
 	/* Clear the umask for this process so it doesn't
-	 * * screw up the permissions on the mkdir and chown. */
+	 * screw up the permissions on the mkdir and chown. */
 	umask(0);
-	if (!(flags & DONT_MAKE_HOME)) {
+	if (!(option_mask32 & OPT_DONT_MAKE_HOME)) {
 		/* Set the owner and group so it is owned by the new user,
 		   then fix up the permissions to 2755. Can't do it before
 		   since chown will clear the setgid bit */
@@ -141,7 +148,7 @@ static int adduser(struct passwd *p, unsigned long flags)
 		}
 	}
 
-	if (!(flags & DONT_SET_PASS)) {
+	if (!(option_mask32 & OPT_DONT_SET_PASS)) {
 		/* interactively set passwd */
 		passwd_wrapper(p->pw_name);
 	}
@@ -158,37 +165,36 @@ static int adduser(struct passwd *p, unsigned long flags)
  *
  * can be customized via command-line parameters.
  */
+int adduser_main(int argc, char **argv);
 int adduser_main(int argc, char **argv)
 {
 	struct passwd pw;
 	const char *usegroup = NULL;
-	unsigned long flags;
 
 	/* got root? */
 	if (geteuid()) {
 		bb_error_msg_and_die(bb_msg_perm_denied_are_you_root);
 	}
 
-	pw.pw_gecos = "Linux User,,,";
+	pw.pw_gecos = (char *)"Linux User,,,";
 	pw.pw_shell = (char *)DEFAULT_SHELL;
 	pw.pw_dir = NULL;
 
-	/* check for min, max and missing args and exit on error */
-	opt_complementary = "-1:?1:?";
-	flags = getopt32(argc, argv, "h:g:s:G:DSH", &pw.pw_dir, &pw.pw_gecos, &pw.pw_shell, &usegroup);
-
-	/* create string for $HOME if not specified already */
-	if (!pw.pw_dir) {
-		snprintf(bb_common_bufsiz1, BUFSIZ, "/home/%s", argv[optind]);
-		pw.pw_dir = bb_common_bufsiz1;
-	}
+	/* exactly one non-option arg */
+	opt_complementary = "=1";
+	getopt32(argv, "h:g:s:G:DSH", &pw.pw_dir, &pw.pw_gecos, &pw.pw_shell, &usegroup);
+	argv += optind;
 
 	/* create a passwd struct */
-	pw.pw_name = argv[optind];
-	pw.pw_passwd = "x";
+	pw.pw_name = argv[0];
+	if (!pw.pw_dir) {
+		/* create string for $HOME if not specified already */
+		pw.pw_dir = xasprintf("/home/%s", argv[0]);
+	}
+	pw.pw_passwd = (char *)"x";
 	pw.pw_uid = 0;
 	pw.pw_gid = usegroup ? xgroup2gid(usegroup) : 0; /* exits on failure */
 
 	/* grand finale */
-	return adduser(&pw, flags);
+	return adduser(&pw);
 }
