@@ -3,16 +3,28 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 #include <utmp.h>
 #include <sys/resource.h>
 #include <syslog.h>
 
-#ifdef CONFIG_SELINUX
+#if ENABLE_SELINUX
 #include <selinux/selinux.h>  /* for is_selinux_enabled()  */
 #include <selinux/get_context_list.h> /* for get_default_context() */
 #include <selinux/flask.h> /* for security class definitions  */
-#include <errno.h>
+#endif
+
+#if ENABLE_PAM
+/* PAM may include <locale.h>. We may need to undefine bbox's stub define: */
+#undef setlocale
+/* For some obscure reason, PAM is not in pam/xxx, but in security/xxx.
+ * Apparently they like to confuse people. */
+#include <security/pam_appl.h>
+#include <security/pam_misc.h>
+static const struct pam_conv conv = {
+	misc_conv,
+	NULL
+};
 #endif
 
 enum {
@@ -22,8 +34,7 @@ enum {
 	TTYNAME_SIZE = 32,
 };
 
-static char full_tty[TTYNAME_SIZE];
-static char* short_tty = full_tty;
+static char* short_tty;
 
 #if ENABLE_FEATURE_UTMP
 /* vv  Taken from tinylogin utmp.c  vv */
@@ -41,9 +52,7 @@ static char* short_tty = full_tty;
  *	command line flags.
  */
 
-static struct utmp utent;
-
-static void read_or_build_utent(int picky)
+static void read_or_build_utent(struct utmp *utptr, int picky)
 {
 	struct utmp *ut;
 	pid_t pid = getpid();
@@ -58,23 +67,23 @@ static void read_or_build_utent(int picky)
 
 	/* If there is one, just use it, otherwise create a new one.  */
 	if (ut) {
-		utent = *ut;
+		*utptr = *ut;
 	} else {
 		if (picky)
 			bb_error_msg_and_die("no utmp entry found");
 
-		memset(&utent, 0, sizeof(utent));
-		utent.ut_type = LOGIN_PROCESS;
-		utent.ut_pid = pid;
-		strncpy(utent.ut_line, short_tty, sizeof(utent.ut_line));
+		memset(utptr, 0, sizeof(*utptr));
+		utptr->ut_type = LOGIN_PROCESS;
+		utptr->ut_pid = pid;
+		strncpy(utptr->ut_line, short_tty, sizeof(utptr->ut_line));
 		/* This one is only 4 chars wide. Try to fit something
 		 * remotely meaningful by skipping "tty"... */
-		strncpy(utent.ut_id, short_tty + 3, sizeof(utent.ut_id));
-		strncpy(utent.ut_user, "LOGIN", sizeof(utent.ut_user));
-		utent.ut_time = time(NULL);
+		strncpy(utptr->ut_id, short_tty + 3, sizeof(utptr->ut_id));
+		strncpy(utptr->ut_user, "LOGIN", sizeof(utptr->ut_user));
+		utptr->ut_time = time(NULL);
 	}
 	if (!picky)	/* root login */
-		memset(utent.ut_host, 0, sizeof(utent.ut_host));
+		memset(utptr->ut_host, 0, sizeof(utptr->ut_host));
 }
 
 /*
@@ -83,36 +92,37 @@ static void read_or_build_utent(int picky)
  *	write_utent changes the type of the current utmp entry to
  *	USER_PROCESS.  the wtmp file will be updated as well.
  */
-static void write_utent(const char *username)
+static void write_utent(struct utmp *utptr, const char *username)
 {
-	utent.ut_type = USER_PROCESS;
-	strncpy(utent.ut_user, username, sizeof(utent.ut_user));
-	utent.ut_time = time(NULL);
+	utptr->ut_type = USER_PROCESS;
+	strncpy(utptr->ut_user, username, sizeof(utptr->ut_user));
+	utptr->ut_time = time(NULL);
 	/* other fields already filled in by read_or_build_utent above */
 	setutent();
-	pututline(&utent);
+	pututline(utptr);
 	endutent();
 #if ENABLE_FEATURE_WTMP
 	if (access(bb_path_wtmp_file, R_OK|W_OK) == -1) {
 		close(creat(bb_path_wtmp_file, 0664));
 	}
-	updwtmp(bb_path_wtmp_file, &utent);
+	updwtmp(bb_path_wtmp_file, utptr);
 #endif
 }
 #else /* !ENABLE_FEATURE_UTMP */
-static inline void read_or_build_utent(int ATTRIBUTE_UNUSED picky) {}
-static inline void write_utent(const char ATTRIBUTE_UNUSED *username) {}
+#define read_or_build_utent(utptr, picky) ((void)0)
+#define write_utent(utptr, username) ((void)0)
 #endif /* !ENABLE_FEATURE_UTMP */
 
+#if ENABLE_FEATURE_NOLOGIN
 static void die_if_nologin_and_non_root(int amroot)
 {
 	FILE *fp;
 	int c;
 
-	if (access(bb_path_nologin_file, F_OK))
+	if (access("/etc/nologin", F_OK))
 		return;
 
-	fp = fopen(bb_path_nologin_file, "r");
+	fp = fopen("/etc/nologin", "r");
 	if (fp) {
 		while ((c = getc(fp)) != EOF)
 			putchar((c=='\n') ? '\r' : c);
@@ -122,28 +132,31 @@ static void die_if_nologin_and_non_root(int amroot)
 		puts("\r\nSystem closed for routine maintenance\r");
 	if (!amroot)
 		exit(1);
-	puts("\r\n[Disconnect bypassed -- root login allowed.]\r");
+	puts("\r\n[Disconnect bypassed -- root login allowed]\r");
 }
+#else
+static ALWAYS_INLINE void die_if_nologin_and_non_root(int amroot) {}
+#endif
 
-#if ENABLE_FEATURE_SECURETTY
+#if ENABLE_FEATURE_SECURETTY && !ENABLE_PAM
 static int check_securetty(void)
 {
 	FILE *fp;
 	int i;
-	char buf[BUFSIZ];
+	char buf[256];
 
-	fp = fopen(bb_path_securetty_file, "r");
+	fp = fopen("/etc/securetty", "r");
 	if (!fp) {
 		/* A missing securetty file is not an error. */
 		return 1;
 	}
 	while (fgets(buf, sizeof(buf)-1, fp)) {
-		for (i = strlen(buf)-1; i>=0; --i) {
+		for (i = strlen(buf)-1; i >= 0; --i) {
 			if (!isspace(buf[i]))
 				break;
 		}
 		buf[++i] = '\0';
-		if ((buf[0]=='\0') || (buf[0]=='#'))
+		if (!buf[0] || (buf[0] == '#'))
 			continue;
 		if (strcmp(buf, short_tty) == 0) {
 			fclose(fp);
@@ -154,16 +167,17 @@ static int check_securetty(void)
 	return 0;
 }
 #else
-static inline int check_securetty(void) { return 1; }
+static ALWAYS_INLINE int check_securetty(void) { return 1; }
 #endif
 
 static void get_username_or_die(char *buf, int size_buf)
 {
 	int c, cntdown;
+
 	cntdown = EMPTY_USERNAME_COUNT;
-prompt:
-	/* skip whitespace */
+ prompt:
 	print_login_prompt();
+	/* skip whitespace */
 	do {
 		c = getchar();
 		if (c == EOF) exit(1);
@@ -184,33 +198,28 @@ prompt:
 
 static void motd(void)
 {
-	FILE *fp;
-	int c;
+	int fd;
 
-	fp = fopen(bb_path_motd_file, "r");
-	if (fp) {
-		while ((c = getc(fp)) != EOF)
-			putchar(c);
-		fclose(fp);
+	fd = open(bb_path_motd_file, O_RDONLY);
+	if (fd) {
+		fflush(stdout);
+		bb_copyfd_eof(fd, STDOUT_FILENO);
+		close(fd);
 	}
 }
 
-static void nonblock(int fd)
-{
-	fcntl(fd, F_SETFL, O_NONBLOCK | fcntl(fd, F_GETFL));
-}
-
 static void alarm_handler(int sig ATTRIBUTE_UNUSED)
 {
 	/* This is the escape hatch!  Poor serial line users and the like
 	 * arrive here when their connection is broken.
 	 * We don't want to block here */
-	nonblock(1);
-	nonblock(2);
-	bb_info_msg("\r\nLogin timed out after %d seconds\r", TIMEOUT);
+	ndelay_on(1);
+	ndelay_on(2);
+	printf("\r\nLogin timed out after %d seconds\r\n", TIMEOUT);
 	exit(EXIT_SUCCESS);
 }
 
+int login_main(int argc, char **argv);
 int login_main(int argc, char **argv)
 {
 	enum {
@@ -218,7 +227,7 @@ int login_main(int argc, char **argv)
 		LOGIN_OPT_h = (1<<1),
 		LOGIN_OPT_p = (1<<2),
 	};
-	char fromhost[512];
+	char *fromhost;
 	char username[USERNAME_SIZE];
 	const char *tmp;
 	int amroot;
@@ -227,14 +236,26 @@ int login_main(int argc, char **argv)
 	struct passwd *pw;
 	char *opt_host = NULL;
 	char *opt_user = NULL;
+	char full_tty[TTYNAME_SIZE];
 	USE_SELINUX(security_context_t user_sid = NULL;)
+	USE_FEATURE_UTMP(struct utmp utent;)
+	USE_PAM(pam_handle_t *pamh;)
+	USE_PAM(int pamret;)
+	USE_PAM(const char *failed_msg;)
 
+	short_tty = full_tty;
 	username[0] = '\0';
 	amroot = (getuid() == 0);
 	signal(SIGALRM, alarm_handler);
 	alarm(TIMEOUT);
 
-	opt = getopt32(argc, argv, "f:h:p", &opt_user, &opt_host);
+	/* Mandatory paranoia for suid applet:
+	 * ensure that fd# 0,1,2 are opened (at least to /dev/null)
+	 * and any extra open fd's are closed.
+	 * (The name of the function is misleading. Not daemonizing here.) */
+	bb_daemonize_or_rexec(DAEMON_ONLY_SANITIZE | DAEMON_CLOSE_EXTRA_FDS, NULL);
+
+	opt = getopt32(argv, "f:h:p", &opt_user, &opt_host);
 	if (opt & LOGIN_OPT_f) {
 		if (!amroot)
 			bb_error_msg_and_die("-f is for root only");
@@ -254,19 +275,18 @@ int login_main(int argc, char **argv)
 			short_tty = full_tty + 5;
 	}
 
-	read_or_build_utent(!amroot);
+	read_or_build_utent(&utent, !amroot);
 
 	if (opt_host) {
 		USE_FEATURE_UTMP(
 			safe_strncpy(utent.ut_host, opt_host, sizeof(utent.ut_host));
 		)
-		snprintf(fromhost, sizeof(fromhost)-1, " on '%.100s' from "
-					"'%.200s'", short_tty, opt_host);
-	}
-	else
-		snprintf(fromhost, sizeof(fromhost)-1, " on '%.100s'", short_tty);
+		fromhost = xasprintf(" on '%s' from '%s'", short_tty, opt_host);
+	} else
+		fromhost = xasprintf(" on '%s'", short_tty);
 
-	bb_setpgrp;
+	/* Was breaking "login <username>" from shell command line: */
+	/*bb_setpgrp();*/
 
 	openlog(applet_name, LOG_PID | LOG_CONS | LOG_NOWAIT, LOG_AUTH);
 
@@ -274,10 +294,58 @@ int login_main(int argc, char **argv)
 		if (!username[0])
 			get_username_or_die(username, sizeof(username));
 
+#if ENABLE_PAM
+		pamret = pam_start("login", username, &conv, &pamh);
+		if (pamret != PAM_SUCCESS) {
+			failed_msg = "pam_start";
+			goto pam_auth_failed;
+		}
+		/* set TTY (so things like securetty work) */
+		pamret = pam_set_item(pamh, PAM_TTY, short_tty);
+		if (pamret != PAM_SUCCESS) {
+			failed_msg = "pam_set_item(TTY)";
+			goto pam_auth_failed;
+		}
+		pamret = pam_authenticate(pamh, 0);
+		if (pamret != PAM_SUCCESS) {
+			failed_msg = "pam_authenticate";
+			goto pam_auth_failed;
+			/* TODO: or just "goto auth_failed"
+			 * since user seems to enter wrong password
+			 * (in this case pamret == 7)
+			 */
+		}
+		/* check that the account is healthy */
+		pamret = pam_acct_mgmt(pamh, 0);
+		if (pamret != PAM_SUCCESS) {
+			failed_msg = "account setup";
+			goto pam_auth_failed;
+		}
+		/* read user back */
+		{
+			const char *pamuser;
+			/* gcc: "dereferencing type-punned pointer breaks aliasing rules..."
+			 * thus we cast to (void*) */
+			if (pam_get_item(pamh, PAM_USER, (void*)&pamuser) != PAM_SUCCESS) {
+				failed_msg = "pam_get_item(USER)";
+				goto pam_auth_failed;
+			}
+			safe_strncpy(username, pamuser, sizeof(username));
+		}
+		/* If we get here, the user was authenticated, and is
+		 * granted access. */
+		pw = getpwnam(username);
+		if (pw)
+			break;
+		goto auth_failed;
+ pam_auth_failed:
+		bb_error_msg("%s failed: %s (%d)", failed_msg, pam_strerror(pamh, pamret), pamret);
+		safe_strncpy(username, "UNKNOWN", sizeof(username));
+#else /* not PAM */
 		pw = getpwnam(username);
 		if (!pw) {
-			safe_strncpy(username, "UNKNOWN", sizeof(username));
-			goto auth_failed;
+			strcpy(username, "UNKNOWN");
+			goto fake_it;
 		}
 
 		if (pw->pw_passwd[0] == '!' || pw->pw_passwd[0] == '*')
@@ -292,14 +360,15 @@ int login_main(int argc, char **argv)
 		/* Don't check the password if password entry is empty (!) */
 		if (!pw->pw_passwd[0])
 			break;
-
+ fake_it:
 		/* authorization takes place here */
 		if (correct_password(pw))
 			break;
-
-auth_failed:
+#endif /* ENABLE_PAM */
+ auth_failed:
 		opt &= ~LOGIN_OPT_f;
 		bb_do_delay(FAIL_DELAY);
+		/* TODO: doesn't sound like correct English phrase to me */
 		puts("Login incorrect");
 		if (++count == 3) {
 			syslog(LOG_WARNING, "invalid password for '%s'%s",
@@ -312,9 +381,9 @@ auth_failed:
 	alarm(0);
 	die_if_nologin_and_non_root(pw->pw_uid == 0);
 
-	write_utent(username);
+	write_utent(&utent, username);
 
-#ifdef CONFIG_SELINUX
+#if ENABLE_SELINUX
 	if (is_selinux_enabled()) {
 		security_context_t old_tty_sid, new_tty_sid;
 
@@ -342,25 +411,20 @@ auth_failed:
 	fchown(0, pw->pw_uid, pw->pw_gid);
 	fchmod(0, 0600);
 
-	/* TODO: be nommu-friendly, use spawn? */
 	if (ENABLE_LOGIN_SCRIPTS) {
-		char *script = getenv("LOGIN_PRE_SUID_SCRIPT");
-		if (script) {
-			char *t_argv[2] = { script, NULL };
-			switch (fork()) {
-			case -1: break;
-			case 0: /* child */
-				xchdir("/");
-				setenv("LOGIN_TTY", full_tty, 1);
-				setenv("LOGIN_USER", pw->pw_name, 1);
-				setenv("LOGIN_UID", utoa(pw->pw_uid), 1);
-				setenv("LOGIN_GID", utoa(pw->pw_gid), 1);
-				setenv("LOGIN_SHELL", pw->pw_shell, 1);
-				execvp(script, t_argv);
-				exit(1);
-			default: /* parent */
-				wait(NULL);
-			}
+		char *t_argv[2];
+
+		t_argv[0] = getenv("LOGIN_PRE_SUID_SCRIPT");
+		if (t_argv[0]) {
+			t_argv[1] = NULL;
+			xsetenv("LOGIN_TTY", full_tty);
+			xsetenv("LOGIN_USER", pw->pw_name);
+			xsetenv("LOGIN_UID", utoa(pw->pw_uid));
+			xsetenv("LOGIN_GID", utoa(pw->pw_gid));
+			xsetenv("LOGIN_SHELL", pw->pw_shell);
+			xspawn(t_argv); /* NOMMU-friendly */
+			/* All variables are unset by setup_environment */
+			wait(NULL);
 		}
 	}
 
@@ -368,13 +432,15 @@ auth_failed:
 	tmp = pw->pw_shell;
 	if (!tmp || !*tmp)
 		tmp = DEFAULT_SHELL;
+	/* setup_environment params: shell, loginshell, changeenv, pw */
 	setup_environment(tmp, 1, !(opt & LOGIN_OPT_p), pw);
+	/* FIXME: login shell = 1 -> 3rd parameter is ignored! */
 
 	motd();
 
 	if (pw->pw_uid == 0)
 		syslog(LOG_INFO, "root login%s", fromhost);
-#ifdef CONFIG_SELINUX
+#if ENABLE_SELINUX
 	/* well, a simple setexeccon() here would do the job as well,
 	 * but let's play the game for now */
 	set_current_security_context(user_sid);
@@ -385,6 +451,9 @@ auth_failed:
 	// setsid();
 	// /* TIOCSCTTY: steal tty from other process group */
 	// if (ioctl(0, TIOCSCTTY, 1)) error_msg...
+	// BBox login used to do this (see above):
+	// bb_setpgrp();
+	// If this stuff is really needed, add it and explain why!
 
 	/* set signals to defaults */
 	signal(SIGALRM, SIG_DFL);
@@ -396,7 +465,8 @@ auth_failed:
 	 * should it leave SIGINT etc enabled or disabled? */
 	signal(SIGINT, SIG_DFL);
 
-	run_shell(tmp, 1, 0, 0);	/* exec the shell finally */
+	/* Exec login shell with no additional parameters */
+	run_shell(tmp, 1, NULL, NULL);
 
-	return EXIT_FAILURE;
+	/* return EXIT_FAILURE; - not reached */
 }
