@@ -16,7 +16,7 @@
 /* Fixed by Erik Andersen to do passwords the tinylogin way...
  * It now works with md5, sha1, etc passwords. */
 
-#include "busybox.h"
+#include "libbb.h"
 #include <sys/vt.h>
 
 static struct passwd *pw;
@@ -41,6 +41,7 @@ static void restore_terminal(void)
 	tcsetattr(STDIN_FILENO, TCSANOW, &oterm);
 }
 
+int vlock_main(int argc, char **argv);
 int vlock_main(int argc, char **argv)
 {
 	sigset_t sig;
@@ -57,13 +58,11 @@ int vlock_main(int argc, char **argv)
 		bb_show_usage();
 	}
 
-	o_lock_all = getopt32(argc, argv, "a");
+	o_lock_all = getopt32(argv, "a");
 
 	vfd = xopen(CURRENT_TTY, O_RDWR);
 
-	if (ioctl(vfd, VT_GETMODE, &vtm) < 0) {
-		bb_perror_msg_and_die("VT_GETMODE");
-	}
+	xioctl(vfd, VT_GETMODE, &vtm);
 
 	/* mask a bunch of signals */
 	sigprocmask(SIG_SETMASK, NULL, &sig);
