@@ -21,7 +21,9 @@
  *   redirected input has been read from stdin
  */
 
-#include "busybox.h"
+#include <sched.h>	/* sched_yield() */
+
+#include "libbb.h"
 #if ENABLE_FEATURE_LESS_REGEXP
 #include "xregex.h"
 #endif
@@ -46,8 +48,12 @@ enum {
 	REAL_KEY_LEFT = 'D',
 	REAL_PAGE_UP = '5',
 	REAL_PAGE_DOWN = '6',
-	REAL_KEY_HOME = '7',
+	REAL_KEY_HOME = '7', // vt100? linux vt? or what?
 	REAL_KEY_END = '8',
+	REAL_KEY_HOME_ALT = '1', // ESC [1~ (vt100? linux vt? or what?)
+	REAL_KEY_END_ALT = '4', // ESC [4~
+	REAL_KEY_HOME_XTERM = 'H',
+	REAL_KEY_END_XTERM = 'F',
 
 /* These are the special codes assigned by this program to the special keys */
 	KEY_UP = 20,
@@ -66,27 +72,6 @@ enum {
 	TILDES = 1,
 };
 
-static unsigned max_displayed_line;
-static unsigned width;
-static const char *empty_line_marker = "~";
-
-static char *filename;
-static char **files;
-static unsigned num_files = 1;
-static unsigned current_file = 1;
-static const char **buffer;
-static const char **flines;
-static int cur_fline; /* signed */
-static unsigned max_fline;
-static unsigned max_lineno; /* this one tracks linewrap */
-
-static ssize_t eof_error = 1; /* eof if 0, error if < 0 */
-static char terminated = 1;
-static size_t readpos;
-static size_t readeof;
-/* last position in last line, taking into account tabs */
-static size_t linepos;
-
 /* Command line options */
 enum {
 	FLAG_E = 1,
@@ -98,25 +83,81 @@ enum {
 	LESS_STATE_MATCH_BACKWARDS = 1 << 15,
 };
 
-#if ENABLE_FEATURE_LESS_MARKS
-static unsigned mark_lines[15][2];
-static unsigned num_marks;
-#endif
-
-#if ENABLE_FEATURE_LESS_REGEXP
-static unsigned *match_lines;
-static int match_pos; /* signed! */
-static unsigned num_matches;
-static regex_t pattern;
-static unsigned pattern_valid;
-#else
+#if !ENABLE_FEATURE_LESS_REGEXP
 enum { pattern_valid = 0 };
 #endif
 
-static struct termios term_orig, term_vi;
-
-/* File pointer to get input from */
-static int kbd_fd;
+struct globals {
+	int cur_fline; /* signed */
+	int kbd_fd;  /* fd to get input from */
+/* last position in last line, taking into account tabs */
+	size_t linepos;
+	unsigned max_displayed_line;
+	unsigned max_fline;
+	unsigned max_lineno; /* this one tracks linewrap */
+	unsigned width;
+	ssize_t eof_error; /* eof if 0, error if < 0 */
+	size_t readpos;
+	size_t readeof;
+	const char **buffer;
+	const char **flines;
+	const char *empty_line_marker;
+	unsigned num_files;
+	unsigned current_file;
+	char *filename;
+	char **files;
+#if ENABLE_FEATURE_LESS_MARKS
+	unsigned num_marks;
+	unsigned mark_lines[15][2];
+#endif
+#if ENABLE_FEATURE_LESS_REGEXP
+	unsigned *match_lines;
+	int match_pos; /* signed! */
+	unsigned num_matches;
+	regex_t pattern;
+	smallint pattern_valid;
+#endif
+	smallint terminated;
+	struct termios term_orig, term_less;
+};
+#define G (*ptr_to_globals)
+#define cur_fline           (G.cur_fline         )
+#define kbd_fd              (G.kbd_fd            )
+#define linepos             (G.linepos           )
+#define max_displayed_line  (G.max_displayed_line)
+#define max_fline           (G.max_fline         )
+#define max_lineno          (G.max_lineno        )
+#define width               (G.width             )
+#define eof_error           (G.eof_error         )
+#define readpos             (G.readpos           )
+#define readeof             (G.readeof           )
+#define buffer              (G.buffer            )
+#define flines              (G.flines            )
+#define empty_line_marker   (G.empty_line_marker )
+#define num_files           (G.num_files         )
+#define current_file        (G.current_file      )
+#define filename            (G.filename          )
+#define files               (G.files             )
+#define num_marks           (G.num_marks         )
+#define mark_lines          (G.mark_lines        )
+#if ENABLE_FEATURE_LESS_REGEXP
+#define match_lines         (G.match_lines       )
+#define match_pos           (G.match_pos         )
+#define num_matches         (G.num_matches       )
+#define pattern             (G.pattern           )
+#define pattern_valid       (G.pattern_valid     )
+#endif
+#define terminated          (G.terminated        )
+#define term_orig           (G.term_orig         )
+#define term_less           (G.term_less         )
+#define INIT_G() do { \
+		PTR_TO_GLOBALS = xzalloc(sizeof(G)); \
+		empty_line_marker = "~"; \
+		num_files = 1; \
+		current_file = 1; \
+		eof_error = 1; \
+		terminated = 1; \
+	} while (0)
 
 /* Reset terminal input to normal */
 static void set_tty_cooked(void)
@@ -165,7 +206,29 @@ static void fill_match_lines(unsigned pos);
 #define fill_match_lines(pos) ((void)0)
 #endif
 
-
+/* Devilishly complex routine.
+ *
+ * Has to deal with EOF and EPIPE on input,
+ * with line wrapping, with last line not ending in '\n'
+ * (possibly not ending YET!), with backspace and tabs.
+ * It reads input again if last time we got an EOF (thus supporting
+ * growing files) or EPIPE (watching output of slow process like make).
+ *
+ * Variables used:
+ * flines[] - array of lines already read. Linewrap may cause
+ *      one source file line to occupy several flines[n].
+ * flines[max_fline] - last line, possibly incomplete.
+ * terminated - 1 if flines[max_fline] is 'terminated'
+ *      (if there was '\n' [which isn't stored itself, we just remember
+ *      that it was seen])
+ * max_lineno - last line's number, this one doesn't increment
+ *      on line wrap, only on "real" new lines.
+ * readbuf[0..readeof-1] - small preliminary buffer.
+ * readbuf[readpos] - next character to add to current line.
+ * linepos - screen line position of next char to be read
+ *      (takes into account tabs and backspaces)
+ * eof_error - < 0 error, == 0 EOF, > 0 not EOF/error
+ */
 static void read_lines(void)
 {
 #define readbuf bb_common_bufsiz1
@@ -186,6 +249,7 @@ static void read_lines(void)
 			cp += 8;
 		strcpy(current_line, cp);
 		p += strlen(current_line);
+		/* linepos is still valid from previous read_lines() */
 	} else {
 		linepos = 0;
 	}
@@ -196,17 +260,35 @@ static void read_lines(void)
 		terminated = 0;
 		while (1) {
 			char c;
+			/* if no unprocessed chars left, eat more */
 			if (readpos >= readeof) {
+				smallint yielded = 0;
+
 				ndelay_on(0);
+ read_again:
 				eof_error = safe_read(0, readbuf, sizeof(readbuf));
-				ndelay_off(0);
 				readpos = 0;
 				readeof = eof_error;
 				if (eof_error < 0) {
+					if (errno == EAGAIN && !yielded) {
+			/* We can hit EAGAIN while searching for regexp match.
+			 * Yield is not 100% reliable solution in general,
+			 * but for less it should be good enough -
+			 * we give stdin supplier some CPU time to produce
+			 * more input. We do it just once.
+			 * Currently, we do not stop when we found the Nth
+			 * occurrence we were looking for. We read till end
+			 * (or double EAGAIN). TODO? */
+						sched_yield();
+						yielded = 1;
+						goto read_again;
+					}
 					readeof = 0;
 					if (errno != EAGAIN)
 						print_statusline("read error");
 				}
+				ndelay_off(0);
+
 				if (eof_error <= 0) {
 					goto reached_eof;
 				}
@@ -218,17 +300,27 @@ static void read_lines(void)
 			if (c == '\x8' && linepos && p[-1] != '\t') {
 				readpos++; /* eat it */
 				linepos--;
+			/* was buggy (p could end up <= current_line)... */
 				*--p = '\0';
 				continue;
 			}
-			if (c == '\t')
-				linepos += (linepos^7) & 7;
-			linepos++;
-			if (linepos >= w)
-				break;
+			{
+				size_t new_linepos = linepos + 1;
+				if (c == '\t') {
+					new_linepos += 7;
+					new_linepos &= (~7);
+				}
+				if (new_linepos >= w)
+					break;
+				linepos = new_linepos;
+			}
 			/* ok, we will eat this char */
 			readpos++;
-			if (c == '\n') { terminated = 1; break; }
+			if (c == '\n') {
+				terminated = 1;
+				linepos = 0;
+				break;
+			}
 			/* NUL is substituted by '\n'! */
 			if (c == '\0') c = '\n';
 			*p++ = c;
@@ -255,8 +347,10 @@ static void read_lines(void)
 		} else {
 			flines[max_fline] = xrealloc(current_line, strlen(current_line)+1);
 		}
-		if (max_fline >= MAXLINES)
+		if (max_fline >= MAXLINES) {
+			eof_error = 0; /* Pretend we saw EOF */
 			break;
+		}
 		if (max_fline > cur_fline + max_displayed_line)
 			break;
 		if (eof_error <= 0) {
@@ -339,12 +433,29 @@ static void status_print(void)
 	print_hilite(p);
 }
 
-static char controls[] =
+static void cap_cur_fline(int nlines)
+{
+	int diff;
+	if (cur_fline < 0)
+		cur_fline = 0;
+	if (cur_fline + max_displayed_line > max_fline + TILDES) {
+		cur_fline -= nlines;
+		if (cur_fline < 0)
+			cur_fline = 0;
+		diff = max_fline - (cur_fline + max_displayed_line) + TILDES;
+		/* As the number of lines requested was too large, we just move
+		to the end of the file */
+		if (diff > 0)
+			cur_fline += diff;
+	}
+}
+
+static const char controls[] ALIGN1 =
 	/* NUL: never encountered; TAB: not converted */
 	/**/"\x01\x02\x03\x04\x05\x06\x07\x08"  "\x0a\x0b\x0c\x0d\x0e\x0f"
 	"\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f"
 	"\x7f\x9b"; /* DEL and infamous Meta-ESC :( */
-static char ctrlconv[] =
+static const char ctrlconv[] ALIGN1 =
 	/* '\n': it's a former NUL - subst with '@', not 'J' */
 	"\x40\x41\x42\x43\x44\x45\x46\x47\x48\x49\x40\x4b\x4c\x4d\x4e\x4f"
 	"\x50\x51\x52\x53\x54\x55\x56\x57\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f";
@@ -476,18 +587,9 @@ static void buffer_fill_and_print(void)
 /* Move the buffer up and down in the file in order to scroll */
 static void buffer_down(int nlines)
 {
-	int diff;
 	cur_fline += nlines;
 	read_lines();
-
-	if (cur_fline + max_displayed_line > max_fline + TILDES) {
-		cur_fline -= nlines;
-		diff = max_fline - (cur_fline + max_displayed_line) + TILDES;
-		/* As the number of lines requested was too large, we just move
-		to the end of the file */
-		if (diff > 0)
-			cur_fline += diff;
-	}
+	cap_cur_fline(nlines);
 	buffer_fill_and_print();
 }
 
@@ -507,6 +609,8 @@ static void buffer_line(int linenum)
 	read_lines();
 	if (linenum + max_displayed_line > max_fline)
 		linenum = max_fline - max_displayed_line + TILDES;
+	if (linenum < 0)
+		linenum = 0;
 	cur_fline = linenum;
 	buffer_fill_and_print();
 }
@@ -570,7 +674,7 @@ static void getch_nowait(char* input, int sz)
 		FD_SET(0, &readfds);
 	}
 	FD_SET(kbd_fd, &readfds);
-	tcsetattr(kbd_fd, TCSANOW, &term_vi);
+	tcsetattr(kbd_fd, TCSANOW, &term_less);
 	select(kbd_fd + 1, &readfds, NULL, NULL, NULL);
 
 	input[0] = '\0';
@@ -595,10 +699,11 @@ static int less_getch(void)
 	char input[16];
 	unsigned i;
  again:
+	memset(input, 0, sizeof(input));
 	getch_nowait(input, sizeof(input));
+
 	/* Detect escape sequences (i.e. arrow keys) and handle
 	 * them accordingly */
-
 	if (input[0] == '\033' && input[1] == '[') {
 		set_tty_cooked();
 		i = input[2] - REAL_KEY_UP;
@@ -607,6 +712,14 @@ static int less_getch(void)
 		i = input[2] - REAL_PAGE_UP;
 		if (i < 4)
 			return 24 + i;
+		if (input[2] == REAL_KEY_HOME_XTERM)
+			return KEY_HOME;
+		if (input[2] == REAL_KEY_HOME_ALT)
+			return KEY_HOME;
+		if (input[2] == REAL_KEY_END_XTERM)
+			return KEY_END;
+		if (input[2] == REAL_KEY_END_ALT)
+			return KEY_END;
 		return 0;
 	}
 	/* Reject almost all control chars */
@@ -626,8 +739,9 @@ static char* less_gets(int sz)
 
 		/* I be damned if I know why is it needed *repeatedly*,
 		 * but it is needed. Is it because of stdio? */
-		tcsetattr(kbd_fd, TCSANOW, &term_vi);
+		tcsetattr(kbd_fd, TCSANOW, &term_less);
 
+		c = '\0';
 		read(kbd_fd, &c, 1);
 		if (c == 0x0d)
 			return result;
@@ -735,20 +849,37 @@ static void colon_process(void)
 }
 
 #if ENABLE_FEATURE_LESS_REGEXP
-static int normalize_match_pos(int match)
+static void normalize_match_pos(int match)
 {
-	match_pos = match;
 	if (match >= num_matches)
-		match_pos = num_matches - 1;
+		match = num_matches - 1;
 	if (match < 0)
-		match_pos = 0;
-	return match_pos;
+		match = 0;
+	match_pos = match;
 }
 
 static void goto_match(int match)
 {
-	if (num_matches)
-		buffer_line(match_lines[normalize_match_pos(match)]);
+	int sv;
+
+	if (!pattern_valid)
+		return;
+	if (match < 0)
+		match = 0;
+	sv = cur_fline;
+	/* Try to find next match if eof isn't reached yet */
+	if (match >= num_matches && eof_error > 0) {
+		cur_fline = MAXLINES; /* look as far as needed */
+		read_lines();
+	}
+	if (num_matches) {
+		cap_cur_fline(cur_fline);
+		normalize_match_pos(match);
+		buffer_line(match_lines[match_pos]);
+	} else {
+		cur_fline = sv;
+		print_statusline("No matches found");
+	}
 }
 
 static void fill_match_lines(unsigned pos)
@@ -801,15 +932,10 @@ static void regex_process(void)
 		free(err);
 		return;
 	}
+
 	pattern_valid = 1;
 	match_pos = 0;
-
 	fill_match_lines(0);
-
-	if (num_matches == 0 || max_fline <= max_displayed_line) {
-		buffer_print();
-		return;
-	}
 	while (match_pos < num_matches) {
 		if (match_lines[match_pos] > cur_fline)
 			break;
@@ -817,7 +943,11 @@ static void regex_process(void)
 	}
 	if (option_mask32 & LESS_STATE_MATCH_BACKWARDS)
 		match_pos--;
-	buffer_line(match_lines[normalize_match_pos(match_pos)]);
+
+	/* It's possible that no matches are found yet.
+	 * goto_match() will read input looking for match,
+	 * if needed */
+	goto_match(match_pos);
 }
 #endif
 
@@ -863,13 +993,13 @@ static void number_process(int first_digit)
 		break;
 	case 'g': case '<': case 'G': case '>':
 		cur_fline = num + max_displayed_line;
-	 	read_lines();
+		read_lines();
 		buffer_line(num - 1);
 		break;
 	case 'p': case '%':
 		num = num * (max_fline / 100); /* + max_fline / 2; */
 		cur_fline = num + max_displayed_line;
-	 	read_lines();
+		read_lines();
 		buffer_line(num);
 		break;
 #if ENABLE_FEATURE_LESS_REGEXP
@@ -1181,14 +1311,17 @@ static void sig_catcher(int sig ATTRIBUTE_UNUSED)
 	exit(1);
 }
 
+int less_main(int argc, char **argv);
 int less_main(int argc, char **argv)
 {
 	int keypress;
 
+	INIT_G();
+
 	/* TODO: -x: do not interpret backspace, -xx: tab also */
 	/* -xxx: newline also */
 	/* -w N: assume width N (-xxx -w 32: hex viewer of sorts) */
-	getopt32(argc, argv, "EMmN~");
+	getopt32(argv, "EMmN~");
 	argc -= optind;
 	argv += optind;
 	num_files = argc;
@@ -1198,6 +1331,9 @@ int less_main(int argc, char **argv)
 	 * is not a tty and turns into cat. This makes sense. */
 	if (!isatty(STDOUT_FILENO))
 		return bb_cat(argv);
+	kbd_fd = open(CURRENT_TTY, O_RDONLY);
+	if (kbd_fd < 0)
+		return bb_cat(argv);
 
 	if (!num_files) {
 		if (isatty(STDIN_FILENO)) {
@@ -1208,8 +1344,6 @@ int less_main(int argc, char **argv)
 	} else
 		filename = xstrdup(files[0]);
 
-	kbd_fd = xopen(CURRENT_TTY, O_RDONLY);
-
 	get_terminal_width_height(kbd_fd, &width, &max_displayed_line);
 	/* 20: two tabstops + 4 */
 	if (width < 20 || max_displayed_line < 3)
@@ -1223,16 +1357,16 @@ int less_main(int argc, char **argv)
 	tcgetattr(kbd_fd, &term_orig);
 	signal(SIGTERM, sig_catcher);
 	signal(SIGINT, sig_catcher);
-	term_vi = term_orig;
-	term_vi.c_lflag &= ~(ICANON | ECHO);
-	term_vi.c_iflag &= ~(IXON | ICRNL);
-	/*term_vi.c_oflag &= ~ONLCR;*/
-	term_vi.c_cc[VMIN] = 1;
-	term_vi.c_cc[VTIME] = 0;
+	term_less = term_orig;
+	term_less.c_lflag &= ~(ICANON | ECHO);
+	term_less.c_iflag &= ~(IXON | ICRNL);
+	/*term_less.c_oflag &= ~ONLCR;*/
+	term_less.c_cc[VMIN] = 1;
+	term_less.c_cc[VTIME] = 0;
 
 	/* Want to do it just once, but it doesn't work, */
 	/* so we are redoing it (see code above). Mystery... */
-	/*tcsetattr(kbd_fd, TCSANOW, &term_vi);*/
+	/*tcsetattr(kbd_fd, TCSANOW, &term_less);*/
 
 	reinitialize();
 	while (1) {
