@@ -15,21 +15,31 @@
  * Rani Assaf <rani@magic.metawire.com> 980929:	resolve addresses
  */
 
-#include <string.h>
-
-#include "libbb.h"
+#include "ip_common.h"  /* #include "libbb.h" is inside */
 #include "utils.h"
-#include "ip_common.h"
-
 
 int preferred_family = AF_UNSPEC;
-int oneline = 0;
-char * _SL_ = NULL;
+smallint oneline;
+char _SL_;
 
 void ip_parse_common_args(int *argcp, char ***argvp)
 {
 	int argc = *argcp;
 	char **argv = *argvp;
+	static const char ip_common_commands[] ALIGN1 =
+		"-family\0""inet\0""inet6\0""link\0"
+		"-4\0""-6\0""-0\0""-oneline\0";
+	enum {
+		ARG_family = 1,
+		ARG_inet,
+		ARG_inet6,
+		ARG_link,
+		ARG_IPv4,
+		ARG_IPv6,
+		ARG_packet,
+		ARG_oneline
+	};
+	smalluint arg;
 
 	while (argc > 1) {
 		char *opt = argv[1];
@@ -39,33 +49,32 @@ void ip_parse_common_args(int *argcp, char ***argvp)
 			argv++;
 			break;
 		}
-
 		if (opt[0] != '-')
 			break;
-
 		if (opt[1] == '-')
 			opt++;
-
-		if (matches(opt, "-family") == 0) {
+		arg = index_in_strings(ip_common_commands, opt) + 1;
+		if (arg == ARG_family) {
 			argc--;
 			argv++;
 			if (!argv[1])
 				bb_show_usage();
-			if (strcmp(argv[1], "inet") == 0)
+			arg = index_in_strings(ip_common_commands, argv[1]) + 1;
+			if (arg == ARG_inet)
 				preferred_family = AF_INET;
-			else if (strcmp(argv[1], "inet6") == 0)
+			else if (arg == ARG_inet6)
 				preferred_family = AF_INET6;
-			else if (strcmp(argv[1], "link") == 0)
+			else if (arg == ARG_link)
 				preferred_family = AF_PACKET;
 			else
 				invarg(argv[1], "protocol family");
-		} else if (strcmp(opt, "-4") == 0) {
+		} else if (arg == ARG_IPv4) {
 			preferred_family = AF_INET;
-		} else if (strcmp(opt, "-6") == 0) {
+		} else if (arg == ARG_IPv6) {
 			preferred_family = AF_INET6;
-		} else if (strcmp(opt, "-0") == 0) {
+		} else if (arg == ARG_packet) {
 			preferred_family = AF_PACKET;
-		} else if (matches(opt, "-oneline") == 0) {
+		} else if (arg == ARG_oneline) {
 			++oneline;
 		} else {
 			bb_show_usage();
@@ -73,7 +82,7 @@ void ip_parse_common_args(int *argcp, char ***argvp)
 		argc--;
 		argv++;
 	}
-	_SL_ = oneline ? "\\" : "\n" ;
+	_SL_ = oneline ? '\\' : '\n';
 	*argcp = argc;
 	*argvp = argv;
 }
