@@ -7,142 +7,99 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include "libbb.h"
-
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-
+//#include <sys/ioctl.h>
+//#include <sys/socket.h>
 #include <net/if.h>
 #include <net/if_packet.h>
 #include <netpacket/packet.h>
-
 #include <net/ethernet.h>
 
+#include "ip_common.h"  /* #include "libbb.h" is inside */
 #include "rt_names.h"
 #include "utils.h"
-#include "ip_common.h"
 
-/* take from linux/sockios.h */
+/* taken from linux/sockios.h */
 #define SIOCSIFNAME	0x8923		/* set interface name */
 
-static int on_off(char *msg)
+static void on_off(const char *msg) ATTRIBUTE_NORETURN;
+static void on_off(const char *msg)
 {
-	bb_error_msg("error: argument of \"%s\" must be \"on\" or \"off\"", msg);
-	return -1;
+	bb_error_msg_and_die("error: argument of \"%s\" must be \"on\" or \"off\"", msg);
 }
 
+/* Exits on error */
 static int get_ctl_fd(void)
 {
-	int s_errno;
 	int fd;
 
 	fd = socket(PF_INET, SOCK_DGRAM, 0);
 	if (fd >= 0)
 		return fd;
-	s_errno = errno;
 	fd = socket(PF_PACKET, SOCK_DGRAM, 0);
 	if (fd >= 0)
 		return fd;
-	fd = socket(PF_INET6, SOCK_DGRAM, 0);
-	if (fd >= 0)
-		return fd;
-	errno = s_errno;
-	bb_perror_msg("cannot create control socket");
-	return -1;
+	return xsocket(PF_INET6, SOCK_DGRAM, 0);
 }
 
-static int do_chflags(char *dev, uint32_t flags, uint32_t mask)
+/* Exits on error */
+static void do_chflags(char *dev, uint32_t flags, uint32_t mask)
 {
 	struct ifreq ifr;
 	int fd;
-	int err;
 
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	fd = get_ctl_fd();
-	if (fd < 0)
-		return -1;
-	err = ioctl(fd, SIOCGIFFLAGS, &ifr);
-	if (err) {
-		bb_perror_msg("SIOCGIFFLAGS");
-		close(fd);
-		return -1;
-	}
-	if ((ifr.ifr_flags^flags)&mask) {
+	xioctl(fd, SIOCGIFFLAGS, &ifr);
+	if ((ifr.ifr_flags ^ flags) & mask) {
 		ifr.ifr_flags &= ~mask;
-		ifr.ifr_flags |= mask&flags;
-		err = ioctl(fd, SIOCSIFFLAGS, &ifr);
-		if (err)
-			bb_perror_msg("SIOCSIFFLAGS");
+		ifr.ifr_flags |= mask & flags;
+		xioctl(fd, SIOCSIFFLAGS, &ifr);
 	}
 	close(fd);
-	return err;
 }
 
-static int do_changename(char *dev, char *newdev)
+/* Exits on error */
+static void do_changename(char *dev, char *newdev)
 {
 	struct ifreq ifr;
 	int fd;
-	int err;
 
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	strncpy(ifr.ifr_newname, newdev, sizeof(ifr.ifr_newname));
 	fd = get_ctl_fd();
-	if (fd < 0)
-		return -1;
-	err = ioctl(fd, SIOCSIFNAME, &ifr);
-	if (err) {
-		bb_perror_msg("SIOCSIFNAME");
-		close(fd);
-		return -1;
-	}
+	xioctl(fd, SIOCSIFNAME, &ifr);
 	close(fd);
-	return err;
 }
 
-static int set_qlen(char *dev, int qlen)
+/* Exits on error */
+static void set_qlen(char *dev, int qlen)
 {
 	struct ifreq ifr;
 	int s;
 
 	s = get_ctl_fd();
-	if (s < 0)
-		return -1;
-
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	ifr.ifr_qlen = qlen;
-	if (ioctl(s, SIOCSIFTXQLEN, &ifr) < 0) {
-		bb_perror_msg("SIOCSIFXQLEN");
-		close(s);
-		return -1;
-	}
+	xioctl(s, SIOCSIFTXQLEN, &ifr);
 	close(s);
-
-	return 0;
 }
 
-static int set_mtu(char *dev, int mtu)
+/* Exits on error */
+static void set_mtu(char *dev, int mtu)
 {
 	struct ifreq ifr;
 	int s;
 
 	s = get_ctl_fd();
-	if (s < 0)
-		return -1;
-
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	ifr.ifr_mtu = mtu;
-	if (ioctl(s, SIOCSIFMTU, &ifr) < 0) {
-		bb_perror_msg("SIOCSIFMTU");
-		close(s);
-		return -1;
-	}
+	xioctl(s, SIOCSIFMTU, &ifr);
 	close(s);
-
-	return 0;
 }
 
+/* Exits on error */
 static int get_address(char *dev, int *htype)
 {
 	struct ifreq ifr;
@@ -150,42 +107,29 @@ static int get_address(char *dev, int *htype)
 	socklen_t alen;
 	int s;
 
-	s = socket(PF_PACKET, SOCK_DGRAM, 0);
-	if (s < 0) {
-		bb_perror_msg("socket(PF_PACKET)");
-		return -1;
-	}
+	s = xsocket(PF_PACKET, SOCK_DGRAM, 0);
 
 	memset(&ifr, 0, sizeof(ifr));
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
-	if (ioctl(s, SIOCGIFINDEX, &ifr) < 0) {
-		bb_perror_msg("SIOCGIFINDEX");
-		close(s);
-		return -1;
-	}
+	xioctl(s, SIOCGIFINDEX, &ifr);
 
 	memset(&me, 0, sizeof(me));
 	me.sll_family = AF_PACKET;
 	me.sll_ifindex = ifr.ifr_ifindex;
 	me.sll_protocol = htons(ETH_P_LOOP);
-	if (bind(s, (struct sockaddr*)&me, sizeof(me)) == -1) {
-		bb_perror_msg("bind");
-		close(s);
-		return -1;
-	}
+	xbind(s, (struct sockaddr*)&me, sizeof(me));
 
 	alen = sizeof(me);
 	if (getsockname(s, (struct sockaddr*)&me, &alen) == -1) {
-		bb_perror_msg("getsockname");
-		close(s);
-		return -1;
+		bb_perror_msg_and_die("getsockname");
 	}
 	close(s);
 	*htype = me.sll_hatype;
 	return me.sll_halen;
 }
 
-static int parse_address(char *dev, int hatype, int halen, char *lla, struct ifreq *ifr)
+/* Exits on error */
+static void parse_address(char *dev, int hatype, int halen, char *lla, struct ifreq *ifr)
 {
 	int alen;
 
@@ -194,31 +138,27 @@ static int parse_address(char *dev, int hatype, int halen, char *lla, struct ifr
 	ifr->ifr_hwaddr.sa_family = hatype;
 	alen = ll_addr_a2n((unsigned char *)(ifr->ifr_hwaddr.sa_data), 14, lla);
 	if (alen < 0)
-		return -1;
+		exit(1);
 	if (alen != halen) {
-		bb_error_msg("wrong address (%s) length: expected %d bytes", lla, halen);
-		return -1;
+		bb_error_msg_and_die("wrong address (%s) length: expected %d bytes", lla, halen);
 	}
-	return 0;
 }
 
-static int set_address(struct ifreq *ifr, int brd)
+/* Exits on error */
+static void set_address(struct ifreq *ifr, int brd)
 {
 	int s;
 
 	s = get_ctl_fd();
-	if (s < 0)
-		return -1;
-	if (ioctl(s, brd?SIOCSIFHWBROADCAST:SIOCSIFHWADDR, ifr) < 0) {
-		bb_perror_msg(brd ? "SIOCSIFHWBROADCAST" : "SIOCSIFHWADDR");
-		close(s);
-		return -1;
-	}
+	if (brd)
+		xioctl(s, SIOCSIFHWBROADCAST, ifr);
+	else
+		xioctl(s, SIOCSIFHWADDR, ifr);
 	close(s);
-	return 0;
 }
 
 
+/* Return value becomes exitcode. It's okay to not return at all */
 static int do_set(int argc, char **argv)
 {
 	char *dev = NULL;
@@ -231,46 +171,55 @@ static int do_set(int argc, char **argv)
 	struct ifreq ifr0, ifr1;
 	char *newname = NULL;
 	int htype, halen;
+	static const char keywords[] ALIGN1 =
+		"up\0""down\0""name\0""mtu\0""multicast\0""arp\0""addr\0""dev\0"
+		"on\0""off\0";
+	enum { ARG_up = 1, ARG_down, ARG_name, ARG_mtu, ARG_multicast, ARG_arp,
+		ARG_addr, ARG_dev, PARM_on, PARM_off };
+	smalluint key;
 
 	while (argc > 0) {
-		if (strcmp(*argv, "up") == 0) {
+		key = index_in_strings(keywords, *argv) + 1;
+		if (key == ARG_up) {
 			mask |= IFF_UP;
 			flags |= IFF_UP;
-		} else if (strcmp(*argv, "down") == 0) {
+		} else if (key == ARG_down) {
 			mask |= IFF_UP;
 			flags &= ~IFF_UP;
-		} else if (strcmp(*argv, "name") == 0) {
+		} else if (key == ARG_name) {
 			NEXT_ARG();
 			newname = *argv;
-		} else if (strcmp(*argv, "mtu") == 0) {
+		} else if (key == ARG_mtu) {
 			NEXT_ARG();
 			if (mtu != -1)
 				duparg("mtu", *argv);
 			if (get_integer(&mtu, *argv, 0))
 				invarg(*argv, "mtu");
-		} else if (strcmp(*argv, "multicast") == 0) {
+		} else if (key == ARG_multicast) {
 			NEXT_ARG();
 			mask |= IFF_MULTICAST;
-			if (strcmp(*argv, "on") == 0) {
+			key = index_in_strings(keywords, *argv) + 1;
+			if (key == PARM_on) {
 				flags |= IFF_MULTICAST;
-			} else if (strcmp(*argv, "off") == 0) {
+			} else if (key == PARM_off) {
 				flags &= ~IFF_MULTICAST;
 			} else
-				return on_off("multicast");
-		} else if (strcmp(*argv, "arp") == 0) {
+				on_off("multicast");
+		} else if (key == ARG_arp) {
 			NEXT_ARG();
 			mask |= IFF_NOARP;
-			if (strcmp(*argv, "on") == 0) {
+			key = index_in_strings(keywords, *argv) + 1;
+			if (key == PARM_on) {
 				flags &= ~IFF_NOARP;
-			} else if (strcmp(*argv, "off") == 0) {
+			} else if (key == PARM_off) {
 				flags |= IFF_NOARP;
 			} else
-				return on_off("noarp");
-		} else if (strcmp(*argv, "addr") == 0) {
+				on_off("arp");
+		} else if (key == ARG_addr) {
 			NEXT_ARG();
 			newaddr = *argv;
 		} else {
-			if (strcmp(*argv, "dev") == 0) {
+			if (key == ARG_dev) {
 				NEXT_ARG();
 			}
 			if (dev)
@@ -281,49 +230,39 @@ static int do_set(int argc, char **argv)
 	}
 
 	if (!dev) {
-		bb_error_msg(bb_msg_requires_arg, "\"dev\"");
-		exit(-1);
+		bb_error_msg_and_die(bb_msg_requires_arg, "\"dev\"");
 	}
 
 	if (newaddr || newbrd) {
 		halen = get_address(dev, &htype);
-		if (halen < 0)
-			return -1;
 		if (newaddr) {
-			if (parse_address(dev, htype, halen, newaddr, &ifr0) < 0)
-				return -1;
+			parse_address(dev, htype, halen, newaddr, &ifr0);
 		}
 		if (newbrd) {
-			if (parse_address(dev, htype, halen, newbrd, &ifr1) < 0)
-				return -1;
+			parse_address(dev, htype, halen, newbrd, &ifr1);
 		}
 	}
 
 	if (newname && strcmp(dev, newname)) {
-		if (do_changename(dev, newname) < 0)
-			return -1;
+		do_changename(dev, newname);
 		dev = newname;
 	}
 	if (qlen != -1) {
-		if (set_qlen(dev, qlen) < 0)
-			return -1;
+		set_qlen(dev, qlen);
 	}
 	if (mtu != -1) {
-		if (set_mtu(dev, mtu) < 0)
-			return -1;
+		set_mtu(dev, mtu);
 	}
 	if (newaddr || newbrd) {
 		if (newbrd) {
-			if (set_address(&ifr1, 1) < 0)
-				return -1;
+			set_address(&ifr1, 1);
 		}
 		if (newaddr) {
-			if (set_address(&ifr0, 0) < 0)
-				return -1;
+			set_address(&ifr0, 0);
 		}
 	}
 	if (mask)
-		return do_chflags(dev, flags, mask);
+		do_chflags(dev, flags, mask);
 	return 0;
 }
 
@@ -333,18 +272,20 @@ static int ipaddr_list_link(int argc, char **argv)
 	return ipaddr_list_or_flush(argc, argv, 0);
 }
 
+/* Return value becomes exitcode. It's okay to not return at all */
 int do_iplink(int argc, char **argv)
 {
-	if (argc > 0) {
-		if (matches(*argv, "set") == 0)
-			return do_set(argc-1, argv+1);
-		if (matches(*argv, "show") == 0 ||
-		    matches(*argv, "lst") == 0 ||
-		    matches(*argv, "list") == 0)
-			return ipaddr_list_link(argc-1, argv+1);
-	} else
+	static const char keywords[] ALIGN1 =
+		"set\0""show\0""lst\0""list\0";
+	smalluint key;
+	if (argc <= 0)
 		return ipaddr_list_link(0, NULL);
-
-	bb_error_msg("command \"%s\" is unknown", *argv);
-	exit(-1);
+	key = index_in_substrings(keywords, *argv) + 1;
+	if (key == 0)
+		bb_error_msg_and_die(bb_msg_invalid_arg, *argv, applet_name);
+	argc--; argv++;
+	if (key == 1) /* set */
+		return do_set(argc, argv);
+	else /* show, lst, list */
+		return ipaddr_list_link(argc, argv);
 }
