@@ -14,29 +14,21 @@
  * Phil Karn <karn@ka9q.ampr.org>	990408:	"pmtudisc" flag
  */
 
-#include "libbb.h"
-#include <sys/socket.h>
-#include <sys/ioctl.h>
-
-#include <string.h>
-#include <unistd.h>
-
 #include <netinet/ip.h>
-
 #include <net/if.h>
 #include <net/if_arp.h>
-
 #include <asm/types.h>
 #ifndef __constant_htons
 #define __constant_htons htons
 #endif
 #include <linux/if_tunnel.h>
 
+#include "ip_common.h"  /* #include "libbb.h" is inside */
 #include "rt_names.h"
 #include "utils.h"
-#include "ip_common.h"
 
 
+/* Dies on error */
 static int do_ioctl_get_ifindex(char *dev)
 {
 	struct ifreq ifr;
@@ -44,10 +36,7 @@ static int do_ioctl_get_ifindex(char *dev)
 
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	if (ioctl(fd, SIOCGIFINDEX, &ifr)) {
-		bb_perror_msg("ioctl");
-		return 0;
-	}
+	xioctl(fd, SIOCGIFINDEX, &ifr);
 	close(fd);
 	return ifr.ifr_ifindex;
 }
@@ -56,36 +45,29 @@ static int do_ioctl_get_iftype(char *dev)
 {
 	struct ifreq ifr;
 	int fd;
+	int err;
 
 	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	if (ioctl(fd, SIOCGIFHWADDR, &ifr)) {
-		bb_perror_msg("ioctl");
-		return -1;
-	}
+	err = ioctl_or_warn(fd, SIOCGIFHWADDR, &ifr);
 	close(fd);
-	return ifr.ifr_addr.sa_family;
+	return err ? -1 : ifr.ifr_addr.sa_family;
 }
 
-
 static char *do_ioctl_get_ifname(int idx)
 {
-	static struct ifreq ifr;
+	struct ifreq ifr;
 	int fd;
+	int err;
 
 	ifr.ifr_ifindex = idx;
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	if (ioctl(fd, SIOCGIFNAME, &ifr)) {
-		bb_perror_msg("ioctl");
-		return NULL;
-	}
+	err = ioctl_or_warn(fd, SIOCGIFNAME, &ifr);
 	close(fd);
-	return ifr.ifr_name;
+	return err ? NULL : xstrndup(ifr.ifr_name, sizeof(ifr.ifr_name));
 }
 
-
-
-static int do_get_ioctl(char *basedev, struct ip_tunnel_parm *p)
+static int do_get_ioctl(const char *basedev, struct ip_tunnel_parm *p)
 {
 	struct ifreq ifr;
 	int fd;
@@ -94,19 +76,16 @@ static int do_get_ioctl(char *basedev, struct ip_tunnel_parm *p)
 	strncpy(ifr.ifr_name, basedev, sizeof(ifr.ifr_name));
 	ifr.ifr_ifru.ifru_data = (void*)p;
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	err = ioctl(fd, SIOCGETTUNNEL, &ifr);
-	if (err) {
-		bb_perror_msg("ioctl");
-	}
+	err = ioctl_or_warn(fd, SIOCGETTUNNEL, &ifr);
 	close(fd);
 	return err;
 }
 
-static int do_add_ioctl(int cmd, char *basedev, struct ip_tunnel_parm *p)
+/* Dies on error, otherwise returns 0 */
+static int do_add_ioctl(int cmd, const char *basedev, struct ip_tunnel_parm *p)
 {
 	struct ifreq ifr;
 	int fd;
-	int err;
 
 	if (cmd == SIOCCHGTUNNEL && p->name[0]) {
 		strncpy(ifr.ifr_name, p->name, sizeof(ifr.ifr_name));
@@ -115,19 +94,24 @@ static int do_add_ioctl(int cmd, char *basedev, struct ip_tunnel_parm *p)
 	}
 	ifr.ifr_ifru.ifru_data = (void*)p;
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	err = ioctl(fd, cmd, &ifr);
-	if (err) {
-		bb_perror_msg("ioctl");
-	}
+#if ENABLE_IOCTL_HEX2STR_ERROR
+	/* #define magic will turn ioctl# into string */
+	if (cmd == SIOCCHGTUNNEL)
+		xioctl(fd, SIOCCHGTUNNEL, &ifr);
+	else
+		xioctl(fd, SIOCADDTUNNEL, &ifr);
+#else
+	xioctl(fd, cmd, &ifr);
+#endif
 	close(fd);
-	return err;
+	return 0;
 }
 
-static int do_del_ioctl(char *basedev, struct ip_tunnel_parm *p)
+/* Dies on error, otherwise returns 0 */
+static int do_del_ioctl(const char *basedev, struct ip_tunnel_parm *p)
 {
 	struct ifreq ifr;
 	int fd;
-	int err;
 
 	if (p->name[0]) {
 		strncpy(ifr.ifr_name, p->name, sizeof(ifr.ifr_name));
@@ -136,57 +120,70 @@ static int do_del_ioctl(char *basedev, struct ip_tunnel_parm *p)
 	}
 	ifr.ifr_ifru.ifru_data = (void*)p;
 	fd = xsocket(AF_INET, SOCK_DGRAM, 0);
-	err = ioctl(fd, SIOCDELTUNNEL, &ifr);
-	if (err) {
-		bb_perror_msg("ioctl");
-	}
+	xioctl(fd, SIOCDELTUNNEL, &ifr);
 	close(fd);
-	return err;
+	return 0;
 }
 
-static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
+/* Dies on error */
+static void parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 {
+	static const char keywords[] ALIGN1 =
+		"mode\0""ipip\0""ip/ip\0""gre\0""gre/ip\0""sit\0""ipv6/ip\0"
+		"key\0""ikey\0""okey\0""seq\0""iseq\0""oseq\0"
+		"csum\0""icsum\0""ocsum\0""nopmtudisc\0""pmtudisc\0"
+		"remote\0""any\0""local\0""dev\0"
+		"ttl\0""inherit\0""tos\0""dsfield\0"
+		"name\0";
+	enum {
+		ARG_mode, ARG_ipip, ARG_ip_ip, ARG_gre, ARG_gre_ip, ARG_sit, ARG_ip6_ip,
+		ARG_key, ARG_ikey, ARG_okey, ARG_seq, ARG_iseq, ARG_oseq,
+		ARG_csum, ARG_icsum, ARG_ocsum, ARG_nopmtudisc, ARG_pmtudisc,
+		ARG_remote, ARG_any, ARG_local, ARG_dev,
+		ARG_ttl, ARG_inherit, ARG_tos, ARG_dsfield,
+		ARG_name
+	};
 	int count = 0;
 	char medium[IFNAMSIZ];
+	int key;
+
 	memset(p, 0, sizeof(*p));
 	memset(&medium, 0, sizeof(medium));
 
 	p->iph.version = 4;
 	p->iph.ihl = 5;
 #ifndef IP_DF
-#define IP_DF		0x4000		/* Flag: "Don't Fragment"	*/
+#define IP_DF 0x4000  /* Flag: "Don't Fragment" */
 #endif
 	p->iph.frag_off = htons(IP_DF);
 
 	while (argc > 0) {
-		if (strcmp(*argv, "mode") == 0) {
+		key = index_in_strings(keywords, *argv);
+		if (key == ARG_mode) {
 			NEXT_ARG();
-			if (strcmp(*argv, "ipip") == 0 ||
-			    strcmp(*argv, "ip/ip") == 0) {
+			key = index_in_strings(keywords, *argv);
+			if (key == ARG_ipip ||
+			    key == ARG_ip_ip) {
 				if (p->iph.protocol && p->iph.protocol != IPPROTO_IPIP) {
-					bb_error_msg("you managed to ask for more than one tunnel mode");
-					exit(-1);
+					bb_error_msg_and_die("you managed to ask for more than one tunnel mode");
 				}
 				p->iph.protocol = IPPROTO_IPIP;
-			} else if (strcmp(*argv, "gre") == 0 ||
-				   strcmp(*argv, "gre/ip") == 0) {
+			} else if (key == ARG_gre ||
+				   key == ARG_gre_ip) {
 				if (p->iph.protocol && p->iph.protocol != IPPROTO_GRE) {
-					bb_error_msg("you managed to ask for more than one tunnel mode");
-					exit(-1);
+					bb_error_msg_and_die("you managed to ask for more than one tunnel mode");
 				}
 				p->iph.protocol = IPPROTO_GRE;
-			} else if (strcmp(*argv, "sit") == 0 ||
-				   strcmp(*argv, "ipv6/ip") == 0) {
+			} else if (key == ARG_sit ||
+				   key == ARG_ip6_ip) {
 				if (p->iph.protocol && p->iph.protocol != IPPROTO_IPV6) {
-					bb_error_msg("you managed to ask for more than one tunnel mode");
-					exit(-1);
+					bb_error_msg_and_die("you managed to ask for more than one tunnel mode");
 				}
 				p->iph.protocol = IPPROTO_IPV6;
 			} else {
-				bb_error_msg("cannot guess tunnel mode");
-				exit(-1);
+				bb_error_msg_and_die("cannot guess tunnel mode");
 			}
-		} else if (strcmp(*argv, "key") == 0) {
+		} else if (key == ARG_key) {
 			unsigned uval;
 			NEXT_ARG();
 			p->i_flags |= GRE_KEY;
@@ -195,12 +192,11 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 				p->i_key = p->o_key = get_addr32(*argv);
 			else {
 				if (get_unsigned(&uval, *argv, 0)<0) {
-					bb_error_msg("invalid value of \"key\"");
-					exit(-1);
+					bb_error_msg_and_die("invalid value of \"key\"");
 				}
 				p->i_key = p->o_key = htonl(uval);
 			}
-		} else if (strcmp(*argv, "ikey") == 0) {
+		} else if (key == ARG_ikey) {
 			unsigned uval;
 			NEXT_ARG();
 			p->i_flags |= GRE_KEY;
@@ -208,12 +204,11 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 				p->o_key = get_addr32(*argv);
 			else {
 				if (get_unsigned(&uval, *argv, 0)<0) {
-					bb_error_msg("invalid value of \"ikey\"");
-					exit(-1);
+					bb_error_msg_and_die("invalid value of \"ikey\"");
 				}
 				p->i_key = htonl(uval);
 			}
-		} else if (strcmp(*argv, "okey") == 0) {
+		} else if (key == ARG_okey) {
 			unsigned uval;
 			NEXT_ARG();
 			p->o_flags |= GRE_KEY;
@@ -221,62 +216,65 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 				p->o_key = get_addr32(*argv);
 			else {
 				if (get_unsigned(&uval, *argv, 0)<0) {
-					bb_error_msg("invalid value of \"okey\"");
-					exit(-1);
+					bb_error_msg_and_die("invalid value of \"okey\"");
 				}
 				p->o_key = htonl(uval);
 			}
-		} else if (strcmp(*argv, "seq") == 0) {
+		} else if (key == ARG_seq) {
 			p->i_flags |= GRE_SEQ;
 			p->o_flags |= GRE_SEQ;
-		} else if (strcmp(*argv, "iseq") == 0) {
+		} else if (key == ARG_iseq) {
 			p->i_flags |= GRE_SEQ;
-		} else if (strcmp(*argv, "oseq") == 0) {
+		} else if (key == ARG_oseq) {
 			p->o_flags |= GRE_SEQ;
-		} else if (strcmp(*argv, "csum") == 0) {
+		} else if (key == ARG_csum) {
 			p->i_flags |= GRE_CSUM;
 			p->o_flags |= GRE_CSUM;
-		} else if (strcmp(*argv, "icsum") == 0) {
+		} else if (key == ARG_icsum) {
 			p->i_flags |= GRE_CSUM;
-		} else if (strcmp(*argv, "ocsum") == 0) {
+		} else if (key == ARG_ocsum) {
 			p->o_flags |= GRE_CSUM;
-		} else if (strcmp(*argv, "nopmtudisc") == 0) {
+		} else if (key == ARG_nopmtudisc) {
 			p->iph.frag_off = 0;
-		} else if (strcmp(*argv, "pmtudisc") == 0) {
+		} else if (key == ARG_pmtudisc) {
 			p->iph.frag_off = htons(IP_DF);
-		} else if (strcmp(*argv, "remote") == 0) {
+		} else if (key == ARG_remote) {
 			NEXT_ARG();
-			if (strcmp(*argv, "any"))
+			key = index_in_strings(keywords, *argv);
+			if (key == ARG_any)
 				p->iph.daddr = get_addr32(*argv);
-		} else if (strcmp(*argv, "local") == 0) {
+		} else if (key == ARG_local) {
 			NEXT_ARG();
-			if (strcmp(*argv, "any"))
+			key = index_in_strings(keywords, *argv);
+			if (key == ARG_any)
 				p->iph.saddr = get_addr32(*argv);
-		} else if (strcmp(*argv, "dev") == 0) {
+		} else if (key == ARG_dev) {
 			NEXT_ARG();
 			strncpy(medium, *argv, IFNAMSIZ-1);
-		} else if (strcmp(*argv, "ttl") == 0) {
+		} else if (key == ARG_ttl) {
 			unsigned uval;
 			NEXT_ARG();
-			if (strcmp(*argv, "inherit") != 0) {
+			key = index_in_strings(keywords, *argv);
+			if (key != ARG_inherit) {
 				if (get_unsigned(&uval, *argv, 0))
 					invarg(*argv, "TTL");
 				if (uval > 255)
 					invarg(*argv, "TTL must be <=255");
 				p->iph.ttl = uval;
 			}
-		} else if (strcmp(*argv, "tos") == 0 ||
-			   matches(*argv, "dsfield") == 0) {
+		} else if (key == ARG_tos ||
+			   key == ARG_dsfield) {
 			uint32_t uval;
 			NEXT_ARG();
-			if (strcmp(*argv, "inherit") != 0) {
+			key = index_in_strings(keywords, *argv);
+			if (key != ARG_inherit) {
 				if (rtnl_dsfield_a2n(&uval, *argv))
 					invarg(*argv, "TOS");
 				p->iph.tos = uval;
 			} else
 				p->iph.tos = 1;
 		} else {
-			if (strcmp(*argv, "name") == 0) {
+			if (key == ARG_name) {
 				NEXT_ARG();
 			}
 			if (p->name[0])
@@ -286,15 +284,15 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 				struct ip_tunnel_parm old_p;
 				memset(&old_p, 0, sizeof(old_p));
 				if (do_get_ioctl(*argv, &old_p))
-					return -1;
+					exit(1);
 				*p = old_p;
 			}
 		}
 		count++;
-		argc--; argv++;
+		argc--;
+		argv++;
 	}
 
-
 	if (p->iph.protocol == 0) {
 		if (memcmp(p->name, "gre", 3) == 0)
 			p->iph.protocol = IPPROTO_GRE;
@@ -306,15 +304,12 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 
 	if (p->iph.protocol == IPPROTO_IPIP || p->iph.protocol == IPPROTO_IPV6) {
 		if ((p->i_flags & GRE_KEY) || (p->o_flags & GRE_KEY)) {
-			bb_error_msg("keys are not allowed with ipip and sit");
-			return -1;
+			bb_error_msg_and_die("keys are not allowed with ipip and sit");
 		}
 	}
 
 	if (medium[0]) {
 		p->link = do_ioctl_get_ifindex(medium);
-		if (p->link == 0)
-			return -1;
 	}
 
 	if (p->i_key == 0 && IN_MULTICAST(ntohl(p->iph.daddr))) {
@@ -326,23 +321,20 @@ static int parse_args(int argc, char **argv, int cmd, struct ip_tunnel_parm *p)
 		p->o_flags |= GRE_KEY;
 	}
 	if (IN_MULTICAST(ntohl(p->iph.daddr)) && !p->iph.saddr) {
-		bb_error_msg("broadcast tunnel requires a source address");
-		return -1;
+		bb_error_msg_and_die("broadcast tunnel requires a source address");
 	}
-	return 0;
 }
 
 
+/* Return value becomes exitcode. It's okay to not return at all */
 static int do_add(int cmd, int argc, char **argv)
 {
 	struct ip_tunnel_parm p;
 
-	if (parse_args(argc, argv, cmd, &p) < 0)
-		return -1;
+	parse_args(argc, argv, cmd, &p);
 
 	if (p.iph.ttl && p.iph.frag_off == 0) {
-		bb_error_msg("ttl != 0 and noptmudisc are incompatible");
-		return -1;
+		bb_error_msg_and_die("ttl != 0 and noptmudisc are incompatible");
 	}
 
 	switch (p.iph.protocol) {
@@ -353,18 +345,16 @@ static int do_add(int cmd, int argc, char **argv)
 	case IPPROTO_IPV6:
 		return do_add_ioctl(cmd, "sit0", &p);
 	default:
-		bb_error_msg("cannot determine tunnel mode (ipip, gre or sit)");
-		return -1;
+		bb_error_msg_and_die("cannot determine tunnel mode (ipip, gre or sit)");
 	}
-	return -1;
 }
 
+/* Return value becomes exitcode. It's okay to not return at all */
 static int do_del(int argc, char **argv)
 {
 	struct ip_tunnel_parm p;
 
-	if (parse_args(argc, argv, SIOCDELTUNNEL, &p) < 0)
-		return -1;
+	parse_args(argc, argv, SIOCDELTUNNEL, &p);
 
 	switch (p.iph.protocol) {
 	case IPPROTO_IPIP:
@@ -376,7 +366,6 @@ static int do_del(int argc, char **argv)
 	default:
 		return do_del_ioctl(p.name, &p);
 	}
-	return -1;
 }
 
 static void print_tunnel(struct ip_tunnel_parm *p)
@@ -399,8 +388,10 @@ static void print_tunnel(struct ip_tunnel_parm *p)
 	       p->iph.daddr ? s1 : "any", p->iph.saddr ? s2 : "any");
 	if (p->link) {
 		char *n = do_ioctl_get_ifname(p->link);
-		if (n)
+		if (n) {
 			printf(" dev %s ", n);
+			free(n);
+		}
 	}
 	if (p->iph.ttl)
 		printf(" ttl %d ", p->iph.ttl);
@@ -428,16 +419,16 @@ static void print_tunnel(struct ip_tunnel_parm *p)
 	}
 
 	if (p->i_flags & GRE_SEQ)
-		printf("%s  Drop packets out of sequence.\n", _SL_);
+		printf("%c  Drop packets out of sequence.\n", _SL_);
 	if (p->i_flags & GRE_CSUM)
-		printf("%s  Checksum in received packet is required.", _SL_);
+		printf("%c  Checksum in received packet is required.", _SL_);
 	if (p->o_flags & GRE_SEQ)
-		printf("%s  Sequence packets on output.", _SL_);
+		printf("%c  Sequence packets on output.", _SL_);
 	if (p->o_flags & GRE_CSUM)
-		printf("%s  Checksum output packets.", _SL_);
+		printf("%c  Checksum output packets.", _SL_);
 }
 
-static int do_tunnels_list(struct ip_tunnel_parm *p)
+static void do_tunnels_list(struct ip_tunnel_parm *p)
 {
 	char name[IFNAMSIZ];
 	unsigned long rx_bytes, rx_packets, rx_errs, rx_drops,
@@ -447,11 +438,10 @@ static int do_tunnels_list(struct ip_tunnel_parm *p)
 	int type;
 	struct ip_tunnel_parm p1;
 	char buf[512];
-	FILE *fp = fopen("/proc/net/dev", "r");
+	FILE *fp = fopen_or_warn("/proc/net/dev", "r");
 
 	if (fp == NULL) {
-		perror("fopen");
-		return -1;
+		return;
 	}
 
 	fgets(buf, sizeof(buf), fp);
@@ -464,8 +454,8 @@ static int do_tunnels_list(struct ip_tunnel_parm *p)
 		ptr = strchr(buf, ':');
 		if (ptr == NULL ||
 		    (*ptr++ = 0, sscanf(buf, "%s", name) != 1)) {
-			bb_error_msg("wrong format of /proc/net/dev. Sorry");
-			return -1;
+			bb_error_msg("wrong format of /proc/net/dev");
+			return;
 		}
 		if (sscanf(ptr, "%lu%lu%lu%lu%lu%lu%lu%*d%lu%lu%lu%lu%lu%lu%lu",
 			   &rx_bytes, &rx_packets, &rx_errs, &rx_drops,
@@ -477,7 +467,7 @@ static int do_tunnels_list(struct ip_tunnel_parm *p)
 			continue;
 		type = do_ioctl_get_iftype(name);
 		if (type == -1) {
-			bb_error_msg("failed to get type of [%s]", name);
+			bb_error_msg("cannot get type of [%s]", name);
 			continue;
 		}
 		if (type != ARPHRD_TUNNEL && type != ARPHRD_IPGRE && type != ARPHRD_SIT)
@@ -494,16 +484,15 @@ static int do_tunnels_list(struct ip_tunnel_parm *p)
 		print_tunnel(&p1);
 		puts("");
 	}
-	return 0;
 }
 
+/* Return value becomes exitcode. It's okay to not return at all */
 static int do_show(int argc, char **argv)
 {
 	int err;
 	struct ip_tunnel_parm p;
 
-	if (parse_args(argc, argv, SIOCGETTUNNEL, &p) < 0)
-		return -1;
+	parse_args(argc, argv, SIOCGETTUNNEL, &p);
 
 	switch (p.iph.protocol) {
 	case IPPROTO_IPIP:
@@ -527,21 +516,26 @@ static int do_show(int argc, char **argv)
 	return 0;
 }
 
+/* Return value becomes exitcode. It's okay to not return at all */
 int do_iptunnel(int argc, char **argv)
 {
-	if (argc > 0) {
-		if (matches(*argv, "add") == 0)
-			return do_add(SIOCADDTUNNEL, argc-1, argv+1);
-		if (matches(*argv, "change") == 0)
-			return do_add(SIOCCHGTUNNEL, argc-1, argv+1);
-		if (matches(*argv, "del") == 0)
-			return do_del(argc-1, argv+1);
-		if (matches(*argv, "show") == 0 ||
-		    matches(*argv, "lst") == 0 ||
-		    matches(*argv, "list") == 0)
-			return do_show(argc-1, argv+1);
-	} else
-		return do_show(0, NULL);
+	static const char keywords[] ALIGN1 =
+		"add\0""change\0""delete\0""show\0""list\0""lst\0";
+	enum { ARG_add = 0, ARG_change, ARG_del, ARG_show, ARG_list, ARG_lst };
+	int key;
 
-	bb_error_msg_and_die("command \"%s\" is unknown", *argv);
+	if (argc) {
+		key = index_in_substrings(keywords, *argv);
+		if (key < 0)
+			bb_error_msg_and_die(bb_msg_invalid_arg, *argv, applet_name);
+		--argc;
+		++argv;
+		if (key == ARG_add)
+			return do_add(SIOCADDTUNNEL, argc, argv);
+		if (key == ARG_change)
+			return do_add(SIOCCHGTUNNEL, argc, argv);
+		if (key == ARG_del)
+			return do_del(argc, argv);
+	}
+	return do_show(argc, argv);
 }
