@@ -11,28 +11,34 @@
  *
  */
 
-#include "libbb.h"
-#include <string.h>
+#include <net/if.h>	/* struct ifreq and co. */
 
+#include "libbb.h"
 #include "libnetlink.h"
 #include "ll_map.h"
 
-#include <sys/socket.h>	/* socket() */
-#include <net/if.h>	/* struct ifreq and co. */
-#include <sys/ioctl.h>	/* ioctl() & SIOCGIFINDEX */
-
 struct idxmap {
-	struct idxmap * next;
-	int		index;
-	int		type;
-	int		alen;
-	unsigned	flags;
-	unsigned char	addr[8];
-	char		name[16];
+	struct idxmap *next;
+	int            index;
+	int            type;
+	int            alen;
+	unsigned       flags;
+	unsigned char  addr[8];
+	char           name[16];
 };
 
 static struct idxmap *idxmap[16];
 
+static struct idxmap *find_by_index(int idx)
+{
+	struct idxmap *im;
+
+	for (im = idxmap[idx & 0xF]; im; im = im->next)
+		if (im->index == idx)
+			return im;
+	return NULL;
+}
+
 int ll_remember_index(struct sockaddr_nl *who, struct nlmsghdr *n, void *arg)
 {
 	int h;
@@ -46,25 +52,22 @@ int ll_remember_index(struct sockaddr_nl *who, struct nlmsghdr *n, void *arg)
 	if (n->nlmsg_len < NLMSG_LENGTH(sizeof(ifi)))
 		return -1;
 
-
 	memset(tb, 0, sizeof(tb));
 	parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), IFLA_PAYLOAD(n));
 	if (tb[IFLA_IFNAME] == NULL)
 		return 0;
 
-	h = ifi->ifi_index&0xF;
+	h = ifi->ifi_index & 0xF;
 
-	for (imp=&idxmap[h]; (im=*imp)!=NULL; imp = &im->next)
+	for (imp = &idxmap[h]; (im = *imp) != NULL; imp = &im->next)
 		if (im->index == ifi->ifi_index)
-			break;
-
-	if (im == NULL) {
-		im = xmalloc(sizeof(*im));
-		im->next = *imp;
-		im->index = ifi->ifi_index;
-		*imp = im;
-	}
+			goto found;
 
+	im = xmalloc(sizeof(*im));
+	im->next = *imp;
+	im->index = ifi->ifi_index;
+	*imp = im;
+ found:
 	im->type = ifi->ifi_type;
 	im->flags = ifi->ifi_flags;
 	if (tb[IFLA_ADDRESS]) {
@@ -87,9 +90,9 @@ const char *ll_idx_n2a(int idx, char *buf)
 
 	if (idx == 0)
 		return "*";
-	for (im = idxmap[idx&0xF]; im; im = im->next)
-		if (im->index == idx)
-			return im->name;
+	im = find_by_index(idx);
+	if (im)
+		return im->name;
 	snprintf(buf, 16, "if%d", idx);
 	return buf;
 }
@@ -102,17 +105,19 @@ const char *ll_index_to_name(int idx)
 	return ll_idx_n2a(idx, nbuf);
 }
 
+#ifdef UNUSED
 int ll_index_to_type(int idx)
 {
 	struct idxmap *im;
 
 	if (idx == 0)
 		return -1;
-	for (im = idxmap[idx&0xF]; im; im = im->next)
-		if (im->index == idx)
-			return im->type;
+	im = find_by_index(idx);
+	if (im)
+		return im->type;
 	return -1;
 }
+#endif
 
 unsigned ll_index_to_flags(int idx)
 {
@@ -120,31 +125,38 @@ unsigned ll_index_to_flags(int idx)
 
 	if (idx == 0)
 		return 0;
-
-	for (im = idxmap[idx&0xF]; im; im = im->next)
-		if (im->index == idx)
-			return im->flags;
+	im = find_by_index(idx);
+	if (im)
+		return im->flags;
 	return 0;
 }
 
-int ll_name_to_index(char *name)
+int xll_name_to_index(const char *const name)
 {
+	int ret = 0;
+	int sock_fd;
+
+/* caching is not warranted - no users which repeatedly call it */
+#ifdef UNUSED
 	static char ncache[16];
 	static int icache;
+
 	struct idxmap *im;
-	int sock_fd;
 	int i;
 
 	if (name == NULL)
-		return 0;
-	if (icache && strcmp(name, ncache) == 0)
-		return icache;
-	for (i=0; i<16; i++) {
+		goto out;
+	if (icache && strcmp(name, ncache) == 0) {
+		ret = icache;
+		goto out;
+	}
+	for (i = 0; i < 16; i++) {
 		for (im = idxmap[i]; im; im = im->next) {
 			if (strcmp(im->name, name) == 0) {
 				icache = im->index;
 				strcpy(ncache, name);
-				return im->index;
+				ret = im->index;
+				goto out;
 			}
 		}
 	}
@@ -155,32 +167,32 @@ int ll_name_to_index(char *name)
 	 * we explicitely request it (check for dev_load() in net/core/dev.c).
 	 * I can think of other similar scenario, but they are less common...
 	 * Jean II */
+#endif
+
 	sock_fd = socket(AF_INET, SOCK_DGRAM, 0);
 	if (sock_fd) {
 		struct ifreq ifr;
-		int ret;
+		int tmp;
+
 		strncpy(ifr.ifr_name, name, IFNAMSIZ);
 		ifr.ifr_ifindex = -1;
-		ret = ioctl(sock_fd, SIOCGIFINDEX, &ifr);
+		tmp = ioctl(sock_fd, SIOCGIFINDEX, &ifr);
 		close(sock_fd);
-		if (ret >= 0)
+		if (tmp >= 0)
 			/* In theory, we should redump the interface list
 			 * to update our cache, this is left as an exercise
 			 * to the reader... Jean II */
-			return ifr.ifr_ifindex;
+			ret = ifr.ifr_ifindex;
 	}
-
-	return 0;
+/* out:*/
+	if (ret <= 0)
+		bb_error_msg_and_die("cannot find device \"%s\"", name);
+	return ret;
 }
 
 int ll_init_map(struct rtnl_handle *rth)
 {
-	if (rtnl_wilddump_request(rth, AF_UNSPEC, RTM_GETLINK) < 0) {
-		bb_perror_msg_and_die("cannot send dump request");
-	}
-
-	if (rtnl_dump_filter(rth, ll_remember_index, &idxmap, NULL, NULL) < 0) {
-		bb_error_msg_and_die("dump terminated");
-	}
+	xrtnl_wilddump_request(rth, AF_UNSPEC, RTM_GETLINK);
+	xrtnl_dump_filter(rth, ll_remember_index, &idxmap);
 	return 0;
 }
