@@ -11,7 +11,6 @@
  */
 
 #include "libbb.h"
-
 #include "rt_names.h"
 #include "utils.h"
 
@@ -24,7 +23,7 @@
 #define __PF(f,n) { ETH_P_##f, #n },
 static struct {
 	int id;
-	char *name;
+	const char *name;
 } llproto_names[] = {
 __PF(LOOP,loop)
 __PF(PUP,pup)
@@ -97,7 +96,7 @@ const char * ll_proto_n2a(unsigned short id, char *buf, int len)
 
 	id = ntohs(id);
 
-	for (i=0; i<sizeof(llproto_names)/sizeof(llproto_names[0]); i++) {
+	for (i=0; i < ARRAY_SIZE(llproto_names); i++) {
 		 if (llproto_names[i].id == id)
 			return llproto_names[i].name;
 	}
@@ -108,7 +107,7 @@ const char * ll_proto_n2a(unsigned short id, char *buf, int len)
 int ll_proto_a2n(unsigned short *id, char *buf)
 {
 	int i;
-	for (i=0; i<sizeof(llproto_names)/sizeof(llproto_names[0]); i++) {
+	for (i=0; i < ARRAY_SIZE(llproto_names); i++) {
 		 if (strcasecmp(llproto_names[i].name, buf) == 0) {
 			 *id = htons(llproto_names[i].id);
 			 return 0;
