@@ -9,7 +9,7 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 #include <syslog.h>
 #include <net/if.h>
 #include <netinet/ether.h>
@@ -55,6 +55,7 @@ static struct ether_addr *cc_macaddr(const char *str_macaddr)
 	return mac;
 }
 
+int nameif_main(int argc, char **argv);
 int nameif_main(int argc, char **argv)
 {
 	mactable_t *clist = NULL;
@@ -65,7 +66,7 @@ int nameif_main(int argc, char **argv)
 	int if_index = 1;
 	mactable_t *ch;
 
-	if (1 & getopt32(argc, argv, "sc:", &fname)) {
+	if (1 & getopt32(argv, "sc:", &fname)) {
 		openlog(applet_name, 0, LOG_LOCAL0);
 		logmode = LOGMODE_SYSLOG;
 	}
@@ -147,9 +148,9 @@ int nameif_main(int argc, char **argv)
 			continue;
 
 		strcpy(ifr.ifr_newname, ch->ifname);
-		if (ioctl(ctl_sk, SIOCSIFNAME, &ifr) < 0)
-			bb_perror_msg_and_die("cannot change ifname %s to %s",
-				   ifr.ifr_name, ch->ifname);
+		ioctl_or_perror_and_die(ctl_sk, SIOCSIFNAME, &ifr,
+					"cannot change ifname %s to %s",
+					ifr.ifr_name, ch->ifname);
 
 		/* Remove list entry of renamed interface */
 		if (ch->prev != NULL) {
