@@ -24,19 +24,18 @@
 #include <termios.h>
 #include <arpa/telnet.h>
 #include <netinet/in.h>
-#include "busybox.h"
+#include "libbb.h"
 
 #ifdef DOTRACE
-#include <arpa/inet.h> /* for inet_ntoa()... */
 #define TRACE(x, y) do { if (x) printf y; } while (0)
 #else
 #define TRACE(x, y)
 #endif
 
-#define DATABUFSIZE  128
-#define IACBUFSIZE   128
-
 enum {
+	DATABUFSIZE = 128,
+	IACBUFSIZE  = 128,
+
 	CHM_TRY = 0,
 	CHM_ON = 1,
 	CHM_OFF = 2,
@@ -51,40 +50,40 @@ enum {
 	TS_SUB2 = 5,
 };
 
-#define WriteCS(fd, str) write(fd, str, sizeof str -1)
-
 typedef unsigned char byte;
 
-/* use globals to reduce size ??? */ /* test this hypothesis later */
-static struct Globalvars {
-	int		netfd; /* console fd:s are 0 and 1 (and 2) */
-    /* same buffer used both for network and console read/write */
-	char    buf[DATABUFSIZE]; /* allocating so static size is smaller */
+struct globals {
+	int	netfd; /* console fd:s are 0 and 1 (and 2) */
+	short	iaclen; /* could even use byte */
 	byte	telstate; /* telnet negotiation state from network input */
 	byte	telwish;  /* DO, DONT, WILL, WONT */
 	byte    charmode;
 	byte    telflags;
 	byte	gotsig;
 	byte	do_termios;
+#if ENABLE_FEATURE_TELNET_TTYPE
+	char	*ttype;
+#endif
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
+	const char *autologin;
+#endif
+#if ENABLE_FEATURE_AUTOWIDTH
+	int	win_width, win_height;
+#endif
+	/* same buffer used both for network and console read/write */
+	char    buf[DATABUFSIZE];
 	/* buffer to handle telnet negotiations */
 	char    iacbuf[IACBUFSIZE];
-	short	iaclen; /* could even use byte */
 	struct termios termios_def;
 	struct termios termios_raw;
-} G;
-
-#define xUSE_GLOBALVAR_PTR /* xUSE... -> don't use :D (makes smaller code) */
-
-#ifdef USE_GLOBALVAR_PTR
-struct Globalvars * Gptr;
-#define G (*Gptr)
-#endif
-
-static void iacflush(void)
-{
-	write(G.netfd, G.iacbuf, G.iaclen);
-	G.iaclen = 0;
-}
+};
+#define G (*(struct globals*)&bb_common_bufsiz1)
+void BUG_telnet_globals_too_big(void);
+#define INIT_G() do { \
+	if (sizeof(G) > COMMON_BUFSIZE) \
+		BUG_telnet_globals_too_big(); \
+	/* memset(&G, 0, sizeof G); - already is */ \
+} while (0)
 
 /* Function prototypes */
 static void rawmode(void);
@@ -94,20 +93,13 @@ static void will_charmode(void);
 static void telopt(byte c);
 static int subneg(byte c);
 
-/* Some globals */
-static const int one = 1;
+static void iacflush(void)
+{
+	write(G.netfd, G.iacbuf, G.iaclen);
+	G.iaclen = 0;
+}
 
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
-static char *ttype;
-#endif
-
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
-static const char *autologin;
-#endif
-
-#ifdef CONFIG_FEATURE_AUTOWIDTH
-static int win_width, win_height;
-#endif
+#define write_str(fd, str) write(fd, str, sizeof(str) - 1)
 
 static void doexit(int ev)
 {
@@ -122,7 +114,7 @@ static void conescape(void)
 	if (G.gotsig)	/* came from line  mode... go raw */
 		rawmode();
 
-	WriteCS(1, "\r\nConsole escape. Commands are:\r\n\n"
+	write_str(1, "\r\nConsole escape. Commands are:\r\n\n"
 			" l	go to line mode\r\n"
 			" c	go to character mode\r\n"
 			" z	suspend telnet\r\n"
@@ -131,18 +123,15 @@ static void conescape(void)
 	if (read(0, &b, 1) <= 0)
 		doexit(1);
 
-	switch (b)
-	{
+	switch (b) {
 	case 'l':
-		if (!G.gotsig)
-		{
+		if (!G.gotsig) {
 			do_linemode();
 			goto rrturn;
 		}
 		break;
 	case 'c':
-		if (G.gotsig)
-		{
+		if (G.gotsig) {
 			will_charmode();
 			goto rrturn;
 		}
@@ -156,7 +145,7 @@ static void conescape(void)
 		doexit(0);
 	}
 
-	WriteCS(1, "continuing...\r\n");
+	write_str(1, "continuing...\r\n");
 
 	if (G.gotsig)
 		cookmode();
@@ -165,6 +154,7 @@ static void conescape(void)
 	G.gotsig = 0;
 
 }
+
 static void handlenetoutput(int len)
 {
 	/*	here we could do smart tricks how to handle 0xFF:s in output
@@ -201,11 +191,10 @@ static void handlenetoutput(int len)
 		else if (*p == 0x0d)
 			outbuf[j++] = 0x00;
 	}
-	if (j > 0 )
+	if (j > 0)
 		write(G.netfd, outbuf, j);
 }
 
-
 static void handlenetinput(int len)
 {
 	int i;
@@ -280,15 +269,11 @@ static void handlenetinput(int len)
 		write(1, G.buf, len);
 }
 
-
-/* ******************************* */
-
 static void putiac(int c)
 {
 	G.iacbuf[G.iaclen++] = c;
 }
 
-
 static void putiac2(byte wwdd, byte c)
 {
 	if (G.iaclen + 3 > IACBUFSIZE)
@@ -299,7 +284,7 @@ static void putiac2(byte wwdd, byte c)
 	putiac(c);
 }
 
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
+#if ENABLE_FEATURE_TELNET_TTYPE
 static void putiac_subopt(byte c, char *str)
 {
 	int	len = strlen(str) + 6;   // ( 2 + 1 + 1 + strlen + 2 )
@@ -320,11 +305,11 @@ static void putiac_subopt(byte c, char *str)
 }
 #endif
 
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
 static void putiac_subopt_autologin(void)
 {
-	int len = strlen(autologin) + 6;	// (2 + 1 + 1 + strlen + 2)
-	char *user = "USER";
+	int len = strlen(G.autologin) + 6;	// (2 + 1 + 1 + strlen + 2)
+	const char *user = "USER";
 
 	if (G.iaclen + len > IACBUFSIZE)
 		iacflush();
@@ -340,15 +325,15 @@ static void putiac_subopt_autologin(void)
 
 	putiac(NEW_ENV_VALUE);
 
-	while (*autologin)
-		putiac(*autologin++);
+	while (*G.autologin)
+		putiac(*G.autologin++);
 
 	putiac(IAC);
 	putiac(SE);
 }
 #endif
 
-#ifdef CONFIG_FEATURE_AUTOWIDTH
+#if ENABLE_FEATURE_AUTOWIDTH
 static void putiac_naws(byte c, int x, int y)
 {
 	if (G.iaclen + 9 > IACBUFSIZE)
@@ -368,24 +353,17 @@ static void putiac_naws(byte c, int x, int y)
 }
 #endif
 
-/* void putiacstring (subneg strings) */
-
-/* ******************************* */
-
-static char const escapecharis[] = "\r\nEscape character is ";
+static char const escapecharis[] ALIGN1 = "\r\nEscape character is ";
 
 static void setConMode(void)
 {
-	if (G.telflags & UF_ECHO)
-	{
+	if (G.telflags & UF_ECHO) {
 		if (G.charmode == CHM_TRY) {
 			G.charmode = CHM_ON;
 			printf("\r\nEntering character mode%s'^]'.\r\n", escapecharis);
 			rawmode();
 		}
-	}
-	else
-	{
+	} else {
 		if (G.charmode != CHM_OFF) {
 			G.charmode = CHM_OFF;
 			printf("\r\nEntering line mode%s'^C'.\r\n", escapecharis);
@@ -394,8 +372,6 @@ static void setConMode(void)
 	}
 }
 
-/* ******************************* */
-
 static void will_charmode(void)
 {
 	G.charmode = CHM_TRY;
@@ -418,28 +394,29 @@ static void do_linemode(void)
 	iacflush();
 }
 
-/* ******************************* */
-
 static void to_notsup(char c)
 {
-	if      (G.telwish == WILL)	putiac2(DONT, c);
-	else if (G.telwish == DO)	putiac2(WONT, c);
+	if (G.telwish == WILL)
+		putiac2(DONT, c);
+	else if (G.telwish == DO)
+		putiac2(WONT, c);
 }
 
 static void to_echo(void)
 {
 	/* if server requests ECHO, don't agree */
-	if      (G.telwish == DO) {	putiac2(WONT, TELOPT_ECHO);	return; }
-	else if (G.telwish == DONT)	return;
+	if (G.telwish == DO) {
+		putiac2(WONT, TELOPT_ECHO);
+		return;
+	}
+	if (G.telwish == DONT)
+		return;
 
-	if (G.telflags & UF_ECHO)
-	{
+	if (G.telflags & UF_ECHO) {
 		if (G.telwish == WILL)
 			return;
-	}
-	else
-		if (G.telwish == WONT)
-			return;
+	} else if (G.telwish == WONT)
+		return;
 
 	if (G.charmode != CHM_OFF)
 		G.telflags ^= UF_ECHO;
@@ -450,107 +427,97 @@ static void to_echo(void)
 		putiac2(DONT, TELOPT_ECHO);
 
 	setConMode();
-	WriteCS(1, "\r\n");  /* sudden modec */
+	write_str(1, "\r\n");  /* sudden modec */
 }
 
 static void to_sga(void)
 {
 	/* daemon always sends will/wont, client do/dont */
 
-	if (G.telflags & UF_SGA)
-	{
+	if (G.telflags & UF_SGA) {
 		if (G.telwish == WILL)
 			return;
-	}
-	else
-		if (G.telwish == WONT)
-			return;
+	} else if (G.telwish == WONT)
+		return;
 
 	if ((G.telflags ^= UF_SGA) & UF_SGA) /* toggle */
 		putiac2(DO, TELOPT_SGA);
 	else
 		putiac2(DONT, TELOPT_SGA);
-
-	return;
 }
 
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
+#if ENABLE_FEATURE_TELNET_TTYPE
 static void to_ttype(void)
 {
 	/* Tell server we will (or won't) do TTYPE */
 
-	if(ttype)
+	if (G.ttype)
 		putiac2(WILL, TELOPT_TTYPE);
 	else
 		putiac2(WONT, TELOPT_TTYPE);
-
-	return;
 }
 #endif
 
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
 static void to_new_environ(void)
 {
 	/* Tell server we will (or will not) do AUTOLOGIN */
 
-	if (autologin)
+	if (G.autologin)
 		putiac2(WILL, TELOPT_NEW_ENVIRON);
 	else
 		putiac2(WONT, TELOPT_NEW_ENVIRON);
-
-	return;
 }
 #endif
 
-#ifdef CONFIG_FEATURE_AUTOWIDTH
+#if ENABLE_FEATURE_AUTOWIDTH
 static void to_naws(void)
 {
 	/* Tell server we will do NAWS */
 	putiac2(WILL, TELOPT_NAWS);
-	return;
 }
 #endif
 
 static void telopt(byte c)
 {
-	switch (c)
-	{
-		case TELOPT_ECHO:		to_echo();	break;
-		case TELOPT_SGA:		to_sga();	break;
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
-		case TELOPT_TTYPE:		to_ttype();break;
+	switch (c) {
+	case TELOPT_ECHO:
+		to_echo(); break;
+	case TELOPT_SGA:
+		to_sga(); break;
+#if ENABLE_FEATURE_TELNET_TTYPE
+	case TELOPT_TTYPE:
+		to_ttype(); break;
 #endif
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
-		case TELOPT_NEW_ENVIRON:	to_new_environ();	break;
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
+	case TELOPT_NEW_ENVIRON:
+		to_new_environ(); break;
 #endif
-#ifdef CONFIG_FEATURE_AUTOWIDTH
-		case TELOPT_NAWS:		to_naws();
-								putiac_naws(c, win_width, win_height);
-								break;
+#if ENABLE_FEATURE_AUTOWIDTH
+	case TELOPT_NAWS:
+		to_naws();
+		putiac_naws(c, G.win_width, G.win_height);
+		break;
 #endif
-		default:				to_notsup(c);
-								break;
+	default:
+		to_notsup(c);
+		break;
 	}
 }
 
-
-/* ******************************* */
-
 /* subnegotiation -- ignore all (except TTYPE,NAWS) */
-
 static int subneg(byte c)
 {
-	switch (G.telstate)
-	{
+	switch (G.telstate) {
 	case TS_SUB1:
 		if (c == IAC)
 			G.telstate = TS_SUB2;
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
+#if ENABLE_FEATURE_TELNET_TTYPE
 		else
 		if (c == TELOPT_TTYPE)
-			putiac_subopt(TELOPT_TTYPE,ttype);
+			putiac_subopt(TELOPT_TTYPE, G.ttype);
 #endif
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
 		else
 		if (c == TELOPT_NEW_ENVIRON)
 			putiac_subopt_autologin();
@@ -565,8 +532,6 @@ static int subneg(byte c)
 	return FALSE;
 }
 
-/* ******************************* */
-
 static void fgotsig(int sig)
 {
 	G.gotsig = sig;
@@ -575,14 +540,17 @@ static void fgotsig(int sig)
 
 static void rawmode(void)
 {
-	if (G.do_termios) tcsetattr(0, TCSADRAIN, &G.termios_raw);
+	if (G.do_termios)
+		tcsetattr(0, TCSADRAIN, &G.termios_raw);
 }
 
 static void cookmode(void)
 {
-	if (G.do_termios) tcsetattr(0, TCSADRAIN, &G.termios_def);
+	if (G.do_termios)
+		tcsetattr(0, TCSADRAIN, &G.termios_def);
 }
 
+int telnet_main(int argc, char** argv);
 int telnet_main(int argc, char** argv)
 {
 	char *host;
@@ -595,16 +563,16 @@ int telnet_main(int argc, char** argv)
 	int maxfd;
 #endif
 
-#ifdef CONFIG_FEATURE_AUTOWIDTH
-	get_terminal_width_height(0, &win_width, &win_height);
+	INIT_G();
+
+#if ENABLE_FEATURE_AUTOWIDTH
+	get_terminal_width_height(0, &G.win_width, &G.win_height);
 #endif
 
-#ifdef CONFIG_FEATURE_TELNET_TTYPE
-	ttype = getenv("TERM");
+#if ENABLE_FEATURE_TELNET_TTYPE
+	G.ttype = getenv("TERM");
 #endif
 
-	/* memset(&G, 0, sizeof G); - already is */
-
 	if (tcgetattr(0, &G.termios_def) >= 0) {
 		G.do_termios = 1;
 		G.termios_raw = G.termios_def;
@@ -614,9 +582,9 @@ int telnet_main(int argc, char** argv)
 	if (argc < 2)
 		bb_show_usage();
 
-#ifdef CONFIG_FEATURE_TELNET_AUTOLOGIN
-	if (1 & getopt32(argc, argv, "al:", &autologin))
-		autologin = getenv("USER");
+#if ENABLE_FEATURE_TELNET_AUTOLOGIN
+	if (1 & getopt32(argv, "al:", &G.autologin))
+		G.autologin = getenv("USER");
 	argv += optind;
 #else
 	argv++;
@@ -630,7 +598,7 @@ int telnet_main(int argc, char** argv)
 
 	G.netfd = create_and_connect_stream_or_die(host, port);
 
-	setsockopt(G.netfd, SOL_SOCKET, SO_KEEPALIVE, &one, sizeof one);
+	setsockopt(G.netfd, SOL_SOCKET, SO_KEEPALIVE, &const_int_1, sizeof(const_int_1));
 
 	signal(SIGINT, fgotsig);
 
@@ -671,12 +639,9 @@ int telnet_main(int argc, char** argv)
 #endif
 			{
 				len = read(0, G.buf, DATABUFSIZE);
-
 				if (len <= 0)
 					doexit(0);
-
 				TRACE(0, ("Read con: %d\n", len));
-
 				handlenetoutput(len);
 			}
 
@@ -687,13 +652,11 @@ int telnet_main(int argc, char** argv)
 #endif
 			{
 				len = read(G.netfd, G.buf, DATABUFSIZE);
-
 				if (len <= 0) {
-					WriteCS(1, "Connection closed by foreign host.\r\n");
+					write_str(1, "Connection closed by foreign host\r\n");
 					doexit(1);
 				}
 				TRACE(0, ("Read netfd (%d): %d\n", G.netfd, len));
-
 				handlenetinput(len);
 			}
 		}
