@@ -9,29 +9,33 @@
  */
 
 #include <getopt.h>
+#include <syslog.h>
 
+/* Override ENABLE_FEATURE_PIDFILE - ifupdown needs our pidfile to always exist */
+#define WANT_PIDFILE 1
 #include "common.h"
 #include "dhcpd.h"
 #include "dhcpc.h"
 #include "options.h"
 
 
-static int state;
 /* Something is definitely wrong here. IPv4 addresses
  * in variables of type long?? BTW, we use inet_ntoa()
  * in the code. Manpage says that struct in_addr has a member of type long (!)
  * which holds IPv4 address, and the struct is passed by value (!!)
  */
-static unsigned long requested_ip; /* = 0 */
+static unsigned timeout;
+static uint32_t requested_ip; /* = 0 */
 static uint32_t server_addr;
-static unsigned long timeout;
 static int packet_num; /* = 0 */
-static int fd = -1;
+static int sockfd = -1;
 
 #define LISTEN_NONE 0
 #define LISTEN_KERNEL 1
 #define LISTEN_RAW 2
-static int listen_mode;
+static smallint listen_mode;
+
+static smallint state;
 
 struct client_config_t client_config;
 
@@ -41,8 +45,10 @@ static void change_mode(int new_mode)
 {
 	DEBUG("entering %s listen mode",
 		new_mode ? (new_mode == 1 ? "kernel" : "raw") : "none");
-	if (fd >= 0) close(fd);
-	fd = -1;
+	if (sockfd >= 0) {
+		close(sockfd);
+		sockfd = -1;
+	}
 	listen_mode = new_mode;
 }
 
@@ -80,13 +86,13 @@ static void perform_renew(void)
 /* perform a release */
 static void perform_release(void)
 {
-	char buffer[16];
+	char buffer[sizeof("255.255.255.255")];
 	struct in_addr temp_addr;
 
 	/* send release packet */
 	if (state == BOUND || state == RENEWING || state == REBINDING) {
 		temp_addr.s_addr = server_addr;
-		sprintf(buffer, "%s", inet_ntoa(temp_addr));
+		strcpy(buffer, inet_ntoa(temp_addr));
 		temp_addr.s_addr = requested_ip;
 		bb_info_msg("Unicasting a release of %s to %s",
 				inet_ntoa(temp_addr), buffer);
@@ -97,14 +103,26 @@ static void perform_release(void)
 
 	change_mode(LISTEN_NONE);
 	state = RELEASED;
-	timeout = 0x7fffffff;
+	timeout = INT_MAX;
 }
 
 
 static void client_background(void)
 {
-	udhcp_background(client_config.pidfile);
-	client_config.foreground = 1; /* Do not fork again. */
+#if !BB_MMU
+	bb_error_msg("cannot background in uclinux (yet)");
+/* ... mainly because udhcpc calls client_background()
+ * in _the _middle _of _udhcpc _run_, not at the start!
+ * If that will be properly disabled for NOMMU, client_background()
+ * will work on NOMMU too */
+#else
+	bb_daemonize(0);
+	logmode &= ~LOGMODE_STDIO;
+	/* rewrite pidfile, as our pid is different now */
+	write_pidfile(client_config.pidfile);
+#endif
+	/* Do not fork again. */
+	client_config.foreground = 1;
 	client_config.background_if_no_lease = 0;
 }
 
@@ -122,23 +140,24 @@ static uint8_t* alloc_dhcp_option(int code, const char *str, int extra)
 }
 
 
-int udhcpc_main(int argc, char *argv[])
+int udhcpc_main(int argc, char **argv);
+int udhcpc_main(int argc, char **argv)
 {
 	uint8_t *temp, *message;
 	char *str_c, *str_V, *str_h, *str_F, *str_r, *str_T, *str_t;
-	unsigned long t1 = 0, t2 = 0, xid = 0;
-	unsigned long start = 0, lease = 0;
-	long now;
+	uint32_t xid = 0;
+	uint32_t lease = 0; /* can be given as 32-bit quantity */
+	unsigned t1 = 0, t2 = 0; /* what a wonderful names */
+	unsigned start = 0;
+	unsigned now;
 	unsigned opt;
 	int max_fd;
-	int sig;
 	int retval;
 	int len;
-	int no_clientid = 0;
-	fd_set rfds;
 	struct timeval tv;
-	struct dhcpMessage packet;
 	struct in_addr temp_addr;
+	struct dhcpMessage packet;
+	fd_set rfds;
 
 	enum {
 		OPT_c = 1 << 0,
@@ -159,29 +178,30 @@ int udhcpc_main(int argc, char *argv[])
 		OPT_T = 1 << 15,
 		OPT_t = 1 << 16,
 		OPT_v = 1 << 17,
+		OPT_S = 1 << 18,
 	};
 #if ENABLE_GETOPT_LONG
-	static const struct option arg_options[] = {
-		{ "clientid",   required_argument,      0, 'c' },
-		{ "clientid-none", no_argument,         0, 'C' },
-		{ "vendorclass", required_argument,     0, 'V' },
-		{ "foreground", no_argument,            0, 'f' },
-		{ "background", no_argument,            0, 'b' },
-		{ "hostname",   required_argument,      0, 'H' },
-		{ "hostname",   required_argument,      0, 'h' },
-		{ "fqdn",       required_argument,      0, 'F' },
-		{ "interface",  required_argument,      0, 'i' },
-		{ "now",        no_argument,            0, 'n' },
-		{ "pidfile",    required_argument,      0, 'p' },
-		{ "quit",       no_argument,            0, 'q' },
-		{ "release",    no_argument,            0, 'R' },
-		{ "request",    required_argument,      0, 'r' },
-		{ "script",     required_argument,      0, 's' },
-		{ "timeout",    required_argument,      0, 'T' },
-		{ "version",    no_argument,            0, 'v' },
-		{ "retries",    required_argument,      0, 't' },
-		{ 0, 0, 0, 0 }
-	};
+	static const char udhcpc_longopts[] ALIGN1 =
+		"clientid\0"      Required_argument "c"
+		"clientid-none\0" No_argument       "C"
+		"vendorclass\0"   Required_argument "V"
+		"foreground\0"    No_argument       "f"
+		"background\0"    No_argument       "b"
+		"hostname\0"      Required_argument "H"
+		"hostname\0"      Required_argument "h"
+		"fqdn\0"          Required_argument "F"
+		"interface\0"     Required_argument "i"
+		"now\0"           No_argument       "n"
+		"pidfile\0"       Required_argument "p"
+		"quit\0"          No_argument       "q"
+		"release\0"       No_argument       "R"
+		"request\0"       Required_argument "r"
+		"script\0"        Required_argument "s"
+		"timeout\0"       Required_argument "T"
+		"version\0"       No_argument       "v"
+		"retries\0"       Required_argument "t"
+		"syslog\0"        No_argument       "S"
+		;
 #endif
 	/* Default options. */
 	client_config.interface = "eth0";
@@ -190,12 +210,12 @@ int udhcpc_main(int argc, char *argv[])
 	client_config.timeout = 3;
 
 	/* Parse command line */
-	opt_complementary = "?:c--C:C--c" // mutually exclusive
+	opt_complementary = "c--C:C--c" // mutually exclusive
 	                    ":hH:Hh"; // -h and -H are the same
 #if ENABLE_GETOPT_LONG
-	applet_long_options = arg_options;
+	applet_long_options = udhcpc_longopts;
 #endif
-	opt = getopt32(argc, argv, "c:CV:fbH:h:F:i:np:qRr:s:T:t:v",
+	opt = getopt32(argv, "c:CV:fbH:h:F:i:np:qRr:s:T:t:vS",
 		&str_c, &str_V, &str_h, &str_h, &str_F,
 		&client_config.interface, &client_config.pidfile, &str_r,
 		&client_config.script, &str_T, &str_t
@@ -203,8 +223,7 @@ int udhcpc_main(int argc, char *argv[])
 
 	if (opt & OPT_c)
 		client_config.clientid = alloc_dhcp_option(DHCP_CLIENT_ID, str_c, 0);
-	if (opt & OPT_C)
-		no_clientid = 1;
+	//if (opt & OPT_C)
 	if (opt & OPT_V)
 		client_config.vendorclass = alloc_dhcp_option(DHCP_VENDOR, str_V, 0);
 	if (opt & OPT_f)
@@ -241,19 +260,33 @@ int udhcpc_main(int argc, char *argv[])
 	if (opt & OPT_t)
 		client_config.retries = xatoi_u(str_t);
 	if (opt & OPT_v) {
-		printf("version %s\n\n", BB_VER);
+		printf("version %s\n", BB_VER);
 		return 0;
 	}
 
-	/* Start the log, sanitize fd's, and write a pid file */
-	udhcp_start_log_and_pid(client_config.pidfile);
+	if (opt & OPT_S) {
+		openlog(applet_name, LOG_PID, LOG_LOCAL0);
+		logmode |= LOGMODE_SYSLOG;
+	}
 
 	if (read_interface(client_config.interface, &client_config.ifindex,
-			   NULL, client_config.arp) < 0)
+			   NULL, client_config.arp))
 		return 1;
 
+	/* Make sure fd 0,1,2 are open */
+	bb_sanitize_stdio();
+	/* Equivalent of doing a fflush after every \n */
+	setlinebuf(stdout);
+
+	/* Create pidfile */
+	write_pidfile(client_config.pidfile);
+	/* if (!..) bb_perror_msg("cannot create pidfile %s", pidfile); */
+
+	/* Goes to stdout and possibly syslog */
+	bb_info_msg("%s (v%s) started", applet_name, BB_VER);
+
 	/* if not set, and not suppressed, setup the default client ID */
-	if (!client_config.clientid && !no_clientid) {
+	if (!client_config.clientid && !(opt & OPT_C)) {
 		client_config.clientid = alloc_dhcp_option(DHCP_CLIENT_ID, "", 7);
 		client_config.clientid[OPT_DATA] = 1;
 		memcpy(client_config.clientid + OPT_DATA+1, client_config.arp, 6);
@@ -268,26 +301,36 @@ int udhcpc_main(int argc, char *argv[])
 	state = INIT_SELECTING;
 	udhcp_run_script(NULL, "deconfig");
 	change_mode(LISTEN_RAW);
+	tv.tv_sec = 0;
+	goto jump_in;
 
 	for (;;) {
-		tv.tv_sec = timeout - uptime();
+		tv.tv_sec = timeout - monotonic_sec();
+ jump_in:
 		tv.tv_usec = 0;
 
-		if (listen_mode != LISTEN_NONE && fd < 0) {
+		if (listen_mode != LISTEN_NONE && sockfd < 0) {
 			if (listen_mode == LISTEN_KERNEL)
-				fd = listen_socket(INADDR_ANY, CLIENT_PORT, client_config.interface);
+				sockfd = listen_socket(/*INADDR_ANY,*/ CLIENT_PORT, client_config.interface);
 			else
-				fd = raw_socket(client_config.ifindex);
+				sockfd = raw_socket(client_config.ifindex);
 		}
-		max_fd = udhcp_sp_fd_set(&rfds, fd);
+		max_fd = udhcp_sp_fd_set(&rfds, sockfd);
 
+		retval = 0; /* If we already timed out, fall through, else... */
 		if (tv.tv_sec > 0) {
 			DEBUG("Waiting on select...");
 			retval = select(max_fd + 1, &rfds, NULL, NULL, &tv);
-		} else retval = 0; /* If we already timed out, fall through */
+		}
 
-		now = uptime();
-		if (retval == 0) {
+		now = monotonic_sec();
+		if (retval < 0) {
+			/* EINTR? signal was caught, don't panic */
+			if (errno != EINTR) {
+				/* Else: an error occured, panic! */
+				bb_perror_msg_and_die("select");
+			}
+		} else if (retval == 0) {
 			/* timeout dropped to zero */
 			switch (state) {
 			case INIT_SELECTING:
@@ -307,7 +350,8 @@ int udhcpc_main(int argc, char *argv[])
 						client_background();
 					} else if (client_config.abort_if_no_lease) {
 						bb_info_msg("No lease, failing");
-						return 1;
+						retval = 1;
+						goto ret;
 					}
 					/* wait to try again */
 					packet_num = 0;
@@ -326,7 +370,8 @@ int udhcpc_main(int argc, char *argv[])
 					packet_num++;
 				} else {
 					/* timed out, go back to init state */
-					if (state == RENEW_REQUESTED) udhcp_run_script(NULL, "deconfig");
+					if (state == RENEW_REQUESTED)
+						udhcp_run_script(NULL, "deconfig");
 					state = INIT_SELECTING;
 					timeout = now;
 					packet_num = 0;
@@ -349,9 +394,8 @@ int udhcpc_main(int argc, char *argv[])
 				} else {
 					/* send a request packet */
 					send_renew(xid, server_addr, requested_ip); /* unicast */
-
 					t1 = (t2 - t1) / 2 + t1;
-					timeout = t1 + start;
+					timeout = start + t1;
 				}
 				break;
 			case REBINDING:
@@ -367,22 +411,21 @@ int udhcpc_main(int argc, char *argv[])
 				} else {
 					/* send a request packet */
 					send_renew(xid, 0, requested_ip); /* broadcast */
-
 					t2 = (lease - t2) / 2 + t2;
-					timeout = t2 + start;
+					timeout = start + t2;
 				}
 				break;
 			case RELEASED:
 				/* yah, I know, *you* say it would never happen */
-				timeout = 0x7fffffff;
+				timeout = INT_MAX;
 				break;
 			}
-		} else if (retval > 0 && listen_mode != LISTEN_NONE && FD_ISSET(fd, &rfds)) {
+		} else if (listen_mode != LISTEN_NONE && FD_ISSET(sockfd, &rfds)) {
 			/* a packet is ready, read it */
 
 			if (listen_mode == LISTEN_KERNEL)
-				len = udhcp_get_packet(&packet, fd);
-			else len = get_raw_packet(&packet, fd);
+				len = udhcp_get_packet(&packet, sockfd);
+			else len = get_raw_packet(&packet, sockfd);
 
 			if (len == -1 && errno != EINTR) {
 				DEBUG("error on read, %s, reopening socket", strerror(errno));
@@ -391,8 +434,8 @@ int udhcpc_main(int argc, char *argv[])
 			if (len < 0) continue;
 
 			if (packet.xid != xid) {
-				DEBUG("Ignoring XID %lx (our xid is %lx)",
-					(unsigned long) packet.xid, xid);
+				DEBUG("Ignoring XID %x (our xid is %x)",
+					(unsigned)packet.xid, (unsigned)xid);
 				continue;
 			}
 
@@ -402,7 +445,8 @@ int udhcpc_main(int argc, char *argv[])
 				continue;
 			}
 
-			if ((message = get_option(&packet, DHCP_MESSAGE_TYPE)) == NULL) {
+			message = get_option(&packet, DHCP_MESSAGE_TYPE);
+			if (message == NULL) {
 				bb_error_msg("cannot get option from packet - ignoring");
 				continue;
 			}
@@ -446,12 +490,12 @@ int udhcpc_main(int argc, char *argv[])
 					t1 = lease / 2;
 
 					/* little fixed point for n * .875 */
-					t2 = (lease * 0x7) >> 3;
+					t2 = (lease * 7) >> 3;
 					temp_addr.s_addr = packet.yiaddr;
-					bb_info_msg("Lease of %s obtained, lease time %ld",
-						inet_ntoa(temp_addr), lease);
+					bb_info_msg("Lease of %s obtained, lease time %u",
+						inet_ntoa(temp_addr), (unsigned)lease);
 					start = now;
-					timeout = t1 + start;
+					timeout = start + t1;
 					requested_ip = packet.yiaddr;
 					udhcp_run_script(&packet,
 						   ((state == RENEWING || state == REBINDING) ? "renew" : "bound"));
@@ -461,7 +505,7 @@ int udhcpc_main(int argc, char *argv[])
 					if (client_config.quit_after_lease) {
 						if (client_config.release_on_quit)
 							perform_release();
-						return 0;
+						goto ret0;
 					}
 					if (!client_config.foreground)
 						client_background();
@@ -482,8 +526,9 @@ int udhcpc_main(int argc, char *argv[])
 				break;
 			/* case BOUND, RELEASED: - ignore all packets */
 			}
-		} else if (retval > 0 && (sig = udhcp_sp_read(&rfds))) {
-			switch (sig) {
+		} else {
+			int signo = udhcp_sp_read(&rfds);
+			switch (signo) {
 			case SIGUSR1:
 				perform_renew();
 				break;
@@ -494,15 +539,14 @@ int udhcpc_main(int argc, char *argv[])
 				bb_info_msg("Received SIGTERM");
 				if (client_config.release_on_quit)
 					perform_release();
-				return 0;
+				goto ret0;
 			}
-		} else if (retval == -1 && errno == EINTR) {
-			/* a signal was caught */
-		} else {
-			/* An error occured */
-			bb_perror_msg("select");
 		}
-
-	}
-	return 0;
+	} /* for (;;) */
+ ret0:
+	retval = 0;
+ ret:
+	/*if (client_config.pidfile) - remove_pidfile has it's own check */
+		remove_pidfile(client_config.pidfile);
+	return retval;
 }
