@@ -8,25 +8,42 @@
 #include "dhcpd.h"
 
 
-uint8_t blank_chaddr[] = {[0 ... 15] = 0};
-
-/* clear every lease out that chaddr OR yiaddr matches and is nonzero */
-void clear_lease(uint8_t *chaddr, uint32_t yiaddr)
+/* Find the oldest expired lease, NULL if there are no expired leases */
+static struct dhcpOfferedAddr *oldest_expired_lease(void)
 {
-	unsigned int i, j;
-
-	for (j = 0; j < 16 && !chaddr[j]; j++);
+	struct dhcpOfferedAddr *oldest = NULL;
+// TODO: use monotonic_sec()
+	unsigned long oldest_lease = time(0);
+	unsigned i;
 
 	for (i = 0; i < server_config.max_leases; i++)
-		if ((j != 16 && !memcmp(leases[i].chaddr, chaddr, 16)) ||
-		    (yiaddr && leases[i].yiaddr == yiaddr)) {
-			memset(&(leases[i]), 0, sizeof(struct dhcpOfferedAddr));
+		if (oldest_lease > leases[i].expires) {
+			oldest_lease = leases[i].expires;
+			oldest = &(leases[i]);
+		}
+	return oldest;
+}
+
+
+/* clear every lease out that chaddr OR yiaddr matches and is nonzero */
+static void clear_lease(const uint8_t *chaddr, uint32_t yiaddr)
+{
+	unsigned i, j;
+
+	for (j = 0; j < 16 && !chaddr[j]; j++)
+		continue;
+
+	for (i = 0; i < server_config.max_leases; i++)
+		if ((j != 16 && memcmp(leases[i].chaddr, chaddr, 16) == 0)
+		 || (yiaddr && leases[i].yiaddr == yiaddr)
+		) {
+			memset(&(leases[i]), 0, sizeof(leases[i]));
 		}
 }
 
 
 /* add a lease into the table, clearing out any old ones */
-struct dhcpOfferedAddr *add_lease(uint8_t *chaddr, uint32_t yiaddr, unsigned long lease)
+struct dhcpOfferedAddr *add_lease(const uint8_t *chaddr, uint32_t yiaddr, unsigned long lease)
 {
 	struct dhcpOfferedAddr *oldest;
 
@@ -52,31 +69,14 @@ int lease_expired(struct dhcpOfferedAddr *lease)
 }
 
 
-/* Find the oldest expired lease, NULL if there are no expired leases */
-struct dhcpOfferedAddr *oldest_expired_lease(void)
-{
-	struct dhcpOfferedAddr *oldest = NULL;
-	unsigned long oldest_lease = time(0);
-	unsigned int i;
-
-
-	for (i = 0; i < server_config.max_leases; i++)
-		if (oldest_lease > leases[i].expires) {
-			oldest_lease = leases[i].expires;
-			oldest = &(leases[i]);
-		}
-	return oldest;
-
-}
-
-
 /* Find the first lease that matches chaddr, NULL if no match */
-struct dhcpOfferedAddr *find_lease_by_chaddr(uint8_t *chaddr)
+struct dhcpOfferedAddr *find_lease_by_chaddr(const uint8_t *chaddr)
 {
-	unsigned int i;
+	unsigned i;
 
 	for (i = 0; i < server_config.max_leases; i++)
-		if (!memcmp(leases[i].chaddr, chaddr, 16)) return &(leases[i]);
+		if (!memcmp(leases[i].chaddr, chaddr, 16))
+			return &(leases[i]);
 
 	return NULL;
 }
@@ -85,59 +85,61 @@ struct dhcpOfferedAddr *find_lease_by_chaddr(uint8_t *chaddr)
 /* Find the first lease that matches yiaddr, NULL is no match */
 struct dhcpOfferedAddr *find_lease_by_yiaddr(uint32_t yiaddr)
 {
-	unsigned int i;
+	unsigned i;
 
 	for (i = 0; i < server_config.max_leases; i++)
-		if (leases[i].yiaddr == yiaddr) return &(leases[i]);
+		if (leases[i].yiaddr == yiaddr)
+			return &(leases[i]);
 
 	return NULL;
 }
 
 
 /* check is an IP is taken, if it is, add it to the lease table */
-static int check_ip(uint32_t addr)
+static int nobody_responds_to_arp(uint32_t addr)
 {
-	struct in_addr temp;
+	static const uint8_t blank_chaddr[16]; /* 16 zero bytes */
 
-	if (arpping(addr, server_config.server, server_config.arp, server_config.interface) == 0) {
-		temp.s_addr = addr;
-		bb_info_msg("%s belongs to someone, reserving it for %ld seconds",
-			inet_ntoa(temp), server_config.conflict_time);
-		add_lease(blank_chaddr, addr, server_config.conflict_time);
-		return 1;
-	} else return 0;
+	struct in_addr temp;
+	int r;
+
+	r = arpping(addr, server_config.server, server_config.arp, server_config.interface);
+	if (r)
+		return r;
+
+	temp.s_addr = addr;
+	bb_info_msg("%s belongs to someone, reserving it for %u seconds",
+		inet_ntoa(temp), (unsigned)server_config.conflict_time);
+	add_lease(blank_chaddr, addr, server_config.conflict_time);
+	return 0;
 }
 
 
-/* find an assignable address, it check_expired is true, we check all the expired leases as well.
+/* find an assignable address, if check_expired is true, we check all the expired leases as well.
  * Maybe this should try expired leases by age... */
 uint32_t find_address(int check_expired)
 {
 	uint32_t addr, ret;
 	struct dhcpOfferedAddr *lease = NULL;
 
-	addr = ntohl(server_config.start); /* addr is in host order here */
-	for (;addr <= ntohl(server_config.end); addr++) {
-
+	addr = server_config.start_ip; /* addr is in host order here */
+	for (; addr <= server_config.end_ip; addr++) {
 		/* ie, 192.168.55.0 */
-		if (!(addr & 0xFF)) continue;
-
+		if (!(addr & 0xFF))
+			continue;
 		/* ie, 192.168.55.255 */
-		if ((addr & 0xFF) == 0xFF) continue;
-
-		/* Only do if it isn't an assigned as a static lease */
-		if (!reservedIp(server_config.static_leases, htonl(addr))) {
-
+		if ((addr & 0xFF) == 0xFF)
+			continue;
+		/* Only do if it isn't assigned as a static lease */
+		ret = htonl(addr);
+		if (!reservedIp(server_config.static_leases, ret)) {
 			/* lease is not taken */
-			ret = htonl(addr);
 			lease = find_lease_by_yiaddr(ret);
-
 			/* no lease or it expired and we are checking for expired leases */
-			if ( (!lease || (check_expired && lease_expired(lease)))
-			 && /* and it isn't on the network */ !check_ip(ret)
+			if ((!lease || (check_expired && lease_expired(lease)))
+			 && nobody_responds_to_arp(ret) /* it isn't used on the network */
 			) {
 				return ret;
-				break;
 			}
 		}
 	}
