@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /* script.c
  *
  * Functions to call the DHCP client notification scripts
@@ -7,26 +8,19 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <sys/types.h>
-#include <sys/wait.h>
-
 #include "common.h"
-#include "options.h"
-#include "dhcpd.h"
 #include "dhcpc.h"
+#include "options.h"
+
 
 /* get a rough idea of how long an option will be (rounding up...) */
-static const int max_option_length[] = {
+static const uint8_t max_option_length[] = {
 	[OPTION_IP] =		sizeof("255.255.255.255 "),
 	[OPTION_IP_PAIR] =	sizeof("255.255.255.255 ") * 2,
 	[OPTION_STRING] =	1,
+#if ENABLE_FEATURE_RFC3397
+	[OPTION_STR1035] =	1,
+#endif
 	[OPTION_BOOLEAN] =	sizeof("yes "),
 	[OPTION_U8] =		sizeof("255 "),
 	[OPTION_U16] =		sizeof("65535 "),
@@ -43,7 +37,7 @@ static inline int upper_length(int length, int opt_index)
 }
 
 
-static int sprintip(char *dest, char *pre, uint8_t *ip)
+static int sprintip(char *dest, const char *pre, const uint8_t *ip)
 {
 	return sprintf(dest, "%s%d.%d.%d.%d", pre, ip[0], ip[1], ip[2], ip[3]);
 }
@@ -61,25 +55,28 @@ static int mton(struct in_addr *mask)
 }
 
 
-/* Fill dest with the text of option 'option'. */
-static void fill_options(char *dest, uint8_t *option, struct dhcp_option *type_p)
+/* Allocate and fill with the text of option 'option'. */
+static char *alloc_fill_opts(uint8_t *option, const struct dhcp_option *type_p)
 {
-	int type, optlen;
+	int len, type, optlen;
 	uint16_t val_u16;
 	int16_t val_s16;
 	uint32_t val_u32;
 	int32_t val_s32;
-	int len = option[OPT_LEN - 2];
-
-	dest += sprintf(dest, "%s=", type_p->name);
+	char *dest, *ret;
 
+	len = option[OPT_LEN - 2];
 	type = type_p->flags & TYPE_MASK;
 	optlen = option_lengths[type];
-	for(;;) {
+
+	dest = ret = xmalloc(upper_length(len, type) + strlen(type_p->name) + 2);
+	dest += sprintf(ret, "%s=", type_p->name);
+
+	for (;;) {
 		switch (type) {
 		case OPTION_IP_PAIR:
 			dest += sprintip(dest, "", option);
-			*(dest++) = '/';
+			*dest++ = '/';
 			option += 4;
 			optlen = 4;
 		case OPTION_IP:	/* Works regardless of host byte order. */
@@ -110,13 +107,21 @@ static void fill_options(char *dest, uint8_t *option, struct dhcp_option *type_p
 		case OPTION_STRING:
 			memcpy(dest, option, len);
 			dest[len] = '\0';
-			return;	 /* Short circuit this case */
+			return ret;	 /* Short circuit this case */
+#if ENABLE_FEATURE_RFC3397
+		case OPTION_STR1035:
+			/* unpack option into dest; use ret for prefix (i.e., "optname=") */
+			dest = dname_dec(option, len, ret);
+			free(ret);
+			return dest;
+#endif
 		}
 		option += optlen;
 		len -= optlen;
 		if (len <= 0) break;
 		dest += sprintf(dest, " ");
 	}
+	return ret;
 }
 
 
@@ -126,50 +131,56 @@ static char **fill_envp(struct dhcpMessage *packet)
 	int num_options = 0;
 	int i, j;
 	char **envp;
+	char *var;
 	uint8_t *temp;
 	struct in_addr subnet;
 	char over = 0;
 
-	if (packet == NULL)
-		num_options = 0;
-	else {
-		for (i = 0; dhcp_options[i].code; i++)
+	if (packet) {
+		for (i = 0; dhcp_options[i].code; i++) {
 			if (get_option(packet, dhcp_options[i].code)) {
 				num_options++;
 				if (dhcp_options[i].code == DHCP_SUBNET)
 					num_options++; /* for mton */
 			}
-		if (packet->siaddr) num_options++;
-		if ((temp = get_option(packet, DHCP_OPTION_OVER)))
+		}
+		if (packet->siaddr)
+			num_options++;
+		temp = get_option(packet, DHCP_OPTION_OVER);
+		if (temp)
 			over = *temp;
-		if (!(over & FILE_FIELD) && packet->file[0]) num_options++;
-		if (!(over & SNAME_FIELD) && packet->sname[0]) num_options++;
+		if (!(over & FILE_FIELD) && packet->file[0])
+			num_options++;
+		if (!(over & SNAME_FIELD) && packet->sname[0])
+			num_options++;
 	}
 
 	envp = xzalloc(sizeof(char *) * (num_options + 5));
 	j = 0;
-	envp[j++] = bb_xasprintf("interface=%s", client_config.interface);
-	envp[j++] = bb_xasprintf("PATH=%s",
-		getenv("PATH") ? : "/bin:/usr/bin:/sbin:/usr/sbin");
-	envp[j++] = bb_xasprintf("HOME=%s", getenv("HOME") ? : "/");
+	envp[j++] = xasprintf("interface=%s", client_config.interface);
+	var = getenv("PATH");
+	if (var)
+		envp[j++] = xasprintf("PATH=%s", var);
+	var = getenv("HOME");
+	if (var)
+		envp[j++] = xasprintf("HOME=%s", var);
 
-	if (packet == NULL) return envp;
+	if (packet == NULL)
+		return envp;
 
 	envp[j] = xmalloc(sizeof("ip=255.255.255.255"));
 	sprintip(envp[j++], "ip=", (uint8_t *) &packet->yiaddr);
 
-
 	for (i = 0; dhcp_options[i].code; i++) {
-		if (!(temp = get_option(packet, dhcp_options[i].code)))
+		temp = get_option(packet, dhcp_options[i].code);
+		if (!temp)
 			continue;
-		envp[j] = xmalloc(upper_length(temp[OPT_LEN - 2],
-			dhcp_options[i].flags & TYPE_MASK) + strlen(dhcp_options[i].name) + 2);
-		fill_options(envp[j++], temp, &dhcp_options[i]);
+		envp[j++] = alloc_fill_opts(temp, &dhcp_options[i]);
 
 		/* Fill in a subnet bits option for things like /24 */
 		if (dhcp_options[i].code == DHCP_SUBNET) {
 			memcpy(&subnet, temp, 4);
-			envp[j++] = bb_xasprintf("mask=%d", mton(&subnet));
+			envp[j++] = xasprintf("mask=%d", mton(&subnet));
 		}
 	}
 	if (packet->siaddr) {
@@ -179,12 +190,12 @@ static char **fill_envp(struct dhcpMessage *packet)
 	if (!(over & FILE_FIELD) && packet->file[0]) {
 		/* watch out for invalid packets */
 		packet->file[sizeof(packet->file) - 1] = '\0';
-		envp[j++] = bb_xasprintf("boot_file=%s", packet->file);
+		envp[j++] = xasprintf("boot_file=%s", packet->file);
 	}
 	if (!(over & SNAME_FIELD) && packet->sname[0]) {
 		/* watch out for invalid packets */
 		packet->sname[sizeof(packet->sname) - 1] = '\0';
-		envp[j++] = bb_xasprintf("sname=%s", packet->sname);
+		envp[j++] = xasprintf("sname=%s", packet->sname);
 	}
 	return envp;
 }
@@ -199,23 +210,23 @@ void udhcp_run_script(struct dhcpMessage *packet, const char *name)
 	if (client_config.script == NULL)
 		return;
 
-	DEBUG(LOG_INFO, "vforking and execle'ing %s", client_config.script);
+	DEBUG("vfork'ing and execle'ing %s", client_config.script);
 
 	envp = fill_envp(packet);
-	/* call script */
-	pid = vfork();
-	if (pid) {
-		waitpid(pid, NULL, 0);
-		for (curr = envp; *curr; curr++) free(*curr);
-		free(envp);
-		return;
-	} else if (pid == 0) {
-		/* close fd's? */
 
+	/* call script */
+// can we use wait4pid(spawn(...)) here?
+	pid = vfork();
+	if (pid < 0) return;
+	if (pid == 0) {
+		/* close fd's? */
 		/* exec script */
 		execle(client_config.script, client_config.script,
 		       name, NULL, envp);
-		LOG(LOG_ERR, "script %s failed: %m", client_config.script);
-		exit(1);
+		bb_perror_msg_and_die("script %s failed", client_config.script);
 	}
+	waitpid(pid, NULL, 0);
+	for (curr = envp; *curr; curr++)
+		free(*curr);
+	free(envp);
 }
