@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /* serverpacket.c
  *
  * Construct and send DHCP server packets
@@ -19,24 +20,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <string.h>
-#include <time.h>
-
 #include "common.h"
-#include "serverpacket.h"
 #include "dhcpd.h"
 #include "options.h"
-#include "static_leases.h"
+
 
 /* send a packet to giaddr using the kernel ip stack */
 static int send_packet_to_relay(struct dhcpMessage *payload)
 {
-	DEBUG(LOG_INFO, "Forwarding packet to relay");
+	DEBUG("Forwarding packet to relay");
 
-	return kernel_packet(payload, server_config.server, SERVER_PORT,
+	return udhcp_kernel_packet(payload, server_config.server, SERVER_PORT,
 			payload->giaddr, SERVER_PORT);
 }
 
@@ -44,27 +38,27 @@ static int send_packet_to_relay(struct dhcpMessage *payload)
 /* send a packet to a specific arp address and ip address by creating our own ip packet */
 static int send_packet_to_client(struct dhcpMessage *payload, int force_broadcast)
 {
-	uint8_t *chaddr;
+	const uint8_t *chaddr;
 	uint32_t ciaddr;
 
 	if (force_broadcast) {
-		DEBUG(LOG_INFO, "broadcasting packet to client (NAK)");
+		DEBUG("broadcasting packet to client (NAK)");
 		ciaddr = INADDR_BROADCAST;
 		chaddr = MAC_BCAST_ADDR;
 	} else if (payload->ciaddr) {
-		DEBUG(LOG_INFO, "unicasting packet to client ciaddr");
+		DEBUG("unicasting packet to client ciaddr");
 		ciaddr = payload->ciaddr;
 		chaddr = payload->chaddr;
 	} else if (ntohs(payload->flags) & BROADCAST_FLAG) {
-		DEBUG(LOG_INFO, "broadcasting packet to client (requested)");
+		DEBUG("broadcasting packet to client (requested)");
 		ciaddr = INADDR_BROADCAST;
 		chaddr = MAC_BCAST_ADDR;
 	} else {
-		DEBUG(LOG_INFO, "unicasting packet to client yiaddr");
+		DEBUG("unicasting packet to client yiaddr");
 		ciaddr = payload->yiaddr;
 		chaddr = payload->chaddr;
 	}
-	return raw_packet(payload, server_config.server, SERVER_PORT,
+	return udhcp_raw_packet(payload, server_config.server, SERVER_PORT,
 			ciaddr, CLIENT_PORT, chaddr, server_config.ifindex);
 }
 
@@ -83,7 +77,7 @@ static int send_packet(struct dhcpMessage *payload, int force_broadcast)
 
 static void init_packet(struct dhcpMessage *packet, struct dhcpMessage *oldpacket, char type)
 {
-	init_header(packet, type);
+	udhcp_init_header(packet, type);
 	packet->xid = oldpacket->xid;
 	memcpy(packet->chaddr, oldpacket->chaddr, 16);
 	packet->flags = oldpacket->flags;
@@ -121,65 +115,57 @@ int sendOffer(struct dhcpMessage *oldpacket)
 	static_lease_ip = getIpByMac(server_config.static_leases, oldpacket->chaddr);
 
 	/* ADDME: if static, short circuit */
-	if(!static_lease_ip)
-	{
-	/* the client is in our lease/offered table */
-	if ((lease = find_lease_by_chaddr(oldpacket->chaddr))) {
-		if (!lease_expired(lease))
-			lease_time_align = lease->expires - time(0);
-		packet.yiaddr = lease->yiaddr;
-
-	/* Or the client has a requested ip */
-	} else if ((req = get_option(oldpacket, DHCP_REQUESTED_IP)) &&
-
-		   /* Don't look here (ugly hackish thing to do) */
-		   memcpy(&req_align, req, 4) &&
-
-		   /* and the ip is in the lease range */
-		   ntohl(req_align) >= ntohl(server_config.start) &&
-		   ntohl(req_align) <= ntohl(server_config.end) &&
-
-			!static_lease_ip &&  /* Check that its not a static lease */
-			/* and is not already taken/offered */
-		   ((!(lease = find_lease_by_yiaddr(req_align)) ||
-
-		   /* or its taken, but expired */ /* ADDME: or maybe in here */
-		   lease_expired(lease)))) {
-				packet.yiaddr = req_align; /* FIXME: oh my, is there a host using this IP? */
-
+	if (!static_lease_ip) {
+		/* the client is in our lease/offered table */
+		lease = find_lease_by_chaddr(oldpacket->chaddr);
+		if (lease) {
+			if (!lease_expired(lease))
+				lease_time_align = lease->expires - time(0);
+			packet.yiaddr = lease->yiaddr;
+		/* Or the client has a requested ip */
+		} else if ((req = get_option(oldpacket, DHCP_REQUESTED_IP))
+		 /* Don't look here (ugly hackish thing to do) */
+		 && memcpy(&req_align, req, 4)
+		 /* and the ip is in the lease range */
+		 && ntohl(req_align) >= server_config.start_ip
+		 && ntohl(req_align) <= server_config.end_ip
+		 && !static_lease_ip /* Check that its not a static lease */
+		 /* and is not already taken/offered */
+		 && (!(lease = find_lease_by_yiaddr(req_align))
+			/* or its taken, but expired */ /* ADDME: or maybe in here */
+			|| lease_expired(lease))
+		) {
+			packet.yiaddr = req_align; /* FIXME: oh my, is there a host using this IP? */
 			/* otherwise, find a free IP */
-	} else {
+		} else {
 			/* Is it a static lease? (No, because find_address skips static lease) */
-		packet.yiaddr = find_address(0);
+			packet.yiaddr = find_address(0);
+			/* try for an expired lease */
+			if (!packet.yiaddr)
+				packet.yiaddr = find_address(1);
+		}
 
-		/* try for an expired lease */
-		if (!packet.yiaddr) packet.yiaddr = find_address(1);
-	}
+		if (!packet.yiaddr) {
+			bb_error_msg("no IP addresses to give - OFFER abandoned");
+			return -1;
+		}
+		if (!add_lease(packet.chaddr, packet.yiaddr, server_config.offer_time)) {
+			bb_error_msg("lease pool is full - OFFER abandoned");
+			return -1;
+		}
+		lease_time = get_option(oldpacket, DHCP_LEASE_TIME);
+		if (lease_time) {
+			memcpy(&lease_time_align, lease_time, 4);
+			lease_time_align = ntohl(lease_time_align);
+			if (lease_time_align > server_config.lease)
+				lease_time_align = server_config.lease;
+		}
 
-	if(!packet.yiaddr) {
-		LOG(LOG_WARNING, "no IP addresses to give -- OFFER abandoned");
-		return -1;
-	}
-
-	if (!add_lease(packet.chaddr, packet.yiaddr, server_config.offer_time)) {
-		LOG(LOG_WARNING, "lease pool is full -- OFFER abandoned");
-		return -1;
-	}
-
-	if ((lease_time = get_option(oldpacket, DHCP_LEASE_TIME))) {
-		memcpy(&lease_time_align, lease_time, 4);
-		lease_time_align = ntohl(lease_time_align);
-		if (lease_time_align > server_config.lease)
+		/* Make sure we aren't just using the lease time from the previous offer */
+		if (lease_time_align < server_config.min_lease)
 			lease_time_align = server_config.lease;
-	}
-
-	/* Make sure we aren't just using the lease time from the previous offer */
-	if (lease_time_align < server_config.min_lease)
-		lease_time_align = server_config.lease;
-	}
-	/* ADDME: end of short circuit */
-	else
-	{
+		/* ADDME: end of short circuit */
+	} else {
 		/* It is a static lease... use it */
 		packet.yiaddr = static_lease_ip;
 	}
@@ -196,7 +182,7 @@ int sendOffer(struct dhcpMessage *oldpacket)
 	add_bootp_options(&packet);
 
 	addr.s_addr = packet.yiaddr;
-	LOG(LOG_INFO, "sending OFFER of %s", inet_ntoa(addr));
+	bb_info_msg("Sending OFFER of %s", inet_ntoa(addr));
 	return send_packet(&packet, 0);
 }
 
@@ -207,7 +193,7 @@ int sendNAK(struct dhcpMessage *oldpacket)
 
 	init_packet(&packet, oldpacket, DHCPNAK);
 
-	DEBUG(LOG_INFO, "sending NAK");
+	DEBUG("Sending NAK");
 	return send_packet(&packet, 1);
 }
 
@@ -223,7 +209,8 @@ int sendACK(struct dhcpMessage *oldpacket, uint32_t yiaddr)
 	init_packet(&packet, oldpacket, DHCPACK);
 	packet.yiaddr = yiaddr;
 
-	if ((lease_time = get_option(oldpacket, DHCP_LEASE_TIME))) {
+	lease_time = get_option(oldpacket, DHCP_LEASE_TIME);
+	if (lease_time) {
 		memcpy(&lease_time_align, lease_time, 4);
 		lease_time_align = ntohl(lease_time_align);
 		if (lease_time_align > server_config.lease)
@@ -244,12 +231,16 @@ int sendACK(struct dhcpMessage *oldpacket, uint32_t yiaddr)
 	add_bootp_options(&packet);
 
 	addr.s_addr = packet.yiaddr;
-	LOG(LOG_INFO, "sending ACK to %s", inet_ntoa(addr));
+	bb_info_msg("Sending ACK to %s", inet_ntoa(addr));
 
 	if (send_packet(&packet, 0) < 0)
 		return -1;
 
 	add_lease(packet.chaddr, packet.yiaddr, lease_time_align);
+	if (ENABLE_FEATURE_UDHCPD_WRITE_LEASES_EARLY) {
+		/* rewrite the file with leases at every new acceptance */
+		write_leases();
+	}
 
 	return 0;
 }
