@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /* signalpipe.c
  *
  * Signal pipe infrastructure. A reliable way of delivering signals.
@@ -19,23 +20,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <unistd.h>
-#include <fcntl.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/select.h>
-
-
-#include "signalpipe.h"
 #include "common.h"
 
+
 static int signal_pipe[2];
 
 static void signal_handler(int sig)
 {
-	if (send(signal_pipe[1], &sig, sizeof(sig), MSG_DONTWAIT) < 0)
-		DEBUG(LOG_ERR, "Could not send signal: %m");
+	unsigned char ch = sig; /* use char, avoid dealing with partial writes */
+	if (write(signal_pipe[1], &ch, 1) != 1)
+		bb_perror_msg("cannot send signal");
 }
 
 
@@ -43,9 +37,11 @@ static void signal_handler(int sig)
  * and installs the signal handler */
 void udhcp_sp_setup(void)
 {
-	socketpair(AF_UNIX, SOCK_STREAM, 0, signal_pipe);
+	/* was socketpair, but it needs AF_UNIX in kernel */
+	xpipe(signal_pipe);
 	fcntl(signal_pipe[0], F_SETFD, FD_CLOEXEC);
 	fcntl(signal_pipe[1], F_SETFD, FD_CLOEXEC);
+	fcntl(signal_pipe[1], F_SETFL, O_NONBLOCK);
 	signal(SIGUSR1, signal_handler);
 	signal(SIGUSR2, signal_handler);
 	signal(SIGTERM, signal_handler);
@@ -72,12 +68,12 @@ int udhcp_sp_fd_set(fd_set *rfds, int extra_fd)
  * your signal on success */
 int udhcp_sp_read(fd_set *rfds)
 {
-	int sig;
+	unsigned char sig;
 
 	if (!FD_ISSET(signal_pipe[0], rfds))
 		return 0;
 
-	if (read(signal_pipe[0], &sig, sizeof(sig)) < 0)
+	if (read(signal_pipe[0], &sig, 1) != 1)
 		return -1;
 
 	return sig;
