@@ -14,16 +14,7 @@
  *
  */
 
-#include "busybox.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <dirent.h>
-#include <string.h>
-#include <errno.h>
-#include <fcntl.h>
+#include "libbb.h"
 
 /*
  *    Function Prototypes
@@ -36,24 +27,27 @@ static int sysctl_display_all(const char *path, int output, int show_table);
 /*
  *    Globals...
  */
-static const char PROC_PATH[] = "/proc/sys/";
-static const char DEFAULT_PRELOAD[] = "/etc/sysctl.conf";
+static const char PROC_PATH[] ALIGN1 = "/proc/sys/";
+static const char DEFAULT_PRELOAD[] ALIGN1 = "/etc/sysctl.conf";
 
 /* error messages */
-static const char ERR_UNKNOWN_PARAMETER[] = "error: Unknown parameter '%s'\n";
-static const char ERR_MALFORMED_SETTING[] = "error: Malformed setting '%s'\n";
-static const char ERR_NO_EQUALS[] =
+static const char ERR_UNKNOWN_PARAMETER[] ALIGN1 =
+	"error: Unknown parameter '%s'\n";
+static const char ERR_MALFORMED_SETTING[] ALIGN1 =
+	"error: Malformed setting '%s'\n";
+static const char ERR_NO_EQUALS[] ALIGN1 =
 	"error: '%s' must be of the form name=value\n";
-static const char ERR_INVALID_KEY[] = "error: '%s' is an unknown key\n";
-static const char ERR_UNKNOWN_WRITING[] =
+static const char ERR_INVALID_KEY[] ALIGN1 =
+	"error: '%s' is an unknown key\n";
+static const char ERR_UNKNOWN_WRITING[] ALIGN1 =
 	"error: unknown error %d setting key '%s'\n";
-static const char ERR_UNKNOWN_READING[] =
+static const char ERR_UNKNOWN_READING[] ALIGN1 =
 	"error: unknown error %d reading key '%s'\n";
-static const char ERR_PERMISSION_DENIED[] =
+static const char ERR_PERMISSION_DENIED[] ALIGN1 =
 	"error: permission denied on key '%s'\n";
-static const char ERR_PRELOAD_FILE[] =
-	"error: unable to open preload file '%s'\n";
-static const char WARN_BAD_LINE[] =
+static const char ERR_PRELOAD_FILE[] ALIGN1 =
+	"error: cannot open preload file '%s'\n";
+static const char WARN_BAD_LINE[] ALIGN1 =
 	"warning: %s(%d): invalid syntax, continuing...\n";
 
 
@@ -65,6 +59,7 @@ static void dwrite_str(int fd, const char *buf)
 /*
  *    sysctl_main()...
  */
+int sysctl_main(int argc, char **argv);
 int sysctl_main(int argc, char **argv)
 {
 	int retval = 0;
@@ -138,7 +133,7 @@ int sysctl_preload_file(const char *filename, int output)
 	}
 
 	while (fgets(oneline, sizeof(oneline) - 1, fp)) {
-		oneline[sizeof(oneline) - 1] = 0;
+		oneline[sizeof(oneline) - 1] = '\0';
 		lineno++;
 		trim(oneline);
 		ptr = (char *) oneline;
@@ -165,9 +160,8 @@ int sysctl_preload_file(const char *filename, int output)
 
 		while ((*value == ' ' || *value == '\t') && *value != 0)
 			value++;
-		strcpy(buffer, name);
-		strcat(buffer, "=");
-		strcat(buffer, value);
+		/* safe because sizeof(oneline) == sizeof(buffer) */
+		sprintf(buffer, "%s=%s", name, value);
 		sysctl_write_setting(buffer, output);
 	}
 	fclose(fp);
@@ -202,8 +196,8 @@ int sysctl_write_setting(const char *setting, int output)
 		return -2;
 	}
 
-	tmpname = bb_xasprintf("%s%.*s", PROC_PATH, (int)(equals - name), name);
-	outname = bb_xstrdup(tmpname + strlen(PROC_PATH));
+	tmpname = xasprintf("%s%.*s", PROC_PATH, (int)(equals - name), name);
+	outname = xstrdup(tmpname + strlen(PROC_PATH));
 
 	while ((cptr = strchr(tmpname, '.')) != NULL)
 		*cptr = '/';
@@ -211,7 +205,8 @@ int sysctl_write_setting(const char *setting, int output)
 	while ((cptr = strchr(outname, '/')) != NULL)
 		*cptr = '.';
 
-	if ((fd = open(tmpname, O_WRONLY | O_CREAT | O_TRUNC, 0666)) < 0) {
+	fd = open(tmpname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+	if (fd < 0) {
 		switch (errno) {
 		case ENOENT:
 			bb_error_msg(ERR_INVALID_KEY, outname);
@@ -258,7 +253,7 @@ int sysctl_read_setting(const char *setting, int output)
 		bb_error_msg(ERR_INVALID_KEY, setting);
 
 	tmpname = concat_path_file(PROC_PATH, name);
-	outname = bb_xstrdup(tmpname + strlen(PROC_PATH));
+	outname = xstrdup(tmpname + strlen(PROC_PATH));
 
 	while ((cptr = strchr(tmpname, '.')) != NULL)
 		*cptr = '/';
@@ -309,14 +304,16 @@ int sysctl_display_all(const char *path, int output, int show_table)
 	char *tmpdir;
 	struct stat ts;
 
-	if (!(dp = bb_opendir(path))) {
+	dp = opendir(path);
+	if (!dp) {
 		retval = -1;
 	} else {
 		while ((de = readdir(dp)) != NULL) {
 			tmpdir = concat_subpath_file(path, de->d_name);
-			if(tmpdir == NULL)
+			if (tmpdir == NULL)
 				continue;
-			if ((retval2 = stat(tmpdir, &ts)) != 0)
+			retval2 = stat(tmpdir, &ts);
+			if (retval2 != 0)
 				bb_perror_msg(tmpdir);
 			else {
 				if (S_ISDIR(ts.st_mode)) {
