@@ -28,11 +28,11 @@
  * GNU Library General Public License
  */
 
-#include "busybox.h"
+#include "libbb.h"
 
 
-typedef struct {
-	unsigned long rss;
+typedef struct top_status_t {
+	unsigned long vsz;
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
 	unsigned long ticks;
 	unsigned pcpu; /* delta of ticks */
@@ -42,24 +42,60 @@ typedef struct {
 	char state[4];
 	char comm[COMM_LEN];
 } top_status_t;
-static top_status_t *top;
-static int ntop;
+
+typedef struct jiffy_counts_t {
+	unsigned long long usr,nic,sys,idle,iowait,irq,softirq,steal;
+	unsigned long long total;
+	unsigned long long busy;
+} jiffy_counts_t;
+
 /* This structure stores some critical information from one frame to
    the next. Used for finding deltas. */
-struct save_hist {
+typedef struct save_hist {
 	unsigned long ticks;
 	unsigned pid;
+} save_hist;
+
+typedef int (*cmp_funcp)(top_status_t *P, top_status_t *Q);
+
+enum { SORT_DEPTH = 3 };
+
+struct globals {
+	top_status_t *top;
+	int ntop;
+#if ENABLE_FEATURE_USE_TERMIOS
+	struct termios initial_settings;
+#endif
+#if !ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
+	cmp_funcp sort_function;
+#else
+	cmp_funcp sort_function[SORT_DEPTH];
+	struct save_hist *prev_hist;
+	int prev_hist_count;
+	jiffy_counts_t jif, prev_jif;
+	/* int hist_iterations; */
+	unsigned total_pcpu;
+	/* unsigned long total_vsz; */
+#endif
 };
+#define G (*(struct globals*)&bb_common_bufsiz1)
+#define top              (G.top               )
+#define ntop             (G.ntop              )
+#if ENABLE_FEATURE_USE_TERMIOS
+#define initial_settings (G. initial_settings )
+#endif
+#define sort_function    (G.sort_function     )
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
-static struct save_hist *prev_hist;
-static int prev_hist_count;
-/* static int hist_iterations; */
-static unsigned total_pcpu;
-/* static unsigned long total_rss; */
+#define prev_hist        (G.prev_hist         )
+#define prev_hist_count  (G.prev_hist_count   )
+#define jif              (G.jif               )
+#define prev_jif         (G.prev_jif          )
+#define total_pcpu       (G.total_pcpu        )
 #endif
 
 #define OPT_BATCH_MODE (option_mask32 & 0x4)
 
+
 #if ENABLE_FEATURE_USE_TERMIOS
 static int pid_sort(top_status_t *P, top_status_t *Q)
 {
@@ -72,22 +108,12 @@ static int pid_sort(top_status_t *P, top_status_t *Q)
 static int mem_sort(top_status_t *P, top_status_t *Q)
 {
 	/* We want to avoid unsigned->signed and truncation errors */
-	if (Q->rss < P->rss) return -1;
-	return Q->rss != P->rss; /* 0 if ==, 1 if > */
+	if (Q->vsz < P->vsz) return -1;
+	return Q->vsz != P->vsz; /* 0 if ==, 1 if > */
 }
 
 
-typedef int (*cmp_funcp)(top_status_t *P, top_status_t *Q);
-
-#if !ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
-
-static cmp_funcp sort_function;
-
-#else
-
-enum { SORT_DEPTH = 3 };
-
-static cmp_funcp sort_function[SORT_DEPTH];
+#if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
 
 static int pcpu_sort(top_status_t *P, top_status_t *Q)
 {
@@ -103,7 +129,8 @@ static int time_sort(top_status_t *P, top_status_t *Q)
 	return Q->ticks != P->ticks; /* 0 if ==, 1 if > */
 }
 
-static int mult_lvl_cmp(void* a, void* b) {
+static int mult_lvl_cmp(void* a, void* b)
+{
 	int i, cmp_val;
 
 	for (i = 0; i < SORT_DEPTH; i++) {
@@ -115,12 +142,6 @@ static int mult_lvl_cmp(void* a, void* b) {
 }
 
 
-typedef struct {
-	unsigned long long usr,nic,sys,idle,iowait,irq,softirq,steal;
-	unsigned long long total;
-	unsigned long long busy;
-} jiffy_counts_t;
-static jiffy_counts_t jif, prev_jif;
 static void get_jiffy_counts(void)
 {
 	FILE* fp = xfopen("stat", "r");
@@ -147,7 +168,7 @@ static void do_stats(void)
 
 	get_jiffy_counts();
 	total_pcpu = 0;
-	/* total_rss = 0; */
+	/* total_vsz = 0; */
 	new_hist = xmalloc(sizeof(struct save_hist)*ntop);
 	/*
 	 * Make a pass through the data to get stats.
@@ -179,7 +200,7 @@ static void do_stats(void)
 			i = (i+1) % prev_hist_count;
 			/* hist_iterations++; */
 		} while (i != last_i);
-		/* total_rss += cur->rss; */
+		/* total_vsz += cur->vsz; */
 	}
 
 	/*
@@ -191,6 +212,30 @@ static void do_stats(void)
 }
 #endif /* FEATURE_TOP_CPU_USAGE_PERCENTAGE */
 
+#if ENABLE_FEATURE_TOP_CPU_GLOBAL_PERCENTS && ENABLE_FEATURE_TOP_DECIMALS
+/* formats 7 char string (8 with terminating NUL) */
+static char *fmt_100percent_8(char pbuf[8], unsigned value, unsigned total)
+{
+	unsigned t;
+	if (value >= total) { /* 100% ? */
+		strcpy(pbuf, "  100% ");
+		return pbuf;
+	}
+	/* else generate " [N/space]N.N% " string */
+	value = 1000 * value / total;
+	t = value / 100;
+	value = value % 100;
+	pbuf[0] = ' ';
+	pbuf[1] = t ? t + '0' : ' ';
+	pbuf[2] = '0' + (value / 10);
+	pbuf[3] = '.';
+	pbuf[4] = '0' + (value % 10);
+	pbuf[5] = '%';
+	pbuf[6] = ' ';
+	pbuf[7] = '\0';
+	return pbuf;
+}
+#endif
 
 /* display generic info (meminfo / loadavg) */
 static unsigned long display_generic(int scr_width)
@@ -198,9 +243,10 @@ static unsigned long display_generic(int scr_width)
 	FILE *fp;
 	char buf[80];
 	char scrbuf[80];
-	char *end;
 	unsigned long total, used, mfree, shared, buffers, cached;
-	unsigned int needs_conversion = 1;
+#if ENABLE_FEATURE_TOP_CPU_GLOBAL_PERCENTS
+	unsigned total_diff;
+#endif
 
 	/* read memory info */
 	fp = xfopen("meminfo", "r");
@@ -219,14 +265,20 @@ static unsigned long display_generic(int scr_width)
 		fgets(buf, sizeof(buf), fp);    /* skip first line */
 
 		fscanf(fp, "Mem: %lu %lu %lu %lu %lu %lu",
-		   &total, &used, &mfree, &shared, &buffers, &cached);
+			&total, &used, &mfree, &shared, &buffers, &cached);
+		/* convert to kilobytes */
+		used /= 1024;
+		mfree /= 1024;
+		shared /= 1024;
+		buffers /= 1024;
+		cached /= 1024;
+		total /= 1024;
 	} else {
 		/*
 		 * Revert to manual parsing, which incidentally already has the
 		 * sizes in kilobytes. This should be safe for both 2.4 and
 		 * 2.6.
 		 */
-		needs_conversion = 0;
 
 		fscanf(fp, "MemFree: %lu %s\n", &mfree, buf);
 
@@ -244,79 +296,119 @@ static unsigned long display_generic(int scr_width)
 	}
 	fclose(fp);
 
-	/* read load average as a string */
-	buf[0] = '\0';
-	open_read_close("loadavg", buf, sizeof(buf));
-	end = strchr(buf, ' ');
-	if (end) end = strchr(end+1, ' ');
-	if (end) end = strchr(end+1, ' ');
-	if (end) *end = '\0';
-
-	if (needs_conversion) {
-		/* convert to kilobytes */
-		used /= 1024;
-		mfree /= 1024;
-		shared /= 1024;
-		buffers /= 1024;
-		cached /= 1024;
-		total /= 1024;
-	}
-
-	/* output memory info and load average */
-	/* clear screen & go to top */
+	/* output memory info */
 	if (scr_width > sizeof(scrbuf))
 		scr_width = sizeof(scrbuf);
 	snprintf(scrbuf, scr_width,
-		"Mem: %ldK used, %ldK free, %ldK shrd, %ldK buff, %ldK cached",
+		"Mem: %luK used, %luK free, %luK shrd, %luK buff, %luK cached",
 		used, mfree, shared, buffers, cached);
-
+	/* clear screen & go to top */
 	printf(OPT_BATCH_MODE ? "%s\n" : "\e[H\e[J%s\n", scrbuf);
 
+#if ENABLE_FEATURE_TOP_CPU_GLOBAL_PERCENTS
+	/*
+	 * xxx% = (jif.xxx - prev_jif.xxx) / (jif.total - prev_jif.total) * 100%
+	 */
+	/* using (unsigned) casts to make operations cheaper */
+	total_diff = ((unsigned)(jif.total - prev_jif.total) ? : 1);
+#if ENABLE_FEATURE_TOP_DECIMALS
+/* Generated code is approx +0.3k */
+#define CALC_STAT(xxx) char xxx[8]
+#define SHOW_STAT(xxx) fmt_100percent_8(xxx, (unsigned)(jif.xxx - prev_jif.xxx), total_diff)
+#define FMT "%s"
+#else
+#define CALC_STAT(xxx) unsigned xxx = 100 * (unsigned)(jif.xxx - prev_jif.xxx) / total_diff
+#define SHOW_STAT(xxx) xxx
+#define FMT "%4u%% "
+#endif
+	{ /* need block: CALC_STAT are declarations */
+		CALC_STAT(usr);
+		CALC_STAT(sys);
+		CALC_STAT(nic);
+		CALC_STAT(idle);
+		CALC_STAT(iowait);
+		CALC_STAT(irq);
+		CALC_STAT(softirq);
+		//CALC_STAT(steal);
+
+		snprintf(scrbuf, scr_width,
+			/* Barely fits in 79 chars when in "decimals" mode. */
+			"CPU:"FMT"usr"FMT"sys"FMT"nice"FMT"idle"FMT"io"FMT"irq"FMT"softirq",
+			SHOW_STAT(usr), SHOW_STAT(sys), SHOW_STAT(nic), SHOW_STAT(idle),
+			SHOW_STAT(iowait), SHOW_STAT(irq), SHOW_STAT(softirq)
+			//, SHOW_STAT(steal) - what is this 'steal' thing?
+			// I doubt anyone wants to know it
+		);
+	}
+	puts(scrbuf);
+#undef SHOW_STAT
+#undef CALC_STAT
+#undef FMT
+#endif
+
+	/* read load average as a string */
+	buf[0] = '\0';
+	open_read_close("loadavg", buf, sizeof("N.NN N.NN N.NN")-1);
+	buf[sizeof("N.NN N.NN N.NN")-1] = '\0';
 	snprintf(scrbuf, scr_width, "Load average: %s", buf);
-	printf("%s\n", scrbuf);
+	puts(scrbuf);
 
 	return total;
 }
 
-
 /* display process statuses */
 static void display_status(int count, int scr_width)
 {
 	enum {
-		bits_per_int = sizeof(int)*8
+		BITS_PER_INT = sizeof(int)*8
 	};
 
 	top_status_t *s = top;
-	char rss_str_buf[8];
-	unsigned long total_memory = display_generic(scr_width); /* or use total_rss? */
-	unsigned pmem_shift, pmem_scale;
-
+	char vsz_str_buf[8];
+	unsigned long total_memory = display_generic(scr_width); /* or use total_vsz? */
+	/* xxx_shift and xxx_scale variables allow us to replace
+	 * expensive divides with multiply and shift */
+	unsigned pmem_shift, pmem_scale, pmem_half;
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
-	unsigned pcpu_shift, pcpu_scale;
+	unsigned pcpu_shift, pcpu_scale, pcpu_half;
 	unsigned busy_jifs;
 
 	/* what info of the processes is shown */
 	printf(OPT_BATCH_MODE ? "%.*s" : "\e[7m%.*s\e[0m", scr_width,
-		"  PID USER     STATUS   RSS  PPID %CPU %MEM COMMAND");
+		"  PID  PPID USER     STAT   VSZ %MEM %CPU COMMAND");
 #define MIN_WIDTH \
-	sizeof( "  PID USER     STATUS   RSS  PPID %CPU %MEM C")
+	sizeof( "  PID  PPID USER     STAT   VSZ %MEM %CPU C")
 #else
+
+	/* !CPU_USAGE_PERCENTAGE */
 	printf(OPT_BATCH_MODE ? "%.*s" : "\e[7m%.*s\e[0m", scr_width,
-		"  PID USER     STATUS   RSS  PPID %MEM COMMAND");
+		"  PID  PPID USER     STAT   VSZ %MEM COMMAND");
 #define MIN_WIDTH \
-	sizeof( "  PID USER     STATUS   RSS  PPID %MEM C")
+	sizeof( "  PID  PPID USER     STAT   VSZ %MEM C")
 #endif
 
+#if ENABLE_FEATURE_TOP_DECIMALS
+#define UPSCALE 1000
+#define CALC_STAT(name, val) div_t name = div((val), 10)
+#define SHOW_STAT(name) name.quot, '0'+name.rem
+#define FMT "%3u.%c"
+#else
+#define UPSCALE 100
+#define CALC_STAT(name, val) unsigned name = (val)
+#define SHOW_STAT(name) name
+#define FMT "%4u%%"
+#endif
 	/*
-	 * MEM% = s->rss/MemTotal
+	 * MEM% = s->vsz/MemTotal
 	 */
-	pmem_shift = bits_per_int-11;
-	pmem_scale = 1000*(1U<<(bits_per_int-11)) / total_memory;
-	/* s->rss is in kb. we want (s->rss * pmem_scale) to never overflow */
+	pmem_shift = BITS_PER_INT-11;
+	pmem_scale = UPSCALE*(1U<<(BITS_PER_INT-11)) / total_memory;
+	/* s->vsz is in kb. we want (s->vsz * pmem_scale) to never overflow */
 	while (pmem_scale >= 512) {
 		pmem_scale /= 4;
 		pmem_shift -= 2;
 	}
+	pmem_half = (1U << pmem_shift) / (ENABLE_FEATURE_TOP_DECIMALS? 20 : 2);
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
 	busy_jifs = jif.busy - prev_jif.busy;
 	/* This happens if there were lots of short-lived processes
@@ -332,8 +424,8 @@ static void display_status(int count, int scr_width)
 	 * we assume that unsigned is at least 32-bit.
 	 */
 	pcpu_shift = 6;
-	pcpu_scale = (1000*64*(uint16_t)busy_jifs ? : 1);
-	while (pcpu_scale < (1U<<(bits_per_int-2))) {
+	pcpu_scale = (UPSCALE*64*(uint16_t)busy_jifs ? : 1);
+	while (pcpu_scale < (1U<<(BITS_PER_INT-2))) {
 		pcpu_scale *= 4;
 		pcpu_shift += 2;
 	}
@@ -343,30 +435,40 @@ static void display_status(int count, int scr_width)
 		pcpu_scale /= 4;
 		pcpu_shift -= 2;
 	}
+	pcpu_half = (1U << pcpu_shift) / (ENABLE_FEATURE_TOP_DECIMALS? 20 : 2);
 	/* printf(" pmem_scale=%u pcpu_scale=%u ", pmem_scale, pcpu_scale); */
 #endif
-	while (count-- > 0) {
-		div_t pmem = div((s->rss*pmem_scale) >> pmem_shift, 10);
-		int col = scr_width+1;
-		USE_FEATURE_TOP_CPU_USAGE_PERCENTAGE(div_t pcpu;)
 
-		if (s->rss >= 100*1024)
-			sprintf(rss_str_buf, "%6ldM", s->rss/1024);
+	/* Ok, all prelim data is ready, go thru the list */
+	while (count-- > 0) {
+		int col = scr_width;
+		CALC_STAT(pmem, (s->vsz*pmem_scale + pmem_half) >> pmem_shift);
+#if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
+		CALC_STAT(pcpu, (s->pcpu*pcpu_scale + pcpu_half) >> pcpu_shift);
+#endif
+
+		if (s->vsz >= 100*1024)
+			sprintf(vsz_str_buf, "%6ldM", s->vsz/1024);
 		else
-			sprintf(rss_str_buf, "%7ld", s->rss);
-		USE_FEATURE_TOP_CPU_USAGE_PERCENTAGE(
-		pcpu = div((s->pcpu*pcpu_scale) >> pcpu_shift, 10);
-		)
-		col -= printf("\n%5u %-8s %s  "
-				"%s%6u"
-				USE_FEATURE_TOP_CPU_USAGE_PERCENTAGE("%3u.%c")
-				"%3u.%c ",
-				s->pid, get_cached_username(s->uid), s->state,
-				rss_str_buf, s->ppid,
-				USE_FEATURE_TOP_CPU_USAGE_PERCENTAGE(pcpu.quot, '0'+pcpu.rem,)
-				pmem.quot, '0'+pmem.rem);
-		if (col > 0)
-			printf("%.*s", col, s->comm);
+			sprintf(vsz_str_buf, "%7ld", s->vsz);
+		// PID PPID USER STAT VSZ %MEM [%CPU] COMMAND
+		col -= printf("\n" "%5u%6u %-8.8s %s%s" FMT
+#if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
+				FMT
+#endif
+				" ",
+				s->pid, s->ppid, get_cached_username(s->uid),
+				s->state, vsz_str_buf,
+				SHOW_STAT(pmem)
+#if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
+				, SHOW_STAT(pcpu)
+#endif
+		);
+		if (col > 0) {
+			char buf[col + 1];
+			read_cmdline(buf, col, s->pid, s->comm);
+			fputs(buf, stdout);
+		}
 		/* printf(" %d/%d %lld/%lld", s->pcpu, total_pcpu,
 			jif.busy - prev_jif.busy, jif.total - prev_jif.total); */
 		s++;
@@ -375,6 +477,10 @@ static void display_status(int count, int scr_width)
 	putchar(OPT_BATCH_MODE ? '\n' : '\r');
 	fflush(stdout);
 }
+#undef UPSCALE
+#undef SHOW_STAT
+#undef CALC_STAT
+#undef FMT
 
 
 static void clearmems(void)
@@ -390,8 +496,6 @@ static void clearmems(void)
 #include <termios.h>
 #include <signal.h>
 
-static struct termios initial_settings;
-
 static void reset_term(void)
 {
 	tcsetattr(0, TCSANOW, (void *) &initial_settings);
@@ -411,6 +515,7 @@ static void sig_catcher(int sig ATTRIBUTE_UNUSED)
 #endif /* FEATURE_USE_TERMIOS */
 
 
+int top_main(int argc, char **argv);
 int top_main(int argc, char **argv)
 {
 	int count, lines, col;
@@ -424,10 +529,11 @@ int top_main(int argc, char **argv)
 	unsigned char c;
 #endif /* FEATURE_USE_TERMIOS */
 
-	/* do normal option parsing */
 	interval = 5;
+
+	/* do normal option parsing */
 	opt_complementary = "-";
-	getopt32(argc, argv, "d:n:b", &sinterval, &siterations);
+	getopt32(argv, "d:n:b", &sinterval, &siterations);
 	if (option_mask32 & 0x1) interval = xatou(sinterval); // -d
 	if (option_mask32 & 0x2) iterations = xatou(siterations); // -n
 	//if (option_mask32 & 0x4) // -b
@@ -458,7 +564,7 @@ int top_main(int argc, char **argv)
 		procps_status_t *p = NULL;
 
 		/* Default to 25 lines - 5 lines for status */
-		lines = 24 - 3;
+		lines = 24 - 3 USE_FEATURE_TOP_CPU_GLOBAL_PERCENTS( - 1);
 		col = 79;
 #if ENABLE_FEATURE_USE_TERMIOS
 		get_terminal_width_height(0, &col, &lines);
@@ -466,14 +572,14 @@ int top_main(int argc, char **argv)
 			sleep(interval);
 			continue;
 		}
-		lines -= 3;
+		lines -= 3 USE_FEATURE_TOP_CPU_GLOBAL_PERCENTS( + 1);
 #endif /* FEATURE_USE_TERMIOS */
 
 		/* read process IDs & status for all the processes */
 		while ((p = procps_scan(p, 0
 				| PSSCAN_PID
 				| PSSCAN_PPID
-				| PSSCAN_RSS
+				| PSSCAN_VSZ
 				| PSSCAN_STIME
 				| PSSCAN_UTIME
 				| PSSCAN_STATE
@@ -482,10 +588,10 @@ int top_main(int argc, char **argv)
 				| PSSCAN_UIDGID
 		))) {
 			int n = ntop;
-			top = xrealloc(top, (++ntop)*sizeof(top_status_t));
+			top = xrealloc(top, (++ntop) * sizeof(*top));
 			top[n].pid = p->pid;
 			top[n].ppid = p->ppid;
-			top[n].rss = p->rss;
+			top[n].vsz = p->vsz;
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
 			top[n].ticks = p->stime + p->utime;
 #endif
@@ -494,7 +600,7 @@ int top_main(int argc, char **argv)
 			strcpy(top[n].comm, p->comm);
 		}
 		if (ntop == 0) {
-			bb_error_msg_and_die("can't find process info in /proc");
+			bb_error_msg_and_die("no process info in /proc");
 		}
 #if ENABLE_FEATURE_TOP_CPU_USAGE_PERCENTAGE
 		if (!prev_hist_count) {
@@ -504,6 +610,7 @@ int top_main(int argc, char **argv)
 			continue;
 		}
 		do_stats();
+/* TODO: we don't need to sort all 10000 processes, we need to find top 24! */
 		qsort(top, ntop, sizeof(top_status_t), (void*)mult_lvl_cmp);
 #else
 		qsort(top, ntop, sizeof(top_status_t), (void*)sort_function);
