@@ -20,23 +20,25 @@
 //For debugging/development on the shell only...
 //#define DEBUG_SHELL
 
-
-#include "busybox.h"
 #include <getopt.h>
-#include "cmdedit.h"
-
 #include <glob.h>
+
+#include "busybox.h" /* for struct bb_applet */
+
 #define expand_t	glob_t
 
 /* Always enable for the moment... */
 #define CONFIG_LASH_PIPE_N_REDIRECTS
 #define CONFIG_LASH_JOB_CONTROL
+#define ENABLE_LASH_PIPE_N_REDIRECTS 1
+#define ENABLE_LASH_JOB_CONTROL      1
 
-static const int MAX_READ = 128;	/* size of input buffer for `read' builtin */
+
+enum { MAX_READ = 128 }; /* size of input buffer for 'read' builtin */
 #define JOB_STATUS_FORMAT "[%d] %-22s %.40s\n"
 
 
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 enum redir_type { REDIRECT_INPUT, REDIRECT_OVERWRITE,
 	REDIRECT_APPEND
 };
@@ -53,7 +55,7 @@ enum {
 #define LASH_OPT_DONE (1)
 #define LASH_OPT_SAW_QUOTE (2)
 
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 struct redir_struct {
 	enum redir_type type;	/* type of redirection */
 	int fd;						/* file descriptor being redirected */
@@ -67,7 +69,7 @@ struct child_prog {
 	int num_redirects;			/* elements in redirection array */
 	int is_stopped;				/* is the program currently running? */
 	struct job *family;			/* pointer back to the child's parent job */
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 	struct redir_struct *redirects;	/* I/O redirects */
 #endif
 };
@@ -92,8 +94,8 @@ struct job {
 };
 
 struct built_in_command {
-	char *cmd;					/* name */
-	char *descr;				/* description */
+	const char *cmd;   /* name */
+	const char *descr; /* description */
 	int (*function) (struct child_prog *);	/* function ptr */
 };
 
@@ -114,7 +116,7 @@ static int builtin_read(struct child_prog *cmd);
 /* function prototypes for shell stuff */
 static void checkjobs(struct jobset *job_list);
 static void remove_job(struct jobset *j_list, struct job *job);
-static int get_command(FILE * source, char *command);
+static int get_command_bufsiz(FILE * source, char *command);
 static int parse_command(char **command_ptr, struct job *job, int *inbg);
 static int run_command(struct job *newjob, int inbg, int outpipe[2]);
 static int pseudo_exec(struct child_prog *cmd) ATTRIBUTE_NORETURN;
@@ -124,35 +126,35 @@ static int busy_loop(FILE * input);
 /* Table of built-in functions (these are non-forking builtins, meaning they
  * can change global variables in the parent shell process but they will not
  * work with pipes and redirects; 'unset foo | whatever' will not work) */
-static struct built_in_command bltins[] = {
-	{"bg", "Resume a job in the background", builtin_fg_bg},
-	{"cd", "Change working directory", builtin_cd},
-	{"exec", "Exec command, replacing this shell with the exec'd process", builtin_exec},
-	{"exit", "Exit from shell()", builtin_exit},
-	{"fg", "Bring job into the foreground", builtin_fg_bg},
-	{"jobs", "Lists the active jobs", builtin_jobs},
+static const struct built_in_command bltins[] = {
+	{"bg"    , "Resume a job in the background", builtin_fg_bg},
+	{"cd"    , "Change working directory", builtin_cd},
+	{"exec"  , "Exec command, replacing this shell with the exec'd process", builtin_exec},
+	{"exit"  , "Exit from shell()", builtin_exit},
+	{"fg"    , "Bring job into the foreground", builtin_fg_bg},
+	{"jobs"  , "Lists the active jobs", builtin_jobs},
 	{"export", "Set environment variable", builtin_export},
-	{"unset", "Unset environment variable", builtin_unset},
-	{"read", "Input environment variable", builtin_read},
-	{".", "Source-in and run commands in a file", builtin_source},
+	{"unset" , "Unset environment variable", builtin_unset},
+	{"read"  , "Input environment variable", builtin_read},
+	{"."     , "Source-in and run commands in a file", builtin_source},
+	/* These were "forked applets", but distinction was nuked */
+	/* Original comment retained: */
+	/* Table of forking built-in functions (things that fork cannot change global
+	 * variables in the parent process, such as the current working directory) */
+	{"pwd"   , "Print current directory", builtin_pwd},
+	{"help"  , "List shell built-in commands", builtin_help},
 	/* to do: add ulimit */
-	{NULL, NULL, NULL}
 };
 
-/* Table of forking built-in functions (things that fork cannot change global
- * variables in the parent process, such as the current working directory) */
-static struct built_in_command bltins_forking[] = {
-	{"pwd", "Print current directory", builtin_pwd},
-	{"help", "List shell built-in commands", builtin_help},
-	{NULL, NULL, NULL}
-};
+
+#define VEC_LAST(v) v[ARRAY_SIZE(v)-1]
 
 
 static int shell_context;  /* Type prompt trigger (PS1 or PS2) */
 
 
 /* Globals that are static to this file */
-static const char *cwd;
+static char *cwd;
 static char *local_pending_command;
 static struct jobset job_list = { NULL, NULL };
 static int argc;
@@ -162,8 +164,8 @@ static int last_return_code;
 static int last_bg_pid;
 static unsigned int last_jobid;
 static int shell_terminal;
-static char *PS1;
-static char *PS2 = "> ";
+static const char *PS1;
+static const char *PS2 = "> ";
 
 
 #ifdef DEBUG_SHELL
@@ -206,6 +208,14 @@ So   cmd->text        becomes  child->family->text
      job_list          becomes  child->family->job_list
  */
 
+
+static void update_cwd(void)
+{
+	cwd = xrealloc_getcwd_or_warn(cwd);
+	if (!cwd)
+		cwd = xstrdup(bb_msg_unknown);
+}
+
 /* built-in 'cd <path>' handler */
 static int builtin_cd(struct child_prog *child)
 {
@@ -219,9 +229,7 @@ static int builtin_cd(struct child_prog *child)
 		bb_perror_msg("cd: %s", newdir);
 		return EXIT_FAILURE;
 	}
-	cwd = xgetcwd((char *)cwd);
-	if (!cwd)
-		cwd = bb_msg_unknown;
+	update_cwd();
 	return EXIT_SUCCESS;
 }
 
@@ -231,7 +239,8 @@ static int builtin_exec(struct child_prog *child)
 	if (child->argv[1] == NULL)
 		return EXIT_SUCCESS;   /* Really? */
 	child->argv++;
-	while(close_me_list) close((long)llist_pop(&close_me_list));
+	while (close_me_list)
+		close((long)llist_pop(&close_me_list));
 	pseudo_exec(child);
 	/* never returns */
 }
@@ -242,42 +251,37 @@ static int builtin_exit(struct child_prog *child)
 	if (child->argv[1] == NULL)
 		exit(EXIT_SUCCESS);
 
-	exit (atoi(child->argv[1]));
+	exit(atoi(child->argv[1]));
 }
 
 /* built-in 'fg' and 'bg' handler */
 static int builtin_fg_bg(struct child_prog *child)
 {
 	int i, jobnum;
-	struct job *job=NULL;
+	struct job *job;
 
 	/* If they gave us no args, assume they want the last backgrounded task */
 	if (!child->argv[1]) {
 		for (job = child->family->job_list->head; job; job = job->next) {
 			if (job->jobid == last_jobid) {
-				break;
+				goto found;
 			}
 		}
-		if (!job) {
-			bb_error_msg("%s: no current job", child->argv[0]);
-			return EXIT_FAILURE;
-		}
-	} else {
-		if (sscanf(child->argv[1], "%%%d", &jobnum) != 1) {
-			bb_error_msg(bb_msg_invalid_arg, child->argv[1], child->argv[0]);
-			return EXIT_FAILURE;
-		}
-		for (job = child->family->job_list->head; job; job = job->next) {
-			if (job->jobid == jobnum) {
-				break;
-			}
-		}
-		if (!job) {
-			bb_error_msg("%s: %d: no such job", child->argv[0], jobnum);
-			return EXIT_FAILURE;
+		bb_error_msg("%s: no current job", child->argv[0]);
+		return EXIT_FAILURE;
+	}
+	if (sscanf(child->argv[1], "%%%d", &jobnum) != 1) {
+		bb_error_msg(bb_msg_invalid_arg, child->argv[1], child->argv[0]);
+		return EXIT_FAILURE;
+	}
+	for (job = child->family->job_list->head; job; job = job->next) {
+		if (job->jobid == jobnum) {
+			goto found;
 		}
 	}
-
+	bb_error_msg("%s: %d: no such job", child->argv[0], jobnum);
+	return EXIT_FAILURE;
+ found:
 	if (*child->argv[0] == 'f') {
 		/* Put the job into the foreground.  */
 		tcsetpgrp(shell_terminal, job->pgrp);
@@ -291,8 +295,9 @@ static int builtin_fg_bg(struct child_prog *child)
 
 	job->stopped_progs = 0;
 
-	if ( (i=kill(- job->pgrp, SIGCONT)) < 0) {
-		if (i == ESRCH) {
+	i = kill(- job->pgrp, SIGCONT);
+	if (i < 0) {
+		if (errno == ESRCH) {
 			remove_job(&job_list, job);
 		} else {
 			bb_perror_msg("kill (SIGCONT)");
@@ -305,17 +310,12 @@ static int builtin_fg_bg(struct child_prog *child)
 /* built-in 'help' handler */
 static int builtin_help(struct child_prog ATTRIBUTE_UNUSED *dummy)
 {
-	struct built_in_command *x;
+	const struct built_in_command *x;
 
 	printf("\nBuilt-in commands:\n"
-		   "-------------------\n");
-	for (x = bltins; x->cmd; x++) {
-		if (x->descr==NULL)
-			continue;
-		printf("%s\t%s\n", x->cmd, x->descr);
-	}
-	for (x = bltins_forking; x->cmd; x++) {
-		if (x->descr==NULL)
+	       "-------------------\n");
+	for (x = bltins; x <= &VEC_LAST(bltins); x++) {
+		if (x->descr == NULL)
 			continue;
 		printf("%s\t%s\n", x->cmd, x->descr);
 	}
@@ -327,7 +327,7 @@ static int builtin_help(struct child_prog ATTRIBUTE_UNUSED *dummy)
 static int builtin_jobs(struct child_prog *child)
 {
 	struct job *job;
-	char *status_string;
+	const char *status_string;
 
 	for (job = child->family->job_list->head; job; job = job->next) {
 		if (job->running_progs == job->stopped_progs)
@@ -344,9 +344,7 @@ static int builtin_jobs(struct child_prog *child)
 /* built-in 'pwd' handler */
 static int builtin_pwd(struct child_prog ATTRIBUTE_UNUSED *dummy)
 {
-	cwd = xgetcwd((char *)cwd);
-	if (!cwd)
-		cwd = bb_msg_unknown;
+	update_cwd();
 	puts(cwd);
 	return EXIT_SUCCESS;
 }
@@ -367,16 +365,16 @@ static int builtin_export(struct child_prog *child)
 	res = putenv(v);
 	if (res)
 		bb_perror_msg("export");
-#ifdef CONFIG_FEATURE_SH_FANCY_PROMPT
-	if (strncmp(v, "PS1=", 4)==0)
+#if ENABLE_FEATURE_EDITING_FANCY_PROMPT
+	if (strncmp(v, "PS1=", 4) == 0)
 		PS1 = getenv("PS1");
 #endif
 
-#ifdef CONFIG_LOCALE_SUPPORT
+#if ENABLE_LOCALE_SUPPORT
 	// TODO: why getenv? "" would be just as good...
-	if(strncmp(v, "LC_ALL=", 7)==0)
+	if (strncmp(v, "LC_ALL=", 7) == 0)
 		setlocale(LC_ALL, getenv("LC_ALL"));
-	if(strncmp(v, "LC_CTYPE=", 9)==0)
+	if (strncmp(v, "LC_CTYPE=", 9) == 0)
 		setlocale(LC_CTYPE, getenv("LC_CTYPE"));
 #endif
 
@@ -406,12 +404,12 @@ static int builtin_read(struct child_prog *child)
 		** the string resides in a static buffer!)
 		*/
 		res = -1;
-		if ((s = strdup(string)))
+		s = strdup(string);
+		if (s)
 			res = putenv(s);
 		if (res)
 			bb_perror_msg("read");
-	}
-	else
+	} else
 		fgets(string, sizeof(string), stdin);
 
 	return res;
@@ -447,7 +445,7 @@ static int builtin_unset(struct child_prog *child)
 	return EXIT_SUCCESS;
 }
 
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 /* free up all memory from a job */
 static void free_job(struct job *cmd)
 {
@@ -456,7 +454,7 @@ static void free_job(struct job *cmd)
 
 	for (i = 0; i < cmd->num_progs; i++) {
 		free(cmd->progs[i].argv);
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 		if (cmd->progs[i].redirects)
 			free(cmd->progs[i].redirects);
 #endif
@@ -511,7 +509,7 @@ static void checkjobs(struct jobset *j_list)
 		}
 
 		/* This happens on backticked commands */
-		if(job==NULL)
+		if (job == NULL)
 			return;
 
 		if (WIFEXITED(status) || WIFSIGNALED(status)) {
@@ -521,7 +519,7 @@ static void checkjobs(struct jobset *j_list)
 
 			if (!job->running_progs) {
 				printf(JOB_STATUS_FORMAT, job->jobid, "Done", job->text);
-				last_jobid=0;
+				last_jobid = 0;
 				remove_job(j_list, job);
 			}
 		} else {
@@ -546,7 +544,7 @@ static void remove_job(struct jobset *j_list, struct job *job)
 }
 #endif
 
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 /* squirrel != NULL means we squirrel away copies of stdin, stdout,
  * and stderr if they are redirected. */
 static int setup_redirects(struct child_prog *prog, int squirrel[])
@@ -569,18 +567,17 @@ static int setup_redirects(struct child_prog *prog, int squirrel[])
 			break;
 		}
 
-		openfd = open(redir->filename, mode, 0666);
+		openfd = open3_or_warn(redir->filename, mode, 0666);
 		if (openfd < 0) {
 			/* this could get lost if stderr has been redirected, but
 			   bash and ash both lose it as well (though zsh doesn't!) */
-			bb_perror_msg("error opening %s", redir->filename);
 			return 1;
 		}
 
 		if (openfd != redir->fd) {
 			if (squirrel && redir->fd < 3) {
 				squirrel[redir->fd] = dup(redir->fd);
-				fcntl (squirrel[redir->fd], F_SETFD, FD_CLOEXEC);
+				fcntl(squirrel[redir->fd], F_SETFD, FD_CLOEXEC);
 			}
 			dup2(openfd, redir->fd);
 			close(openfd);
@@ -593,7 +590,7 @@ static int setup_redirects(struct child_prog *prog, int squirrel[])
 static void restore_redirects(int squirrel[])
 {
 	int i, fd;
-	for (i=0; i<3; i++) {
+	for (i = 0; i < 3; i++) {
 		fd = squirrel[i];
 		if (fd != -1) {
 			/* No error checking.  I sure wouldn't know what
@@ -615,41 +612,46 @@ static inline void restore_redirects(int squirrel[])
 
 static inline void cmdedit_set_initial_prompt(void)
 {
-#ifndef CONFIG_FEATURE_SH_FANCY_PROMPT
+#if !ENABLE_FEATURE_EDITING_FANCY_PROMPT
 	PS1 = NULL;
 #else
 	PS1 = getenv("PS1");
-	if(PS1==0)
+	if (PS1 == 0)
 		PS1 = "\\w \\$ ";
 #endif
 }
 
-static inline void setup_prompt_string(char **prompt_str)
+static inline const char* setup_prompt_string(void)
 {
-#ifndef CONFIG_FEATURE_SH_FANCY_PROMPT
+#if !ENABLE_FEATURE_EDITING_FANCY_PROMPT
 	/* Set up the prompt */
 	if (shell_context == 0) {
-		free(PS1);
-		PS1=xmalloc(strlen(cwd)+4);
-		sprintf(PS1, "%s %c ", cwd, ( geteuid() != 0 ) ? '$': '#');
-		*prompt_str = PS1;
+		char *ns;
+		free((char*)PS1);
+		ns = xmalloc(strlen(cwd)+4);
+		sprintf(ns, "%s %c ", cwd, (geteuid() != 0) ? '$': '#');
+		PS1 = ns;
+		return ns;
 	} else {
-		*prompt_str = PS2;
+		return PS2;
 	}
 #else
-	*prompt_str = (shell_context==0)? PS1 : PS2;
+	return (shell_context == 0)? PS1 : PS2;
 #endif
 }
 
-static int get_command(FILE * source, char *command)
+#if ENABLE_FEATURE_EDITING
+static line_input_t *line_input_state;
+#endif
+
+static int get_command_bufsiz(FILE * source, char *command)
 {
-	char *prompt_str;
+	const char *prompt_str;
 
 	if (source == NULL) {
 		if (local_pending_command) {
 			/* a command specified (-c option): return it & mark it done */
-			strcpy(command, local_pending_command);
-			free(local_pending_command);
+			strncpy(command, local_pending_command, BUFSIZ);
 			local_pending_command = NULL;
 			return 0;
 		}
@@ -657,16 +659,16 @@ static int get_command(FILE * source, char *command)
 	}
 
 	if (source == stdin) {
-		setup_prompt_string(&prompt_str);
+		prompt_str = setup_prompt_string();
 
-#ifdef CONFIG_FEATURE_COMMAND_EDITING
+#if ENABLE_FEATURE_EDITING
 		/*
 		** enable command line editing only while a command line
 		** is actually being read; otherwise, we'll end up bequeathing
 		** atexit() handlers and other unwanted stuff to our
 		** child processes (rob@sysgo.de)
 		*/
-		cmdedit_read_input(prompt_str, command);
+		read_line_input(prompt_str, command, BUFSIZ, line_input_state);
 		return 0;
 #else
 		fputs(prompt_str, stdout);
@@ -682,10 +684,10 @@ static int get_command(FILE * source, char *command)
 	return 0;
 }
 
-static char * strsep_space( char *string, int * ix)
+static char * strsep_space(char *string, int * ix)
 {
 	/* Short circuit the trivial case */
-	if ( !string || ! string[*ix])
+	if (!string || ! string[*ix])
 		return NULL;
 
 	/* Find the end of the token. */
@@ -709,11 +711,12 @@ static char * strsep_space( char *string, int * ix)
 
 static int expand_arguments(char *command)
 {
-	int total_length=0, length, i, retval, ix = 0;
+	static const char out_of_space[] ALIGN1 = "out of space during expansion";
+
+	int total_length = 0, length, i, retval, ix = 0;
 	expand_t expand_result;
 	char *tmpcmd, *cmd, *cmd_copy;
 	char *src, *dst, *var;
-	const char * const out_of_space = "out of space during expansion";
 	int flags = GLOB_NOCHECK
 #ifdef GLOB_BRACE
 		| GLOB_BRACE
@@ -727,7 +730,7 @@ static int expand_arguments(char *command)
 	chomp(command);
 
 	/* Fix up escape sequences to be the Real Thing(tm) */
-	while( command && command[ix]) {
+	while (command && command[ix]) {
 		if (command[ix] == '\\') {
 			const char *tmp = command+ix+1;
 			command[ix] = bb_process_escape_sequence(  &tmp );
@@ -746,7 +749,7 @@ static int expand_arguments(char *command)
 	cmd = cmd_copy = xstrdup(command);
 	*command = '\0';
 	for (ix = 0, tmpcmd = cmd;
-			(tmpcmd = strsep_space(cmd, &ix)) != NULL; cmd += ix, ix=0) {
+			(tmpcmd = strsep_space(cmd, &ix)) != NULL; cmd += ix, ix = 0) {
 		if (*tmpcmd == '\0')
 			break;
 		/* we need to trim() the result for glob! */
@@ -755,7 +758,7 @@ static int expand_arguments(char *command)
 		free(tmpcmd); /* Free mem allocated by strsep_space */
 		if (retval == GLOB_NOSPACE) {
 			/* Mem may have been allocated... */
-			globfree (&expand_result);
+			globfree(&expand_result);
 			bb_error_msg(out_of_space);
 			return FALSE;
 		} else if (retval != 0) {
@@ -767,18 +770,18 @@ static int expand_arguments(char *command)
 		} else {
 			/* Convert from char** (one word per string) to a simple char*,
 			 * but don't overflow command which is BUFSIZ in length */
-			for (i=0; i < expand_result.gl_pathc; i++) {
-				length=strlen(expand_result.gl_pathv[i]);
+			for (i = 0; i < expand_result.gl_pathc; i++) {
+				length = strlen(expand_result.gl_pathv[i]);
 				if (total_length+length+1 >= BUFSIZ) {
 					bb_error_msg(out_of_space);
 					return FALSE;
 				}
 				strcat(command+total_length, " ");
-				total_length+=1;
+				total_length += 1;
 				strcat(command+total_length, expand_result.gl_pathv[i]);
-				total_length+=length;
+				total_length += length;
 			}
-			globfree (&expand_result);
+			globfree(&expand_result);
 		}
 	}
 	free(cmd_copy);
@@ -787,15 +790,15 @@ static int expand_arguments(char *command)
 	/* Now do the shell variable substitutions which
 	 * wordexp can't do for us, namely $? and $! */
 	src = command;
-	while((dst = strchr(src,'$')) != NULL){
+	while ((dst = strchr(src,'$')) != NULL) {
 		var = NULL;
 		switch (*(dst+1)) {
 			case '?':
 				var = itoa(last_return_code);
 				break;
 			case '!':
-				if (last_bg_pid==-1)
-					*(var)='\0';
+				if (last_bg_pid == -1)
+					*var = '\0';
 				else
 					var = itoa(last_bg_pid);
 				break;
@@ -813,9 +816,9 @@ static int expand_arguments(char *command)
 			case '0':case '1':case '2':case '3':case '4':
 			case '5':case '6':case '7':case '8':case '9':
 				{
-					int ixx=*(dst+1)-48+1;
+					int ixx = *(dst+1)-48+1;
 					if (ixx >= argc) {
-						var='\0';
+						var = '\0';
 					} else {
 						var = argv[ixx];
 					}
@@ -826,32 +829,32 @@ static int expand_arguments(char *command)
 		if (var) {
 			/* a single character construction was found, and
 			 * already handled in the case statement */
-			src=dst+2;
+			src = dst + 2;
 		} else {
 			/* Looks like an environment variable */
 			char delim_hold;
-			int num_skip_chars=0;
+			int num_skip_chars = 0;
 			int dstlen = strlen(dst);
 			/* Is this a ${foo} type variable? */
-			if (dstlen >=2 && *(dst+1) == '{') {
-				src=strchr(dst+1, '}');
-				num_skip_chars=1;
+			if (dstlen >= 2 && *(dst+1) == '{') {
+				src = strchr(dst+1, '}');
+				num_skip_chars = 1;
 			} else {
-				src=dst+1;
-				while((isalnum)(*src) || *src=='_') src++;
+				src = dst + 1;
+				while ((isalnum)(*src) || *src == '_') src++;
 			}
 			if (src == NULL) {
 				src = dst+dstlen;
 			}
-			delim_hold=*src;
-			*src='\0';  /* temporary */
+			delim_hold = *src;
+			*src = '\0';  /* temporary */
 			var = getenv(dst + 1 + num_skip_chars);
-			*src=delim_hold;
+			*src = delim_hold;
 			src += num_skip_chars;
 		}
 		if (var == NULL) {
 			/* Seems we got an un-expandable variable.  So delete it. */
-			var = "";
+			var = (char*)"";
 		}
 		{
 			int subst_len = strlen(var);
@@ -887,7 +890,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 	int argv_alloced;
 	char quote = '\0';
 	struct child_prog *prog;
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 	int i;
 	char *chptr;
 #endif
@@ -897,7 +900,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 
 	/* this handles empty lines or leading '#' characters */
 	if (!**command_ptr || (**command_ptr == '#')) {
-		job->num_progs=0;
+		job->num_progs = 0;
 		return 0;
 	}
 
@@ -919,7 +922,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 	prog->num_redirects = 0;
 	prog->is_stopped = 0;
 	prog->family = job;
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 	prog->redirects = NULL;
 #endif
 
@@ -951,14 +954,13 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 					   *src == ']') *buf++ = '\\';
 			*buf++ = *src;
 		} else if (isspace(*src)) {
-			if (*prog->argv[argc_l] || flag & LASH_OPT_SAW_QUOTE) {
+			if (*prog->argv[argc_l] || (flag & LASH_OPT_SAW_QUOTE)) {
 				buf++, argc_l++;
 				/* +1 here leaves room for the NULL which ends argv */
 				if ((argc_l + 1) == argv_alloced) {
 					argv_alloced += 5;
 					prog->argv = xrealloc(prog->argv,
-										  sizeof(*prog->argv) *
-										  argv_alloced);
+							sizeof(*prog->argv) * argv_alloced);
 				}
 				prog->argv[argc_l] = buf;
 				flag ^= LASH_OPT_SAW_QUOTE;
@@ -978,13 +980,12 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 					flag |= LASH_OPT_DONE;
 				break;
 
-#ifdef CONFIG_LASH_PIPE_N_REDIRECTS
+#if ENABLE_LASH_PIPE_N_REDIRECTS
 			case '>':			/* redirects */
 			case '<':
 				i = prog->num_redirects++;
 				prog->redirects = xrealloc(prog->redirects,
-											  sizeof(*prog->redirects) *
-											  (i + 1));
+						sizeof(*prog->redirects) * (i + 1));
 
 				prog->redirects[i].fd = -1;
 				if (buf != prog->argv[argc_l]) {
@@ -1023,7 +1024,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 				if (!*chptr) {
 					bb_error_msg("file name expected after %c", *(src-1));
 					free_job(job);
-					job->num_progs=0;
+					job->num_progs = 0;
 					return 1;
 				}
 
@@ -1047,7 +1048,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 				/* and start the next */
 				job->num_progs++;
 				job->progs = xrealloc(job->progs,
-									  sizeof(*job->progs) * job->num_progs);
+						sizeof(*job->progs) * job->num_progs);
 				prog = job->progs + (job->num_progs - 1);
 				prog->num_redirects = 0;
 				prog->redirects = NULL;
@@ -1066,7 +1067,7 @@ static int parse_command(char **command_ptr, struct job *job, int *inbg)
 empty_command_in_pipe:
 					bb_error_msg("empty command in pipe");
 					free_job(job);
-					job->num_progs=0;
+					job->num_progs = 0;
 					return 1;
 				}
 				src--;			/* we'll ++ it at the end of the loop */
@@ -1074,7 +1075,7 @@ empty_command_in_pipe:
 				break;
 #endif
 
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 			case '&':			/* background */
 				*inbg = 1;
 				/* fallthrough */
@@ -1126,26 +1127,20 @@ empty_command_in_pipe:
  */
 static int pseudo_exec(struct child_prog *child)
 {
-	struct built_in_command *x;
+	const struct built_in_command *x;
 
 	/* Check if the command matches any of the non-forking builtins.
 	 * Depending on context, this might be redundant.  But it's
 	 * easier to waste a few CPU cycles than it is to figure out
 	 * if this is one of those cases.
 	 */
-	for (x = bltins; x->cmd; x++) {
-		if (strcmp(child->argv[0], x->cmd) == 0 ) {
+	/* Check if the command matches any of the forking builtins. */
+	for (x = bltins; x <= &VEC_LAST(bltins); x++) {
+		if (strcmp(child->argv[0], x->cmd) == 0) {
 			_exit(x->function(child));
 		}
 	}
 
-	/* Check if the command matches any of the forking builtins. */
-	for (x = bltins_forking; x->cmd; x++) {
-		if (strcmp(child->argv[0], x->cmd) == 0) {
-			applet_name=x->cmd;
-			_exit (x->function(child));
-		}
-	}
 
 	/* Check if the command matches any busybox internal
 	 * commands ("applets") here.  Following discussions from
@@ -1158,14 +1153,8 @@ static int pseudo_exec(struct child_prog *child)
 	 * /bin/foo invocation will fork and exec /bin/foo, even if
 	 * /bin/foo is a symlink to busybox.
 	 */
-
-	if (ENABLE_FEATURE_SH_STANDALONE_SHELL) {
-		char **argv_l = child->argv;
-		int argc_l;
-
-		for (argc_l=0; *argv_l; argv_l++, argc_l++);
-		optind = 1;
-		run_applet_by_name(child->argv[0], argc_l, child->argv);
+	if (ENABLE_FEATURE_SH_STANDALONE) {
+		run_applet_and_exit(child->argv[0], child->argv);
 	}
 
 	execvp(child->argv[0], child->argv);
@@ -1179,7 +1168,7 @@ static int pseudo_exec(struct child_prog *child)
 static void insert_job(struct job *newjob, int inbg)
 {
 	struct job *thejob;
-	struct jobset *j_list=newjob->job_list;
+	struct jobset *j_list = newjob->job_list;
 
 	/* find the ID for thejob to use */
 	newjob->jobid = 1;
@@ -1201,14 +1190,14 @@ static void insert_job(struct job *newjob, int inbg)
 	thejob->running_progs = thejob->num_progs;
 	thejob->stopped_progs = 0;
 
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 	if (inbg) {
 		/* we don't wait for background thejobs to return -- append it
 		   to the list of backgrounded thejobs and leave it alone */
 		printf("[%d] %d\n", thejob->jobid,
 			   newjob->progs[newjob->num_progs - 1].pid);
 		last_jobid = newjob->jobid;
-		last_bg_pid=newjob->progs[newjob->num_progs - 1].pid;
+		last_bg_pid = newjob->progs[newjob->num_progs - 1].pid;
 	} else {
 		newjob->job_list->fg = thejob;
 
@@ -1225,25 +1214,21 @@ static int run_command(struct job *newjob, int inbg, int outpipe[2])
 	int i;
 	int nextin, nextout;
 	int pipefds[2];				/* pipefd[0] is for reading */
-	struct built_in_command *x;
+	const struct built_in_command *x;
 	struct child_prog *child;
 
-	nextin = 0, nextout = 1;
+	nextin = 0;
 	for (i = 0; i < newjob->num_progs; i++) {
-		child = & (newjob->progs[i]);
+		child = &(newjob->progs[i]);
 
+		nextout = 1;
 		if ((i + 1) < newjob->num_progs) {
-			if (pipe(pipefds)<0) bb_perror_msg_and_die("pipe");
+			xpipe(pipefds);
 			nextout = pipefds[1];
-		} else {
-			if (outpipe[1]!=-1) {
-				nextout = outpipe[1];
-			} else {
-				nextout = 1;
-			}
+		} else if (outpipe[1] != -1) {
+			nextout = outpipe[1];
 		}
 
-
 		/* Check if the command matches any non-forking builtins,
 		 * but only if this is a simple command.
 		 * Non-forking builtins within pipes have to fork anyway,
@@ -1251,30 +1236,42 @@ static int run_command(struct job *newjob, int inbg, int outpipe[2])
 		 * is doomed to failure, and doesn't work on bash, either.
 		 */
 		if (newjob->num_progs == 1) {
+			int rcode;
+			int squirrel[] = {-1, -1, -1};
+
 			/* Check if the command sets an environment variable. */
 			if (strchr(child->argv[0], '=') != NULL) {
 				child->argv[1] = child->argv[0];
 				return builtin_export(child);
 			}
 
-			for (x = bltins; x->cmd; x++) {
-				if (strcmp(child->argv[0], x->cmd) == 0 ) {
-					int rcode;
-					int squirrel[] = {-1, -1, -1};
+			for (x = bltins; x <= &VEC_LAST(bltins); x++) {
+				if (strcmp(child->argv[0], x->cmd) == 0) {
 					setup_redirects(child, squirrel);
 					rcode = x->function(child);
 					restore_redirects(squirrel);
 					return rcode;
 				}
 			}
+#if ENABLE_FEATURE_SH_STANDALONE
+			{
+				const struct bb_applet *a = find_applet_by_name(child->argv[i]);
+				if (a && a->nofork) {
+					setup_redirects(child, squirrel);
+					rcode = run_nofork_applet(a, child->argv + i);
+					restore_redirects(squirrel);
+					return rcode;
+				}
+			}
+#endif
 		}
 
-#if !defined(__UCLIBC__) || defined(__ARCH_HAS_MMU__)
-		if (!(child->pid = fork()))
+#if BB_MMU
+		child->pid = fork();
 #else
-		if (!(child->pid = vfork()))
+		child->pid = vfork();
 #endif
-		{
+		if (!child->pid) {
 			/* Set the handling for job control signals back to the default.  */
 			signal(SIGINT, SIG_DFL);
 			signal(SIGQUIT, SIG_DFL);
@@ -1284,9 +1281,10 @@ static int run_command(struct job *newjob, int inbg, int outpipe[2])
 			signal(SIGCHLD, SIG_DFL);
 
 			/* Close all open filehandles. */
-			while(close_me_list) close((long)llist_pop(&close_me_list));
+			while (close_me_list)
+				close((long)llist_pop(&close_me_list));
 
-			if (outpipe[1]!=-1) {
+			if (outpipe[1] != -1) {
 				close(outpipe[0]);
 			}
 			if (nextin != 0) {
@@ -1306,7 +1304,7 @@ static int run_command(struct job *newjob, int inbg, int outpipe[2])
 
 			pseudo_exec(child);
 		}
-		if (outpipe[1]!=-1) {
+		if (outpipe[1] != -1) {
 			close(outpipe[1]);
 		}
 
@@ -1338,7 +1336,7 @@ static int busy_loop(FILE * input)
 	int i;
 	int inbg = 0;
 	int status;
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 	pid_t  parent_pgrp;
 	/* save current owner of TTY so we can restore it on exit */
 	parent_pgrp = tcgetpgrp(shell_terminal);
@@ -1356,12 +1354,12 @@ static int busy_loop(FILE * input)
 			checkjobs(&job_list);
 
 			if (!next_command) {
-				if (get_command(input, command))
+				if (get_command_bufsiz(input, command))
 					break;
 				next_command = command;
 			}
 
-			if (! expand_arguments(next_command)) {
+			if (!expand_arguments(next_command)) {
 				free(command);
 				command = xzalloc(BUFSIZ);
 				next_command = NULL;
@@ -1370,8 +1368,8 @@ static int busy_loop(FILE * input)
 
 			if (!parse_command(&next_command, &newjob, &inbg) &&
 				newjob.num_progs) {
-				int pipefds[2] = {-1,-1};
-				debug_printf( "job=%p fed to run_command by busy_loop()'\n",
+				int pipefds[2] = { -1, -1 };
+				debug_printf("job=%p fed to run_command by busy_loop()'\n",
 						&newjob);
 				run_command(&newjob, inbg, pipefds);
 			}
@@ -1386,9 +1384,9 @@ static int busy_loop(FILE * input)
 			while (!job_list.fg->progs[i].pid ||
 				   job_list.fg->progs[i].is_stopped == 1) i++;
 
-			if (waitpid(job_list.fg->progs[i].pid, &status, WUNTRACED)<0) {
+			if (waitpid(job_list.fg->progs[i].pid, &status, WUNTRACED) < 0) {
 				if (errno != ECHILD) {
-					bb_perror_msg_and_die("waitpid(%d)",job_list.fg->progs[i].pid);
+					bb_perror_msg_and_die("waitpid(%d)", job_list.fg->progs[i].pid);
 				}
 			}
 
@@ -1397,7 +1395,7 @@ static int busy_loop(FILE * input)
 				job_list.fg->running_progs--;
 				job_list.fg->progs[i].pid = 0;
 
-				last_return_code=WEXITSTATUS(status);
+				last_return_code = WEXITSTATUS(status);
 
 				if (!job_list.fg->running_progs) {
 					/* child exited */
@@ -1405,7 +1403,7 @@ static int busy_loop(FILE * input)
 					job_list.fg = NULL;
 				}
 			}
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 			else {
 				/* the child was stopped */
 				job_list.fg->stopped_progs++;
@@ -1429,7 +1427,7 @@ static int busy_loop(FILE * input)
 	}
 	free(command);
 
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 	/* return controlling TTY back to parent process group before exiting */
 	if (tcsetpgrp(shell_terminal, parent_pgrp) && errno != ENOTTY)
 		bb_perror_msg("tcsetpgrp");
@@ -1442,14 +1440,10 @@ static int busy_loop(FILE * input)
 	return 0;
 }
 
-#ifdef CONFIG_FEATURE_CLEAN_UP
+#if ENABLE_FEATURE_CLEAN_UP
 static void free_memory(void)
 {
-	if (cwd && cwd!=bb_msg_unknown) {
-		free((char*)cwd);
-	}
-	if (local_pending_command)
-		free(local_pending_command);
+	free(cwd);
 
 	if (job_list.fg && !job_list.fg->running_progs) {
 		remove_job(&job_list, job_list.fg);
@@ -1459,7 +1453,7 @@ static void free_memory(void)
 void free_memory(void);
 #endif
 
-#ifdef CONFIG_LASH_JOB_CONTROL
+#if ENABLE_LASH_JOB_CONTROL
 /* Make sure we have a controlling tty.  If we get started under a job
  * aware app (like bash for example), make sure we are now in charge so
  * we don't fight over who gets the foreground */
@@ -1469,11 +1463,12 @@ static void setup_job_control(void)
 	pid_t shell_pgrp;
 
 	/* Loop until we are in the foreground.  */
-	while ((status = tcgetpgrp (shell_terminal)) >= 0) {
-		if (status == (shell_pgrp = getpgrp ())) {
+	while ((status = tcgetpgrp(shell_terminal)) >= 0) {
+		shell_pgrp = getpgrp();
+		if (status == shell_pgrp) {
 			break;
 		}
-		kill (- shell_pgrp, SIGTTIN);
+		kill(- shell_pgrp, SIGTTIN);
 	}
 
 	/* Ignore interactive and job-control signals.  */
@@ -1498,6 +1493,7 @@ static inline void setup_job_control(void)
 }
 #endif
 
+int lash_main(int argc_l, char **argv_l);
 int lash_main(int argc_l, char **argv_l)
 {
 	unsigned opt;
@@ -1505,12 +1501,16 @@ int lash_main(int argc_l, char **argv_l)
 	argc = argc_l;
 	argv = argv_l;
 
+#if ENABLE_FEATURE_EDITING
+	line_input_state = new_line_input_t(FOR_SHELL);
+#endif
+
 	/* These variables need re-initializing when recursing */
 	last_jobid = 0;
 	close_me_list = NULL;
 	job_list.head = NULL;
 	job_list.fg = NULL;
-	last_return_code=1;
+	last_return_code = 1;
 
 	if (argv[0] && argv[0][0] == '-') {
 		FILE *prof_input;
@@ -1524,9 +1524,9 @@ int lash_main(int argc_l, char **argv_l)
 		}
 	}
 
-	opt = getopt32(argc_l, argv_l, "+ic:", &local_pending_command);
+	opt = getopt32(argv_l, "+ic:", &local_pending_command);
 #define LASH_OPT_i (1<<0)
-#define LASH_OPT_c (1<<2)
+#define LASH_OPT_c (1<<1)
 	if (opt & LASH_OPT_c) {
 		input = NULL;
 		optind++;
@@ -1539,18 +1539,18 @@ int lash_main(int argc_l, char **argv_l)
 	 *    standard input is a terminal
 	 *    standard output is a terminal
 	 *    Refer to Posix.2, the description of the `sh' utility. */
-	if (argv[optind]==NULL && input==stdin &&
-			isatty(STDIN_FILENO) && isatty(STDOUT_FILENO))
-	{
+	if (argv[optind] == NULL && input == stdin
+	 && isatty(STDIN_FILENO) && isatty(STDOUT_FILENO)
+	) {
 		opt |= LASH_OPT_i;
 	}
 	setup_job_control();
 	if (opt & LASH_OPT_i) {
 		/* Looks like they want an interactive shell */
 		if (!ENABLE_FEATURE_SH_EXTRA_QUIET) {
-			printf("\n\n%s Built-in shell (lash)\n"
+			printf("\n\n%s built-in shell (lash)\n"
 					"Enter 'help' for a list of built-in commands.\n\n",
-					BB_BANNER);
+					bb_banner);
 		}
 	} else if (!local_pending_command && argv[optind]) {
 		//printf( "optind=%d  argv[optind]='%s'\n", optind, argv[optind]);
@@ -1560,14 +1560,12 @@ int lash_main(int argc_l, char **argv_l)
 	}
 
 	/* initialize the cwd -- this is never freed...*/
-	cwd = xgetcwd(0);
-	if (!cwd)
-		cwd = bb_msg_unknown;
+	update_cwd();
 
 	if (ENABLE_FEATURE_CLEAN_UP) atexit(free_memory);
 
-	if (ENABLE_FEATURE_COMMAND_EDITING) cmdedit_set_initial_prompt();
+	if (ENABLE_FEATURE_EDITING) cmdedit_set_initial_prompt();
 	else PS1 = NULL;
 
-	return (busy_loop(input));
+	return busy_loop(input);
 }
