@@ -13,63 +13,121 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include "busybox.h"
+#include "libbb.h"
 #include <paths.h>
 #include <sys/un.h>
 
-/* SYSLOG_NAMES defined to pull some extra junk from syslog.h */
+/* SYSLOG_NAMES defined to pull prioritynames[] and facilitynames[]
+ * from syslog.h. Grrrr - glibc puts those in _rwdata_! :( */
 #define SYSLOG_NAMES
+#define SYSLOG_NAMES_CONST /* uclibc is saner :) */
 #include <sys/syslog.h>
 #include <sys/uio.h>
 
-#define DEBUG 0
-
-/* Path to the unix socket */
-static char *dev_log_name;
-
-/* Path for the file where all log messages are written */
-static const char *logFilePath = "/var/log/messages";
-static int logFD = -1;
-
-/* interval between marks in seconds */
-static int markInterval = 20 * 60;
-
-/* level of messages to be locally logged */
-static int logLevel = 8;
-
-/* localhost's name */
-static char localHostName[64];
-
-#if ENABLE_FEATURE_ROTATE_LOGFILE
-/* max size of message file before being rotated */
-static unsigned logFileSize = 200 * 1024;
-/* number of rotated message files */
-static unsigned logFileRotate = 1;
-static unsigned curFileSize;
-static smallint isRegular;
-#endif
-
 #if ENABLE_FEATURE_REMOTE_LOG
 #include <netinet/in.h>
-/* udp socket for logging to remote host */
-static int remoteFD = -1;
-static len_and_sockaddr* remoteAddr;
 #endif
 
-/* We are using bb_common_bufsiz1 for buffering: */
-enum { MAX_READ = (BUFSIZ/6) & ~0xf };
-/* We recv into RECVBUF... (size: MAX_READ ~== BUFSIZ/6) */
-#define RECVBUF  bb_common_bufsiz1
-/* ...then copy to PARSEBUF, escaping control chars */
-/* (can grow x2 max ~== BUFSIZ/3) */
-#define PARSEBUF (bb_common_bufsiz1 + MAX_READ)
-/* ...then sprintf into PRINTBUF, adding timestamp (15 chars),
- * host (64), fac.prio (20) to the message */
-/* (growth by: 15 + 64 + 20 + delims = ~110) */
-#define PRINTBUF (bb_common_bufsiz1 + 3*MAX_READ)
-/* totals: BUFSIZ == BUFSIZ/6 + BUFSIZ/3 + (BUFSIZ/3+BUFSIZ/6)
- * -- we have BUFSIZ/6 extra at the ent of PRINTBUF
- * which covers needed ~110 extra bytes (and much more) */
+#if ENABLE_FEATURE_IPC_SYSLOG
+#include <sys/ipc.h>
+#include <sys/sem.h>
+#include <sys/shm.h>
+#endif
+
+
+#define DEBUG 0
+
+/* MARK code is not very useful, is bloat, and broken:
+ * can deadlock if alarmed to make MARK while writing to IPC buffer
+ * (semaphores are down but do_mark routine tries to down them again) */
+#undef SYSLOGD_MARK
+
+enum { MAX_READ = 256 };
+
+/* Semaphore operation structures */
+struct shbuf_ds {
+	int32_t size;   /* size of data - 1 */
+	int32_t tail;   /* end of message list */
+	char data[1];   /* data/messages */
+};
+
+/* Allows us to have smaller initializer. Ugly. */
+#define GLOBALS \
+	const char *logFilePath;                \
+	int logFD;                              \
+	/* interval between marks in seconds */ \
+	/*int markInterval;*/                   \
+	/* level of messages to be logged */    \
+	int logLevel;                           \
+USE_FEATURE_ROTATE_LOGFILE( \
+	/* max size of file before rotation */  \
+	unsigned logFileSize;                   \
+	/* number of rotated message files */   \
+	unsigned logFileRotate;                 \
+	unsigned curFileSize;                   \
+	smallint isRegular;                     \
+) \
+USE_FEATURE_REMOTE_LOG( \
+	/* udp socket for remote logging */     \
+	int remoteFD;                           \
+	len_and_sockaddr* remoteAddr;           \
+) \
+USE_FEATURE_IPC_SYSLOG( \
+	int shmid; /* ipc shared memory id */   \
+	int s_semid; /* ipc semaphore id */     \
+	int shm_size;                           \
+	struct sembuf SMwup[1];                 \
+	struct sembuf SMwdn[3];                 \
+)
+
+struct init_globals {
+	GLOBALS
+};
+
+struct globals {
+	GLOBALS
+#if ENABLE_FEATURE_IPC_SYSLOG
+	struct shbuf_ds *shbuf;
+#endif
+	time_t last_log_time;
+	/* localhost's name */
+	char localHostName[64];
+
+	/* We recv into recvbuf... */
+	char recvbuf[MAX_READ];
+	/* ...then copy to parsebuf, escaping control chars */
+	/* (can grow x2 max) */
+	char parsebuf[MAX_READ*2];
+	/* ...then sprintf into printbuf, adding timestamp (15 chars),
+	 * host (64), fac.prio (20) to the message */
+	/* (growth by: 15 + 64 + 20 + delims = ~110) */
+	char printbuf[MAX_READ*2 + 128];
+};
+
+static const struct init_globals init_data = {
+	.logFilePath = "/var/log/messages",
+	.logFD = -1,
+#ifdef SYSLOGD_MARK
+	.markInterval = 20 * 60,
+#endif
+	.logLevel = 8,
+#if ENABLE_FEATURE_ROTATE_LOGFILE
+	.logFileSize = 200 * 1024,
+	.logFileRotate = 1,
+#endif
+#if ENABLE_FEATURE_REMOTE_LOG
+	.remoteFD = -1,
+#endif
+#if ENABLE_FEATURE_IPC_SYSLOG
+	.shmid = -1,
+	.s_semid = -1,
+	.shm_size = ((CONFIG_FEATURE_IPC_SYSLOG_BUFFER_SIZE)*1024), // default shm size
+	.SMwup = { {1, -1, IPC_NOWAIT} },
+	.SMwdn = { {0, 0}, {1, 0}, {1, +1} },
+#endif
+};
+
+#define G (*ptr_to_globals)
 
 
 /* Options */
@@ -107,7 +165,7 @@ enum {
 	USE_FEATURE_ROTATE_LOGFILE(,*opt_b) \
 	USE_FEATURE_REMOTE_LOG(    ,*opt_R) \
 	USE_FEATURE_IPC_SYSLOG(    ,*opt_C = NULL)
-#define OPTION_PARAM &opt_m, &logFilePath, &opt_l \
+#define OPTION_PARAM &opt_m, &G.logFilePath, &opt_l \
 	USE_FEATURE_ROTATE_LOGFILE(,&opt_s) \
 	USE_FEATURE_ROTATE_LOGFILE(,&opt_b) \
 	USE_FEATURE_REMOTE_LOG(    ,&opt_R) \
@@ -117,68 +175,52 @@ enum {
 /* circular buffer variables/structures */
 #if ENABLE_FEATURE_IPC_SYSLOG
 
-
 #if CONFIG_FEATURE_IPC_SYSLOG_BUFFER_SIZE < 4
 #error Sorry, you must set the syslogd buffer size to at least 4KB.
 #error Please check CONFIG_FEATURE_IPC_SYSLOG_BUFFER_SIZE
 #endif
 
-#include <sys/ipc.h>
-#include <sys/sem.h>
-#include <sys/shm.h>
-
 /* our shared key */
 #define KEY_ID ((long)0x414e4547) /* "GENA" */
 
-// Semaphore operation structures
-static struct shbuf_ds {
-	int32_t size;   // size of data written
-	int32_t head;   // start of message list
-	int32_t tail;   // end of message list
-	char data[1];   // data/messages
-} *shbuf;               // shared memory pointer
-
-static int shmid = -1;   // ipc shared memory id
-static int s_semid = -1; // ipc semaphore id
-static int shm_size = ((CONFIG_FEATURE_IPC_SYSLOG_BUFFER_SIZE)*1024);	// default shm size
-
 static void ipcsyslog_cleanup(void)
 {
-	if (shmid != -1) {
-		shmdt(shbuf);
+	if (G.shmid != -1) {
+		shmdt(G.shbuf);
 	}
-	if (shmid != -1) {
-		shmctl(shmid, IPC_RMID, NULL);
+	if (G.shmid != -1) {
+		shmctl(G.shmid, IPC_RMID, NULL);
 	}
-	if (s_semid != -1) {
-		semctl(s_semid, 0, IPC_RMID, 0);
+	if (G.s_semid != -1) {
+		semctl(G.s_semid, 0, IPC_RMID, 0);
 	}
 }
 
 static void ipcsyslog_init(void)
 {
 	if (DEBUG)
-		printf("shmget(%lx, %d,...)\n", KEY_ID, shm_size);
+		printf("shmget(%lx, %d,...)\n", KEY_ID, G.shm_size);
 
-	shmid = shmget(KEY_ID, shm_size, IPC_CREAT | 1023);
-	if (shmid == -1) {
+	G.shmid = shmget(KEY_ID, G.shm_size, IPC_CREAT | 0644);
+	if (G.shmid == -1) {
 		bb_perror_msg_and_die("shmget");
 	}
 
-	shbuf = shmat(shmid, NULL, 0);
-	if (!shbuf) {
+	G.shbuf = shmat(G.shmid, NULL, 0);
+	if (!G.shbuf) {
 		bb_perror_msg_and_die("shmat");
 	}
 
-	shbuf->size = shm_size - offsetof(struct shbuf_ds, data);
-	shbuf->head = shbuf->tail = 0;
+	memset(G.shbuf, 0, G.shm_size);
+	G.shbuf->size = G.shm_size - offsetof(struct shbuf_ds, data) - 1;
+	/*G.shbuf->tail = 0;*/
 
 	// we'll trust the OS to set initial semval to 0 (let's hope)
-	s_semid = semget(KEY_ID, 2, IPC_CREAT | IPC_EXCL | 1023);
-	if (s_semid == -1) {
+	G.s_semid = semget(KEY_ID, 2, IPC_CREAT | IPC_EXCL | 1023);
+	if (G.s_semid == -1) {
 		if (errno == EEXIST) {
-			s_semid = semget(KEY_ID, 2, 0);
-			if (s_semid != -1)
+			G.s_semid = semget(KEY_ID, 2, 0);
+			if (G.s_semid != -1)
 				return;
 		}
 		bb_perror_msg_and_die("semget");
@@ -188,75 +230,41 @@ static void ipcsyslog_init(void)
 /* Write message to shared mem buffer */
 static void log_to_shmem(const char *msg, int len)
 {
-	/* Why libc insists on these being rw? */
-	static struct sembuf SMwup[1] = { {1, -1, IPC_NOWAIT} };
-	static struct sembuf SMwdn[3] = { {0, 0}, {1, 0}, {1, +1} };
-
 	int old_tail, new_tail;
-	char *c;
 
-	if (semop(s_semid, SMwdn, 3) == -1) {
+	if (semop(G.s_semid, G.SMwdn, 3) == -1) {
 		bb_perror_msg_and_die("SMwdn");
 	}
 
 	/* Circular Buffer Algorithm:
 	 * --------------------------
 	 * tail == position where to store next syslog message.
-	 * head == position of next message to retrieve ("print").
-	 * if head == tail, there is no "unprinted" messages left.
-	 * head is typically advanced by separate "reader" program,
-	 * but if there isn't one, we have to do it ourself.
-	 * messages are NUL-separated.
+	 * tail's max value is (shbuf->size - 1)
+	 * Last byte of buffer is never used and remains NUL.
 	 */
 	len++; /* length with NUL included */
  again:
-	old_tail = shbuf->tail;
+	old_tail = G.shbuf->tail;
 	new_tail = old_tail + len;
-	if (new_tail < shbuf->size) {
-		/* No need to move head if shbuf->head <= old_tail,
-		 * else... */
-		if (old_tail < shbuf->head && shbuf->head <= new_tail) {
-			/* ...need to move head forward */
-			c = memchr(shbuf->data + new_tail, '\0',
-					   shbuf->size - new_tail);
-			if (!c) /* no NUL ahead of us, wrap around */
-				c = memchr(shbuf->data, '\0', old_tail);
-			if (!c) { /* still nothing? point to this msg... */
-				shbuf->head = old_tail;
-			} else {
-				/* convert pointer to offset + skip NUL */
-				shbuf->head = c - shbuf->data + 1;
-			}
-		}
+	if (new_tail < G.shbuf->size) {
 		/* store message, set new tail */
-		memcpy(shbuf->data + old_tail, msg, len);
-		shbuf->tail = new_tail;
+		memcpy(G.shbuf->data + old_tail, msg, len);
+		G.shbuf->tail = new_tail;
 	} else {
-		/* we need to break up the message and wrap it around */
 		/* k == available buffer space ahead of old tail */
-		int k = shbuf->size - old_tail - 1;
-		if (shbuf->head > old_tail) {
-			/* we are going to overwrite head, need to
-			 * move it out of the way */
-			c = memchr(shbuf->data, '\0', old_tail);
-			if (!c) { /* nothing? point to this msg... */
-				shbuf->head = old_tail;
-			} else { /* convert pointer to offset + skip NUL */
-				shbuf->head = c - shbuf->data + 1;
-			}
-		}
+		int k = G.shbuf->size - old_tail;
 		/* copy what fits to the end of buffer, and repeat */
-		memcpy(shbuf->data + old_tail, msg, k);
+		memcpy(G.shbuf->data + old_tail, msg, k);
 		msg += k;
 		len -= k;
-		shbuf->tail = 0;
+		G.shbuf->tail = 0;
 		goto again;
 	}
-	if (semop(s_semid, SMwup, 1) == -1) {
+	if (semop(G.s_semid, G.SMwup, 1) == -1) {
 		bb_perror_msg_and_die("SMwup");
 	}
 	if (DEBUG)
-		printf("head:%d tail:%d\n", shbuf->head, shbuf->tail);
+		printf("tail:%d\n", G.shbuf->tail);
 }
 #else
 void ipcsyslog_cleanup(void);
@@ -268,29 +276,28 @@ void log_to_shmem(const char *msg);
 /* Print a message to the log file. */
 static void log_locally(char *msg)
 {
-	static time_t last;
 	struct flock fl;
 	int len = strlen(msg);
 
 #if ENABLE_FEATURE_IPC_SYSLOG
-	if ((option_mask32 & OPT_circularlog) && shbuf) {
+	if ((option_mask32 & OPT_circularlog) && G.shbuf) {
 		log_to_shmem(msg, len);
 		return;
 	}
 #endif
-	if (logFD >= 0) {
+	if (G.logFD >= 0) {
 		time_t cur;
 		time(&cur);
-		if (last != cur) {
-			last = cur; /* reopen log file every second */
-			close(logFD);
+		if (G.last_log_time != cur) {
+			G.last_log_time = cur; /* reopen log file every second */
+			close(G.logFD);
 			goto reopen;
 		}
 	} else {
  reopen:
-		logFD = device_open(logFilePath, O_WRONLY | O_CREAT
+		G.logFD = device_open(G.logFilePath, O_WRONLY | O_CREAT
 					| O_NOCTTY | O_APPEND | O_NONBLOCK);
-		if (logFD < 0) {
+		if (G.logFD < 0) {
 			/* cannot open logfile? - print to /dev/console then */
 			int fd = device_open(_PATH_CONSOLE, O_WRONLY | O_NOCTTY | O_NONBLOCK);
 			if (fd < 0)
@@ -302,11 +309,10 @@ static void log_locally(char *msg)
 		}
 #if ENABLE_FEATURE_ROTATE_LOGFILE
 		{
-		struct stat statf;
-
-		isRegular = (fstat(logFD, &statf) == 0 && (statf.st_mode & S_IFREG));
-		/* bug (mostly harmless): can wrap around if file > 4gb */
-		curFileSize = statf.st_size;
+			struct stat statf;
+			G.isRegular = (fstat(G.logFD, &statf) == 0 && S_ISREG(statf.st_mode));
+			/* bug (mostly harmless): can wrap around if file > 4gb */
+			G.curFileSize = statf.st_size;
 		}
 #endif
 	}
@@ -315,41 +321,41 @@ static void log_locally(char *msg)
 	fl.l_start = 0;
 	fl.l_len = 1;
 	fl.l_type = F_WRLCK;
-	fcntl(logFD, F_SETLKW, &fl);
+	fcntl(G.logFD, F_SETLKW, &fl);
 
 #if ENABLE_FEATURE_ROTATE_LOGFILE
-	if (logFileSize && isRegular && curFileSize > logFileSize) {
-		if (logFileRotate) { /* always 0..99 */
-			int i = strlen(logFilePath) + 3 + 1;
+	if (G.logFileSize && G.isRegular && G.curFileSize > G.logFileSize) {
+		if (G.logFileRotate) { /* always 0..99 */
+			int i = strlen(G.logFilePath) + 3 + 1;
 			char oldFile[i];
 			char newFile[i];
-			i = logFileRotate - 1;
+			i = G.logFileRotate - 1;
 			/* rename: f.8 -> f.9; f.7 -> f.8; ... */
 			while (1) {
-				sprintf(newFile, "%s.%d", logFilePath, i);
+				sprintf(newFile, "%s.%d", G.logFilePath, i);
 				if (i == 0) break;
-				sprintf(oldFile, "%s.%d", logFilePath, --i);
+				sprintf(oldFile, "%s.%d", G.logFilePath, --i);
 				rename(oldFile, newFile);
 			}
 			/* newFile == "f.0" now */
-			rename(logFilePath, newFile);
+			rename(G.logFilePath, newFile);
 			fl.l_type = F_UNLCK;
-			fcntl(logFD, F_SETLKW, &fl);
-			close(logFD);
+			fcntl(G.logFD, F_SETLKW, &fl);
+			close(G.logFD);
 			goto reopen;
 		}
-		ftruncate(logFD, 0);
+		ftruncate(G.logFD, 0);
 	}
-	curFileSize +=
+	G.curFileSize +=
 #endif
-	                full_write(logFD, msg, len);
+	                full_write(G.logFD, msg, len);
 	fl.l_type = F_UNLCK;
-	fcntl(logFD, F_SETLKW, &fl);
+	fcntl(G.logFD, F_SETLKW, &fl);
 }
 
 static void parse_fac_prio_20(int pri, char *res20)
 {
-	CODE *c_pri, *c_fac;
+	const CODE *c_pri, *c_fac;
 
 	if (pri != 0) {
 		c_fac = facilitynames;
@@ -395,15 +401,15 @@ static void timestamp_and_log(int pri, char *msg, int len)
 
 	/* Log message locally (to file or shared mem) */
 	if (!ENABLE_FEATURE_REMOTE_LOG || (option_mask32 & OPT_locallog)) {
-		if (LOG_PRI(pri) < logLevel) {
+		if (LOG_PRI(pri) < G.logLevel) {
 			if (option_mask32 & OPT_small)
-				sprintf(PRINTBUF, "%s %s\n", timestamp, msg);
+				sprintf(G.printbuf, "%s %s\n", timestamp, msg);
 			else {
 				char res[20];
 				parse_fac_prio_20(pri, res);
-				sprintf(PRINTBUF, "%s %s %s %s\n", timestamp, localHostName, res, msg);
+				sprintf(G.printbuf, "%s %s %s %s\n", timestamp, G.localHostName, res, msg);
 			}
-			log_locally(PRINTBUF);
+			log_locally(G.printbuf);
 		}
 	}
 }
@@ -415,7 +421,7 @@ static void split_escape_and_log(char *tmpbuf, int len)
 	tmpbuf += len;
 	while (p < tmpbuf) {
 		char c;
-		char *q = PARSEBUF;
+		char *q = G.parsebuf;
 		int pri = (LOG_USER | LOG_NOTICE);
 
 		if (*p == '<') {
@@ -430,7 +436,7 @@ static void split_escape_and_log(char *tmpbuf, int len)
 		while ((c = *p++)) {
 			if (c == '\n')
 				c = ' ';
-			if (!(c & ~0x1f)) {
+			if (!(c & ~0x1f) && c != '\t') {
 				*q++ = '^';
 				c += '@'; /* ^@, ^A, ^B... */
 			}
@@ -438,35 +444,36 @@ static void split_escape_and_log(char *tmpbuf, int len)
 		}
 		*q = '\0';
 		/* Now log it */
-		timestamp_and_log(pri, PARSEBUF, q - PARSEBUF);
+		timestamp_and_log(pri, G.parsebuf, q - G.parsebuf);
 	}
 }
 
 static void quit_signal(int sig)
 {
-	timestamp_and_log(LOG_SYSLOG | LOG_INFO, "syslogd exiting", 0);
+	timestamp_and_log(LOG_SYSLOG | LOG_INFO, (char*)"syslogd exiting", 0);
 	puts("syslogd exiting");
-	unlink(dev_log_name);
 	if (ENABLE_FEATURE_IPC_SYSLOG)
 		ipcsyslog_cleanup();
 	exit(1);
 }
 
+#ifdef SYSLOGD_MARK
 static void do_mark(int sig)
 {
-	if (markInterval) {
-		timestamp_and_log(LOG_SYSLOG | LOG_INFO, "-- MARK --", 0);
-		alarm(markInterval);
+	if (G.markInterval) {
+		timestamp_and_log(LOG_SYSLOG | LOG_INFO, (char*)"-- MARK --", 0);
+		alarm(G.markInterval);
 	}
 }
+#endif
 
 static void do_syslogd(void) ATTRIBUTE_NORETURN;
 static void do_syslogd(void)
 {
 	struct sockaddr_un sunx;
-	socklen_t addr_len;
 	int sock_fd;
 	fd_set fds;
+	char *dev_log_name;
 
 	/* Set up signal handlers */
 	signal(SIGINT, quit_signal);
@@ -477,31 +484,46 @@ static void do_syslogd(void)
 #ifdef SIGCLD
 	signal(SIGCLD, SIG_IGN);
 #endif
+#ifdef SYSLOGD_MARK
 	signal(SIGALRM, do_mark);
-	alarm(markInterval);
-
-	dev_log_name = xmalloc_realpath(_PATH_LOG);
-	if (!dev_log_name)
-		dev_log_name = _PATH_LOG;
-
-	/* Unlink old /dev/log (or object it points to) */
-	unlink(dev_log_name);
+	alarm(G.markInterval);
+#endif
+	remove_pidfile("/var/run/syslogd.pid");
 
 	memset(&sunx, 0, sizeof(sunx));
 	sunx.sun_family = AF_UNIX;
-	strncpy(sunx.sun_path, dev_log_name, sizeof(sunx.sun_path));
-	sock_fd = xsocket(AF_UNIX, SOCK_DGRAM, 0);
-	addr_len = sizeof(sunx.sun_family) + strlen(sunx.sun_path);
-	xbind(sock_fd, (struct sockaddr *) &sunx, addr_len);
+	strcpy(sunx.sun_path, "/dev/log");
 
-	if (chmod(dev_log_name, 0666) < 0) {
-		bb_perror_msg_and_die("cannot set permission on %s", dev_log_name);
+	/* Unlink old /dev/log or object it points to. */
+	/* (if it exists, bind will fail) */
+	logmode = LOGMODE_NONE;
+	dev_log_name = xmalloc_readlink_or_warn("/dev/log");
+	logmode = LOGMODE_STDIO;
+	if (dev_log_name) {
+		int fd = xopen(".", O_NONBLOCK);
+		xchdir("/dev");
+		/* we do not check whether this is a link also */
+		unlink(dev_log_name);
+		fchdir(fd);
+		close(fd);
+		safe_strncpy(sunx.sun_path, dev_log_name, sizeof(sunx.sun_path));
+		free(dev_log_name);
+	} else {
+		unlink("/dev/log");
+	}
+
+	sock_fd = xsocket(AF_UNIX, SOCK_DGRAM, 0);
+	xbind(sock_fd, (struct sockaddr *) &sunx, sizeof(sunx));
+
+	if (chmod("/dev/log", 0666) < 0) {
+		bb_perror_msg_and_die("cannot set permission on /dev/log");
 	}
 	if (ENABLE_FEATURE_IPC_SYSLOG && (option_mask32 & OPT_circularlog)) {
 		ipcsyslog_init();
 	}
 
-	timestamp_and_log(LOG_SYSLOG | LOG_INFO, "syslogd started: BusyBox v" BB_VER, 0);
+	timestamp_and_log(LOG_SYSLOG | LOG_INFO,
+			(char*)"syslogd started: BusyBox v" BB_VER, 0);
 
 	for (;;) {
 		FD_ZERO(&fds);
@@ -517,60 +539,65 @@ static void do_syslogd(void)
 
 		if (FD_ISSET(sock_fd, &fds)) {
 			int i;
-			i = recv(sock_fd, RECVBUF, MAX_READ - 1, 0);
+			i = recv(sock_fd, G.recvbuf, MAX_READ - 1, 0);
 			if (i <= 0)
 				bb_perror_msg_and_die("UNIX socket error");
 			/* TODO: maybe suppress duplicates? */
 #if ENABLE_FEATURE_REMOTE_LOG
 			/* We are not modifying log messages in any way before send */
 			/* Remote site cannot trust _us_ anyway and need to do validation again */
-			if (remoteAddr) {
-				if (-1 == remoteFD) {
-					remoteFD = socket(remoteAddr->sa.sa_family, SOCK_DGRAM, 0);
+			if (G.remoteAddr) {
+				if (-1 == G.remoteFD) {
+					G.remoteFD = socket(G.remoteAddr->sa.sa_family, SOCK_DGRAM, 0);
 				}
-				if (-1 != remoteFD) {
+				if (-1 != G.remoteFD) {
 					/* send message to remote logger, ignore possible error */
-					sendto(remoteFD, RECVBUF, i, MSG_DONTWAIT,
-						&remoteAddr->sa, remoteAddr->len);
+					sendto(G.remoteFD, G.recvbuf, i, MSG_DONTWAIT,
+						&G.remoteAddr->sa, G.remoteAddr->len);
 				}
 			}
 #endif
-			RECVBUF[i] = '\0';
-			split_escape_and_log(RECVBUF, i);
+			G.recvbuf[i] = '\0';
+			split_escape_and_log(G.recvbuf, i);
 		} /* FD_ISSET() */
 	} /* for */
 }
 
+int syslogd_main(int argc, char **argv);
 int syslogd_main(int argc, char **argv)
 {
 	char OPTION_DECL;
 	char *p;
 
+	PTR_TO_GLOBALS = memcpy(xzalloc(sizeof(G)), &init_data, sizeof(init_data));
+
 	/* do normal option parsing */
 	opt_complementary = "=0"; /* no non-option params */
-	getopt32(argc, argv, OPTION_STR, OPTION_PARAM);
+	getopt32(argv, OPTION_STR, OPTION_PARAM);
+#ifdef SYSLOGD_MARK
 	if (option_mask32 & OPT_mark) // -m
-		markInterval = xatou_range(opt_m, 0, INT_MAX/60) * 60;
+		G.markInterval = xatou_range(opt_m, 0, INT_MAX/60) * 60;
+#endif
 	//if (option_mask32 & OPT_nofork) // -n
 	//if (option_mask32 & OPT_outfile) // -O
 	if (option_mask32 & OPT_loglevel) // -l
-		logLevel = xatou_range(opt_l, 1, 8);
+		G.logLevel = xatou_range(opt_l, 1, 8);
 	//if (option_mask32 & OPT_small) // -S
 #if ENABLE_FEATURE_ROTATE_LOGFILE
 	if (option_mask32 & OPT_filesize) // -s
-		logFileSize = xatou_range(opt_s, 0, INT_MAX/1024) * 1024;
+		G.logFileSize = xatou_range(opt_s, 0, INT_MAX/1024) * 1024;
 	if (option_mask32 & OPT_rotatecnt) // -b
-		logFileRotate = xatou_range(opt_b, 0, 99);
+		G.logFileRotate = xatou_range(opt_b, 0, 99);
 #endif
 #if ENABLE_FEATURE_REMOTE_LOG
 	if (option_mask32 & OPT_remotelog) { // -R
-		remoteAddr = host2sockaddr(opt_R, 514);
+		G.remoteAddr = xhost2sockaddr(opt_R, 514);
 	}
 	//if (option_mask32 & OPT_locallog) // -L
 #endif
 #if ENABLE_FEATURE_IPC_SYSLOG
 	if (opt_C) // -Cn
-		shm_size = xatoul_range(opt_C, 4, INT_MAX/1024) * 1024;
+		G.shm_size = xatoul_range(opt_C, 4, INT_MAX/1024) * 1024;
 #endif
 
 	/* If they have not specified remote logging, then log locally */
@@ -578,20 +605,17 @@ int syslogd_main(int argc, char **argv)
 		option_mask32 |= OPT_locallog;
 
 	/* Store away localhost's name before the fork */
-	gethostname(localHostName, sizeof(localHostName));
-	p = strchr(localHostName, '.');
+	gethostname(G.localHostName, sizeof(G.localHostName));
+	p = strchr(G.localHostName, '.');
 	if (p) {
 		*p = '\0';
 	}
 
 	if (!(option_mask32 & OPT_nofork)) {
-#ifdef BB_NOMMU
-		vfork_daemon_rexec(0, 1, argc, argv, "-n");
-#else
-		bb_daemonize();
-#endif
+		bb_daemonize_or_rexec(DAEMON_CHDIR_ROOT, argv);
 	}
 	umask(0);
+	write_pidfile("/var/run/syslogd.pid");
 	do_syslogd();
 	/* return EXIT_SUCCESS; */
 }
